@@ -63,14 +63,28 @@ namespace IKVM.Reflection.Emit
 		private readonly List<Module> addedModules = new List<Module>();
 		private readonly List<CustomAttributeBuilder> customAttributes = new List<CustomAttributeBuilder>();
 		private readonly List<CustomAttributeBuilder> declarativeSecurity = new List<CustomAttributeBuilder>();
-		private readonly List<Type> typeForwarders = new List<Type>();
+		private readonly List<TypeForwarder> typeForwarders = new List<TypeForwarder>();
+
+		struct TypeForwarder
+		{
+			internal readonly Type Type;
+			internal readonly bool IncludeNested;
+
+			internal TypeForwarder(Type type, bool includeNested)
+			{
+				this.Type = type;
+				this.IncludeNested = includeNested;
+			}
+		}
 
 		private struct ResourceFile
 		{
 			internal string Name;
 			internal string FileName;
 			internal ResourceAttributes Attributes;
+#if !CORECLR
 			internal ResourceWriter Writer;
+#endif
 		}
 
 		internal AssemblyBuilder(Universe universe, AssemblyName name, string dir, IEnumerable<CustomAttributeBuilder> customAttributes)
@@ -265,7 +279,12 @@ namespace IKVM.Reflection.Emit
 
 		public void __AddTypeForwarder(Type type)
 		{
-			typeForwarders.Add(type);
+			__AddTypeForwarder(type, true);
+		}
+
+		public void __AddTypeForwarder(Type type, bool includeNested)
+		{
+			typeForwarders.Add(new TypeForwarder(type, includeNested));
 		}
 
 		public void SetEntryPoint(MethodInfo entryMethod)
@@ -356,9 +375,9 @@ namespace IKVM.Reflection.Emit
 				foreach (CustomAttributeBuilder cab in customAttributes)
 				{
 					// .NET doesn't support copying blob custom attributes into the version info
-					if (!cab.HasBlob)
+					if (!cab.HasBlob || universe.DecodeVersionInfoAttributeBlobs)
 					{
-						versionInfo.SetAttribute(cab);
+						versionInfo.SetAttribute(this, cab);
 					}
 				}
 				ByteBuffer versionInfoData = new ByteBuffer(512);
@@ -389,18 +408,20 @@ namespace IKVM.Reflection.Emit
 
 			manifestModule.AddDeclarativeSecurity(0x20000001, declarativeSecurity);
 
-			foreach (Type type in typeForwarders)
+			foreach (TypeForwarder fwd in typeForwarders)
 			{
-				manifestModule.AddTypeForwarder(type);
+				manifestModule.AddTypeForwarder(fwd.Type, fwd.IncludeNested);
 			}
 
 			foreach (ResourceFile resfile in resourceFiles)
 			{
+#if !CORECLR
 				if (resfile.Writer != null)
 				{
 					resfile.Writer.Generate();
 					resfile.Writer.Close();
 				}
+#endif
 				int fileToken = AddFile(manifestModule, resfile.FileName, 1 /*ContainsNoMetaData*/);
 				ManifestResourceTable.Record rec = new ManifestResourceTable.Record();
 				rec.Offset = 0;
@@ -431,6 +452,7 @@ namespace IKVM.Reflection.Emit
 					}
 					moduleBuilder.ExportTypes(fileToken, manifestModule);
 				}
+				moduleBuilder.CloseResources();
 			}
 
 			foreach (Module module in addedModules)
@@ -481,6 +503,7 @@ namespace IKVM.Reflection.Emit
 			resourceFiles.Add(resfile);
 		}
 
+#if !CORECLR
 		public IResourceWriter DefineResource(string name, string description, string fileName)
 		{
 			return DefineResource(name, description, fileName, ResourceAttributes.Public);
@@ -504,6 +527,7 @@ namespace IKVM.Reflection.Emit
 			resourceFiles.Add(resfile);
 			return rw;
 		}
+#endif
 
 		public void DefineVersionInfoResource()
 		{
