@@ -515,16 +515,17 @@ namespace System.Net.Security {
                             uint count = issuerList.cIssuers;
                             issuers = new string[issuerList.cIssuers];
                             _CERT_CHAIN_ELEMENT* pIL = (_CERT_CHAIN_ELEMENT*)issuerList.aIssuers.DangerousGetHandle();
-                            for (int i =0; i<count; ++i) {
+                            for (uint i =0; i<count; ++i) {
                                 _CERT_CHAIN_ELEMENT* pIL2 = pIL + i;
                                 uint size = pIL2->cbSize;
                                 byte* ptr = (byte*)(pIL2->pCertContext);
                                 byte[] x = new byte[size];
-                                for (int j=0; j<size; j++) {
+                                for (uint j=0; j<size; j++) {
                                     x[j] = *(ptr + j);
                                 }
                                 // Oid oid = new Oid();
                                 // oid.Value = "1.3.6.1.5.5.7.3.2";
+                                // Value of issuers[i] can be an empty string when size of x is 0.
                                 X500DistinguishedName x500DistinguishedName = new X500DistinguishedName(x);
                                 issuers[i] = x500DistinguishedName.Name;
                                 GlobalLog.Print("SecureChannel#" + ValidationHelper.HashString(this) + "::GetIssuers() IssuerListEx[" + i + "]:" + issuers[i]);
@@ -795,7 +796,8 @@ namespace System.Net.Security {
                 {
                     SecureCredential.Flags flags = SecureCredential.Flags.ValidateManual | SecureCredential.Flags.NoDefaultCred;
                     if (!ServicePointManager.DisableStrongCrypto 
-                        && ((m_ProtocolFlags & (SchProtocols.Tls10 | SchProtocols.Tls11 | SchProtocols.Tls12)) != 0))
+                        && ((m_ProtocolFlags & (SchProtocols.Tls10 | SchProtocols.Tls11 | SchProtocols.Tls12)) != 0)
+                        && (m_EncryptionPolicy != EncryptionPolicy.AllowNoEncryption) && (m_EncryptionPolicy != EncryptionPolicy.NoEncryption))
                     {
                         flags |= SecureCredential.Flags.UseStrongCrypto;
                     }
@@ -1112,6 +1114,7 @@ namespace System.Net.Security {
             GlobalLog.Enter("SecureChannel#" + ValidationHelper.HashString(this) + "::ProcessHandshakeSuccess");
 #if MONO
             m_HeaderSize = m_TrailerSize = 0;
+            m_ConnectionInfo = SSPIWrapper.GetConnectionInfo(m_SecModule, m_SecurityContext);
 #else
             StreamSizes streamSizes = SSPIWrapper.QueryContextAttributes(m_SecModule, m_SecurityContext, ContextAttribute.StreamSizes) as StreamSizes;
             if (streamSizes != null) {
@@ -1296,13 +1299,6 @@ namespace System.Net.Security {
             // we don't catch exceptions in this method, so it's safe for "accepted" be initialized with true
             bool success = false;
 
-#if MONO
-            if (IsServer)
-                // FIXME
-                success = true;
-            else
-                success = SSPIWrapper.CheckRemoteCertificate(m_SecurityContext);
-#else
             X509Chain chain = null;
             X509Certificate2 remoteCertificateEx = null;
 
@@ -1316,6 +1312,7 @@ namespace System.Net.Security {
                     GlobalLog.Leave("SecureChannel#" + ValidationHelper.HashString(this) + "::VerifyRemoteCertificate (no remote cert)", (!m_RemoteCertRequired).ToString());
                     sslPolicyErrors |= SslPolicyErrors.RemoteCertificateNotAvailable;
                 }
+#if !MONO
                 else
                 {
                     chain = new X509Chain();
@@ -1368,6 +1365,9 @@ namespace System.Net.Security {
                     else
                         success = (sslPolicyErrors == SslPolicyErrors.None);
                 }
+#else
+		success = SSPIWrapper.CheckRemoteCertificate(m_SecurityContext);
+#endif
 
                 if (Logging.On) {
                     if (sslPolicyErrors != SslPolicyErrors.None)
@@ -1406,7 +1406,6 @@ namespace System.Net.Security {
                     remoteCertificateEx.Reset();
             }
             GlobalLog.Leave("SecureChannel#" + ValidationHelper.HashString(this) + "::VerifyRemoteCertificate", success.ToString());
-#endif
             return success;
         }
 
