@@ -32,7 +32,7 @@ namespace System.Globalization {
 
     [Serializable]
     [System.Runtime.InteropServices.ComVisible(true)]
-    public class TextInfo : ICloneable, IDeserializationCallback
+    public partial class TextInfo : ICloneable, IDeserializationCallback
     {
         //--------------------------------------------------------------------//
         //                        Internal Information                        //
@@ -73,8 +73,10 @@ namespace System.Globalization {
         private String                          m_cultureName;      // Name of the culture that created this text info
         [NonSerialized]private CultureData      m_cultureData;      // Data record for the culture that made us, not for this textinfo
         [NonSerialized]private String           m_textInfoName;     // Name of the text info we're using (ie: m_cultureData.STEXTINFO)
+#if !MONO
         [NonSerialized]private IntPtr           m_dataHandle;       // Sort handle
         [NonSerialized]private IntPtr           m_handleOrigin;
+#endif
         [NonSerialized]private bool?            m_IsAsciiCasingSameAsInvariant;
 
 
@@ -103,7 +105,7 @@ namespace System.Globalization {
             this.m_cultureData = cultureData;
             this.m_cultureName = this.m_cultureData.CultureName;
             this.m_textInfoName = this.m_cultureData.STEXTINFO;
-#if !FEATURE_CORECLR
+#if !FEATURE_CORECLR && !MONO
             IntPtr handleOrigin;
             this.m_dataHandle = CompareInfo.InternalInitSortHandle(m_textInfoName, out handleOrigin);
             this.m_handleOrigin = handleOrigin;
@@ -178,7 +180,7 @@ namespace System.Globalization {
                 // Get the text info name belonging to that culture
                 this.m_cultureData = CultureInfo.GetCultureInfo(m_cultureName).m_cultureData;
                 this.m_textInfoName = this.m_cultureData.STEXTINFO;
-#if !FEATURE_CORECLR
+#if !FEATURE_CORECLR && !MONO
                 IntPtr handleOrigin;
                 this.m_dataHandle = CompareInfo.InternalInitSortHandle(m_textInfoName, out handleOrigin);
                 this.m_handleOrigin = handleOrigin;
@@ -231,7 +233,7 @@ namespace System.Globalization {
             // (not necessarily true for sorting, but OK for casing & then we apply normal hash code rules)
             return (Invariant.GetCaseInsensitiveHashCode(s, forceRandomizedHashing, additionalEntropy));
         }
-
+#if !MONO
         [System.Security.SecuritySafeCritical]
         [ResourceExposure(ResourceScope.None)]
         [ResourceConsumption(ResourceScope.Process, ResourceScope.Process)]
@@ -239,7 +241,7 @@ namespace System.Globalization {
         {
             return InternalTryFindStringOrdinalIgnoreCase(searchFlags, source, count, startIndex, value, value.Length, ref foundIndex);
         }
-
+#endif
         // This function doesn't check arguments. Please do check in the caller.
         // The underlying unmanaged code will assert the sanity of arguments.
         [System.Security.SecuritySafeCritical]  // auto-generated
@@ -247,14 +249,8 @@ namespace System.Globalization {
         [ResourceConsumption(ResourceScope.Process, ResourceScope.Process)]
         internal static unsafe int CompareOrdinalIgnoreCase(String str1, String str2)
         {
-#if __APPLE__
-            // ToUpper (invariant) here before going to the PAL since Mac OS 10.4 does this step
-            // wrong (CFCompareString uses the current OS locale and does not let you specify a specific locale)
-            return String.CompareOrdinal(str1.ToUpper(CultureInfo.InvariantCulture), str2.ToUpper(CultureInfo.InvariantCulture));
-#else
             // Compare the whole string and ignore case.
             return InternalCompareStringOrdinalIgnoreCase(str1, 0, str2, 0, str1.Length, str2.Length);
-#endif
         }
 
         // This function doesn't check arguments. Please do check in the caller.
@@ -266,13 +262,7 @@ namespace System.Globalization {
         {
             Contract.Assert(strA.Length >= indexA + lengthA,  "[TextInfo.CompareOrdinalIgnoreCaseEx] Caller should've validated strA.Length >= indexA + lengthA");
             Contract.Assert(strB.Length >= indexB + lengthB, "[TextInfo.CompareOrdinalIgnoreCaseEx]  Caller should've validated strB.Length >= indexB + lengthB");
-#if __APPLE__
-            // ToUpper (invariant) here before going to the PAL since Mac OS 10.4 does this step
-            // wrong (CFCompareString uses the current OS locale and does not let you specify a specific locale)
-            return String.CompareOrdinal(strA.ToUpper(CultureInfo.InvariantCulture), indexA, strB.ToUpper(CultureInfo.InvariantCulture), indexB, Math.Max(lengthA, lengthB));
-#else
             return InternalCompareStringOrdinalIgnoreCase(strA, indexA, strB, indexB, lengthA, lengthB);
-#endif
         }
 
         [ResourceExposure(ResourceScope.None)]
@@ -289,11 +279,8 @@ namespace System.Globalization {
                 return 0;
             }
 
-#if __APPLE__
-            string sourceUpper = source.ToUpper(CultureInfo.InvariantCulture);
-            string valueUpper = value.ToUpper(CultureInfo.InvariantCulture);
-#else
             // fast path
+#if !MONO
             int ret = -1;
             if (TryFastFindStringOrdinalIgnoreCase(Microsoft.Win32.Win32Native.FIND_FROMSTART, source, startIndex, value, count, ref ret))
                 return ret;
@@ -312,14 +299,10 @@ namespace System.Globalization {
 
             for (; startIndex <= maxStartIndex; startIndex++)
             {
-#if __APPLE__
-                if (String.CompareOrdinal(sourceUpper, startIndex, valueUpper, 0, value.Length)==0)
-#else
                 // We should always have the same or more characters left to search than our actual pattern
                 Contract.Assert(end - startIndex >= value.Length);
                 // since this is an ordinal comparison, we can assume that the lengths must match
                 if (CompareOrdinalIgnoreCaseEx(source, startIndex, value, 0, value.Length, value.Length) == 0)
-#endif
                 {
                     return startIndex;
                 }
@@ -329,16 +312,6 @@ namespace System.Globalization {
             return -1;
         }
 
-#if FEATURE_CORECLR
-        private static bool LegacyMode 
-        {
-            get 
-            {            
-                return CompatibilitySwitches.IsAppEarlierThanSilverlight4;
-            }
-        }
-#endif // FEATURE_CORECLR
-
         [ResourceExposure(ResourceScope.None)]
         [ResourceConsumption(ResourceScope.Process, ResourceScope.Process)]
         internal static int LastIndexOfStringOrdinalIgnoreCase(String source, String value, int startIndex, int count)
@@ -349,21 +322,13 @@ namespace System.Globalization {
             Contract.Assert(startIndex <= source.Length, "[TextInfo.LastIndexOfStringOrdinalIgnoreCase] Caller should've validated startIndex <= source.Length");
 
             // If value is Empty, the return value is startIndex
-            // <STRIP>we accidently shipped Silverlight 2 and 3 without this if-check</STRIP>
-            if (value.Length == 0
-#if FEATURE_CORECLR
-                && !LegacyMode
-#endif // FEATURE_CORECLR
-               )
+            if (value.Length == 0)
             {
                 return startIndex;
             }
 
-#if __APPLE__
-            string sourceUpper = source.ToUpper(CultureInfo.InvariantCulture);
-            string valueUpper = value.ToUpper(CultureInfo.InvariantCulture);
-#else
             // fast path
+#if !MONO
             int ret = -1;
             if (TryFastFindStringOrdinalIgnoreCase(Microsoft.Win32.Win32Native.FIND_FROMEND, source, startIndex, value, count, ref ret))
                 return ret;
@@ -382,11 +347,7 @@ namespace System.Globalization {
 
             for (; startIndex >= minIndex; startIndex--)
             {
-#if __APPLE__
-                if (String.CompareOrdinal(sourceUpper, startIndex, valueUpper, 0, value.Length)==0)
-#else
                 if (CompareOrdinalIgnoreCaseEx(source, startIndex, value, 0, value.Length, value.Length) == 0)
-#endif
                 {
                     return startIndex;
                 }
@@ -583,16 +544,17 @@ namespace System.Globalization {
         ////////////////////////////////////////////////////////////////////////
 
         [System.Security.SecuritySafeCritical]  // auto-generated
-#if !FEATURE_CORECLR
-        [TargetedPatchingOptOut("Performance critical to inline across NGen image boundaries")]
-#endif
         public unsafe virtual char ToLower(char c) 
         {
             if(IsAscii(c) && IsAsciiCasingSameAsInvariant)
             {
                 return ToLowerAsciiInvariant(c);
             }
+#if MONO
+            return ToLowerInternal (c);
+#else
             return (InternalChangeCaseChar(this.m_dataHandle, this.m_handleOrigin, this.m_textInfoName, c, false));
+#endif
         }
 
         [System.Security.SecuritySafeCritical]  // auto-generated
@@ -601,30 +563,11 @@ namespace System.Globalization {
             if (str == null) { throw new ArgumentNullException("str"); }
             Contract.EndContractBlock();
 
-            String toLower = InternalChangeCaseString(this.m_dataHandle, this.m_handleOrigin, this.m_textInfoName, str, false);
-#if __APPLE__
-            //
-            // Mac OS X has a documented list of "illegal" (precomposed) characters
-            // http://developer.apple.com/technotes/tn/tn1150table.html
-            //
-            // These characters are mostly in the EXTENDED_GREEK range.  Apple decomposes
-            // these characters into a sequence of two or more characters that is a
-            // canonical or compatibility equivalent.
-            //
-            // In the extremely unlikely event that an illegal character is in the String,
-            // fallback to using slower Char routines since they do not decompose
-            // the illegal characters.   
-            //
-            if (toLower.Length != str.Length) {
-                char[] chars = new char[str.Length];
-                for (int i = 0; i < str.Length; i++) {
-                    chars[i] = this.ToLower(str[i]);
-                }
-                toLower = new String(chars);
-            }
+#if MONO
+            return ToLowerInternal (str);
+#else
+            return InternalChangeCaseString(this.m_dataHandle, this.m_handleOrigin, this.m_textInfoName, str, false);
 #endif
-            return toLower;
-
         }
 
         static private Char ToLowerAsciiInvariant(Char c)
@@ -646,16 +589,18 @@ namespace System.Globalization {
         ////////////////////////////////////////////////////////////////////////
 
         [System.Security.SecuritySafeCritical]  // auto-generated
-#if !FEATURE_CORECLR
-        [TargetedPatchingOptOut("Performance critical to inline across NGen image boundaries")]
-#endif
         public unsafe virtual char ToUpper(char c) 
         {
             if (IsAscii(c) && IsAsciiCasingSameAsInvariant)
             {
                 return ToUpperAsciiInvariant(c);
             }
+
+#if MONO
+            return ToUpperInternal (c);
+#else
             return (InternalChangeCaseChar(this.m_dataHandle, this.m_handleOrigin, this.m_textInfoName, c, true));
+#endif
         }
 
 
@@ -664,29 +609,11 @@ namespace System.Globalization {
         {
             if (str == null) { throw new ArgumentNullException("str"); }
             Contract.EndContractBlock();
-            String toUpper = InternalChangeCaseString(this.m_dataHandle, this.m_handleOrigin, this.m_textInfoName, str, true);
-#if __APPLE__
-            //
-            // Mac OS X has a documented list of "illegal" (precomposed) characters
-            // http://developer.apple.com/technotes/tn/tn1150table.html
-            //
-            // These characters are mostly in the EXTENDED_GREEK range.  Apple decomposes
-            // these characters into a sequence of two or more characters that is a
-            // canonical or compatibility equivalent.
-            //
-            // In the extremely unlikely event that an illegal character is in the String,
-            // fallback to using slower Char routines since they do not decompose
-            // the illegal characters.   
-            //
-            if (toUpper.Length != str.Length) {
-                char[] chars = new char[str.Length];
-                for (int i = 0; i < str.Length; i++) {
-                    chars[i] = this.ToUpper(str[i]);
-                }
-                toUpper = new String(chars);
-            }
+#if MONO
+            return ToUpperInternal (str);
+#else
+            return InternalChangeCaseString(this.m_dataHandle, this.m_handleOrigin, this.m_textInfoName, str, true);
 #endif
-            return toUpper;
         }
 
         static private Char ToUpperAsciiInvariant(Char c)
@@ -709,10 +636,14 @@ namespace System.Globalization {
             {
                 if (m_IsAsciiCasingSameAsInvariant == null)
                 {
+#if MONO
+                    m_IsAsciiCasingSameAsInvariant = !(m_cultureData.SISO639LANGNAME == "az" || m_cultureData.SISO639LANGNAME == "tr");
+#else
                     m_IsAsciiCasingSameAsInvariant =
                         CultureInfo.GetCultureInfo(m_textInfoName).CompareInfo.Compare("abcdefghijklmnopqrstuvwxyz",
                                                                              "ABCDEFGHIJKLMNOPQRSTUVWXYZ",
                                                                              CompareOptions.IgnoreCase) == 0;
+#endif
                 }
                 return (bool)m_IsAsciiCasingSameAsInvariant;
             }
@@ -997,7 +928,6 @@ namespace System.Globalization {
 #endif
 
 
-#if !FEATURE_CORECLR
         // IsRightToLeft
         //
         // Returns true if the dominant direction of text and UI such as the relative position of buttons and scroll bars
@@ -1005,13 +935,11 @@ namespace System.Globalization {
         [System.Runtime.InteropServices.ComVisible(false)]
         public bool IsRightToLeft
         {
-            [TargetedPatchingOptOut("Performance critical to inline across NGen image boundaries")]
             get
             {
                 return this.m_cultureData.IsRightToLeft;
             }
         }
-#endif
 
 #if FEATURE_SERIALIZATION
         /// <internalonly/>
@@ -1044,10 +972,31 @@ namespace System.Globalization {
             }
             Contract.EndContractBlock();
 
+#if MONO
+			return this == s_Invariant ? GetInvariantCaseInsensitiveHashCode (str) : StringComparer.CurrentCultureIgnoreCase.GetHashCode (str);
+#else
             // Return our result
             return (InternalGetCaseInsHash(this.m_dataHandle, this.m_handleOrigin, this.m_textInfoName, str, forceRandomizedHashing, additionalEntropy));
+#endif
         }
-
+#if MONO
+		unsafe int GetInvariantCaseInsensitiveHashCode (string str)
+		{
+			fixed (char * c = str) {
+				char * cc = c;
+				char * end = cc + str.Length - 1;
+				int h = 0;
+				for (;cc < end; cc += 2) {
+					h = (h << 5) - h + Char.ToUpperInvariant (*cc);
+					h = (h << 5) - h + Char.ToUpperInvariant (cc [1]);
+				}
+				++end;
+				if (cc < end)
+					h = (h << 5) - h + Char.ToUpperInvariant (*cc);
+				return h;
+			}
+		}
+#else
         // Change case (ToUpper/ToLower) -- COMNlsInfo::InternalChangeCaseChar
         [System.Security.SecurityCritical]  // auto-generated
         [ResourceExposure(ResourceScope.None)]
@@ -1083,6 +1032,7 @@ namespace System.Globalization {
         [SuppressUnmanagedCodeSecurity]
         [return: MarshalAs(UnmanagedType.Bool)]
         private static unsafe extern bool InternalTryFindStringOrdinalIgnoreCase(int searchFlags, String source, int sourceCount, int startIndex, String target, int targetCount, ref int foundIndex);
+#endif
     }
 
 }
