@@ -468,7 +468,19 @@ namespace System.Resources {
                     }
                     else {
 #endif //IA64
-                    s = new String(charPtr, 0, byteLen/2);
+                    if (!BitConverter.IsLittleEndian) {
+                        byte* bytePtr = (byte*) charPtr;
+                        var dest = new byte[byteLen];
+                        for (int i = 0; i < byteLen; i += 2) {
+                            dest[i] = *(bytePtr+i+1);
+                            dest[i+1] = *(bytePtr+i);
+                        }
+
+                        fixed(byte *pDest = dest) {
+                            s = new String((char *)pDest, 0, byteLen/2);
+                        }
+                    } else
+                        s = new String(charPtr, 0, byteLen/2);
 #if IA64
                     }
 #endif //IA64
@@ -1386,22 +1398,24 @@ namespace System.Resources {
 
                     String key;
                     Object value = null;
-                    lock (_reader._resCache) {
-                        key = _reader.AllocateStringForNameIndex(_currentName, out _dataPosition);
-                        ResourceLocator locator;
-                        if (_reader._resCache.TryGetValue(key, out locator)) {
-                            value = locator.Value;
-                        }
-                        if (value == null) {
-                            if (_dataPosition == -1) 
-                                value = _reader.GetValueForNameIndex(_currentName);
-                            else 
-                                value = _reader.LoadObject(_dataPosition);
-                            // If enumeration and subsequent lookups happen very
-                            // frequently in the same process, add a ResourceLocator
-                            // to _resCache here.  But [....] enumerates and
-                            // just about everyone else does lookups.  So caching
-                            // here may bloat working set.
+                    lock (_reader) { // locks should be taken in the same order as in RuntimeResourceSet.GetObject to avoid deadlock
+                        lock (_reader._resCache) {
+                            key = _reader.AllocateStringForNameIndex(_currentName, out _dataPosition); // AllocateStringForNameIndex could lock on _reader
+                            ResourceLocator locator;
+                            if (_reader._resCache.TryGetValue(key, out locator)) {
+                                value = locator.Value;
+                            }
+                            if (value == null) {
+                                if (_dataPosition == -1) 
+                                    value = _reader.GetValueForNameIndex(_currentName);
+                                else 
+                                    value = _reader.LoadObject(_dataPosition);
+                                // If enumeration and subsequent lookups happen very
+                                // frequently in the same process, add a ResourceLocator
+                                // to _resCache here.  But [....] enumerates and
+                                // just about everyone else does lookups.  So caching
+                                // here may bloat working set.
+                            }
                         }
                     }
                     return new DictionaryEntry(key, value);
