@@ -1,3 +1,6 @@
+#if MONO_COM
+#define FEATURE_COMINTEROP
+#endif
 // ==++==
 // 
 //   Copyright (c) Microsoft Corporation.  All rights reserved.
@@ -76,10 +79,8 @@ namespace System.Runtime.InteropServices{
         InterfaceIsIUnknown = 1,
         InterfaceIsIDispatch = 2,
 
-#if FEATURE_COMINTEROP
         [System.Runtime.InteropServices.ComVisible(false)]
         InterfaceIsIInspectable = 3,
-#endif // FEATURE_COMINTEROP
     }
 
     [AttributeUsage(AttributeTargets.Interface, Inherited = false)]
@@ -162,7 +163,6 @@ namespace System.Runtime.InteropServices{
         public String Value { get { return _importClassName; } }
     }
 
-#if FEATURE_COMINTEROP
     [AttributeUsage(AttributeTargets.Method, Inherited = false)] 
     [System.Runtime.InteropServices.ComVisible(true)]
     public sealed class LCIDConversionAttribute : Attribute
@@ -272,6 +272,7 @@ namespace System.Runtime.InteropServices{
         public String Value { get {return _val;} }  
     }    
 
+#if FEATURE_COMINTEROP || MOBILE_LEGACY
     [AttributeUsage(AttributeTargets.All, Inherited = false)] 
     [System.Runtime.InteropServices.ComVisible(true)]
     public sealed class ComConversionLossAttribute : Attribute
@@ -280,7 +281,8 @@ namespace System.Runtime.InteropServices{
         {
         }
     }
-    
+#endif
+#if FEATURE_COMINTEROP || MOBILE_LEGACY
 [Serializable]
 [Flags()]
     [System.Runtime.InteropServices.ComVisible(true)]
@@ -389,7 +391,6 @@ namespace System.Runtime.InteropServices{
         }
         public TypeLibVarFlags Value { get {return _val;} } 
     }   
-#endif //FEATURE_COMINTEROP
 
     [Serializable]
     [System.Runtime.InteropServices.ComVisible(true)]
@@ -404,9 +405,7 @@ namespace System.Runtime.InteropServices{
         VT_CY = 6,
         VT_DATE = 7,
         VT_BSTR = 8,
-#if FEATURE_COMINTEROP
         VT_DISPATCH         = 9,
-#endif // FEATURE_COMINTEROP
         VT_ERROR = 10,
         VT_BOOL = 11,
         VT_VARIANT = 12,
@@ -472,9 +471,7 @@ namespace System.Runtime.InteropServices{
 
         Currency = 0xf,         // A currency
 
-#if FEATURE_COMINTEROP || FEATURE_LEGACYNETCF
         BStr             = 0x13,        // OLE Unicode BSTR
-#endif //FEATURE_COMINTEROP || FEATURE_LEGACYNETCF
 
         LPStr = 0x14,        // Ptr to SBCS string
 
@@ -486,17 +483,13 @@ namespace System.Runtime.InteropServices{
 
         IUnknown = 0x19,        // COM IUnknown pointer. 
 
-#if FEATURE_COMINTEROP || FEATURE_LEGACYNETCF
         IDispatch        = 0x1a,        // COM IDispatch pointer
-#endif //FEATURE_COMINTEROP || FEATURE_LEGACYNETCF
 
         Struct = 0x1b,        // Structure
 
-#if FEATURE_COMINTEROP || FEATURE_LEGACYNETCF
         Interface        = 0x1c,        // COM interface
 
         SafeArray        = 0x1d,        // OLE SafeArray
-#endif //FEATURE_COMINTEROP || FEATURE_LEGACYNETCF
 
         ByValArray = 0x1e,        // Array of fixed size (only valid in structs)
 
@@ -504,7 +497,6 @@ namespace System.Runtime.InteropServices{
 
         SysUInt = 0x20,
 
-#if FEATURE_COMINTEROP || FEATURE_LEGACYNETCF
         VBByRefStr       = 0x22,         
 
         AnsiBStr         = 0x23,        // OLE BSTR containing SBCS characters
@@ -512,7 +504,6 @@ namespace System.Runtime.InteropServices{
         TBStr            = 0x24,        // Ptr to OS preferred (SBCS/Unicode) BSTR
 
         VariantBool      = 0x25,        // OLE defined BOOLEAN (2 bytes, true == -1, false == 0)
-#endif //FEATURE_COMINTEROP || FEATURE_LEGACYNETCF
 
         FunctionPtr = 0x26,        // Function pointer
 
@@ -522,26 +513,28 @@ namespace System.Runtime.InteropServices{
 
         LPStruct = 0x2b,        // Pointer to a structure
 
-#if FEATURE_COMINTEROP || FEATURE_LEGACYNETCF
         CustomMarshaler  = 0x2c,        
-#endif //FEATURE_COMINTEROP || FEATURE_LEGACYNETCF
 
         Error = 0x2d,
 
-#if FEATURE_COMINTEROP
         [System.Runtime.InteropServices.ComVisible(false)]
         IInspectable     = 0x2e,
         
         [System.Runtime.InteropServices.ComVisible(false)]
         HString          = 0x2f,        // Windows Runtime HSTRING
-
-#endif //FEATURE_COMINTEROP
     }
 
+#if !MONO
     [AttributeUsage(AttributeTargets.Parameter | AttributeTargets.Field | AttributeTargets.ReturnValue, Inherited = false)]
     [System.Runtime.InteropServices.ComVisible(true)]
     public unsafe sealed class MarshalAsAttribute : Attribute
     {
+#if MONO
+        internal MarshalAsAttribute Copy ()
+        {
+            return (MarshalAsAttribute)this.MemberwiseClone ();
+        }
+#else
         [System.Security.SecurityCritical]  // auto-generated
         internal static Attribute GetCustomAttribute(RuntimeParameterInfo parameter)
         {
@@ -607,11 +600,9 @@ namespace System.Runtime.InteropServices{
             short sizeParamIndex, int sizeConst, string marshalType, RuntimeType marshalTypeRef, string marshalCookie, int iidParamIndex)
         {
             _val = val;
-#if FEATURE_COMINTEROP
             SafeArraySubType = safeArraySubType;
             SafeArrayUserDefinedSubType = safeArrayUserDefinedSubType;
             IidParameterIndex = iidParamIndex;
-#endif // FEATURE_COMINTEROP
             ArraySubType = arraySubType;
             SizeParamIndex = sizeParamIndex;
             SizeConst = sizeConst;
@@ -619,7 +610,7 @@ namespace System.Runtime.InteropServices{
             MarshalTypeRef = marshalTypeRef;
             MarshalCookie = marshalCookie;
         }
-
+#endif
         internal UnmanagedType _val;
         public MarshalAsAttribute(UnmanagedType unmanagedType)
         {
@@ -631,14 +622,12 @@ namespace System.Runtime.InteropServices{
         }
         public UnmanagedType Value { get { return _val; } }
 
-#if FEATURE_COMINTEROP
         // Fields used with SubType = SafeArray.
         public VarEnum            SafeArraySubType;
         public Type               SafeArrayUserDefinedSubType;
 
         // Field used with iid_is attribute (interface pointers).
         public int                IidParameterIndex;
-#endif // FEATURE_COMINTEROP
 
         // Fields used with SubType = ByValArray and LPArray.
         // Array size =  parameter(PI) * PM + C
@@ -653,10 +642,13 @@ namespace System.Runtime.InteropServices{
         public Type MarshalTypeRef;           // Type of marshaler class
         public String MarshalCookie;            // cookie to pass to marshaler
     }
-
+#endif
     [AttributeUsage(AttributeTargets.Class | AttributeTargets.Interface, Inherited = false)]
     [System.Runtime.InteropServices.ComVisible(true)]
-    public sealed class ComImportAttribute : Attribute
+#if !MONOTOUCH
+    public
+#endif
+    sealed class ComImportAttribute : Attribute
     {
         internal static Attribute GetCustomAttribute(RuntimeType type)
         {
@@ -799,12 +791,18 @@ namespace System.Runtime.InteropServices{
             if ((method.Attributes & MethodAttributes.PinvokeImpl) == 0)
                 return null;
 
+#if !MONO
             MetadataImport scope = ModuleHandle.GetMetadataImport(method.Module.ModuleHandle.GetRuntimeModule());
+#endif
             string entryPoint, dllName = null;
             int token = method.MetadataToken;
             PInvokeAttributes flags = 0;
 
+#if MONO
+            ((MonoMethod)method).GetPInvoke(out flags, out entryPoint, out dllName);
+#else
             scope.GetPInvokeMap(token, out flags, out entryPoint, out dllName);
+#endif
 
             CharSet charSet = CharSet.None;
 
@@ -891,7 +889,7 @@ namespace System.Runtime.InteropServices{
         private const int DEFAULT_PACKING_SIZE = 8;
 
         [System.Security.SecurityCritical]  // auto-generated
-        internal static Attribute GetCustomAttribute(RuntimeType type)
+        internal static StructLayoutAttribute GetCustomAttribute(RuntimeType type)
         {
             if (!IsDefined(type))
                 return null;
@@ -914,7 +912,12 @@ namespace System.Runtime.InteropServices{
                 case TypeAttributes.UnicodeClass: charSet = CharSet.Unicode; break;
                 default: Contract.Assume(false); break;
             }
+
+#if MONO
+            type.GetPacking (out pack, out size);
+#else
             type.GetRuntimeModule().MetadataImport.GetClassLayout(type.MetadataToken, out pack, out size);
+#endif
 
             // Metadata parameter checking should not have allowed 0 for packing size.
             // The runtime later converts a packing size of 0 to 8 so do the same here
@@ -967,7 +970,11 @@ namespace System.Runtime.InteropServices{
             int fieldOffset;
 
             if (field.DeclaringType != null &&
+#if MONO
+                (fieldOffset = field.GetFieldOffset ()) >= 0)
+#else
                 field.GetRuntimeModule().MetadataImport.GetFieldOffset(field.DeclaringType.MetadataToken, field.MetadataToken, out fieldOffset))
+#endif
                 return new FieldOffsetAttribute(fieldOffset);
 
             return null;
@@ -987,7 +994,7 @@ namespace System.Runtime.InteropServices{
         public int Value { get { return _val; } }
     }
 
-#if FEATURE_COMINTEROP
+#if FEATURE_COMINTEROP || MOBILE_LEGACY
     [AttributeUsage(AttributeTargets.Parameter | AttributeTargets.Field | AttributeTargets.Property | AttributeTargets.ReturnValue, Inherited = false)] 
     [System.Runtime.InteropServices.ComVisible(true)]
     public sealed class ComAliasNameAttribute : Attribute
@@ -1028,7 +1035,6 @@ namespace System.Runtime.InteropServices{
         public int MajorVersion { get {return _major;} }
         public int MinorVersion { get {return _minor;} }
     }
-#endif //FEATURE_COMINTEROP
 
     [AttributeUsage(AttributeTargets.Interface, Inherited = false)]
     [System.Runtime.InteropServices.ComVisible(true)]
@@ -1044,8 +1050,6 @@ namespace System.Runtime.InteropServices{
         public Type CoClass { get { return _CoClass; } }
     }
 
-#if FEATURE_COMINTEROP
-
     [AttributeUsage(AttributeTargets.Interface, Inherited = false)]
     [System.Runtime.InteropServices.ComVisible(true)]
     public sealed class ComEventInterfaceAttribute : Attribute
@@ -1062,7 +1066,8 @@ namespace System.Runtime.InteropServices{
         public Type SourceInterface { get {return _SourceInterface;} }       
         public Type EventProvider { get {return _EventProvider;} }
     }
-
+#endif
+#if FEATURE_COMINTEROP || MOBILE_LEGACY
     [AttributeUsage(AttributeTargets.Assembly, Inherited = false)] 
     [System.Runtime.InteropServices.ComVisible(true)]
     public sealed class TypeLibVersionAttribute : Attribute
@@ -1103,8 +1108,6 @@ namespace System.Runtime.InteropServices{
         public int RevisionNumber { get {return _revision;} }
     }
 
-#endif //FEATURE_COMINTEROP
-
     [AttributeUsage(AttributeTargets.Assembly | AttributeTargets.Interface | AttributeTargets.Class | AttributeTargets.Struct, Inherited = false)]
     [System.Runtime.InteropServices.ComVisible(true)]
     public sealed class BestFitMappingAttribute : Attribute
@@ -1144,7 +1147,6 @@ namespace System.Runtime.InteropServices{
         }
     }
 
-#if FEATURE_COMINTEROP
     [AttributeUsage(AttributeTargets.Method, Inherited = false, AllowMultiple = false)]
     [System.Runtime.InteropServices.ComVisible(false)]
     public sealed class ManagedToNativeComInteropStubAttribute : Attribute
@@ -1161,7 +1163,6 @@ namespace System.Runtime.InteropServices{
         public Type ClassType { get { return _classType; } }
         public String MethodName { get { return _methodName; } }
     }    
-#endif // FEATURE_COMINTEROP
-
+#endif
+#endif
 }
-
