@@ -87,7 +87,7 @@ namespace System.Security.Cryptography
         internal const int CALG_RC4               = (ALG_CLASS_DATA_ENCRYPT | ALG_TYPE_STREAM | 1);
 #endif // FEATURE_CRYPTO
 
-                                        internal const int PROV_RSA_FULL          = 1;
+        internal const int PROV_RSA_FULL          = 1;
         internal const int PROV_DSS_DH            = 13;
         internal const int PROV_RSA_AES           = 24;
 
@@ -134,14 +134,25 @@ namespace System.Security.Cryptography
     }
 
     internal static class Utils {
-        
+
 #if !FEATURE_PAL && FEATURE_CRYPTO
         [SecuritySafeCritical]
 #endif
         static Utils()
         {
         }
+#if MONO
+        // The default provider value must remain '1' for Mono, otherwise we won't be able
+        // to locate keypairs that were serialized by Mono versions 4.0 and lower.
+        // (The ProviderType property in the CspParameters class affects serialization)
+        internal const int DefaultRsaProviderType = 1;
+#else
+        // Provider type to use by default for RSA operations. We want to use RSA-AES CSP
+        // since it enables access to SHA-2 operations. All currently supported OSes support RSA-AES.
+        internal const int DefaultRsaProviderType = Constants.PROV_RSA_AES;
+#endif
 
+#if !MONO
 #if FEATURE_CRYPTO || FEATURE_LEGACYNETCFCRYPTO
         // Private object for locking instead of locking on a public type for SQL reliability work.
         private static Object s_InternalSyncObject = new Object();
@@ -149,33 +160,7 @@ namespace System.Security.Cryptography
         private static Object InternalSyncObject {
             get { return s_InternalSyncObject; }
         }
-#endif // FEATURE_CRYPTO || FEATURE_LEGACYNETCFCRYPTO
 
-        // Provider type to use by default for RSA operations.  On systems which support the RSA-AES CSP, we
-        // want to use that since it enables access to SHA-2 operations, downlevel we fall back to the
-        // RSA-FULL CSP.
-        private static volatile int _defaultRsaProviderType;
-        private static volatile bool _haveDefaultRsaProviderType;
-        internal static int DefaultRsaProviderType
-        {
-            get {
-                if (!_haveDefaultRsaProviderType)
-                {
-                    // The AES CSP is only supported on WinXP and higher
-                    bool osSupportsAesCsp = Environment.OSVersion.Platform == PlatformID.Win32NT &&
-                                            (Environment.OSVersion.Version.Major > 5 ||
-                                            (Environment.OSVersion.Version.Major == 5 && Environment.OSVersion.Version.Minor >= 1));
-
-                    _defaultRsaProviderType = osSupportsAesCsp ? Constants.PROV_RSA_AES : Constants.PROV_RSA_FULL;
-                    _haveDefaultRsaProviderType = true;
-                }
-
-                return _defaultRsaProviderType;
-            }
-        }
-
-#if FEATURE_CRYPTO || FEATURE_LEGACYNETCFCRYPTO
-#if !FEATURE_PAL
         [System.Security.SecurityCritical] // auto-generated
         private static volatile SafeProvHandle _safeProvHandle;
         internal static SafeProvHandle StaticProvHandle {
@@ -184,16 +169,13 @@ namespace System.Security.Cryptography
                 if (_safeProvHandle == null) {
                     lock (InternalSyncObject) {
                         if (_safeProvHandle == null) {
-                            SafeProvHandle safeProvHandle = AcquireProvHandle(new CspParameters(DefaultRsaProviderType));
-                            Thread.MemoryBarrier();
-                            _safeProvHandle = safeProvHandle;
+                            _safeProvHandle = AcquireProvHandle(new CspParameters(DefaultRsaProviderType));
                         }
                     }
                 }
                 return _safeProvHandle;
             }
         }
-#endif // !FEATURE_PAL
 
         [System.Security.SecurityCritical] // auto-generated
         private static volatile SafeProvHandle _safeDssProvHandle;
@@ -203,9 +185,7 @@ namespace System.Security.Cryptography
                 if (_safeDssProvHandle == null) {
                     lock (InternalSyncObject) {
                         if (_safeDssProvHandle == null) {
-                            SafeProvHandle safeProvHandle = CreateProvHandle(new CspParameters(Constants.PROV_DSS_DH), true);
-                            Thread.MemoryBarrier();
-                            _safeDssProvHandle = safeProvHandle;
+                            _safeDssProvHandle = CreateProvHandle(new CspParameters(Constants.PROV_DSS_DH), true);
                         }
                     }
                 }
@@ -505,8 +485,9 @@ namespace System.Security.Cryptography
             }
         }
 #endif // FEATURE_CRYPTO
+#endif
+        private static volatile RNGCryptoServiceProvider _rng;
 
-        private static volatile RNGCryptoServiceProvider _rng = null;
         internal static RNGCryptoServiceProvider StaticRandomNumberGenerator {
             get {
                 if (_rng == null)
@@ -524,7 +505,7 @@ namespace System.Security.Cryptography
             StaticRandomNumberGenerator.GetBytes(key);
             return key;
         }
-
+#if !MONO
 #if FEATURE_CRYPTO
         /// <summary>
         ///     Read the FIPS policy from the pre-Vista registry key
@@ -570,11 +551,14 @@ namespace System.Security.Cryptography
             }
         }
 #endif //FEATURE_CRYPTO
-
+#endif
 #if FEATURE_CRYPTO || FEATURE_LEGACYNETCFCRYPTO
         // dwKeySize = 0 means the default key size
         [System.Security.SecurityCritical]  // auto-generated
         internal static bool HasAlgorithm (int dwCalg, int dwKeySize) {
+#if MONO
+            return true;
+#else
             bool r = false;
             // We need to take a lock here since we are querying the provider handle in a loop.
             // If multiple threads are racing in this code, not all algorithms/key sizes combinations
@@ -583,8 +567,9 @@ namespace System.Security.Cryptography
                 r = SearchForAlgorithm(StaticProvHandle, dwCalg, dwKeySize);
             }
             return r;
+#endif
         }
-
+#if !MONO
         internal static int ObjToAlgId(object hashAlg, OidGroup group) {
             if (hashAlg == null)
                 throw new ArgumentNullException("hashAlg");
@@ -636,7 +621,7 @@ namespace System.Security.Cryptography
 
             return hash;
         }
-
+#endif
         internal static String DiscardWhiteSpaces (string inputBuffer) {
             return DiscardWhiteSpaces(inputBuffer, 0, inputBuffer.Length);
         }
@@ -814,10 +799,14 @@ namespace System.Security.Cryptography
             return unchecked(new byte[] { (byte)(i >> 24), (byte)(i >> 16), (byte)(i >> 8), (byte)i });
         }
 
-
 #if FEATURE_CRYPTO || FEATURE_LEGACYNETCFCRYPTO
         [System.Security.SecurityCritical]  // auto-generated
         internal static byte[] RsaOaepEncrypt (RSA rsa, HashAlgorithm hash, PKCS1MaskGenerationMethod mgf, RandomNumberGenerator rng, byte[] data) {
+#if MONO
+            // It looks like .net managed implementation is buggy. It returns quite different
+            // result compare to old mono code, even PSLength calculation is quite different
+            return Mono.Security.Cryptography.PKCS1.Encrypt_OAEP (rsa, hash, rng, data);
+#else
             int cb = rsa.KeySize / 8;
 
             //  1. Hash the parameters to get PHash
@@ -827,7 +816,7 @@ namespace System.Security.Cryptography
             hash.ComputeHash(EmptyArray<Byte>.Value); // Use an empty octet string
 
             //  2.  Create DB object
-            byte[] DB = new byte[cb - cbHash];
+            byte[] DB = new byte[cb - cbHash + 1];
 
             //  Structure is as follows:
             //      pHash || PS || 01 || M
@@ -863,10 +852,17 @@ namespace System.Security.Cryptography
             Buffer.InternalBlockCopy(DB, 0, pad, seed.Length, DB.Length);
 
             return rsa.EncryptValue(pad);
+#endif
         }
 
         [System.Security.SecurityCritical]  // auto-generated
         internal static byte[] RsaOaepDecrypt (RSA rsa, HashAlgorithm hash, PKCS1MaskGenerationMethod mgf, byte[] encryptedData) {
+#if MONO
+            var result = Mono.Security.Cryptography.PKCS1.Decrypt_OAEP (rsa, hash, encryptedData);
+            if (result == null)
+                throw new CryptographicException(Environment.GetResourceString("Cryptography_OAEPDecoding"));
+            return result;
+#else
             int cb = rsa.KeySize / 8;
 
             // 1. Decode the input data
@@ -890,11 +886,12 @@ namespace System.Security.Cryptography
             if (zeros < 0 || zeros >= cbHash)
                 throw new CryptographicException(Environment.GetResourceString("Cryptography_OAEPDecoding"));
 
+            const int leading_zeros = 0;
             byte[] seed = new byte[cbHash];
-            Buffer.InternalBlockCopy(data, 0, seed, zeros, seed.Length - zeros);
+            Buffer.InternalBlockCopy(data, leading_zeros, seed, zeros, seed.Length - zeros);
 
-            byte[] DB = new byte[data.Length - seed.Length + zeros];
-            Buffer.InternalBlockCopy(data, seed.Length - zeros, DB, 0, DB.Length);
+            byte[] DB = new byte[data.Length - seed.Length + zeros - leading_zeros];
+            Buffer.InternalBlockCopy(data, seed.Length - zeros + leading_zeros, DB, 0, DB.Length);
 
             //  4.  seedMask = MGF(maskedDB, hLen);
             byte[] mask = mgf.GenerateMask(DB, seed.Length);
@@ -942,6 +939,7 @@ namespace System.Security.Cryptography
             byte[] output = new byte[DB.Length - i];
             Buffer.InternalBlockCopy(DB, i, output, 0, output.Length);
             return output;
+ #endif
         }
 
         [System.Security.SecurityCritical]  // auto-generated
@@ -1013,7 +1011,7 @@ namespace System.Security.Cryptography
             }
             return true;
         }
-
+#if !MONO
         [System.Security.SecurityCritical]  // auto-generated
         [ResourceExposure(ResourceScope.None)]  // Creates a process resource, but it can't be scoped.
         [DllImport(JitHelpers.QCall, CharSet = CharSet.Unicode), SuppressUnmanagedCodeSecurity]
@@ -1128,7 +1126,9 @@ namespace System.Security.Cryptography
         [ResourceExposure(ResourceScope.None)]
         [MethodImplAttribute(MethodImplOptions.InternalCall)]
         internal static extern void _GenerateKey(SafeProvHandle hProv, int algid, CspProviderFlags flags, int keySize, ref SafeKeyHandle hKey);
+#endif
 #endif // FEATURE_CRYPTO
+#if !MONO
         [System.Security.SecurityCritical]  // auto-generated
         [ResourceExposure(ResourceScope.None)]
         [MethodImpl(MethodImplOptions.InternalCall)]
@@ -1176,6 +1176,17 @@ namespace System.Security.Cryptography
         [ResourceExposure(ResourceScope.Machine)]
         [MethodImplAttribute(MethodImplOptions.InternalCall)]
         internal static extern void _AcquireCSP(CspParameters param, ref SafeProvHandle hProv);
+
+#else
+        internal static bool _ProduceLegacyHmacValues()
+        {
+#if FULL_AOT_RUNTIME
+            return false;
+#else
+            return Environment.GetEnvironmentVariable ("legacyHMACMode") == "1";
+#endif
+        }
+#endif
     }
 }
 
