@@ -33,7 +33,20 @@
 namespace System.Threading
 {
     using System.Security;
+    using System.Runtime.Remoting;
+    using System.Security.Permissions;
+    using System;
+    using System.Runtime.CompilerServices;
+    using System.Runtime.ConstrainedExecution;
+    using System.Runtime.InteropServices;
+    using System.Runtime.Versioning;
+    using System.Collections.Generic;
     using System.Diagnostics.Contracts;
+    using System.Diagnostics.CodeAnalysis;
+    using System.Diagnostics.Tracing;
+#if !MONO
+    using Microsoft.Win32;
+#endif
 
     //
     // Interface to something that can be queued to the TP.  This is implemented by
@@ -62,25 +75,6 @@ namespace System.Threading
     [CLSCompliant(false)]
     [System.Runtime.InteropServices.ComVisible(true)]
     unsafe public delegate void IOCompletionCallback(uint errorCode, uint numBytes, NativeOverlapped* pOVERLAP);
-}
-
-namespace System.Threading.Microsoft
-{
-    using System.Security;
-    using System.Runtime.Remoting;
-    using System.Security.Permissions;
-    using System;
-    using System.Runtime.CompilerServices;
-    using System.Runtime.ConstrainedExecution;
-    using System.Runtime.InteropServices;
-    using System.Runtime.Versioning;
-    using System.Collections.Generic;
-    using System.Diagnostics.Contracts;
-    using System.Diagnostics.CodeAnalysis;
-    using System.Diagnostics.Tracing;
-#if !MONO
-    using Microsoft.Win32;
-#endif
 
     internal static class ThreadPoolGlobals
     {
@@ -585,9 +579,7 @@ namespace System.Threading.Microsoft
         // The head and tail of the queue.  We enqueue to the head, and dequeue from the tail.
         internal volatile QueueSegment queueHead;
         internal volatile QueueSegment queueTail;
-#if !FEATURE_CORECLR
         internal bool loggingEnabled;
-#endif
 
         internal static SparseArray<WorkStealingQueue> allThreadQueues = new SparseArray<WorkStealingQueue>(16); //
 
@@ -596,7 +588,7 @@ namespace System.Threading.Microsoft
         public ThreadPoolWorkQueue()
         {
             queueTail = queueHead = new QueueSegment();
-#if !MONO && !FEATURE_CORECLR
+#if !MONO
             loggingEnabled = FrameworkEventSource.Log.IsEnabled(EventLevel.Verbose, FrameworkEventSource.Keywords.ThreadPool|FrameworkEventSource.Keywords.ThreadTransfer);
 #endif
         }
@@ -658,7 +650,7 @@ namespace System.Threading.Microsoft
             if (!forceGlobal)
                 tl = ThreadPoolWorkQueueThreadLocals.threadLocals;
 
-#if !MONO && !FEATURE_CORECLR
+#if !MONO
             if (loggingEnabled)
                 System.Diagnostics.Tracing.FrameworkEventSource.Log.ThreadPoolEnqueueWorkObject(callback);
 #endif
@@ -770,7 +762,7 @@ namespace System.Threading.Microsoft
             //
             workQueue.MarkThreadRequestSatisfied();
 
-#if !MONO && !FEATURE_CORECLR
+#if !MONO
             // Has the desire for logging changed since the last time we entered?
             workQueue.loggingEnabled = FrameworkEventSource.Log.IsEnabled(EventLevel.Verbose, FrameworkEventSource.Keywords.ThreadPool|FrameworkEventSource.Keywords.ThreadTransfer);
 #endif
@@ -832,7 +824,7 @@ namespace System.Threading.Microsoft
                     }
                     else
                     {
-#if !MONO && !FEATURE_CORECLR
+#if !MONO
                         if (workQueue.loggingEnabled)
                             System.Diagnostics.Tracing.FrameworkEventSource.Log.ThreadPoolDequeueWorkObject(workItem);
 #endif
@@ -1196,9 +1188,27 @@ namespace System.Threading.Microsoft
     //
     internal static class _ThreadPoolWaitCallback
     {
+#if FEATURE_INTERCEPTABLE_THREADPOOL_CALLBACK
+        // This feature is used by Xamarin.iOS to use an NSAutoreleasePool
+        // for every task done by the threadpool.
+        static Func<Func<bool>, bool> dispatcher;
+
+        internal static void SetDispatcher (Func<Func<bool>, bool> value)
+        {
+            dispatcher = value;
+        }
+#endif
+
         [System.Security.SecurityCritical]
         static internal bool PerformWaitCallback()
         {
+#if FEATURE_INTERCEPTABLE_THREADPOOL_CALLBACK
+            // store locally first to ensure another thread doesn't clear the field between checking for null and using it.
+            var dispatcher = _ThreadPoolWaitCallback.dispatcher;
+            if (dispatcher != null)
+                return dispatcher (ThreadPoolWorkQueue.Dispatch);
+#endif
+
             return ThreadPoolWorkQueue.Dispatch();
         }
     }
@@ -1373,8 +1383,6 @@ namespace System.Threading.Microsoft
     [HostProtection(Synchronization=true, ExternalThreading=true)]
     public static class ThreadPool
     {
-        internal static readonly bool UseMicrosoftThreadPool = Environment.GetEnvironmentVariable ("MONO_THREADPOOL") == "microsoft";
-
         #if FEATURE_CORECLR
         [System.Security.SecurityCritical] // auto-generated
         #else
@@ -1897,6 +1905,7 @@ namespace System.Threading.Microsoft
         [MethodImplAttribute(MethodImplOptions.InternalCall)]
         private static extern void InitializeVMTp(ref bool enableWorkerTracking);
 
+#if !MONO
         [System.Security.SecurityCritical]  // auto-generated
         [ResourceExposure(ResourceScope.None)]
         [MethodImplAttribute(MethodImplOptions.InternalCall)]
@@ -1909,6 +1918,7 @@ namespace System.Threading.Microsoft
              ref StackCrawlMark     stackMark,
              bool                   compressStack   
              );
+#endif
 
 #if !FEATURE_CORECLR
         [System.Security.SecuritySafeCritical]  // auto-generated
