@@ -25,9 +25,8 @@
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-#if NET_2_0
-
 using System;
+using System.Linq;
 using System.Collections;
 using System.Collections.Generic;
 using System.Collections.Specialized;
@@ -41,7 +40,7 @@ using Mono.XBuild.Utilities;
 namespace Microsoft.Build.BuildEngine {
 	public class BuildItem {
 
-		BuildItemGroup	child_items;
+		List<BuildItem> child_items;
 		XmlElement	itemElement;
 		string		finalItemSpec;
 		bool		isImported;
@@ -52,6 +51,9 @@ namespace Microsoft.Build.BuildEngine {
 		//string		recursiveDir;
 		IDictionary	evaluatedMetadata;
 		IDictionary	unevaluatedMetadata;
+		bool		isDynamic;
+		bool		keepDuplicates = true;
+		string		removeMetadata, keepMetadata;
 
 		BuildItem ()
 		{
@@ -64,7 +66,7 @@ namespace Microsoft.Build.BuildEngine {
 
 			this.name = itemName;
 			this.finalItemSpec = taskItem.ItemSpec;
-			this.itemInclude = Utilities.Escape (taskItem.ItemSpec);
+			this.itemInclude = MSBuildUtils.Escape (taskItem.ItemSpec);
 			this.evaluatedMetadata = (Hashtable) taskItem.CloneCustomMetadata ();
 			this.unevaluatedMetadata = (Hashtable) taskItem.CloneCustomMetadata ();
 		}
@@ -82,27 +84,65 @@ namespace Microsoft.Build.BuildEngine {
 			unevaluatedMetadata = CollectionsUtil.CreateCaseInsensitiveHashtable ();
 			evaluatedMetadata = CollectionsUtil.CreateCaseInsensitiveHashtable ();
 		}
-		
+
 		internal BuildItem (XmlElement itemElement, BuildItemGroup parentItemGroup)
 		{
-			child_items = new BuildItemGroup ();
+			child_items = new List<BuildItem> ();
 			isImported = parentItemGroup.IsImported;
 			unevaluatedMetadata = CollectionsUtil.CreateCaseInsensitiveHashtable ();
 			evaluatedMetadata = CollectionsUtil.CreateCaseInsensitiveHashtable ();
 			this.parent_item_group = parentItemGroup;
 			
 			this.itemElement = itemElement;
-			
-			if (Include == String.Empty)
-				throw new InvalidProjectFileException (String.Format ("The required attribute \"Include\" is missing from element <{0}>.", Name));
+			isDynamic = parentItemGroup.IsDynamic;
+
+			if (IsDynamic) {
+				if (!string.IsNullOrEmpty (Remove)) {
+					if (!string.IsNullOrEmpty (Include) || !string.IsNullOrEmpty (Exclude))
+						throw new InvalidProjectFileException (string.Format ("The attribute \"Remove\" in element <{0}> is unrecognized.", Name));
+					if (itemElement.HasChildNodes)
+						throw new InvalidProjectFileException ("Children are not allowed below an item remove element.");
+				}
+				if (string.IsNullOrEmpty (Include) && !string.IsNullOrEmpty (Exclude))
+					throw new InvalidProjectFileException (string.Format ("The attribute \"Exclude\" in element <{0}> is unrecognized.", Name));
+			} else {
+				if (string.IsNullOrEmpty (Include))
+					throw new InvalidProjectFileException (string.Format ("The required attribute \"Include\" is missing from element <{0}>.", Name));
+				if (!string.IsNullOrEmpty (Remove))
+					throw new InvalidProjectFileException (string.Format ("The attribute \"Remove\" in element <{0}> is unrecognized.", Name));
+			}
+
+			foreach (XmlAttribute attr in itemElement.Attributes) {
+				if (attr.Name == "Include" || attr.Name == "Exclude" || attr.Name == "Condition")
+					continue;
+				if (!IsDynamic)
+					throw new InvalidProjectFileException (string.Format ("The attribute \"{0}\" in element <{1}> is unrecognized.", attr.Name, Name));
+
+				switch (attr.Name) {
+				case "Remove":
+					Remove = attr.Value;
+					break;
+				case "KeepDuplicates":
+					KeepDuplicates = bool.Parse (attr.Value);
+					break;
+				case "RemoveMetadata":
+					removeMetadata = attr.Value;
+					break;
+				case "KeepMetadata":
+					keepMetadata = attr.Value;
+					break;
+				default:
+					throw new InvalidProjectFileException (string.Format ("The attribute \"{0}\" in element <{1}> is unrecognized.", attr.Name, Name));
+				}
+			}
 		}
-		
+
 		BuildItem (BuildItem parent)
 		{
 			isImported = parent.isImported;
 			name = parent.Name;
 			parent_item = parent;
-			parent_item.child_items.AddItem (this);
+			parent_item.child_items.Add (this);
 			parent_item_group = parent.parent_item_group;
 			unevaluatedMetadata = CollectionsUtil.CreateCaseInsensitiveHashtable (parent.unevaluatedMetadata);
 			evaluatedMetadata = CollectionsUtil.CreateCaseInsensitiveHashtable (parent.evaluatedMetadata);
@@ -126,8 +166,8 @@ namespace Microsoft.Build.BuildEngine {
 		public string GetEvaluatedMetadata (string metadataName)
 		{
 			if (ReservedNameUtils.IsReservedMetadataName (metadataName)) {
-				string metadata = ReservedNameUtils.GetReservedMetadata (FinalItemSpec, metadataName);
-				return (metadataName.ToLower () == "fullpath") ? Utilities.Escape (metadata) : metadata;
+				string metadata = ReservedNameUtils.GetReservedMetadata (FinalItemSpec, metadataName, evaluatedMetadata);
+				return MSBuildUtils.Unescape (metadata);
 			}
 
 			if (evaluatedMetadata.Contains (metadataName))
@@ -139,8 +179,7 @@ namespace Microsoft.Build.BuildEngine {
 		public string GetMetadata (string metadataName)
 		{
 			if (ReservedNameUtils.IsReservedMetadataName (metadataName)) {
-				string metadata = ReservedNameUtils.GetReservedMetadata (FinalItemSpec, metadataName);
-				return (metadataName.ToLower () == "fullpath") ? Utilities.Escape (metadata) : metadata;
+				return ReservedNameUtils.GetReservedMetadata (FinalItemSpec, metadataName, unevaluatedMetadata);
 			} else if (unevaluatedMetadata.Contains (metadataName))
 				return (string) unevaluatedMetadata [metadataName];
 			else
@@ -169,7 +208,7 @@ namespace Microsoft.Build.BuildEngine {
 					XmlNode node = itemElement [metadataName];
 					itemElement.RemoveChild (node);
 				}
-			} else if (HasParent) {
+			} else if (HasParentItem) {
 				if (parent_item.child_items.Count > 1)
 					SplitParentItem ();
 				parent_item.RemoveMetadata (metadataName);
@@ -198,8 +237,8 @@ namespace Microsoft.Build.BuildEngine {
 				throw new ArgumentException (String.Format ("\"{0}\" is a reserved item meta-data, and cannot be modified or deleted.",
 					metadataName));
 
-			if (treatMetadataValueAsLiteral && !HasParent)
-				metadataValue = Utilities.Escape (metadataValue);
+			if (treatMetadataValueAsLiteral && !HasParentItem)
+				metadataValue = MSBuildUtils.Escape (metadataValue);
 
 			if (FromXml) {
 				XmlElement element = itemElement [metadataName];
@@ -209,12 +248,12 @@ namespace Microsoft.Build.BuildEngine {
 					itemElement.AppendChild (element);
 				} else
 					element.InnerText = metadataValue;
-			} else if (HasParent) {
+			} else if (HasParentItem) {
 				if (parent_item.child_items.Count > 1)
 					SplitParentItem ();
 				parent_item.SetMetadata (metadataName, metadataValue, treatMetadataValueAsLiteral);
 			}
-			if (FromXml || HasParent) {
+			if (FromXml || HasParentItem) {
 				parent_item_group.ParentProject.MarkProjectAsDirty ();
 				parent_item_group.ParentProject.NeedToReevaluate ();
 			}
@@ -225,14 +264,18 @@ namespace Microsoft.Build.BuildEngine {
 
 		void AddMetadata (string name, string value)
 		{
+			var options = IsDynamic ?
+			              ParseOptions.AllowItemsMetadataAndSplit : ParseOptions.AllowItemsNoMetadataAndSplit;
+
 			if (parent_item_group != null) {
 				Expression e = new Expression ();
-				e.Parse (value, true);
-				evaluatedMetadata.Add (name, (string) e.ConvertTo (parent_item_group.ParentProject, typeof (string)));
+				e.Parse (value, options);
+				evaluatedMetadata [name] = (string) e.ConvertTo (parent_item_group.ParentProject,
+						typeof (string), ExpressionOptions.ExpandItemRefs);
 			} else
-				evaluatedMetadata.Add (name, Utilities.Unescape (value));
+				evaluatedMetadata [name] = MSBuildUtils.Unescape (value);
 				
-				unevaluatedMetadata.Add (name, value);
+			unevaluatedMetadata [name] = value;
 		}
 
 		void DeleteMetadata (string name)
@@ -246,50 +289,197 @@ namespace Microsoft.Build.BuildEngine {
 
 		internal void Evaluate (Project project, bool evaluatedTo)
 		{
-			// FIXME: maybe make Expression.ConvertTo (null, ...) work as Utilities.Unescape ()?
+			// FIXME: maybe make Expression.ConvertTo (null, ...) work as MSBuildUtils.Unescape ()?
 			if (project == null) {
-				this.finalItemSpec = Utilities.Unescape (Include);
+				this.finalItemSpec = MSBuildUtils.Unescape (Include);
 				return;
 			}
-			
-			foreach (XmlElement xe in itemElement.ChildNodes)
-				AddMetadata (xe.Name, xe.InnerText);
+
+			foreach (XmlNode xn in itemElement.ChildNodes) {
+				XmlElement xe = xn as XmlElement;
+				if (xe != null && ConditionParser.ParseAndEvaluate (xe.GetAttribute ("Condition"), project))
+					AddMetadata (xe.Name, xe.InnerText);
+			}
+
+			if (IsDynamic) {
+				if (!evaluatedTo)
+					return;
+
+				if (!string.IsNullOrEmpty (Remove)) {
+					RemoveItems (project);
+					return;
+				}
+
+				if (string.IsNullOrEmpty (Include)) {
+					UpdateMetadata (project);
+					return;
+				}
+			}
 
 			DirectoryScanner directoryScanner;
 			Expression includeExpr, excludeExpr;
-			string includes, excludes;
+			ITaskItem[] includes, excludes;
+
+			var options = IsDynamic ?
+				ParseOptions.AllowItemsMetadataAndSplit : ParseOptions.AllowItemsNoMetadataAndSplit;
 
 			includeExpr = new Expression ();
-			includeExpr.Parse (Include, true);
+			includeExpr.Parse (Include, options);
 			excludeExpr = new Expression ();
-			excludeExpr.Parse (Exclude, true);
+			excludeExpr.Parse (Exclude, options);
 			
-			includes = (string) includeExpr.ConvertTo (project, typeof (string));
-			excludes = (string) excludeExpr.ConvertTo (project, typeof (string));
+			includes = (ITaskItem[]) includeExpr.ConvertTo (project, typeof (ITaskItem[]),
+								ExpressionOptions.ExpandItemRefs);
+			excludes = (ITaskItem[]) excludeExpr.ConvertTo (project, typeof (ITaskItem[]),
+								ExpressionOptions.ExpandItemRefs);
+
+			this.finalItemSpec = (string) includeExpr.ConvertTo (project, typeof (string),
+							ExpressionOptions.ExpandItemRefs);
 
-			this.finalItemSpec = includes;
-			
 			directoryScanner = new DirectoryScanner ();
 			
 			directoryScanner.Includes = includes;
 			directoryScanner.Excludes = excludes;
 
+			if (project.FullFileName != String.Empty) {
+				directoryScanner.ProjectFile = project.ThisFileFullPath;
+				directoryScanner.BaseDirectory = new DirectoryInfo (Path.GetDirectoryName (project.FullFileName));
+			}
+			else
+				directoryScanner.BaseDirectory = new DirectoryInfo (Directory.GetCurrentDirectory ());
+			
+			directoryScanner.Scan ();
+			
+			foreach (ITaskItem matchedItem in directoryScanner.MatchedItems)
+				AddEvaluatedItem (project, evaluatedTo, matchedItem);
+		}
+
+		bool CheckCondition (Project project)
+		{
+			if (parent_item_group != null && !ConditionParser.ParseAndEvaluate (parent_item_group.Condition, project))
+				return false;
+			if (parent_item != null && !parent_item.CheckCondition (project))
+				return false;
+			return ConditionParser.ParseAndEvaluate (Condition, project);
+		}
+
+		void UpdateMetadata (Project project)
+		{
+			BuildItemGroup group;
+			if (!project.TryGetEvaluatedItemByNameBatched (Name, out group))
+				return;
+
+			foreach (BuildItem item in group) {
+				if (!item.CheckCondition (project))
+					continue;
+				
+				foreach (string name in evaluatedMetadata.Keys) {
+					item.SetMetadata (name, (string)evaluatedMetadata [name]);
+				}
+
+				AddAndRemoveMetadata (project, item);
+			}
+		}
+
+		void AddAndRemoveMetadata (Project project, BuildItem item)
+		{
+			if (!string.IsNullOrEmpty (removeMetadata)) {
+				var removeExpr = new Expression ();
+				removeExpr.Parse (removeMetadata, ParseOptions.AllowItemsNoMetadataAndSplit);
+
+				var removeSpec = (string[]) removeExpr.ConvertTo (
+					project, typeof (string[]), ExpressionOptions.ExpandItemRefs);
+
+				foreach (var remove in removeSpec) {
+					item.DeleteMetadata (remove);
+				}
+			}
+
+			if (!string.IsNullOrEmpty (keepMetadata)) {
+				var keepExpr = new Expression ();
+				keepExpr.Parse (keepMetadata, ParseOptions.AllowItemsNoMetadataAndSplit);
+
+				var keepSpec = (string[]) keepExpr.ConvertTo (
+					project, typeof (string[]), ExpressionOptions.ExpandItemRefs);
+
+				var metadataNames = new string [item.evaluatedMetadata.Count];
+				item.evaluatedMetadata.Keys.CopyTo (metadataNames, 0);
+
+				foreach (string name in metadataNames) {
+					if (!keepSpec.Contains (name))
+						item.DeleteMetadata (name);
+				}
+			}
+		}
+
+		void RemoveItems (Project project)
+		{
+			BuildItemGroup group;
+			if (!project.TryGetEvaluatedItemByNameBatched (Name, out group))
+				return;
+
+			var removeExpr = new Expression ();
+			removeExpr.Parse (Remove, ParseOptions.AllowItemsNoMetadataAndSplit);
+
+			var removes = (ITaskItem[]) removeExpr.ConvertTo (
+				project, typeof (ITaskItem[]), ExpressionOptions.ExpandItemRefs);
+
+			var directoryScanner = new DirectoryScanner ();
+			
+			directoryScanner.Includes = removes;
+
 			if (project.FullFileName != String.Empty)
 				directoryScanner.BaseDirectory = new DirectoryInfo (Path.GetDirectoryName (project.FullFileName));
 			else
 				directoryScanner.BaseDirectory = new DirectoryInfo (Directory.GetCurrentDirectory ());
 			
 			directoryScanner.Scan ();
-			
-			foreach (string matchedFile in directoryScanner.MatchedFilenames)
-				AddEvaluatedItem (project, evaluatedTo, matchedFile);
+
+			foreach (ITaskItem matchedItem in directoryScanner.MatchedItems) {
+				group.RemoveItem (matchedItem);
+			}
 		}
-		
-		void AddEvaluatedItem (Project project, bool evaluatedTo, string itemSpec)
+
+		bool ContainsItem (Project project, ITaskItem taskItem)
 		{
+			BuildItemGroup group;
+			if (!project.TryGetEvaluatedItemByNameBatched (Name, out group))
+				return false;
+
+			var item = group.FindItem (taskItem);
+			if (item == null)
+				return false;
+
+			foreach (string metadataName in evaluatedMetadata.Keys) {
+				string metadataValue = (string)evaluatedMetadata [metadataName];
+				if (!metadataValue.Equals (item.evaluatedMetadata [metadataName]))
+					return false;
+			}
+			
+			foreach (string metadataName in item.evaluatedMetadata.Keys) {
+				string metadataValue = (string)item.evaluatedMetadata [metadataName];
+				if (!metadataValue.Equals (evaluatedMetadata [metadataName]))
+					return false;
+			}
+
+			return true;
+		}
+
+		void AddEvaluatedItem (Project project, bool evaluatedTo, ITaskItem taskitem)
+		{
+			if (IsDynamic && evaluatedTo && !KeepDuplicates && ContainsItem (project, taskitem))
+				return;
+
 			BuildItemGroup big;			
 			BuildItem bi = new BuildItem (this);
-			bi.finalItemSpec = itemSpec;
+			bi.finalItemSpec = taskitem.ItemSpec;
+
+			foreach (DictionaryEntry de in taskitem.CloneCustomMetadata ()) {
+				bi.unevaluatedMetadata.Add ((string) de.Key, (string) de.Value);
+				bi.evaluatedMetadata.Add ((string) de.Key, (string) de.Value);
+			}
+
+			project.EvaluatedItemsIgnoringCondition.AddItem (bi);
 
 			if (evaluatedTo) {
 				project.EvaluatedItems.AddItem (bi);
@@ -312,17 +502,27 @@ namespace Microsoft.Build.BuildEngine {
 			}
 
 			big.AddItem (bi);
+
+			if (IsDynamic)
+				AddAndRemoveMetadata (project, bi);
 		}
 		
-		internal string ConvertToString (Expression transform)
+		// during item's eval phase, any item refs in this item, have either
+		// already been expanded or are non-existant, so expand can be _false_
+		//
+		// during prop's eval phase, this isn't reached, as it parses expressions
+		// with allowItems=false, so no ItemReferences are created at all
+		//
+		// at other times, item refs have already been expanded, so expand: false
+		internal string ConvertToString (Expression transform, ExpressionOptions options)
 		{
-			return GetItemSpecFromTransform (transform);
+			return GetItemSpecFromTransform (transform, options);
 		}
 		
-		internal ITaskItem ConvertToITaskItem (Expression transform)
+		internal ITaskItem ConvertToITaskItem (Expression transform, ExpressionOptions options)
 		{
 			TaskItem taskItem;
-			taskItem = new TaskItem (GetItemSpecFromTransform (transform), evaluatedMetadata);
+			taskItem = new TaskItem (GetItemSpecFromTransform (transform, options), evaluatedMetadata);
 			return taskItem;
 		}
 
@@ -330,28 +530,44 @@ namespace Microsoft.Build.BuildEngine {
 		{
 			if (FromXml)
 				itemElement.ParentNode.RemoveChild (itemElement);
-			else if (HasParent) {
+			else if (HasParentItem) {
 				if (parent_item.child_items.Count > 1)
 					SplitParentItem ();
 				parent_item.Detach ();
 			}
 		}
 
-		string GetItemSpecFromTransform (Expression transform)
+		string GetItemSpecFromTransform (Expression transform, ExpressionOptions options)
 		{
 			StringBuilder sb;
 		
-			if (transform == null)
-				return finalItemSpec;
-			else {
+			if (transform == null) {
+				if (options == ExpressionOptions.ExpandItemRefs) {
+					// With usual code paths, this will never execute,
+					// but letting this be here, incase BI.ConvertTo*
+					// is called directly
+					Expression expr = new Expression ();
+					expr.Parse (finalItemSpec, ParseOptions.AllowItemsNoMetadataAndSplit);
+
+					return (string) expr.ConvertTo (parent_item_group.ParentProject,
+							typeof (string), ExpressionOptions.ExpandItemRefs);
+				} else {
+					return finalItemSpec;
+				}
+			} else {
+				// Transform, _DONT_ expand itemrefs
 				sb = new StringBuilder ();
 				foreach (object o in transform.Collection) {
 					if (o is string) {
 						sb.Append ((string)o);
 					} else if (o is PropertyReference) {
-						sb.Append (((PropertyReference)o).ConvertToString (parent_item_group.ParentProject));
+						sb.Append (((PropertyReference)o).ConvertToString (
+									parent_item_group.ParentProject,
+									ExpressionOptions.DoNotExpandItemRefs));
 					} else if (o is ItemReference) {
-						sb.Append (((ItemReference)o).ConvertToString (parent_item_group.ParentProject));
+						sb.Append (((ItemReference)o).ConvertToString (
+									parent_item_group.ParentProject,
+									ExpressionOptions.DoNotExpandItemRefs));
 					} else if (o is MetadataReference) {
 						sb.Append (GetMetadata (((MetadataReference)o).MetadataName));
 					}
@@ -383,12 +599,12 @@ namespace Microsoft.Build.BuildEngine {
 			newElement.SetAttribute ("Include", child.FinalItemSpec);
 			if (parent.itemElement.HasAttribute ("Condition"))
 				newElement.SetAttribute ("Condition", parent.itemElement.GetAttribute ("Condition"));
-			foreach (XmlElement xe in parent.itemElement)
-				newElement.AppendChild (xe.Clone ());
+			foreach (XmlNode xn in parent.itemElement)
+				newElement.AppendChild (xn.Clone ());
 			parent.itemElement.ParentNode.InsertAfter (newElement, insertAfter);
 
 			newParent = new BuildItem (newElement, parent.parent_item_group);
-			newParent.child_items.AddItem (child);
+			newParent.child_items.Add (child);
 			child.parent_item = newParent;
 
 			return newParent;
@@ -404,7 +620,7 @@ namespace Microsoft.Build.BuildEngine {
 			set {
 				if (FromXml)
 					itemElement.SetAttribute ("Condition", value);
-				else if (!HasParent)
+				else if (!HasParentItem)
 					throw new InvalidOperationException ("Cannot set a condition on an object not represented by an XML element in the project file.");
 			}
 		}
@@ -432,7 +648,7 @@ namespace Microsoft.Build.BuildEngine {
 			get {
 				if (FromXml)
 					return itemElement.GetAttribute ("Include");
-				else if (HasParent)
+				else if (HasParentItem)
 					return parent_item.Include;
 				else
 					return itemInclude;
@@ -440,7 +656,7 @@ namespace Microsoft.Build.BuildEngine {
 			set {
 				if (FromXml)
 					itemElement.SetAttribute ("Include", value);
-				else if (HasParent) {
+				else if (HasParentItem) {
 					if (parent_item.child_items.Count > 1)
 						SplitParentItem ();
 					parent_item.Include = value;
@@ -449,6 +665,20 @@ namespace Microsoft.Build.BuildEngine {
 			}
 		}
 
+		internal bool IsDynamic {
+			get { return isDynamic; }
+		}
+
+		internal string Remove {
+			get;
+			private set;
+		}
+
+		internal bool KeepDuplicates {
+			get { return keepDuplicates; }
+			private set { keepDuplicates = value; }
+		}
+
 		public bool IsImported {
 			get { return isImported; }
 		}
@@ -457,7 +687,7 @@ namespace Microsoft.Build.BuildEngine {
 			get {
 				if (FromXml)
 					return itemElement.Name;
-				else if (HasParent)
+				else if (HasParentItem)
 					return parent_item.Name;
 				else
 					return name;
@@ -467,11 +697,11 @@ namespace Microsoft.Build.BuildEngine {
 					XmlElement newElement = itemElement.OwnerDocument.CreateElement (value, Project.XmlNamespace);
 					newElement.SetAttribute ("Include", itemElement.GetAttribute ("Include"));
 					newElement.SetAttribute ("Condition", itemElement.GetAttribute ("Condition"));
-					foreach (XmlElement xe in itemElement)
-						newElement.AppendChild (xe.Clone ());
+					foreach (XmlNode xn in itemElement)
+						newElement.AppendChild (xn.Clone ());
 					itemElement.ParentNode.ReplaceChild (newElement, itemElement);
 					itemElement = newElement;
-				} else if (HasParent) {
+				} else if (HasParentItem) {
 					if (parent_item.child_items.Count > 1)
 						SplitParentItem ();
 					parent_item.Name = value;
@@ -483,8 +713,12 @@ namespace Microsoft.Build.BuildEngine {
 		internal bool FromXml {
 			get { return itemElement != null; }
 		}
+
+		internal XmlElement XmlElement {
+			get { return itemElement; }
+		}
 		
-		internal bool HasParent {
+		internal bool HasParentItem {
 			get { return parent_item != null; }
 		}
 
@@ -494,8 +728,7 @@ namespace Microsoft.Build.BuildEngine {
 
 		internal BuildItemGroup ParentItemGroup {
 			get { return parent_item_group; }
+			set { parent_item_group = value; }
 		}
 	}
 }
-
-#endif
