@@ -30,6 +30,8 @@ namespace Mono.Data.Sqlite
     private const string SQLITE_DLL = "SQLite.Interop.DLL";
 #endif // USE_INTEROP_DLL
 
+#elif MONOTOUCH
+	private const string SQLITE_DLL = "/usr/lib/libsqlite3.dylib";
 #else
     private const string SQLITE_DLL = "sqlite3";
 #endif
@@ -133,6 +135,13 @@ namespace Mono.Data.Sqlite
     [DllImport(SQLITE_DLL)]
 #endif
     internal static extern int sqlite3_close(IntPtr db);
+		
+#if !PLATFORM_COMPACTFRAMEWORK
+    [DllImport(SQLITE_DLL, CallingConvention = CallingConvention.Cdecl)]
+#else
+    [DllImport(SQLITE_DLL)]
+#endif
+    internal static extern int sqlite3_close_v2(IntPtr db);
 
 #if !PLATFORM_COMPACTFRAMEWORK
     [DllImport(SQLITE_DLL, CallingConvention = CallingConvention.Cdecl)]
@@ -141,6 +150,13 @@ namespace Mono.Data.Sqlite
 #endif
     internal static extern int sqlite3_create_function(IntPtr db, byte[] strName, int nArgs, int nType, IntPtr pvUser, SQLiteCallback func, SQLiteCallback fstep, SQLiteFinalCallback ffinal);
 
+#if !PLATFORM_COMPACTFRAMEWORK
+		[DllImport(SQLITE_DLL, CallingConvention = CallingConvention.Cdecl)]
+#else
+		[DllImport(SQLITE_DLL)]
+#endif
+		internal static extern int sqlite3_create_function_v2(IntPtr db, byte[] strName, int nArgs, int nType, IntPtr pvUser, SQLiteCallback func, SQLiteCallback fstep, SQLiteFinalCallback ffinal, SQLiteFinalCallback fdestroy);
+
 #if !PLATFORM_COMPACTFRAMEWORK
     [DllImport(SQLITE_DLL, CallingConvention = CallingConvention.Cdecl)]
 #else
@@ -184,6 +200,8 @@ namespace Mono.Data.Sqlite
 #endif
     internal static extern IntPtr sqlite3_bind_parameter_name(IntPtr stmt, int index);
 
+// Apple does not include those symbols in it's libsqlite3
+#if !MONOTOUCH
 #if !PLATFORM_COMPACTFRAMEWORK
     [DllImport(SQLITE_DLL, CallingConvention = CallingConvention.Cdecl)]
 #else
@@ -197,6 +215,7 @@ namespace Mono.Data.Sqlite
     [DllImport(SQLITE_DLL)]
 #endif
     internal static extern IntPtr sqlite3_column_database_name16(IntPtr stmt, int index);
+#endif
 
 #if !PLATFORM_COMPACTFRAMEWORK
     [DllImport(SQLITE_DLL, CallingConvention = CallingConvention.Cdecl)]
@@ -226,6 +245,7 @@ namespace Mono.Data.Sqlite
 #endif
     internal static extern IntPtr sqlite3_column_name16(IntPtr stmt, int index);
 
+#if !MONOTOUCH
 #if !PLATFORM_COMPACTFRAMEWORK
     [DllImport(SQLITE_DLL, CallingConvention = CallingConvention.Cdecl)]
 #else
@@ -253,6 +273,7 @@ namespace Mono.Data.Sqlite
     [DllImport(SQLITE_DLL)]
 #endif
     internal static extern IntPtr sqlite3_column_table_name16(IntPtr stmt, int index);
+#endif // !MONOTOUCH
 
 #if !PLATFORM_COMPACTFRAMEWORK
     [DllImport(SQLITE_DLL, CallingConvention = CallingConvention.Cdecl)]
@@ -630,6 +651,7 @@ namespace Mono.Data.Sqlite
 #endif
     internal static extern void sqlite3_result_text16(IntPtr context, string strName, int nLen, IntPtr pvReserved);
 
+#if !MONOTOUCH
 #if !PLATFORM_COMPACTFRAMEWORK
     [DllImport(SQLITE_DLL, CallingConvention = CallingConvention.Cdecl)]
 #else
@@ -643,6 +665,7 @@ namespace Mono.Data.Sqlite
     [DllImport(SQLITE_DLL)]
 #endif
     internal static extern int sqlite3_rekey(IntPtr db, byte[] key, int keylen);
+#endif
 
 #if !PLATFORM_COMPACTFRAMEWORK
     [DllImport(SQLITE_DLL, CallingConvention = CallingConvention.Cdecl)]
@@ -693,6 +716,20 @@ namespace Mono.Data.Sqlite
 #endif
     internal static extern int sqlite3_config (SQLiteConfig config);
 
+#if !PLATFORM_COMPACTFRAMEWORK
+		[DllImport(SQLITE_DLL, CallingConvention = CallingConvention.Cdecl)]
+#else
+		[DllImport(SQLITE_DLL)]
+#endif
+		internal static extern IntPtr sqlite3_user_data (IntPtr context);
+
+#if !PLATFORM_COMPACTFRAMEWORK
+    [DllImport(SQLITE_DLL, CallingConvention = CallingConvention.Cdecl)]
+#else
+    [DllImport(SQLITE_DLL)]
+#endif
+    internal static extern int sqlite3_free (IntPtr ptr);
+
     #endregion
   }
 
