@@ -32,6 +32,7 @@ using System.Security.Cryptography.X509Certificates;
 using System.Threading;
 
 using Mono.Security.Protocol.Tls.Handshake;
+using Mono.Security.Interface;
 
 namespace Mono.Security.Protocol.Tls
 {
@@ -46,36 +47,6 @@ namespace Mono.Security.Protocol.Tls
 		X509Certificate certificate, 
 		int[]			certificateErrors);
 
-#if INSIDE_SYSTEM
-	internal
-#else
-	public
-#endif
-	class ValidationResult {
-		bool trusted;
-		bool user_denied;
-		int error_code;
-
-		public ValidationResult (bool trusted, bool user_denied, int error_code)
-		{
-			this.trusted = trusted;
-			this.user_denied = user_denied;
-			this.error_code = error_code;
-		}
-
-		public bool Trusted {
-			get { return trusted; }
-		}
-
-		public bool UserDenied {
-			get { return user_denied; }
-		}
-
-		public int ErrorCode {
-			get { return error_code; }
-		}
-	}
-
 #if INSIDE_SYSTEM
 	internal
 #else
@@ -588,14 +559,20 @@ namespace Mono.Security.Protocol.Tls
 			}
 			catch (TlsException ex)
 			{
-				// FIXME: should the send alert also be done asynchronously here and below?
-				this.protocol.SendAlert(ex.Alert);
-				negotiate.SetComplete (new IOException("The authentication or decryption has failed.", ex));
+				try {
+					Exception e = ex;
+					this.protocol.SendAlert(ref e);
+				} catch {
+				}
+				negotiate.SetComplete(new IOException("The authentication or decryption has failed.", ex));
 			}
 			catch (Exception ex)
 			{
-				this.protocol.SendAlert(AlertDescription.InternalError);
-				negotiate.SetComplete (new IOException("The authentication or decryption has failed.", ex));
+				try {
+					this.protocol.SendAlert(AlertDescription.InternalError);
+				} catch {
+				}
+				negotiate.SetComplete(new IOException("The authentication or decryption has failed.", ex));
 			}
 		}
 
