@@ -26,7 +26,6 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_4_0
 
 using System;
 using System.IO;
@@ -214,6 +213,41 @@ namespace MonoTests.System.IO.MemoryMappedFiles {
 			}
 		}
 
+		[Test]
+		public unsafe void ViewAccessorReadArrayWithOffset () {
+			var file = MemoryMappedFile.CreateFromFile (fname, FileMode.Open);
+			var offset = 3;
+			var expected = "lo!";
+
+			using (var v = file.CreateViewAccessor (offset, expected.Length)) {
+				// PointerOffset Mono implementation is always 0.
+				// Assert.AreEqual (offset, v.PointerOffset);
+
+				var a = new byte [expected.Length];
+				var n = v.ReadArray (0, a, 0, expected.Length);
+				Assert.AreEqual (expected.Length, n);
+				var s = new string (Array.ConvertAll (a, b => (char)b));
+				Assert.AreEqual (expected, s);
+			}
+		}
+
+		[Test]
+		public unsafe void ViewStreamReadWithOffset () {
+			var file = MemoryMappedFile.CreateFromFile (fname, FileMode.Open);
+			var offset = 3;
+			var expected = "lo!";
+
+			using (var v = file.CreateViewStream (offset, expected.Length)) {
+				// PointerOffset Mono implementation is always 0.
+				// Assert.AreEqual (offset, v.PointerOffset);
+
+				var a = new byte [expected.Length];
+				var n = v.Read (a, 0, expected.Length);
+				Assert.AreEqual (expected.Length, n);
+				var s = new string (Array.ConvertAll (a, b => (char)b));
+				Assert.AreEqual (expected, s);
+			}
+		}
 
 		[Test]
 		public void NamedMappingToInvalidFile ()
@@ -362,8 +396,18 @@ namespace MonoTests.System.IO.MemoryMappedFiles {
 #endif
 			stream.Write (new byte [pageSize], 0, pageSize);
 		}
+
+		[Test] // #30741 #30825
+		public void CreateFromFileNullMapName ()
+		{
+			int size = 100;
+			string f = Path.Combine (tempDir, "null-map-name-file");
+			File.WriteAllBytes (f, new byte [size]);
+
+			FileStream file = File.OpenRead (f);
+			MemoryMappedFile.CreateFromFile (file, null, size, MemoryMappedFileAccess.ReadExecute, null, 0, false);
+		}
 	}
 }
 
-#endif
 
