@@ -48,11 +48,7 @@ using System.Xml;
 
 namespace System.Data.SqlClient
 {
-#if NET_2_0
 	public class SqlDataReader : DbDataReader, IDataReader, IDisposable, IDataRecord
-#else
-	public sealed class SqlDataReader : MarshalByRefObject, IEnumerable, IDataReader, IDisposable, IDataRecord
-#endif // NET_2_0
 	{
 		#region Fields
 
@@ -66,9 +62,7 @@ namespace System.Data.SqlClient
 		bool haveRead;
 		bool readResult;
 		bool readResultUsed;
-#if NET_2_0
 		int visibleFieldCount;
-#endif
 
 		#endregion // Fields
 
@@ -118,17 +112,13 @@ namespace System.Data.SqlClient
 		#region Properties
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		int Depth {
 			get { return 0; }
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		int FieldCount {
 			get {
 				ValidateState ();
@@ -137,33 +127,25 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		bool IsClosed {
 			get { return isClosed; }
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		object this [int i] {
 			get { return GetValue (i); }
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		object this [string name] {
 			get { return GetValue (GetOrdinal (name)); }
 		}
 	
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		int RecordsAffected {
 			get {
 				return command.Tds.RecordsAffected;
@@ -171,9 +153,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		bool HasRows {
 			get {
 				ValidateState ();
@@ -185,7 +165,6 @@ namespace System.Data.SqlClient
 				return readResult;
 			}
 		}
-#if NET_2_0
 		public override int VisibleFieldCount {
 			get { return visibleFieldCount; }
 		}
@@ -197,16 +176,13 @@ namespace System.Data.SqlClient
 		protected bool IsCommandBehavior (CommandBehavior condition) {
 			return condition == command.CommandBehavior;
 		}
-#endif
 
 		#endregion // Properties
 
 		#region Methods
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		void Close ()
 		{
 			if (IsClosed)
@@ -223,9 +199,7 @@ namespace System.Data.SqlClient
 			Type booleanType = typeof (bool);
 			Type stringType = typeof (string);
 			Type intType = typeof (int);
-#if NET_2_0
 			Type typeType = typeof (Type);
-#endif
 			Type shortType = typeof (short);
 
 			DataTable schemaTable = new DataTable ("SchemaTable");
@@ -241,11 +215,7 @@ namespace System.Data.SqlClient
 			schemaTable.Columns.Add ("BaseColumnName", stringType);
 			schemaTable.Columns.Add ("BaseSchemaName", stringType);
 			schemaTable.Columns.Add ("BaseTableName", stringType);
-#if NET_2_0
 			schemaTable.Columns.Add ("DataType", typeType);
-#else
-			schemaTable.Columns.Add ("DataType", typeof (object));
-#endif
 			schemaTable.Columns.Add ("AllowDBNull", booleanType);
 			schemaTable.Columns.Add ("ProviderType", intType);
 			schemaTable.Columns.Add ("IsAliased", booleanType);
@@ -256,7 +226,6 @@ namespace System.Data.SqlClient
 			schemaTable.Columns.Add ("IsHidden", booleanType);
 			schemaTable.Columns.Add ("IsLong", booleanType);
 			schemaTable.Columns.Add ("IsReadOnly", booleanType);
-#if NET_2_0
 			schemaTable.Columns.Add ("ProviderSpecificDataType", typeType);
 			schemaTable.Columns.Add ("DataTypeName", stringType);
 			schemaTable.Columns.Add ("XmlSchemaCollectionDatabase", stringType);
@@ -265,7 +234,6 @@ namespace System.Data.SqlClient
 			schemaTable.Columns.Add ("UdtAssemblyQualifiedName", stringType);
 			schemaTable.Columns.Add ("NonVersionedProviderType", intType);
 			schemaTable.Columns.Add ("IsColumnSet", booleanType);
-#endif
 			
 			return schemaTable;
 		}
@@ -307,17 +275,10 @@ namespace System.Data.SqlClient
 				throw new IndexOutOfRangeException ();
 
 			column = command.Tds.Columns [ordinal];
-#if NET_2_0
 			ctype = (TdsColumnType) column.ColumnType;
 			csize = (int) column.ColumnSize;
 			precision = (short) (column.NumericPrecision ?? 0);
 			scale = (short) (column.NumericScale ?? 0);
-#else
-			ctype = (TdsColumnType) column ["ColumnType"];
-			csize = (int) column ["ColumnSize"];
-			precision = (short) ((byte) column ["NumericPrecision"]);
-			scale = (short) ((byte) column ["NumericScale"]);
-#endif
 			return GetSchemaRowDbType (ctype, csize, precision, scale);
 		}
 
@@ -531,9 +492,7 @@ namespace System.Data.SqlClient
 			}
 		}
 
-#if NET_2_0
 		new
-#endif
 		void Dispose (bool disposing)
 		{
 			if (!disposed) {
@@ -548,9 +507,7 @@ namespace System.Data.SqlClient
 		}
 
 		public 
-#if NET_2_0
 		override
-#endif // NET_2_0
 		bool GetBoolean (int i)
 		{
 			object value = GetValue (i);
@@ -562,9 +519,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		byte GetByte (int i)
 		{
 			object value = GetValue (i);
@@ -576,9 +531,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		long GetBytes (int i, long dataIndex, byte[] buffer, int bufferIndex, int length)
 		{
 			if ((command.CommandBehavior & CommandBehavior.SequentialAccess) != 0) {
@@ -590,11 +543,7 @@ namespace System.Data.SqlClient
 					if (len == -1)
 						throw CreateGetBytesOnInvalidColumnTypeException (i);
 					if (len == -2)
-#if NET_2_0
 						throw new SqlNullValueException ();
-#else
-						return 0;
-#endif
 					return len;
 				} catch (TdsInternalException ex) {
 					command.Connection.Close ();
@@ -611,27 +560,19 @@ namespace System.Data.SqlClient
 						throw new SqlNullValueException ();
 					break;
 				case SqlDbType.Text:
-#if NET_2_0
 					string text = value as string;
 					if (text != null)
 						value = Encoding.Default.GetBytes (text);
 					else
 						value = null;
 					break;
-#else
-					throw new InvalidCastException ();
-#endif
 				case SqlDbType.NText:
-#if NET_2_0
 					string ntext = value as string;
 					if (ntext != null)
 						value = Encoding.Unicode.GetBytes (ntext);
 					else
 						value = null;
 					break;
-#else
-					throw new InvalidCastException ();
-#endif
 				default:
 					throw CreateGetBytesOnInvalidColumnTypeException (i);
 				}
@@ -653,18 +594,14 @@ namespace System.Data.SqlClient
 
 		[EditorBrowsableAttribute (EditorBrowsableState.Never)]
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		char GetChar (int i)
 		{
 			throw new NotSupportedException ();
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		long GetChars (int i, long dataIndex, char[] buffer, int bufferIndex, int length)
 		{
 			if ((command.CommandBehavior & CommandBehavior.SequentialAccess) != 0) {
@@ -734,23 +671,13 @@ namespace System.Data.SqlClient
 			}
 		}
 		
-#if !NET_2_0
-		[EditorBrowsableAttribute (EditorBrowsableState.Never)]
-		public new IDataReader GetData (int i)
-		{
-			return ((IDataReader) this [i]);
-		}
-#endif
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		string GetDataTypeName (int i)
 		{
 			TdsDataColumn column;
 			TdsColumnType ctype;
-			string datatypeName;
 			int csize;
 			short precision;
 			short scale;
@@ -761,24 +688,15 @@ namespace System.Data.SqlClient
 				throw new IndexOutOfRangeException ();
 
 			column = command.Tds.Columns [i];
-#if NET_2_0
 			ctype = (TdsColumnType) column.ColumnType;
 			csize = (int) column.ColumnSize;
 			precision = (short) (column.NumericPrecision ?? 0);
 			scale = (short) (column.NumericScale ?? 0);
-#else
-			ctype = (TdsColumnType) column ["ColumnType"];
-			csize = (int) column ["ColumnSize"];
-			precision = (short) ((byte) column ["NumericPrecision"]);
-			scale = (short) ((byte) column ["NumericScale"]);
-#endif
 			return GetSchemaRowTypeName (ctype, csize, precision, scale);
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		DateTime GetDateTime (int i)
 		{
 			object value = GetValue (i);
@@ -789,10 +707,26 @@ namespace System.Data.SqlClient
 			return (DateTime) value;
 		}
 
+		[MonoTODO]
+		public virtual DateTimeOffset GetDateTimeOffset (int i)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public virtual TimeSpan GetTimeSpan (int i)
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public virtual SqlChars GetSqlChars (int i)
+		{
+			throw new NotImplementedException ();
+		}	
+
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		decimal GetDecimal (int i)
 		{
 			object value = GetValue (i);
@@ -804,9 +738,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		double GetDouble (int i)
 		{
 			object value = GetValue (i);
@@ -818,9 +750,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		Type GetFieldType (int i)
 		{
 			TdsDataColumn column;
@@ -835,25 +765,16 @@ namespace System.Data.SqlClient
 				throw new IndexOutOfRangeException ();
 
 			column = command.Tds.Columns [i];
-#if NET_2_0
 			ctype = (TdsColumnType) column.ColumnType;
 			csize = (int) column.ColumnSize;
 			precision = (short) (column.NumericPrecision ?? 0);
 			scale = (short) (column.NumericScale ?? 0);
-#else
-			ctype = (TdsColumnType) column ["ColumnType"];
-			csize = (int) column ["ColumnSize"];
-			precision = (short) ((byte) column ["NumericPrecision"]);
-			scale = (short) ((byte) column ["NumericScale"]);
-#endif
 			return GetSchemaRowFieldType (ctype, csize, precision,
 				scale);
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		float GetFloat (int i)
 		{
 			object value = GetValue (i);
@@ -865,9 +786,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		Guid GetGuid (int i)
 		{
 			object value = GetValue (i);
@@ -879,9 +798,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		short GetInt16 (int i)
 		{
 			object value = GetValue (i);
@@ -893,9 +810,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		int GetInt32 (int i)
 		{
 			object value = GetValue (i);
@@ -907,9 +822,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		long GetInt64 (int i)
 		{
 			object value = GetValue (i);
@@ -921,26 +834,18 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		string GetName (int i)
 		{
 			ValidateState ();
 
 			if (i < 0 || i >= command.Tds.Columns.Count)
 				throw new IndexOutOfRangeException ();
-#if NET_2_0
 			return (string) command.Tds.Columns[i].ColumnName;
-#else
-			return (string) command.Tds.Columns[i]["ColumnName"];
-#endif
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		int GetOrdinal (string name)
 		{
 			ValidateState ();
@@ -950,23 +855,15 @@ namespace System.Data.SqlClient
 
 			string colName;
 			foreach (TdsDataColumn schema in command.Tds.Columns) {
-#if NET_2_0
 				colName = schema.ColumnName;
 				if (colName.Equals (name) || String.Compare (colName, name, true) == 0)
 					return (int) schema.ColumnOrdinal;
-#else
-				colName = (string) schema["ColumnName"];
-				if (colName.Equals (name) || String.Compare (colName, name, true) == 0)
-					return (int) schema["ColumnOrdinal"];
-#endif
 			}
 			throw new IndexOutOfRangeException ();
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		DataTable GetSchemaTable ()
 		{
 			ValidateState ();
@@ -983,7 +880,6 @@ namespace System.Data.SqlClient
 			foreach (TdsDataColumn schema in command.Tds.Columns) {
 				DataRow row = schemaTable.NewRow ();
 
-#if NET_2_0
 				row [COLUMN_NAME_IDX]		= GetSchemaValue (schema.ColumnName);
 				row [COLUMN_ORDINAL_IDX]		= GetSchemaValue (schema.ColumnOrdinal);
 				row [IS_UNIQUE_IDX]		= GetSchemaValue (schema.IsUnique);
@@ -1010,25 +906,6 @@ namespace System.Data.SqlClient
 				row [UDT_ASMBLY_QUALIFIED_NAME_IDX] = DBNull.Value;
 				row [NON_VER_PROVIDER_TYPE_IDX] = DBNull.Value;
 				row [IS_COLUMN_SET] = DBNull.Value;
-#else
-				row [COLUMN_NAME_IDX]		= GetSchemaValue (schema, "ColumnName");
-				row [COLUMN_ORDINAL_IDX]		= GetSchemaValue (schema, "ColumnOrdinal");
-				row [IS_UNIQUE_IDX]		= GetSchemaValue (schema, "IsUnique");
-				row [IS_AUTO_INCREMENT_IDX]		= GetSchemaValue (schema, "IsAutoIncrement");
-				row [IS_ROW_VERSION_IDX]		= GetSchemaValue (schema, "IsRowVersion");
-				row [IS_HIDDEN_IDX]		= GetSchemaValue (schema, "IsHidden");
-				row [IS_IDENTITY_IDX]		= GetSchemaValue (schema, "IsIdentity");
-				row [IS_KEY_IDX]			= GetSchemaValue (schema, "IsKey");
-				row [IS_ALIASED_IDX]		= GetSchemaValue (schema, "IsAliased");
-				row [IS_EXPRESSION_IDX]		= GetSchemaValue (schema, "IsExpression");
-				row [IS_READ_ONLY_IDX]		= GetSchemaValue (schema, "IsReadOnly");
-				row [BASE_SERVER_NAME_IDX]		= GetSchemaValue (schema, "BaseServerName");
-				row [BASE_CATALOG_NAME_IDX]		= GetSchemaValue (schema, "BaseCatalogName");
-				row [BASE_COLUMN_NAME_IDX]		= GetSchemaValue (schema, "BaseColumnName");
-				row [BASE_SCHEMA_NAME_IDX]		= GetSchemaValue (schema, "BaseSchemaName");
-				row [BASE_TABLE_NAME_IDX]		= GetSchemaValue (schema, "BaseTableName");
-				row [ALLOW_DBNULL_IDX]		= GetSchemaValue (schema, "AllowDBNull");
-#endif
 				// We don't always get the base column name.
 				if (row [BASE_COLUMN_NAME_IDX] == DBNull.Value)
 					row [BASE_COLUMN_NAME_IDX] = row [COLUMN_NAME_IDX];
@@ -1040,17 +917,10 @@ namespace System.Data.SqlClient
 				string typeName;
 				short precision;
 				short scale;
-#if NET_2_0
 				ctype = (TdsColumnType) schema.ColumnType;
 				csize = (int) schema.ColumnSize;
 				precision = (short) GetSchemaValue (schema.NumericPrecision);
 				scale = (short) GetSchemaValue (schema.NumericScale);
-#else
-				ctype = (TdsColumnType) schema ["ColumnType"];
-				csize = (int) schema ["ColumnSize"];
-				precision = (short) ((byte) GetSchemaValue (schema, "NumericPrecision"));
-				scale = (short) ((byte) GetSchemaValue (schema, "NumericScale"));
-#endif
 
 				GetSchemaRowType (ctype, csize, precision, scale,
 					out dbType, out fieldType, out isLong,
@@ -1062,10 +932,8 @@ namespace System.Data.SqlClient
 				row [PROVIDER_TYPE_IDX] = dbType;
 				row [DATA_TYPE_IDX] = fieldType;
 				row [IS_LONG_IDX] = isLong;
-#if NET_2_0
 				if ((bool)row [IS_HIDDEN_IDX] == false)
 					visibleFieldCount += 1;
-#endif
 
 				schemaTable.Rows.Add (row);
 			}
@@ -1081,7 +949,6 @@ namespace System.Data.SqlClient
 				return DBNull.Value;
 		}
 
-#if NET_2_0
 		static object GetSchemaValue (object value)
 		{
 			if (value == null)
@@ -1089,12 +956,9 @@ namespace System.Data.SqlClient
 
 			return value;
 		}
-#endif		
 		
 		public
-#if NET_2_0
 		virtual
-#endif
 		SqlBinary GetSqlBinary (int i)
 		{
 			object value = GetSqlValue (i);
@@ -1104,9 +968,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		virtual
-#endif
 		SqlBoolean GetSqlBoolean (int i) 
 		{
 			object value = GetSqlValue (i);
@@ -1116,9 +978,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		virtual
-#endif
 		SqlByte GetSqlByte (int i)
 		{
 			object value = GetSqlValue (i);
@@ -1128,9 +988,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		virtual
-#endif
 		SqlDateTime GetSqlDateTime (int i)
 		{
 			object value = GetSqlValue (i);
@@ -1140,9 +998,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		virtual
-#endif
 		SqlDecimal GetSqlDecimal (int i)
 		{
 			object value = GetSqlValue (i);
@@ -1152,9 +1008,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		virtual
-#endif
 		SqlDouble GetSqlDouble (int i)
 		{
 			object value = GetSqlValue (i);
@@ -1164,9 +1018,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		virtual
-#endif
 		SqlGuid GetSqlGuid (int i)
 		{
 			object value = GetSqlValue (i);
@@ -1176,9 +1028,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		virtual
-#endif
 		SqlInt16 GetSqlInt16 (int i)
 		{
 			object value = GetSqlValue (i);
@@ -1188,9 +1038,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		virtual
-#endif
 		SqlInt32 GetSqlInt32 (int i)
 		{
 			object value = GetSqlValue (i);
@@ -1200,9 +1048,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		virtual
-#endif
 		SqlInt64 GetSqlInt64 (int i)
 		{
 			object value = GetSqlValue (i);
@@ -1212,9 +1058,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		virtual
-#endif
 		SqlMoney GetSqlMoney (int i)
 		{
 			object value = GetSqlValue (i);
@@ -1224,9 +1068,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		virtual
-#endif
 		SqlSingle GetSqlSingle (int i)
 		{
 			object value = GetSqlValue (i);
@@ -1236,9 +1078,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		virtual
-#endif
 		SqlString GetSqlString (int i)
 		{
 			object value = GetSqlValue (i);
@@ -1247,7 +1087,6 @@ namespace System.Data.SqlClient
 			return (SqlString) value;
 		}
 
-#if NET_2_0
 		public virtual SqlXml GetSqlXml (int i)
 		{
 			object value = GetSqlValue (i);
@@ -1269,21 +1108,14 @@ namespace System.Data.SqlClient
 			}
 			return (SqlXml) value;
 		}
-#endif // NET_2_0
 
 		public
-#if NET_2_0
 		virtual
-#endif
 		object GetSqlValue (int i)
 		{
-			int csize;
-			short precision, scale;
-			TdsColumnType ctype;
-			TdsDataColumn column;
-
 			object value = GetValue (i);
-
+			//Console.WriteLine ("Type of value: {0}", value.GetType ());
+			
 			SqlDbType type = GetSchemaRowDbType (i);
 			switch (type) {
 			case SqlDbType.BigInt:
@@ -1319,7 +1151,9 @@ namespace System.Data.SqlClient
 				if (value == DBNull.Value)
 					return SqlDecimal.Null;
 				if (value is TdsBigDecimal)
-					return SqlDecimal.FromTdsBigDecimal ((TdsBigDecimal) value);
+					return SqlDecimalExtensions.FromTdsBigDecimal ((TdsBigDecimal) value);
+				if (value is Int64)
+					return (SqlDecimal)((long) value);
 				return (SqlDecimal) ((decimal) value);
 			case SqlDbType.Float:
 				if (value == DBNull.Value)
@@ -1350,21 +1184,17 @@ namespace System.Data.SqlClient
 				if (value == DBNull.Value)
 					return SqlByte.Null;
 				return (SqlByte) ((byte) value);
-#if NET_2_0
 			case SqlDbType.Xml:
 				if (value == DBNull.Value)
 					return SqlByte.Null;
 				return (SqlXml) value;
-#endif
 			}
 
 			throw new InvalidOperationException ("The type of this column is unknown.");
 		}
 
 		public
-#if NET_2_0
 		virtual
-#endif
 		int GetSqlValues (object[] values)
 		{
 			ValidateState ();
@@ -1389,9 +1219,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		string GetString (int i)
 		{
 			object value = GetValue (i);
@@ -1403,9 +1231,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		object GetValue (int i)
 		{
 			ValidateState ();
@@ -1427,9 +1253,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		int GetValues (object[] values)
 		{
 			ValidateState ();
@@ -1439,52 +1263,46 @@ namespace System.Data.SqlClient
 				throw new ArgumentNullException ("values");
 
 			int len = values.Length;
-			int bigDecimalIndex = command.Tds.ColumnValues.BigDecimalIndex;
+			var tds = command.Tds;
+			int columns = Math.Min (len, tds.Columns.Count);
 
-			// If a four-byte decimal is stored, then we can't convert to
-			// a native type.  Throw an OverflowException.
-			if (bigDecimalIndex >= 0 && bigDecimalIndex < len)
-				throw new OverflowException ();
-			try {
-				command.Tds.ColumnValues.CopyTo (0, values, 0,
-								 len > command.Tds.ColumnValues.Count ? command.Tds.ColumnValues.Count : len);
-			} catch (TdsInternalException ex) {
-				command.Connection.Close ();
-				throw SqlException.FromTdsInternalException ((TdsInternalException) ex);
+			if ((command.CommandBehavior & CommandBehavior.SequentialAccess) != 0) {
+				for (int i = 0; i < columns; ++i) {
+					values [i] = tds.GetSequentialColumnValue (i);
+				}
+			} else {
+				int bigDecimalIndex = tds.ColumnValues.BigDecimalIndex;
+
+				// If a four-byte decimal is stored, then we can't convert to
+				// a native type.  Throw an OverflowException.
+				if (bigDecimalIndex >= 0 && bigDecimalIndex < len)
+					throw new OverflowException ();
+				try {
+					tds.ColumnValues.CopyTo (0, values, 0, columns);
+				} catch (TdsInternalException ex) {
+					command.Connection.Close ();
+					throw SqlException.FromTdsInternalException ((TdsInternalException)ex);
+				}
 			}
-			return (len < FieldCount ? len : FieldCount);
+
+			return columns;
 		}
 
-#if !NET_2_0
-		void IDisposable.Dispose ()
-		{
-			Dispose (true);
-			GC.SuppressFinalize (this);
-		}
-#endif
 
-#if NET_2_0
 		public override IEnumerator GetEnumerator ()
-#else
-		IEnumerator IEnumerable.GetEnumerator ()
-#endif
 		{
 			return new DbEnumerator (this);
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		bool IsDBNull (int i)
 		{
 			return GetValue (i) == DBNull.Value;
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		bool NextResult ()
 		{
 			ValidateState ();
@@ -1516,9 +1334,7 @@ namespace System.Data.SqlClient
 		}
 
 		public
-#if NET_2_0
 		override
-#endif // NET_2_0
 		bool Read ()
 		{
 			ValidateState ();
@@ -1574,7 +1390,6 @@ namespace System.Data.SqlClient
 			return new InvalidCastException (message);
 		}
 
-#if NET_2_0
 		public override Type GetProviderSpecificFieldType (int i)
 		{
 			return (GetSqlValue (i).GetType());
@@ -1599,7 +1414,17 @@ namespace System.Data.SqlClient
 			SqlBytes sb = new SqlBytes (val);
 			return (sb);
 		}
-#endif // NET_2_0
+
+		public override T GetFieldValue<T> (int i)
+		{
+			return (T)GetValue(i);
+		}
+
+		[MonoTODO]
+		public virtual XmlReader GetXmlReader (int i)
+		{
+			throw new NotImplementedException ();	
+		}
 
 		#endregion // Methods
 	}
