@@ -152,15 +152,16 @@ namespace MonoTests.System.Net.Http
 
 		const int WaitTimeout = 5000;
 
-		string port, TestHost, LocalServer;
+		string TestHost, LocalServer;
+		int port;
 
 		[SetUp]
 		public void SetupFixture ()
 		{
 			if (Environment.OSVersion.Platform == PlatformID.Win32NT) {
-				port = "810";
+				port = 810;
 			} else {
-				port = "8810";
+				port = 8810;
 			}
 
 			TestHost = "localhost:" + port;
@@ -606,6 +607,39 @@ namespace MonoTests.System.Net.Http
 			}
 		}
 
+		[Test]
+		public void Send_Complete_CustomHeaders_Host ()
+		{
+			bool? failed = null;
+			var listener = CreateListener (l => {
+				var request = l.Request;
+
+				try {
+					Assert.AreEqual ("customhost", request.Headers["Host"], "#1");
+					failed = false;
+				} catch {
+					failed = true;
+				}
+			});
+
+			try {
+				var client = new HttpClient ();
+
+				client.DefaultRequestHeaders.Add("Host", "customhost");
+
+				var request = new HttpRequestMessage (HttpMethod.Get, LocalServer);
+
+				var response = client.SendAsync (request, HttpCompletionOption.ResponseHeadersRead).Result;
+
+				Assert.AreEqual ("", response.Content.ReadAsStringAsync ().Result, "#100");
+				Assert.AreEqual (HttpStatusCode.OK, response.StatusCode, "#101");
+				Assert.AreEqual (false, failed, "#102");
+			} finally {
+				listener.Abort ();
+				listener.Close ();
+			}
+		}
+
 		[Test]
 		public void Send_Transfer_Encoding_Chunked ()
 		{
@@ -964,8 +998,8 @@ namespace MonoTests.System.Net.Http
 		public void GetString_Many ()
 		{
 			var client = new HttpClient ();
-			var t1 = client.GetStringAsync ("http://www.google.com");
-			var t2 = client.GetStringAsync ("http://www.google.com");
+			var t1 = client.GetStringAsync ("http://example.org");
+			var t2 = client.GetStringAsync ("http://example.org");
 			Assert.IsTrue (Task.WaitAll (new [] { t1, t2 }, WaitTimeout));		
 		}
 
@@ -1027,9 +1061,20 @@ namespace MonoTests.System.Net.Http
 				var response = l.Response;
 
 				response.StatusCode = (int)HttpStatusCode.Moved;
-				response.RedirectLocation = "http://xamarin.com/";
+				response.RedirectLocation = "http://localhost:8811/";
 			});
 
+			var listener2 = CreateListener (l => {
+				var response = l.Response;
+
+				response.StatusCode = (int)HttpStatusCode.OK;
+				response.OutputStream.WriteByte (0x68);
+				response.OutputStream.WriteByte (0x65);
+				response.OutputStream.WriteByte (0x6c);
+				response.OutputStream.WriteByte (0x6c);
+				response.OutputStream.WriteByte (0x6f);
+			}, 8811);
+
 			try {
 				var chandler = new HttpClientHandler ();
 				chandler.AllowAutoRedirect = true;
@@ -1038,10 +1083,13 @@ namespace MonoTests.System.Net.Http
 				var r = client.GetAsync (LocalServer);
 				Assert.IsTrue (r.Wait (WaitTimeout), "#1");
 				var resp = r.Result;
-				Assert.AreEqual ("http://xamarin.com/", resp.RequestMessage.RequestUri.AbsoluteUri, "#2");
+				Assert.AreEqual ("http://localhost:8811/", resp.RequestMessage.RequestUri.AbsoluteUri, "#2");
+				Assert.AreEqual ("hello", resp.Content.ReadAsStringAsync ().Result, "#3");
 			} finally {
 				listener.Abort ();
 				listener.Close ();
+				listener2.Abort ();
+				listener2.Close ();
 			}
 		}
 
@@ -1098,6 +1146,11 @@ namespace MonoTests.System.Net.Http
 		}
 
 		HttpListener CreateListener (Action<HttpListenerContext> contextAssert)
+		{
+			return CreateListener (contextAssert, port);
+		}
+
+		HttpListener CreateListener (Action<HttpListenerContext> contextAssert, int port)
 		{
 			var l = new HttpListener ();
 			l.Prefixes.Add (string.Format ("http://+:{0}/", port));
