@@ -133,7 +133,6 @@ namespace MonoTests.System.Runtime.Caching
 		{
 			var mc = new MemoryCache ("MyCache");
 			Assert.AreEqual ("MyCache", mc.Name, "#A1");
-			Assert.AreEqual (98, mc.PhysicalMemoryLimit, "#A2");
 			// Value of this property is different from system to system
 			//Assert.AreEqual (0, mc.CacheMemoryLimit, "#A3");
 			Assert.AreEqual (TimeSpan.FromMinutes (2), mc.PollingInterval, "#A4");
@@ -152,7 +151,6 @@ namespace MonoTests.System.Runtime.Caching
 		{
 			var mc = MemoryCache.Default;
 			Assert.AreEqual ("Default", mc.Name, "#A1");
-			Assert.AreEqual (98, mc.PhysicalMemoryLimit, "#A2");
 			// Value of this property is different from system to system
 			//Assert.AreEqual (0, mc.CacheMemoryLimit, "#A3");
 			Assert.AreEqual (TimeSpan.FromMinutes (2), mc.PollingInterval, "#A4");
@@ -170,12 +168,10 @@ namespace MonoTests.System.Runtime.Caching
 		public void ConstructorValues ()
 		{
 			var config = new NameValueCollection ();
-			config.Add ("PhysicalMemoryLimitPercentage", "0");
 			config.Add ("CacheMemoryLimitMegabytes", "1");
 			config.Add ("pollingInterval", "00:10:00");
 
 			var mc = new MemoryCache ("MyCache", config);
-			Assert.AreEqual (98, mc.PhysicalMemoryLimit, "#A1");
 			Assert.AreEqual (1048576, mc.CacheMemoryLimit, "#A2");
 			Assert.AreEqual (TimeSpan.FromMinutes (10), mc.PollingInterval, "#A3");
 
@@ -837,17 +833,9 @@ namespace MonoTests.System.Runtime.Caching
 		{
 			bool expired = false;
 			CacheEntryRemovedReason reason = CacheEntryRemovedReason.CacheSpecificEviction;
-			NameValueCollection config;
-			int sleepPeriod;
-#if !DOTNET
-			config = new NameValueCollection ();
-			config.Add ("__MonoTimerPeriod", "1");
-			sleepPeriod = 1100;
-#else
-			config = null;
-			sleepPeriod = 20100; // 20s is the .NET period - discovered by experimentation
-#endif
-			var mc = new PokerMemoryCache ("MyCache", config);
+			int sleepPeriod = 1100;
+
+			var mc = new PokerMemoryCache ("MyCache");
 			var cip = new CacheItemPolicy ();
 
 			cip.RemovedCallback = (CacheEntryRemovedArguments args) => {
@@ -875,6 +863,7 @@ namespace MonoTests.System.Runtime.Caching
 			mc.Set ("key", "value", cip);
 			Thread.Sleep (sleepPeriod);
 
+			Assert.IsNull (mc.Get ("key"), "#A3-0");
 			Assert.IsTrue (expired, "#A3-1");
 			Assert.AreEqual (CacheEntryRemovedReason.Expired, reason, "#A3-2");
 
@@ -907,6 +896,10 @@ namespace MonoTests.System.Runtime.Caching
 			mc.Set ("key4", "value4", cip);
 			
 			Thread.Sleep (sleepPeriod);
+			Assert.IsNull (mc.Get ("key1"), "#A4-1");
+			Assert.IsNull (mc.Get ("key2"), "#A4-2");
+			Assert.IsNull (mc.Get ("key3"), "#A4-3");
+			Assert.IsNotNull (mc.Get ("key4"), "#A4-4");
 			Assert.AreEqual (3, expiredCount, "#A4");
 		}
 
@@ -974,7 +967,7 @@ namespace MonoTests.System.Runtime.Caching
 			var mc = new PokerMemoryCache ("MyCache");
 
 			AssertExtensions.Throws<ArgumentNullException> (() => {
-				mc.GetValues (null);
+				mc.GetValues ((string[]) null);
 			}, "#A1-1");
 
 			AssertExtensions.Throws<NotSupportedException> (() => {
@@ -1179,11 +1172,10 @@ namespace MonoTests.System.Runtime.Caching
 			for (int i = 0; i < 10; i++)
 				mc.Set ("key" + i.ToString (), "value" + i.ToString (), null);
 
-			// .NET doesn't touch the freshest 10 entries
 			Assert.AreEqual (10, mc.GetCount (), "#A1-1");
 			long trimmed = mc.Trim (50);
-			Assert.AreEqual (0, trimmed, "#A1-2");
-			Assert.AreEqual (10, mc.GetCount (), "#A1-3");
+			Assert.AreEqual (5, trimmed, "#A1-2");
+			Assert.AreEqual (5, mc.GetCount (), "#A1-3");
 
 			mc = new MemoryCache ("MyCache", config);
 			// Only entries 11- are considered for removal
@@ -1192,8 +1184,8 @@ namespace MonoTests.System.Runtime.Caching
 
 			Assert.AreEqual (11, mc.GetCount (), "#A2-1");
 			trimmed = mc.Trim (50);
-			Assert.AreEqual (1, trimmed, "#A2-2");
-			Assert.AreEqual (10, mc.GetCount (), "#A2-3");
+			Assert.AreEqual (6, trimmed, "#A2-2");
+			Assert.AreEqual (5, mc.GetCount (), "#A2-3");
 
 			mc = new MemoryCache ("MyCache", config);
 			// Only entries 11- are considered for removal
@@ -1202,8 +1194,8 @@ namespace MonoTests.System.Runtime.Caching
 
 			Assert.AreEqual (125, mc.GetCount (), "#A3-1");
 			trimmed = mc.Trim (50);
-			Assert.AreEqual (62, trimmed, "#A3-2");
-			Assert.AreEqual (63, mc.GetCount (), "#A3-3");
+			Assert.AreEqual (63, trimmed, "#A3-2");
+			Assert.AreEqual (62, mc.GetCount (), "#A3-3");
 
 			// Testing the removal order
 			mc = new MemoryCache ("MyCache", config);
@@ -1225,11 +1217,176 @@ namespace MonoTests.System.Runtime.Caching
 			Assert.AreEqual (25, trimmed, "#A4-2");
 			Assert.AreEqual (25, removed.Count, "#A4-3");
 
-			// OK, this is odd... The list is correct in terms of entries removed but the entries
-			// are removed in the _MOST_ frequently used order, within the group selected for removal.
-			for (int i = 24; i >= 0; i--) {
-				int idx = 24 - i;
-				Assert.AreEqual ("key" + i.ToString (), removed [idx], "#A5-" + idx.ToString ());
+			for (int i = 0; i < 25; i++)
+				Assert.AreEqual ("key" + i.ToString (), removed [i], "#A5-" + i.ToString ());
+		}
+		
+		[Test]
+		public void TestCacheShrink ()
+		{
+			const int HEAP_RESIZE_THRESHOLD = 8192 + 2;
+			const int HEAP_RESIZE_SHORT_ENTRIES = 2048;
+			const int HEAP_RESIZE_LONG_ENTRIES = HEAP_RESIZE_THRESHOLD - HEAP_RESIZE_SHORT_ENTRIES;			
+			
+			var config = new NameValueCollection ();
+			config["cacheMemoryLimitMegabytes"] = 0.ToString ();
+			config["physicalMemoryLimitPercentage"] = 100.ToString ();
+			config["pollingInterval"] = new TimeSpan (0, 0, 1).ToString ();
+			
+			using (var mc = new MemoryCache ("TestCacheShrink",  config)) {	
+				Assert.AreEqual (0, mc.GetCount (), "#CS1");
+							
+				// add some short duration entries
+				for (int i = 0; i < HEAP_RESIZE_SHORT_ENTRIES; i++) {
+					var expireAt = DateTimeOffset.Now.AddSeconds (3);
+					mc.Add ("short-" + i, i.ToString (), expireAt);
+				}
+				
+				Assert.AreEqual (HEAP_RESIZE_SHORT_ENTRIES, mc.GetCount (), "#CS2");
+							
+				// add some long duration entries				
+				for (int i = 0; i < HEAP_RESIZE_LONG_ENTRIES; i++) {
+					var expireAt = DateTimeOffset.Now.AddSeconds (12);
+					mc.Add ("long-" + i, i.ToString (), expireAt);
+				}															
+				
+				Assert.AreEqual (HEAP_RESIZE_LONG_ENTRIES + HEAP_RESIZE_SHORT_ENTRIES, mc.GetCount(), "#CS3");
+				
+				// wait for the cache thread to expire the short duration items, this will also shrink the size of the cache
+				global::System.Threading.Thread.Sleep (5 * 1000);
+				
+				for (int i = 0; i < HEAP_RESIZE_SHORT_ENTRIES; i++) {
+					Assert.IsNull (mc.Get ("short-" + i), "#CS4-" + i);
+				}
+				Assert.AreEqual (HEAP_RESIZE_LONG_ENTRIES, mc.GetCount (), "#CS4");	
+				
+				// add some new items into the cache, this will grow the cache again
+				for (int i = 0; i < HEAP_RESIZE_LONG_ENTRIES; i++) {				
+					mc.Add("final-" + i, i.ToString (), DateTimeOffset.Now.AddSeconds (4));
+				}			
+				
+				Assert.AreEqual (HEAP_RESIZE_LONG_ENTRIES + HEAP_RESIZE_LONG_ENTRIES, mc.GetCount (), "#CS5");	
+			}
+		}
+
+		[Test]
+		public void TestExpiredGetValues ()
+		{
+			var config = new NameValueCollection ();
+			config["cacheMemoryLimitMegabytes"] = 0.ToString ();
+			config["physicalMemoryLimitPercentage"] = 100.ToString ();
+			config["pollingInterval"] = new TimeSpan (0, 0, 10).ToString ();
+			
+			using (var mc = new MemoryCache ("TestExpiredGetValues",  config)) {
+				Assert.AreEqual (0, mc.GetCount (), "#EGV1");
+
+				var keys = new List<string> ();
+
+				// add some short duration entries
+				for (int i = 0; i < 10; i++) {
+					var key = "short-" + i;
+					var expireAt = DateTimeOffset.Now.AddSeconds (1);
+					mc.Add (key, i.ToString (), expireAt);
+
+					keys.Add (key);
+				}
+
+				Assert.AreEqual (10, mc.GetCount (), "#EGV2");
+
+				global::System.Threading.Thread.Sleep (1000);
+
+				// we have waited but the items won't be expired by the timer since it wont have fired yet
+				Assert.AreEqual (10, mc.GetCount (), "#EGV3");
+
+				// calling GetValues() will expire the items since we are now past their expiresAt
+				mc.GetValues (keys);
+
+				Assert.AreEqual (0, mc.GetCount (), "#EGV4");
+			}
+		}
+
+		[Test]
+		public void TestCacheExpiryOrdering ()
+		{
+			var config = new NameValueCollection ();
+			config["cacheMemoryLimitMegabytes"] = 0.ToString ();
+			config["physicalMemoryLimitPercentage"] = 100.ToString ();
+			config["pollingInterval"] = new TimeSpan (0, 0, 1).ToString ();
+
+			using (var mc = new MemoryCache ("TestCacheExpiryOrdering",  config)) {
+				Assert.AreEqual (0, mc.GetCount (), "#CEO1");
+
+				// add long lived items into the cache first
+				for (int i = 0; i < 100; i++) {
+					var cip = new CacheItemPolicy ();
+					cip.SlidingExpiration = new TimeSpan (0, 0, 10);
+					mc.Add ("long-" + i, i, cip);
+				}
+
+				Assert.AreEqual (100, mc.GetCount (), "#CEO2");
+
+				// add shorter lived items into the cache, these should expire first
+				for (int i = 0; i < 100; i++) {
+					var cip = new CacheItemPolicy ();
+					cip.SlidingExpiration = new TimeSpan(0, 0, 1);
+					mc.Add ("short-" + i, i, cip);
+				}
+
+				Assert.AreEqual (200, mc.GetCount (), "#CEO3");
+
+				global::System.Threading.Thread.Sleep (4 * 1000);
+
+				for (int i = 0; i < 100; i++) {
+					Assert.IsNull (mc.Get ("short-" + i), "#CEO4-" + i);
+				}
+				Assert.AreEqual (100, mc.GetCount (), "#CEO4");
+			}
+		}
+
+		[Test]
+		public void TestCacheSliding ()
+		{    
+			var config = new NameValueCollection ();
+			config["cacheMemoryLimitMegabytes"] = 0.ToString ();
+			config["physicalMemoryLimitPercentage"] = 100.ToString ();
+			config["pollingInterval"] = new TimeSpan (0, 0, 1).ToString ();
+
+			using (var mc = new MemoryCache ("TestCacheSliding",  config)) {
+				Assert.AreEqual (0, mc.GetCount (), "#CSL1");
+
+				var cip = new CacheItemPolicy();
+				// The sliding expiration timeout has to be greater than 1 second because
+				// .NET implementation ignores timeouts updates smaller than
+				// CacheExpires.MIN_UPDATE_DELTA which is equal to 1.
+				cip.SlidingExpiration = new TimeSpan (0, 0, 2);
+				mc.Add("slidingtest", "42", cip);
+
+				mc.Add("expire1", "1", cip);
+				mc.Add("expire2", "2", cip);
+				mc.Add("expire3", "3", cip);
+				mc.Add("expire4", "4", cip);
+				mc.Add("expire5", "5", cip);
+
+				Assert.AreEqual (6, mc.GetCount (), "#CSL2");
+
+				for (int i = 0; i < 50; i++) {
+					global::System.Threading.Thread.Sleep (100);
+
+					var item = mc.Get ("slidingtest");
+					Assert.AreNotEqual (null, item, "#CSL3-" + i);
+				}
+
+				Assert.IsNull (mc.Get ("expire1"), "#CSL4-1");
+				Assert.IsNull (mc.Get ("expire2"), "#CSL4-2");
+				Assert.IsNull (mc.Get ("expire3"), "#CSL4-3");
+				Assert.IsNull (mc.Get ("expire4"), "#CSL4-4");
+				Assert.IsNull (mc.Get ("expire5"), "#CSL4-5");
+				Assert.AreEqual (1, mc.GetCount (), "#CSL4");
+
+				global::System.Threading.Thread.Sleep (4 * 1000);
+
+				Assert.IsNull (mc.Get ("slidingtest"), "#CSL5a");
+				Assert.AreEqual (0, mc.GetCount (), "#CSL5");
 			}
 		}
 	}
