@@ -34,6 +34,8 @@ using System.ServiceModel.Description;
 using System.ServiceModel.Dispatcher;
 using NUnit.Framework;
 
+using MonoTests.Helpers;
+
 namespace MonoTests.System.ServiceModel
 {
 	[TestFixture]
@@ -172,10 +174,11 @@ namespace MonoTests.System.ServiceModel
 		[Test]
 		public void AddServiceEndpoint2_4 ()
 		{
-			ServiceHost host = new ServiceHost (typeof (HogeFuga), new Uri ("http://localhost:37564"));
+			var ep = "http://" + NetworkHelpers.LocalEphemeralEndPoint().ToString();
+			ServiceHost host = new ServiceHost (typeof (HogeFuga), new Uri (ep));
 			var binding = new BasicHttpBinding ();
-			host.AddServiceEndpoint (typeof (IHoge), binding, new Uri ("http://localhost:37564"));
-			host.AddServiceEndpoint (typeof (IFuga), binding, new Uri ("http://localhost:37564"));
+			host.AddServiceEndpoint (typeof (IHoge), binding, new Uri (ep));
+			host.AddServiceEndpoint (typeof (IFuga), binding, new Uri (ep));
 
 			// Use the same binding, results in one ChannelDispatcher (actually two, for metadata/debug behavior).
 			host.Open ();
@@ -203,6 +206,26 @@ namespace MonoTests.System.ServiceModel
 			host.Close (); // should not reach here. It is to make sure to close unexpectedly opened host.
 		}
 
+		[Test]
+		public void Open ()
+		{
+			ServiceHost host = new ServiceHost (typeof (ZeroOperationsImpl));
+			host.AddServiceEndpoint (typeof (IHaveZeroOperarationsContract), new BasicHttpBinding (), "http://localhost/echo");
+
+			try {
+				host.Open ();
+				Assert.Fail ("InvalidOperationException expected");
+			} 
+			catch (InvalidOperationException e) {
+				//"ContractDescription 'IHaveZeroOperarationsContract' has zero operations; a contract must have at least one operation."
+				StringAssert.Contains ("IHaveZeroOperarationsContract", e.Message);
+			}
+			finally {
+				if (host.State == CommunicationState.Opened)
+					host.Close (); // It is to make sure to close unexpectedly opened host if the test fail.
+			}
+		}
+
 		[Test]
 		public void AddServiceEndpoint4 ()
 		{
@@ -228,6 +251,21 @@ namespace MonoTests.System.ServiceModel
 			host.AddServiceEndpoint ("ISuchTypeDoesNotExist", new BasicHttpBinding (), "rel");
 		}
 
+		[Test]
+		public void AddServiceEndpoint7 ()
+		{
+			ServiceHost host = new ServiceHost (typeof (Foo), new Uri ("http://localhost/echo"));
+			var a = host.AddServiceEndpoint (typeof (Foo), new BasicHttpBinding (), "a");
+			Console.WriteLine (a.Address);
+			Assert.AreEqual ("http", a.Address.Uri.Scheme, "#1");
+			Assert.AreEqual ("http://localhost/echo/a", a.Address.Uri.AbsoluteUri, "#2");
+
+			var b = host.AddServiceEndpoint (typeof (Foo), new BasicHttpBinding (), "/b");
+			Console.WriteLine (b.Address);
+			Assert.AreEqual ("http", b.Address.Uri.Scheme, "#3");
+			Assert.AreEqual ("http://localhost/echo/b", b.Address.Uri.AbsoluteUri, "#4");
+		}
+		
 		[Test]
 		[ExpectedException (typeof (InvalidOperationException))]
 		public void AddServiceEndpointMexWithNoImpl ()
@@ -261,13 +299,14 @@ namespace MonoTests.System.ServiceModel
 		[Test]
 		public void InstanceWithNonSingletonMode ()
 		{
+			var ep = NetworkHelpers.LocalEphemeralEndPoint().ToString();
 			ServiceHost host = new ServiceHost (
 				new NonSingletonService ());
 			Assert.IsNotNull (host.Description.Behaviors.Find<ServiceBehaviorAttribute> ().GetWellKnownSingleton (), "premise1");
 			host.AddServiceEndpoint (
 				typeof (NonSingletonService),
 				new BasicHttpBinding (),
-				new Uri ("http://localhost:37564/s1"));
+				new Uri ("http://" + ep + "/s1"));
 
 			// in case Open() didn't fail, we need to close the host.
 			// And even if Close() caused the expected exception,
@@ -288,13 +327,14 @@ namespace MonoTests.System.ServiceModel
 		[Test]
 		public void InstanceWithSingletonMode ()
 		{
+            var ep = NetworkHelpers.LocalEphemeralEndPoint().ToString();
 			SingletonService instance = new SingletonService ();
 			ServiceHost host = new ServiceHost (instance);
 			Assert.IsNotNull (host.Description.Behaviors.Find<ServiceBehaviorAttribute> ().GetWellKnownSingleton (), "#1");
 			host.AddServiceEndpoint (
 				typeof (SingletonService),
 				new BasicHttpBinding (),
-				new Uri ("http://localhost:37564/s2"));
+				new Uri ("http://" + ep + "/s2"));
 
 			// in case Open() didn't fail, we need to close the host.
 			// And even if Close() caused the expected exception,
@@ -313,6 +353,29 @@ namespace MonoTests.System.ServiceModel
 			}
 		}
 
+		[Test]
+		public void InstanceWithSingletonMode_InheritServiceBehavior ()
+		{
+			// # 37035
+
+			var ep = NetworkHelpers.LocalEphemeralEndPoint ().ToString ();
+
+			ChildSingletonService instance = new ChildSingletonService ();
+			ServiceHost host = new ServiceHost (instance);
+
+			host.AddServiceEndpoint (typeof (SingletonService),
+						 new BasicHttpBinding (),
+						 new Uri ("http://" + ep + "/s3"));
+
+			try {
+				host.Open ();
+			} catch (InvalidOperationException ex) {
+				Assert.Fail ("InstanceContextMode was not inherited from parent, exception was: {0}", ex);
+			} finally {
+				host.Close ();
+			}
+		}
+
 		[ServiceContract]
 		interface IBar
 		{
@@ -345,7 +408,21 @@ namespace MonoTests.System.ServiceModel
 			[OperationContract]
 			void DoY ();
 		}
-		
+
+		[ServiceContract]
+		interface IHaveZeroOperarationsContract
+		{
+			string Echo (string source);
+		}
+
+		class ZeroOperationsImpl : IHaveZeroOperarationsContract
+		{
+			public string Echo(string source)
+			{
+				return null;
+			}
+		}
+
 		class HogeFuga : IHoge, IFuga
 		{
 			public void DoX () {}
@@ -392,7 +469,14 @@ namespace MonoTests.System.ServiceModel
 		public class SingletonService
 		{
 			[OperationContract]
-			public void Process (string input)
+			public virtual void Process (string input)
+			{
+			}
+		}
+
+		public class ChildSingletonService : SingletonService
+		{
+			public override void Process (string input)
 			{
 			}
 		}
