@@ -104,6 +104,13 @@ namespace System.Web.Hosting {
 			get { return vpath_provider; }
 		}
 
+		public static bool InClientBuildManager {
+			get {
+				// Mono doesn't have a ClientBuildManager, so we can't be in it. Simple as that.
+				return false;
+			}
+		}
+
 		public static void DecrementBusyCount ()
 		{
 			Interlocked.Decrement (ref busy_count);
