@@ -1,3 +1,4 @@
+#if !MONO_FEATURE_NEW_TLS
 using System.Diagnostics;
 
 namespace System.Net {
@@ -14,6 +15,12 @@ namespace System.Net {
 			}
 		}
 
+		internal static TraceSource HttpListener {
+			get {
+				return null;
+			}
+		}
+
 		[Conditional ("TRACE")]
  		internal static void Enter(TraceSource traceSource, object obj, string method, object paramObject) {
  		}
@@ -21,6 +28,12 @@ namespace System.Net {
 		[Conditional ("TRACE")]
 		internal static void Exit(TraceSource traceSource, object obj, string method, object retObject) {
 		}
+
+		internal static void PrintInfo(TraceSource traceSource, object obj, string method, string msg) {
+		}
+
+		internal static void PrintWarning(TraceSource traceSource, object obj, string method, string msg) {
+		}
 	}
 
 #if MOBILE
@@ -31,4 +44,5 @@ namespace System.Net {
 	}
 
 #endif
-}
\ No newline at end of file
+}
+#endif
