@@ -28,12 +28,27 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_2_0
-
 #if SECURITY_DEP
+#if MONO_SECURITY_ALIAS
+extern alias MonoSecurity;
+#endif
+#if MONO_X509_ALIAS
 extern alias PrebuiltSystem;
 #endif
 
+#if MONO_SECURITY_ALIAS
+using MSI = MonoSecurity::Mono.Security.Interface;
+#else
+using MSI = Mono.Security.Interface;
+#endif
+#if MONO_X509_ALIAS
+using X509CertificateCollection = PrebuiltSystem::System.Security.Cryptography.X509Certificates.X509CertificateCollection;
+#else
+using X509CertificateCollection = System.Security.Cryptography.X509Certificates.X509CertificateCollection;
+#endif
+using System.Security.Cryptography.X509Certificates;
+#endif
+
 using System;
 using System.Collections.Generic;
 using System.ComponentModel;
@@ -42,24 +57,19 @@ using System.IO;
 using System.Net;
 using System.Net.Mime;
 using System.Net.Sockets;
-using System.Security.Cryptography.X509Certificates;
 using System.Text;
 using System.Threading;
-using System.Reflection;
 using System.Net.Configuration;
 using System.Configuration;
 using System.Net.Security;
 using System.Security.Authentication;
-
-#if SECURITY_DEP
-using X509CertificateCollection = PrebuiltSystem::System.Security.Cryptography.X509Certificates.X509CertificateCollection;
-#endif
+using System.Threading.Tasks;
+using Mono.Net.Security;
 
 namespace System.Net.Mail {
+	[Obsolete ("SmtpClient and its network of types are poorly designed, we strongly recommend you use https://github.com/jstedfast/MailKit and https://github.com/jstedfast/MimeKit instead")]
 	public class SmtpClient
-#if NET_4_0
 	: IDisposable
-#endif
 	{
 		#region Fields
 
@@ -121,9 +131,7 @@ namespace System.Net.Mail {
 			if (cfg != null) {
 				this.host = cfg.Network.Host;
 				this.port = cfg.Network.Port;
-#if NET_4_0
 				this.enableSsl = cfg.Network.EnableSsl;
-#endif
 				TargetName = cfg.Network.TargetName;
 				if (this.TargetName == null)
 					TargetName = "SMTPSVC/" + (host != null ? host : "");
@@ -151,6 +159,8 @@ namespace System.Net.Mail {
 
 			if (port != 0)
 				this.port = port;
+			else if (this.port == 0)
+				this.port = 25;
 		}
 
 		#endregion // Constructors
@@ -168,9 +178,7 @@ namespace System.Net.Mail {
 		}
 #endif
 
-#if NET_4_0
 		public
-#endif
 		string TargetName { get; set; }
 
 		public ICredentialsByHost Credentials {
@@ -258,7 +266,6 @@ namespace System.Net.Mail {
 		#endregion // Events 
 
 		#region Methods
-#if NET_4_0
 		public void Dispose ()
 		{
 			Dispose (true);
@@ -269,7 +276,6 @@ namespace System.Net.Mail {
 		{
 			// TODO: We should close all the connections and abort any async operations here
 		}
-#endif
 		private void CheckState ()
 		{
 			if (messageInProcess != null)
@@ -279,7 +285,7 @@ namespace System.Net.Mail {
 		private static string EncodeAddress(MailAddress address)
 		{
 			if (!String.IsNullOrEmpty (address.DisplayName)) {
-				string encodedDisplayName = ContentType.EncodeSubjectRFC2047 (address.DisplayName, Encoding.UTF8);
+				string encodedDisplayName = MailMessage.EncodeSubjectRFC2047 (address.DisplayName, Encoding.UTF8);
 				return "\"" + encodedDisplayName + "\" <" + address.Address + ">";
 			}
 			return address.ToString ();
@@ -301,7 +307,7 @@ namespace System.Net.Mail {
 
 		private string EncodeSubjectRFC2047 (MailMessage message)
 		{
-			return ContentType.EncodeSubjectRFC2047 (message.Subject, message.SubjectEncoding);
+			return MailMessage.EncodeSubjectRFC2047 (message.Subject, message.SubjectEncoding);
 		}
 
 		private string EncodeBody (MailMessage message)
@@ -544,8 +550,12 @@ namespace System.Net.Mail {
 				MailAddress from = message.From;
 				if (from == null)
 					from = defaultFrom;
-				
-				SendHeader (HeaderName.Date, DateTime.Now.ToString ("ddd, dd MMM yyyy HH':'mm':'ss zzz", DateTimeFormatInfo.InvariantInfo));
+
+				string dt = DateTime.Now.ToString("ddd, dd MMM yyyy HH':'mm':'ss zzz", DateTimeFormatInfo.InvariantInfo);
+				// remove ':' from time zone offset (e.g. from "+01:00")
+				dt = dt.Remove(dt.Length - 3, 1);
+				SendHeader(HeaderName.Date, dt);
+
 				SendHeader (HeaderName.From, EncodeAddress(from));
 				SendHeader (HeaderName.To, EncodeAddresses(message.To));
 				if (message.CC.Count > 0)
@@ -581,10 +591,21 @@ namespace System.Net.Mail {
 			
 			// FIXME: parse the list of extensions so we don't bother wasting
 			// our time trying commands if they aren't supported.
-			status = SendCommand ("EHLO " + Dns.GetHostName ());
+			
+			// get the host name (not fully qualified)
+			string fqdn = Dns.GetHostName ();
+			try {
+				// we'll try for the fully qualified name - ref: bug #33551
+				fqdn = Dns.GetHostEntry (fqdn).HostName;
+			}
+			catch (SocketException) {
+				// we could not resolve our name but will continue with the partial name
+				// IOW we won't fail to send email because of this - ref: bug #37246
+			}
+			status = SendCommand ("EHLO " + fqdn);
 			
 			if (IsError (status)) {
-				status = SendCommand ("HELO " + Dns.GetHostName ());
+				status = SendCommand ("HELO " + fqdn);
 				
 				if (IsError (status))
 					throw new SmtpException (status.StatusCode, status.Description);
@@ -601,10 +622,10 @@ namespace System.Net.Mail {
 				ResetExtensions();
 				writer = new StreamWriter (stream);
 				reader = new StreamReader (stream);
-				status = SendCommand ("EHLO " + Dns.GetHostName ());
+				status = SendCommand ("EHLO " + fqdn);
 			
 				if (IsError (status)) {
-					status = SendCommand ("HELO " + Dns.GetHostName ());
+					status = SendCommand ("HELO " + fqdn);
 				
 					if (IsError (status))
 						throw new SmtpException (status.StatusCode, status.Description);
@@ -651,16 +672,8 @@ namespace System.Net.Mail {
 					sfre.Add (new SmtpFailedRecipientException (status.StatusCode, message.Bcc [i].Address));
 			}
 
-#if TARGET_JVM // List<T>.ToArray () is not supported
-			if (sfre.Count > 0) {
-				SmtpFailedRecipientException[] xs = new SmtpFailedRecipientException[sfre.Count];
-				sfre.CopyTo (xs);
-				throw new SmtpFailedRecipientsException ("failed recipients", xs);
-			}
-#else
 			if (sfre.Count >0)
 				throw new SmtpFailedRecipientsException ("failed recipients", sfre.ToArray ());
-#endif
 
 			// DATA
 			status = SendCommand ("DATA");
@@ -704,13 +717,8 @@ namespace System.Net.Mail {
 			if (message.ReplyToList.Count > 0)
 				SendHeader ("Reply-To", EncodeAddresses (message.ReplyToList));
 
-#if NET_4_0
 			foreach (string s in message.Headers.AllKeys)
-				SendHeader (s, ContentType.EncodeSubjectRFC2047 (message.Headers [s], message.HeadersEncoding));
-#else
-			foreach (string s in message.Headers.AllKeys)
-				SendHeader (s, message.Headers [s]);
-#endif
+				SendHeader (s, MailMessage.EncodeSubjectRFC2047 (message.Headers [s], message.HeadersEncoding));
 	
 			AddPriorityHeader (message);
 
@@ -738,6 +746,41 @@ namespace System.Net.Mail {
 			Send (new MailMessage (from, to, subject, body));
 		}
 
+		public Task SendMailAsync (MailMessage message)
+		{
+			var tcs = new TaskCompletionSource<object> ();
+			SendCompletedEventHandler handler = null;
+			handler = (s, e) => SendMailAsyncCompletedHandler (tcs, e, handler, this);
+			SendCompleted += handler;
+			SendAsync (message, tcs);
+			return tcs.Task;
+		}
+
+		public Task SendMailAsync (string from, string recipients, string subject, string body)
+		{
+			return SendMailAsync (new MailMessage (from, recipients, subject, body));
+		}
+
+		static void SendMailAsyncCompletedHandler (TaskCompletionSource<object> source, AsyncCompletedEventArgs e, SendCompletedEventHandler handler, SmtpClient client)
+		{
+			if (source != e.UserState)
+				return;
+
+			client.SendCompleted -= handler;
+
+			if (e.Error != null) {
+				source.SetException (e.Error);
+				return;
+			}
+
+			if (e.Cancelled) {
+				source.SetCanceled ();
+				return;
+			}
+
+			source.SetResult (null);
+		}
+
 		private void SendDot()
 		{
 			writer.Write(".\r\n");
@@ -759,13 +802,8 @@ namespace System.Net.Mail {
 				CheckCancellation ();
 
 				if (escapeDots) {
-					int i;
-					for (i = 0; i < line.Length; i++) {
-						if (line[i] != '.')
-							break;
-					}
-					if (i > 0 && i == line.Length) {
-						line += ".";
+					if (line.Length > 0 && line[0] == '.') {
+						line = "." + line;
 					}
 				}
 				writer.Write (line);
@@ -921,21 +959,17 @@ try {
 					
 					contentType.Parameters ["type"] = av.ContentType.ToString ();
 					StartSection (inner_boundary, contentType);
-					StartSection (alt_boundary, av.ContentType, av.TransferEncoding);
+					StartSection (alt_boundary, av.ContentType, av);
 				} else {
 					contentType = new ContentType (av.ContentType.ToString ());
-					StartSection (inner_boundary, contentType, av.TransferEncoding);
+					StartSection (inner_boundary, contentType, av);
 				}
 
 				switch (av.TransferEncoding) {
 				case TransferEncoding.Base64:
 					byte [] content = new byte [av.ContentStream.Length];
 					av.ContentStream.Read (content, 0, content.Length);
-#if TARGET_JVM
-					SendData (Convert.ToBase64String (content));
-#else
 					    SendData (Convert.ToBase64String (content, Base64FormattingOptions.InsertLineBreaks));
-#endif
 					break;
 				case TransferEncoding.QuotedPrintable:
 					byte [] bytes = new byte [av.ContentStream.Length];
@@ -969,17 +1003,13 @@ try {
 		private void SendLinkedResources (MailMessage message, LinkedResourceCollection resources, string boundary)
 		{
 			foreach (LinkedResource lr in resources) {
-				StartSection (boundary, lr.ContentType, lr.TransferEncoding, lr);
+				StartSection (boundary, lr.ContentType, lr);
 
 				switch (lr.TransferEncoding) {
 				case TransferEncoding.Base64:
 					byte [] content = new byte [lr.ContentStream.Length];
 					lr.ContentStream.Read (content, 0, content.Length);
-#if TARGET_JVM
-					SendData (Convert.ToBase64String (content));
-#else
 					    SendData (Convert.ToBase64String (content, Base64FormattingOptions.InsertLineBreaks));
-#endif
 					break;
 				case TransferEncoding.QuotedPrintable:
 					byte [] bytes = new byte [lr.ContentStream.Length];
@@ -1005,17 +1035,13 @@ try {
 						contentType.CharSet = att.NameEncoding.HeaderName;
 					att.ContentDisposition.FileName = att.Name;
 				}
-				StartSection (boundary, contentType, att.TransferEncoding, att == body ? null : att.ContentDisposition);
+				StartSection (boundary, contentType, att, att != body);
 
 				byte [] content = new byte [att.ContentStream.Length];
 				att.ContentStream.Read (content, 0, content.Length);
 				switch (att.TransferEncoding) {
 				case TransferEncoding.Base64:
-#if TARGET_JVM
-					SendData (Convert.ToBase64String (content));
-#else
 					SendData (Convert.ToBase64String (content, Base64FormattingOptions.InsertLineBreaks));
-#endif
 					break;
 				case TransferEncoding.QuotedPrintable:
 					SendData (ToQuotedPrintable (content));
@@ -1051,35 +1077,27 @@ try {
 			SendData (string.Empty);
 		}
 
-		private void StartSection (string section, ContentType sectionContentType,TransferEncoding transferEncoding)
+		private void StartSection (string section, ContentType sectionContentType, AttachmentBase att)
 		{
 			SendData (String.Format ("--{0}", section));
 			SendHeader ("content-type", sectionContentType.ToString ());
-			SendHeader ("content-transfer-encoding", GetTransferEncodingName (transferEncoding));
+			SendHeader ("content-transfer-encoding", GetTransferEncodingName (att.TransferEncoding));
+			if (!string.IsNullOrEmpty (att.ContentId))
+				SendHeader("content-ID", "<" + att.ContentId + ">");
 			SendData (string.Empty);
 		}
 
-		private void StartSection(string section, ContentType sectionContentType, TransferEncoding transferEncoding, LinkedResource lr)
-		{
-			SendData (String.Format("--{0}", section));
-			SendHeader ("content-type", sectionContentType.ToString ());
-			SendHeader ("content-transfer-encoding", GetTransferEncodingName (transferEncoding));
-
-			if (lr.ContentId != null && lr.ContentId.Length > 0)
-				SendHeader("content-ID", "<" + lr.ContentId + ">");
-
-			SendData (string.Empty);
-		}
-
-		private void StartSection (string section, ContentType sectionContentType, TransferEncoding transferEncoding, ContentDisposition contentDisposition) {
+		private void StartSection (string section, ContentType sectionContentType, Attachment att, bool sendDisposition) {
 			SendData (String.Format ("--{0}", section));
+			if (!string.IsNullOrEmpty (att.ContentId))
+				SendHeader("content-ID", "<" + att.ContentId + ">");
 			SendHeader ("content-type", sectionContentType.ToString ());
-			SendHeader ("content-transfer-encoding", GetTransferEncodingName (transferEncoding));
-			if (contentDisposition != null)
-				SendHeader ("content-disposition", contentDisposition.ToString ());
+			SendHeader ("content-transfer-encoding", GetTransferEncodingName (att.TransferEncoding));
+			if (sendDisposition)
+				SendHeader ("content-disposition", att.ContentDisposition.ToString ());
 			SendData (string.Empty);
 		}
-
+		
 		// use proper encoding to escape input
 		private string ToQuotedPrintable (string input, Encoding enc)
 		{
@@ -1137,21 +1155,6 @@ try {
 			return "unknown";
 		}
 
-#if SECURITY_DEP
-		RemoteCertificateValidationCallback callback = delegate (object sender,
-									 X509Certificate certificate,
-									 X509Chain chain,
-									 SslPolicyErrors sslPolicyErrors) {
-			// honor any exciting callback defined on ServicePointManager
-			if (ServicePointManager.ServerCertificateValidationCallback != null)
-				return ServicePointManager.ServerCertificateValidationCallback (sender, certificate, chain, sslPolicyErrors);
-			// otherwise provide our own
-			if (sslPolicyErrors != SslPolicyErrors.None)
-				throw new InvalidOperationException ("SSL authentication error: " + sslPolicyErrors);
-			return true;
-			};
-#endif
-
 		private void InitiateSecureConnection () {
 			SmtpResponse response = SendCommand ("STARTTLS");
 
@@ -1159,13 +1162,14 @@ try {
 				throw new SmtpException (SmtpStatusCode.GeneralFailure, "Server does not support secure connections.");
 			}
 
-#if TARGET_JVM
-			((NetworkStream) stream).ChangeToSSLSocket ();
-#elif SECURITY_DEP
-			SslStream sslStream = new SslStream (stream, false, callback, null);
+#if SECURITY_DEP
+			var tlsProvider = MonoTlsProviderFactory.GetProviderInternal ();
+			var settings = MSI.MonoTlsSettings.CopyDefaultSettings ();
+			settings.UseServicePointManagerCallback = true;
+			var sslStream = tlsProvider.CreateSslStream (stream, false, settings);
 			CheckCancellation ();
 			sslStream.AuthenticateAsClient (Host, this.ClientCertificates, SslProtocols.Default, false);
-			stream = sslStream;
+			stream = sslStream.AuthenticatedStream;
 
 #else
 			throw new SystemException ("You are using an incomplete System.dll build");
@@ -1298,4 +1302,3 @@ try {
 	}
 }
 
-#endif // NET_2_0
