@@ -4,6 +4,7 @@
 // Authors:
 //	Gonzalo Paniagua Javier (gonzalo@novell.com)
 //	Atsushi Enomoto (atsushi@ximian.com)
+//	Marek Safar (marek.safar@gmail.com)
 //
 // Copyright (c) 2006-2007 Novell, Inc. (http://www.novell.com)
 //
@@ -43,6 +44,11 @@ namespace System.Net.NetworkInformation {
 
 		public static IPGlobalProperties GetIPGlobalProperties ()
 		{
+#if MONODROID
+			return new AndroidIPGlobalProperties ();
+#elif MONOTOUCH || XAMMAC || MOBILE_STATIC
+			return new UnixIPGlobalProperties ();
+#else
 			switch (Environment.OSVersion.Platform) {
 			case PlatformID.Unix:
 				MibIPGlobalProperties impl = null;
@@ -56,10 +62,16 @@ namespace System.Net.NetworkInformation {
 					if (File.Exists (impl.StatisticsFile))
 						return impl;
 				}
-				throw new NotSupportedException ("This platform is not supported");
+				return new UnixIPGlobalProperties ();
 			default:
 				return new Win32IPGlobalProperties ();
 			}
+#endif
+		}
+
+		internal static IPGlobalProperties InternalGetIPGlobalProperties()
+		{
+			return GetIPGlobalProperties ();
 		}
 
 		public abstract TcpConnectionInformation [] GetActiveTcpConnections ();
@@ -81,11 +93,7 @@ namespace System.Net.NetworkInformation {
 		public abstract NetBiosNodeType NodeType { get; }
 	}
 
-	// It expects /proc/net/snmp (or /usr/compat/linux/proc/net/snmp),
-	// formatted like:
-	// http://www.linuxdevcenter.com/linux/2000/11/16/example5.html
-	// http://www.linuxdevcenter.com/linux/2000/11/16/example2.html
-	class MibIPGlobalProperties : IPGlobalProperties
+	abstract class CommonUnixIPGlobalProperties : IPGlobalProperties
 	{
 		[DllImport ("libc")]
 		static extern int gethostname ([MarshalAs (UnmanagedType.LPArray, SizeParamIndex = 1)] byte [] name, int len);
@@ -93,6 +101,114 @@ namespace System.Net.NetworkInformation {
 		[DllImport ("libc")]
 		static extern int getdomainname ([MarshalAs (UnmanagedType.LPArray, SizeParamIndex = 1)] byte [] name, int len);
 
+		public override string DhcpScopeName {
+			get { return String.Empty; }
+		}
+
+		public override string DomainName {
+			get {
+				byte [] bytes = new byte [256];
+				if (getdomainname (bytes, 256) != 0)
+					throw new NetworkInformationException ();
+				int len = Array.IndexOf<byte> (bytes, 0);
+				return Encoding.ASCII.GetString (bytes, 0, len < 0 ? 256 : len);
+			}
+		}
+
+		public override string HostName {
+			get {
+				byte [] bytes = new byte [256];
+				if (gethostname (bytes, 256) != 0)
+					throw new NetworkInformationException ();
+				int len = Array.IndexOf<byte> (bytes, 0);
+				return Encoding.ASCII.GetString (bytes, 0, len < 0 ? 256 : len);
+			}
+		}
+
+		public override bool IsWinsProxy {
+			get { return false; } // no WINS
+		}
+
+		public override NetBiosNodeType NodeType {
+			get { return NetBiosNodeType.Unknown; } // no NetBios
+		}
+	}
+
+	class UnixIPGlobalProperties : CommonUnixIPGlobalProperties
+	{
+		public override TcpConnectionInformation [] GetActiveTcpConnections ()
+		{
+			throw new NotImplementedException ();
+		}
+
+		public override IPEndPoint [] GetActiveTcpListeners ()
+		{
+			throw new NotImplementedException ();
+		}
+
+		public override IPEndPoint [] GetActiveUdpListeners ()
+		{
+			throw new NotImplementedException ();
+		}
+
+		public override IcmpV4Statistics GetIcmpV4Statistics ()
+		{
+			throw new NotImplementedException ();
+		}
+
+		public override IcmpV6Statistics GetIcmpV6Statistics ()
+		{
+			throw new NotImplementedException ();
+		}
+
+		public override IPGlobalStatistics GetIPv4GlobalStatistics ()
+		{
+			throw new NotImplementedException ();
+		}
+
+		public override IPGlobalStatistics GetIPv6GlobalStatistics ()
+		{
+			throw new NotImplementedException ();
+		}
+
+		public override TcpStatistics GetTcpIPv4Statistics ()
+		{
+			throw new NotImplementedException ();
+		}
+
+		public override TcpStatistics GetTcpIPv6Statistics ()
+		{
+			throw new NotImplementedException ();
+		}
+
+		public override UdpStatistics GetUdpIPv4Statistics ()
+		{
+			throw new NotImplementedException ();
+		}
+
+		public override UdpStatistics GetUdpIPv6Statistics ()
+		{
+			throw new NotImplementedException ();
+		}
+	}
+
+#if MONODROID
+	sealed class AndroidIPGlobalProperties : UnixIPGlobalProperties
+	{
+		public override string DomainName {
+			get {
+				return String.Empty;
+			}
+		}
+	}
+#endif
+
+	// It expects /proc/net/snmp (or /usr/compat/linux/proc/net/snmp),
+	// formatted like:
+	// http://www.linuxdevcenter.com/linux/2000/11/16/example5.html
+	// http://www.linuxdevcenter.com/linux/2000/11/16/example2.html
+	class MibIPGlobalProperties : UnixIPGlobalProperties
+	{
 		public const string ProcDir = "/proc";
 		public const string CompatProcDir = "/usr/compat/linux/proc";
 
@@ -289,40 +405,9 @@ namespace System.Net.NetworkInformation {
 		{
 			return new MibUdpStatistics (GetProperties6 ("Udp6"));
 		}
-
-		public override string DhcpScopeName {
-			get { return String.Empty; }
-		}
-
-		public override string DomainName {
-			get {
-				byte [] bytes = new byte [256];
-				if (getdomainname (bytes, 256) != 0)
-					throw new NetworkInformationException ();
-				int len = Array.IndexOf<byte> (bytes, 0);
-				return Encoding.ASCII.GetString (bytes, 0, len < 0 ? 256 : len);
-			}
-		}
-
-		public override string HostName {
-			get {
-				byte [] bytes = new byte [256];
-				if (gethostname (bytes, 256) != 0)
-					throw new NetworkInformationException ();
-				int len = Array.IndexOf<byte> (bytes, 0);
-				return Encoding.ASCII.GetString (bytes, 0, len < 0 ? 256 : len);
-			}
-		}
-
-		public override bool IsWinsProxy {
-			get { return false; } // no WINS
-		}
-
-		public override NetBiosNodeType NodeType {
-			get { return NetBiosNodeType.Unknown; } // no NetBios
-		}
 	}
 
+#if !MOBILE
 	class Win32IPGlobalProperties : IPGlobalProperties
 	{
 		public const int AF_INET = 2;
@@ -550,31 +635,31 @@ namespace System.Net.NetworkInformation {
 
 		// PInvokes
 
-		[DllImport ("Iphlpapi.dll")]
+		[DllImport ("iphlpapi.dll")]
 		static extern int GetTcpTable (byte [] pTcpTable, ref int pdwSize, bool bOrder);
 
-		[DllImport ("Iphlpapi.dll")]
+		[DllImport ("iphlpapi.dll")]
 		static extern int GetTcp6Table (byte [] TcpTable, ref int SizePointer, bool Order);
 
-		[DllImport ("Iphlpapi.dll")]
+		[DllImport ("iphlpapi.dll")]
 		static extern int GetUdpTable (byte [] pUdpTable, ref int pdwSize, bool bOrder);
 
-		[DllImport ("Iphlpapi.dll")]
+		[DllImport ("iphlpapi.dll")]
 		static extern int GetUdp6Table (byte [] Udp6Table, ref int SizePointer, bool Order);
 
-		[DllImport ("Iphlpapi.dll")]
+		[DllImport ("iphlpapi.dll")]
 		static extern int GetTcpStatisticsEx (out Win32_MIB_TCPSTATS pStats, int dwFamily);
 
-		[DllImport ("Iphlpapi.dll")]
+		[DllImport ("iphlpapi.dll")]
 		static extern int GetUdpStatisticsEx (out Win32_MIB_UDPSTATS pStats, int dwFamily);
 
-		[DllImport ("Iphlpapi.dll")]
+		[DllImport ("iphlpapi.dll")]
 		static extern int GetIcmpStatistics (out Win32_MIBICMPINFO pStats, int dwFamily);
 
-		[DllImport ("Iphlpapi.dll")]
+		[DllImport ("iphlpapi.dll")]
 		static extern int GetIcmpStatisticsEx (out Win32_MIB_ICMP_EX pStats, int dwFamily);
 
-		[DllImport ("Iphlpapi.dll")]
+		[DllImport ("iphlpapi.dll")]
 		static extern int GetIpStatisticsEx (out Win32_MIB_IPSTATS pStats, int dwFamily);
 
 		// Win32 structures
@@ -583,7 +668,7 @@ namespace System.Net.NetworkInformation {
 		struct Win32_IN6_ADDR
 		{
 			[FieldOffset (0)]
-			[MarshalAs ((short) UnmanagedType.U1, SizeConst = 16)]
+			[MarshalAs ( UnmanagedType.ByValArray, SizeConst = 16)]
 			public byte [] Bytes;
 		}
 
@@ -656,4 +741,5 @@ namespace System.Net.NetworkInformation {
 			}
 		}
 	}
+#endif
 }
