@@ -6,6 +6,7 @@
 //	Atsushi Enomoto (atsushi@ximian.com)
 //
 // Copyright (c) 2006-2007 Novell, Inc. (http://www.novell.com)
+// Copyright 2015 Xamarin Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -26,7 +27,7 @@
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
-#if NET_2_0
+
 using System;
 using System.IO;
 using System.Text;
@@ -37,11 +38,14 @@ using System.Net.Sockets;
 using System.Security.Principal;
 using System.Security.Cryptography;
 using System.Runtime.InteropServices;
+using System.Threading;
+using System.Threading.Tasks;
 
 namespace System.Net.NetworkInformation {
 	[MonoTODO ("IPv6 support is missing")]
 	public class Ping : Component, IDisposable
 	{
+#if !MONOTOUCH
 		[StructLayout(LayoutKind.Sequential)]
 		struct cap_user_header_t
 		{
@@ -61,9 +65,13 @@ namespace System.Net.NetworkInformation {
 		static readonly string [] PingBinPaths = new string [] {
 			"/bin/ping",
 			"/sbin/ping",
-			"/usr/sbin/ping"
+			"/usr/sbin/ping",
+#if MONODROID
+			"/system/bin/ping"
+#endif
 		};
 		static readonly string PingBinPath;
+#endif
 		const int default_timeout = 4000; // 4 sec.
 		ushort identifier;
 
@@ -77,9 +85,11 @@ namespace System.Net.NetworkInformation {
 
 		BackgroundWorker worker;
 		object user_async_state;
+		CancellationTokenSource cts;
 		
 		public event PingCompletedEventHandler PingCompleted;
-		
+
+#if !MONOTOUCH
 		static Ping ()
 		{
 			if (Environment.OSVersion.Platform == PlatformID.Unix) {
@@ -101,6 +111,7 @@ namespace System.Net.NetworkInformation {
 			if (PingBinPath == null)
 				PingBinPath = "/bin/ping"; // default, fallback value
 		}
+#endif
 		
 		public Ping ()
 		{
@@ -110,7 +121,8 @@ namespace System.Net.NetworkInformation {
 			rng.GetBytes (randomIdentifier);
 			identifier = (ushort)(randomIdentifier [0] + (randomIdentifier [1] << 8));
 		}
-  
+
+#if !MONOTOUCH
 		[DllImport ("libc", EntryPoint="capget")]
 		static extern int capget (ref cap_user_header_t header, ref cap_user_data_t data);
 
@@ -137,6 +149,7 @@ namespace System.Net.NetworkInformation {
 				canSendPrivileged = false;
 			}
 		}
+#endif
 		
 		void IDisposable.Dispose ()
 		{
@@ -144,10 +157,12 @@ namespace System.Net.NetworkInformation {
 
 		protected void OnPingCompleted (PingCompletedEventArgs e)
 		{
-			if (PingCompleted != null)
-				PingCompleted (this, e);
 			user_async_state = null;
 			worker = null;
+			cts = null;
+
+			if (PingCompleted != null)
+				PingCompleted (this, e);
 		}
 
 		// Sync
@@ -208,11 +223,16 @@ namespace System.Net.NetworkInformation {
 				throw new ArgumentException ("buffer");
 			// options can be null.
 
+#if MONOTOUCH
+			throw new InvalidOperationException ();
+#else
 			if (canSendPrivileged)
 				return SendPrivileged (address, timeout, buffer, options);
 			return SendUnprivileged (address, timeout, buffer, options);
+#endif
 		}
 
+#if !MONOTOUCH
 		private PingReply SendPrivileged (IPAddress address, int timeout, byte [] buffer, PingOptions options)
 		{
 			IPEndPoint target = new IPEndPoint (address, 0);
@@ -279,7 +299,8 @@ namespace System.Net.NetworkInformation {
 
 		private PingReply SendUnprivileged (IPAddress address, int timeout, byte [] buffer, PingOptions options)
 		{
-			DateTime sentTime = DateTime.Now;
+#if MONO_FEATURE_PROCESS_START
+			DateTime sentTime = DateTime.UtcNow;
 
 			Process ping = new Process ();
 			string args = BuildPingArgs (address, timeout, options);
@@ -294,6 +315,7 @@ namespace System.Net.NetworkInformation {
 			ping.StartInfo.RedirectStandardOutput = true;
 			ping.StartInfo.RedirectStandardError = true;
 
+			IPStatus status = IPStatus.Unknown;
 			try {
 				ping.Start ();
 
@@ -302,24 +324,26 @@ namespace System.Net.NetworkInformation {
 				string stderr = ping.StandardError.ReadToEnd ();
 #pragma warning restore 219
 				
-				trip_time = (long) (DateTime.Now - sentTime).TotalMilliseconds;
+				trip_time = (long) (DateTime.UtcNow - sentTime).TotalMilliseconds;
 				if (!ping.WaitForExit (timeout) || (ping.HasExited && ping.ExitCode == 2))
-					return new PingReply (address, buffer, options, trip_time, IPStatus.TimedOut); 
-
-				if (ping.ExitCode == 1)
-					return new PingReply (address, buffer, options, trip_time, IPStatus.TtlExpired);
-			} catch (Exception) {
-				return new PingReply (address, buffer, options, trip_time, IPStatus.Unknown);
+					status = IPStatus.TimedOut;
+				else if (ping.ExitCode == 0)
+					status = IPStatus.Success;
+				else if (ping.ExitCode == 1)
+					status = IPStatus.TtlExpired;
+			} catch {
 			} finally {
-				if (ping != null) {
-					if (!ping.HasExited)
-						ping.Kill ();
-					ping.Dispose ();
-				}
+				if (!ping.HasExited)
+					ping.Kill ();
+				ping.Dispose ();
 			}
 
-			return new PingReply (address, buffer, options, trip_time, IPStatus.Success);
+			return new PingReply (address, buffer, options, trip_time, status);
+#else
+			throw new PlatformNotSupportedException ("Ping is not supported on this platform.");
+#endif // MONO_FEATURE_PROCESS_START
 		}
+#endif // !MONOTOUCH
 
 		// Async
 
@@ -361,7 +385,7 @@ namespace System.Net.NetworkInformation {
 
 		public void SendAsync (IPAddress address, int timeout, byte [] buffer, PingOptions options, object userToken)
 		{
-			if (worker != null)
+			if ((worker != null) || (cts != null))
 				throw new InvalidOperationException ("Another SendAsync operation is in progress");
 
 			worker = new BackgroundWorker ();
@@ -385,11 +409,17 @@ namespace System.Net.NetworkInformation {
 
 		public void SendAsyncCancel ()
 		{
+			if (cts != null) {
+				cts.Cancel ();
+				return;
+			}
+
 			if (worker == null)
 				throw new InvalidOperationException ("SendAsync operation is not in progress");
 			worker.CancelAsync ();
 		}
 
+#if !MONOTOUCH
 		// ICMP message
 
 		class IcmpMessage
@@ -509,28 +539,76 @@ namespace System.Net.NetworkInformation {
 			CultureInfo culture = CultureInfo.InvariantCulture;
 			StringBuilder args = new StringBuilder ();
 			uint t = Convert.ToUInt32 (Math.Floor ((timeout + 1000) / 1000.0));
-#if NET_2_0
-			bool is_mac = ((int) Environment.OSVersion.Platform == 6);
+			bool is_mac = Platform.IsMacOS;
 			if (!is_mac)
-#endif
 				args.AppendFormat (culture, "-q -n -c {0} -w {1} -t {2} -M ", DefaultCount, t, options.Ttl);
-#if NET_2_0
 			else
 				args.AppendFormat (culture, "-q -n -c {0} -t {1} -o -m {2} ", DefaultCount, t, options.Ttl);
 			if (!is_mac)
-#endif
 				args.Append (options.DontFragment ? "do " : "dont ");
-#if NET_2_0
 			else if (options.DontFragment)
 				args.Append ("-D ");
-#endif
 
 			args.Append (address.ToString ());
 
 			return args.ToString ();
 		}
+#endif // !MONOTOUCH
 
+		public Task<PingReply> SendPingAsync (IPAddress address, int timeout, byte [] buffer)
+		{
+			return SendPingAsync (address, default_timeout, default_buffer, new PingOptions ());
+		}
+
+		public Task<PingReply> SendPingAsync (IPAddress address, int timeout)
+		{
+			return SendPingAsync (address, default_timeout, default_buffer);
+		}
+
+		public Task<PingReply> SendPingAsync (IPAddress address)
+		{
+			return SendPingAsync (address, default_timeout);
+		}
+
+		public Task<PingReply> SendPingAsync (string hostNameOrAddress, int timeout, byte [] buffer)
+		{
+			return SendPingAsync (hostNameOrAddress, timeout, buffer, new PingOptions ());
+		}
+
+		public Task<PingReply> SendPingAsync (string hostNameOrAddress, int timeout, byte [] buffer, PingOptions options)
+		{
+			IPAddress address = Dns.GetHostEntry (hostNameOrAddress).AddressList [0];
+			return SendPingAsync (address, timeout, buffer, options);
+		}
+
+		public Task<PingReply> SendPingAsync (string hostNameOrAddress, int timeout)
+		{
+			return SendPingAsync (hostNameOrAddress, timeout, default_buffer);
+		}
+
+		public Task<PingReply> SendPingAsync (string hostNameOrAddress)
+		{
+			return SendPingAsync (hostNameOrAddress, default_timeout);
+		}
+
+		public Task<PingReply> SendPingAsync (IPAddress address, int timeout, byte [] buffer, PingOptions options)
+		{
+			if ((worker != null) || (cts != null))
+				throw new InvalidOperationException ("Another SendAsync operation is in progress");
+
+			var task = Task<PingReply>.Factory.StartNew (
+				() => Send (address, timeout, buffer, options), cts.Token);
+
+			task.ContinueWith ((t) => {
+				if (t.IsCanceled)
+					OnPingCompleted (new PingCompletedEventArgs (null, true, null, null));
+				else if (t.IsFaulted)
+					OnPingCompleted (new PingCompletedEventArgs (t.Exception, false, null, null));
+				else
+					OnPingCompleted (new PingCompletedEventArgs (null, false, null, t.Result));
+			});
+
+			return task;
+		}
 	}
 }
-#endif
-
