@@ -4,8 +4,10 @@
 // Author:
 //    Gonzalo Paniagua Javier <gonzalo@ximian.com>
 //    Sridhar Kulkarni (sridharkulkarni@gmail.com)
+//    Marek Safar (marek.safar@gmail.com)
 //
 // Copyright (C) Ximian, Inc. http://www.ximian.com
+// Copyright 2011 Xamarin Inc.
 //
 
 //
@@ -31,6 +33,7 @@
 
 using System;
 using System.Net;
+using System.Threading.Tasks;
 
 namespace System.Net.Sockets
 {
@@ -40,29 +43,20 @@ namespace System.Net.Sockets
 		private bool active = false;
 		private Socket socket;
 		private AddressFamily family = AddressFamily.InterNetwork;
-#if NET_2_0
 		private byte[] recvbuffer;
-#endif
 	
-#region Constructors
 		public UdpClient () : this(AddressFamily.InterNetwork)
 		{
 		}
 
-#if NET_1_1
 		public UdpClient(AddressFamily family)
 		{
 			if(family != AddressFamily.InterNetwork && family != AddressFamily.InterNetworkV6)
-#if NET_2_0
 				throw new ArgumentException ("Family must be InterNetwork or InterNetworkV6", "family");
-#else
-				throw new ArgumentException ("family");
-#endif
 
 			this.family = family;
 			InitSocket (null);
 		}
-#endif
 
 		public UdpClient (int port)
 		{
@@ -85,15 +79,10 @@ namespace System.Net.Sockets
 			InitSocket (localEP);
 		}
 
-#if NET_1_1
 		public UdpClient (int port, AddressFamily family)
 		{
 			if (family != AddressFamily.InterNetwork && family != AddressFamily.InterNetworkV6)
-#if NET_2_0
 				throw new ArgumentException ("Family must be InterNetwork or InterNetworkV6", "family");
-#else
-				throw new ArgumentException ("family");
-#endif
 
 			if (port < IPEndPoint.MinPort ||
 			    port > IPEndPoint.MaxPort) {
@@ -110,7 +99,6 @@ namespace System.Net.Sockets
 				localEP = new IPEndPoint (IPAddress.IPv6Any, port);
 			InitSocket (localEP);
 		}
-#endif
 
 		public UdpClient (string hostname, int port)
 		{
@@ -137,8 +125,6 @@ namespace System.Net.Sockets
 				socket.Bind (localEP);
 		}
 
-#endregion // Constructors
-#region Public methods
 #region Close
 		public void Close ()
 		{
@@ -221,14 +207,11 @@ namespace System.Net.Sockets
 			if(family == AddressFamily.InterNetwork)
 				socket.SetSocketOption (SocketOptionLevel.IP, SocketOptionName.DropMembership,
 					new MulticastOption (multicastAddr));
-#if NET_1_1
 			else
 				socket.SetSocketOption (SocketOptionLevel.IPv6, SocketOptionName.DropMembership,
 					new IPv6MulticastOption (multicastAddr));
-#endif
 		}
 
-#if NET_1_1
 		public void DropMulticastGroup (IPAddress multicastAddr,
 						int ifindex)
 		{
@@ -250,30 +233,22 @@ namespace System.Net.Sockets
 				socket.SetSocketOption (SocketOptionLevel.IPv6, SocketOptionName.DropMembership, new IPv6MulticastOption (multicastAddr, ifindex));
 			}
 		}
-#endif
 		
 		public void JoinMulticastGroup (IPAddress multicastAddr)
 		{
 			CheckDisposed ();
 
 			if (multicastAddr == null)
-#if NET_2_0
 				throw new ArgumentNullException ("multicastAddr");
-#else
-				throw new NullReferenceException ();
-#endif
 
 			if(family == AddressFamily.InterNetwork)
 				socket.SetSocketOption (SocketOptionLevel.IP, SocketOptionName.AddMembership,
 					new MulticastOption (multicastAddr));
-#if NET_1_1
 			else
 				socket.SetSocketOption (SocketOptionLevel.IPv6, SocketOptionName.AddMembership,
 					new IPv6MulticastOption (multicastAddr));
-#endif
 		}
 
-#if NET_1_1
 		public void JoinMulticastGroup (int ifindex,
 						IPAddress multicastAddr)
 		{
@@ -287,7 +262,6 @@ namespace System.Net.Sockets
 			else
 				throw new SocketException ((int) SocketError.OperationNotSupported);
 		}
-#endif
 
 		public void JoinMulticastGroup (IPAddress multicastAddr, int timeToLive)
 		{
@@ -301,14 +275,11 @@ namespace System.Net.Sockets
 			if(family == AddressFamily.InterNetwork)
 				socket.SetSocketOption (SocketOptionLevel.IP, SocketOptionName.MulticastTimeToLive,
 					timeToLive);
-#if NET_1_1
 			else
 				socket.SetSocketOption (SocketOptionLevel.IPv6, SocketOptionName.MulticastTimeToLive,
 					timeToLive);
-#endif
 		}
 
-#if NET_2_0
 		public void JoinMulticastGroup (IPAddress multicastAddr,
 						IPAddress localAddress)
 		{
@@ -319,7 +290,6 @@ namespace System.Net.Sockets
 			else
 				throw new SocketException ((int) SocketError.OperationNotSupported);
 		}
-#endif
 
 		#endregion
 		#region Data I/O
@@ -410,7 +380,6 @@ namespace System.Net.Sockets
 		}
 #endregion
 
-#if NET_2_0
 		IAsyncResult DoBeginSend (byte[] datagram, int bytes,
 					  IPEndPoint endPoint,
 					  AsyncCallback requestCallback,
@@ -480,9 +449,8 @@ namespace System.Net.Sockets
 			
 			return(socket.EndSend (asyncResult));
 		}
-		
-		public IAsyncResult BeginReceive (AsyncCallback callback,
-						  object state)
+
+		public IAsyncResult BeginReceive (AsyncCallback requestCallback, object state)
 		{
 			CheckDisposed ();
 
@@ -499,11 +467,10 @@ namespace System.Net.Sockets
 			return(socket.BeginReceiveFrom (recvbuffer, 0, 8192,
 							SocketFlags.None,
 							ref ep,
-							callback, state));
+							requestCallback, state));
 		}
 		
-		public byte[] EndReceive (IAsyncResult asyncResult,
-					  ref IPEndPoint remoteEP)
+		public byte[] EndReceive (IAsyncResult asyncResult, ref IPEndPoint remoteEP)
 		{
 			CheckDisposed ();
 			
@@ -532,7 +499,6 @@ namespace System.Net.Sockets
 			
 			return(buf);
 		}
-#endif
 				
 #region Properties
 		protected bool Active {
@@ -540,17 +506,11 @@ namespace System.Net.Sockets
 			set { active = value; }
 		}
 
-#if NET_2_0
-		public
-#else
-		protected
-#endif
-		Socket Client {
+		public Socket Client {
 			get { return socket; }
 			set { socket = value; }
 		}
 
-#if NET_2_0
 		public int Available
 		{
 			get {
@@ -558,9 +518,6 @@ namespace System.Net.Sockets
 			}
 		}
 		
-#if TARGET_JVM
-		[MonoNotSupported ("Not supported as Socket.DontFragment is not supported")]
-#endif
 		public bool DontFragment
 		{
 			get {
@@ -571,9 +528,6 @@ namespace System.Net.Sockets
 			}
 		}
 
-#if TARGET_JVM
-		[MonoNotSupported ("Not supported as Socket.EnableBroadcast is not supported")]
-#endif
 		public bool EnableBroadcast
 		{
 			get {
@@ -584,9 +538,6 @@ namespace System.Net.Sockets
 			}
 		}
 		
-#if TARGET_JVM
-		[MonoNotSupported ("Not supported as Socket.ExclusiveAddressUse is not supported")]
-#endif
 		public bool ExclusiveAddressUse
 		{
 			get {
@@ -597,9 +548,6 @@ namespace System.Net.Sockets
 			}
 		}
 		
-#if TARGET_JVM
-		[MonoNotSupported ("Not supported as Socket.MulticastLoopback is not supported")]
-#endif
 		public bool MulticastLoopback
 		{
 			get {
@@ -610,9 +558,6 @@ namespace System.Net.Sockets
 			}
 		}
 		
-#if TARGET_JVM
-		[MonoNotSupported ("Not supported as Socket.Ttl is not supported")]
-#endif
 		public short Ttl
 		{
 			get {
@@ -622,7 +567,6 @@ namespace System.Net.Sockets
 				socket.Ttl = value;
 			}
 		}
-#endif
 
 #endregion
 #region Disposing
@@ -632,10 +576,7 @@ namespace System.Net.Sockets
 			GC.SuppressFinalize (this);
 		}
 
-#if NET_2_0
-		protected virtual
-#endif
-		void Dispose (bool disposing)
+		protected virtual void Dispose (bool disposing)
 		{
 			if (disposed)
 				return;
@@ -660,7 +601,36 @@ namespace System.Net.Sockets
 				throw new ObjectDisposedException (GetType().FullName);
 		}		
 #endregion
-#endregion
+
+		
+		public Task<UdpReceiveResult> ReceiveAsync ()
+		{
+			return Task<UdpReceiveResult>.Factory.FromAsync (BeginReceive, r => {
+				IPEndPoint remoteEndPoint = null;
+				return new UdpReceiveResult (EndReceive (r, ref remoteEndPoint), remoteEndPoint);
+			}, null);
+		}
+
+		public Task<int> SendAsync (byte[] datagram, int bytes)
+		{
+			return Task<int>.Factory.FromAsync (BeginSend, EndSend, datagram, bytes, null);
+		}
+
+		public Task<int> SendAsync (byte[] datagram, int bytes, IPEndPoint endPoint)
+		{
+			return Task<int>.Factory.FromAsync (BeginSend, EndSend, datagram, bytes, endPoint, null);
+		}
+
+		public Task<int> SendAsync (byte[] datagram, int bytes, string hostname, int port)
+		{
+			var t = Tuple.Create (datagram, bytes, hostname, port, this);
+
+			return Task<int>.Factory.FromAsync ((callback, state) => {
+				var d = (Tuple<byte[], int, string, int, UdpClient>) state;
+				return d.Item5.BeginSend (d.Item1, d.Item2, d.Item3, d.Item4, callback, null);
+			}, EndSend, t);
+ 
+		}
 	}
 }
 
