@@ -31,6 +31,15 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
+#if SECURITY_DEP
+#if MONO_SECURITY_ALIAS
+extern alias MonoSecurity;
+using MonoSecurity::Mono.Security.Interface;
+#else
+using Mono.Security.Interface;
+#endif
+#endif
+
 using System;
 using System.Collections;
 using System.Configuration;
@@ -39,11 +48,13 @@ using System.IO;
 using System.Net;
 using System.Net.Cache;
 using System.Net.Sockets;
+using System.Net.Security;
 using System.Runtime.Remoting.Messaging;
 using System.Runtime.Serialization;
 using System.Security.Cryptography.X509Certificates;
 using System.Text;
 using System.Threading;
+using Mono.Net.Security;
 
 namespace System.Net 
 {
@@ -101,6 +112,11 @@ namespace System.Net
 		int maxResponseHeadersLength;
 		static int defaultMaxResponseHeadersLength;
 		int readWriteTimeout = 300000; // ms
+		IMonoTlsProvider tlsProvider;
+#if SECURITY_DEP
+		MonoTlsSettings tlsSettings;
+#endif
+		ServerCertValidationCallback certValidationCallback;
 
 		enum NtlmAuthState {
 			None,
@@ -110,6 +126,9 @@ namespace System.Net
 		AuthorizationState auth_state, proxy_auth_state;
 		string host;
 
+		[NonSerialized]
+		internal Action<Stream> ResendContentFactory;
+
 		// Constructors
 		static HttpWebRequest ()
 		{
@@ -140,6 +159,15 @@ namespace System.Net
 			ThrowOnError = true;
 			ResetAuthorization ();
 		}
+
+#if SECURITY_DEP
+		internal HttpWebRequest (Uri uri, IMonoTlsProvider tlsProvider, MonoTlsSettings settings = null)
+			: this (uri)
+		{
+			this.tlsProvider = tlsProvider;
+			this.tlsSettings = settings;
+		}
+#endif
 		
 		[Obsolete ("Serialization is obsoleted for this type", false)]
 		protected HttpWebRequest (SerializationInfo serializationInfo, StreamingContext streamingContext) 
@@ -200,12 +228,13 @@ namespace System.Net
 			set { allowBuffering = value; }
 		}
 		
-#if NET_4_5
 		public virtual bool AllowReadStreamBuffering {
-			get { return allowBuffering; }
-			set { allowBuffering = value; }
+			get { return false; }
+			set {
+				if (value)
+					throw new InvalidOperationException ();
+			}
 		}
-#endif
 
 		static Exception GetMustImplement ()
 		{
@@ -225,25 +254,41 @@ namespace System.Net
 		
 		internal bool InternalAllowBuffering {
 			get {
-				return (allowBuffering && (method != "HEAD" && method != "GET" &&
-							method != "MKCOL" && method != "CONNECT" &&
-							method != "TRACE"));
+				return allowBuffering && MethodWithBuffer;
 			}
 		}
+
+		bool MethodWithBuffer {
+			get {
+				return method != "HEAD" && method != "GET" &&
+				method != "MKCOL" && method != "CONNECT" &&
+				method != "TRACE";
+			}
+		}
+
+		internal IMonoTlsProvider TlsProvider {
+			get { return tlsProvider; }
+		}
+
+#if SECURITY_DEP
+		internal MonoTlsSettings TlsSettings {
+			get { return tlsSettings; }
+		}
+#endif
 		
 		public X509CertificateCollection ClientCertificates {
 			get {
 				if (certificates == null)
 					certificates = new X509CertificateCollection ();
-
 				return certificates;
 			}
-			[MonoTODO]
 			set {
-				throw GetMustImplement ();
+				if (value == null)
+					throw new ArgumentNullException ("value");
+				certificates = value;
 			}
 		}
-		
+
 		public string Connection {
 			get { return webHeaders ["Connection"]; }
 			set {
@@ -304,9 +349,7 @@ namespace System.Net
 			set { continueDelegate = value; }
 		}
 		
-#if NET_4_5
 		virtual
-#endif
 		public CookieContainer CookieContainer {
 			get { return cookieContainer; }
 			set { cookieContainer = value; }
@@ -316,7 +359,6 @@ namespace System.Net
 			get { return credentials; }
 			set { credentials = value; }
 		}
-#if NET_4_0
 		public DateTime Date {
 			get {
 				string date = webHeaders ["Date"];
@@ -331,7 +373,6 @@ namespace System.Net
 					webHeaders.RemoveAndAdd ("Date", value.ToUniversalTime ().ToString ("r", CultureInfo.InvariantCulture));
 			}
 		}
-#endif
 
 #if !NET_2_1
 		[MonoTODO]
@@ -377,9 +418,7 @@ namespace System.Net
 			}
 		}
 		
-#if NET_4_5
 		virtual
-#endif
 		public bool HaveResponse {
 			get { return haveResponse; }
 		}
@@ -397,11 +436,7 @@ namespace System.Net
 			}
 		}
 		
-#if NET_4_0
 		public
-#else
-		internal
-#endif
 		string Host {
 			get {
 				if (host == null)
@@ -503,13 +538,11 @@ namespace System.Net
 			}
 		}
 		
-#if NET_4_5
 		[MonoTODO]
 		public int ContinueTimeout {
 			get { throw new NotImplementedException (); }
 			set { throw new NotImplementedException (); }
 		}
-#endif
 		
 		public string MediaType {
 			get { return mediaType; }
@@ -560,6 +593,7 @@ namespace System.Net
 				CheckRequestStarted ();
 				proxy = value;
 				servicePoint = null; // we may need a new one
+				GetServicePoint ();
 			}
 		}
 		
@@ -594,7 +628,6 @@ namespace System.Net
 		internal ServicePoint ServicePointNoLock {
 			get { return servicePoint; }
 		}
-#if NET_4_0
 		public virtual bool SupportsCookieContainer { 
 			get {
 				// The managed implementation supports the cookie container
@@ -602,7 +635,6 @@ namespace System.Net
 				return true;
 			}
 		}
-#endif
 		public override int Timeout { 
 			get { return timeout; }
 			set {
@@ -670,7 +702,26 @@ namespace System.Net
 		internal bool ProxyQuery {
 			get { return servicePoint.UsesProxy && !servicePoint.UseConnect; }
 		}
-		
+
+		internal ServerCertValidationCallback ServerCertValidationCallback {
+			get { return certValidationCallback; }
+		}
+
+		public RemoteCertificateValidationCallback ServerCertificateValidationCallback {
+			get {
+				if (certValidationCallback == null)
+					return null;
+				return certValidationCallback.ValidationCallback;
+			}
+			set
+			{
+				if (value == null)
+					certValidationCallback = null;
+				else
+					certValidationCallback = new ServerCertValidationCallback (value);
+			}
+		}
+
 		// Methods
 		
 		internal ServicePoint GetServicePoint ()
@@ -704,31 +755,19 @@ namespace System.Net
 		{
 			AddRange (rangeSpecifier, (long) from, (long) to);
 		}
-#if NET_4_0
 		public
-#else
-		internal
-#endif
 		void AddRange (long range)
 		{
 			AddRange ("bytes", (long) range);
 		}
 
-#if NET_4_0
 		public
-#else
-		internal
-#endif
 		void AddRange (long from, long to)
 		{
 			AddRange ("bytes", from, to);
 		}
 
-#if NET_4_0
 		public
-#else
-		internal
-#endif
 		void AddRange (string rangeSpecifier, long range)
 		{
 			if (rangeSpecifier == null)
@@ -754,11 +793,7 @@ namespace System.Net
 			webHeaders.RemoveAndAdd ("Range", r);
 		}
 
-#if NET_4_0
 		public
-#else
-		internal
-#endif
 		void AddRange (string rangeSpecifier, long from, long to)
 		{
 			if (rangeSpecifier == null)
@@ -869,16 +904,11 @@ namespace System.Net
 		{
 			if (writeStream == null || writeStream.RequestWritten || !InternalAllowBuffering)
 				return false;
-			#if NET_4_0
 			if (contentLength < 0 && writeStream.CanWrite == true && writeStream.WriteBufferLength < 0)
 				return false;
 
 			if (contentLength < 0 && writeStream.WriteBufferLength >= 0)
 				InternalContentLength = writeStream.WriteBufferLength;
-			#else
-			if (contentLength < 0 && writeStream.CanWrite == true)
-				return false;
-			#endif
 
 			// This will write the POST/PUT if the write stream already has the expected
 			// amount of bytes in it (ContentLength) (bug #77753) or if the write stream
@@ -940,11 +970,17 @@ namespace System.Net
 					}
 				}
 
-				if (!requestSent) {
+				if (requestSent)
+					return;
+
+				try {
 					requestSent = true;
 					redirects = 0;
 					servicePoint = GetServicePoint ();
 					abortHandler = servicePoint.SendRequest (this, connectionGroup);
+				} catch (Exception ex) {
+					aread.SetCompleted (synch, ex);
+					aread.DoCallback ();
 				}
 			});
 
@@ -971,13 +1007,11 @@ namespace System.Net
 			return result.Response;
 		}
 		
-#if NET_3_5
 		public Stream EndGetRequestStream (IAsyncResult asyncResult, out TransportContext transportContext)
 		{
 			transportContext = null;
 			return EndGetRequestStream (asyncResult);
 		}
-#endif
 
 		public override WebResponse GetResponse()
 		{
@@ -1131,13 +1165,15 @@ namespace System.Net
 				break;
 			}
 
-			if (method != "GET" && !InternalAllowBuffering)
+			if (method != "GET" && !InternalAllowBuffering && (writeStream.WriteBufferLength > 0 || contentLength > 0))
 				e = new WebException ("The request requires buffering data to succeed.", null, WebExceptionStatus.ProtocolError, webResponse);
 
 			if (e != null)
 				throw e;
 
-			contentLength = -1;
+			if (AllowWriteStreamBuffering || method == "GET")
+				contentLength = -1;
+
 			uriString = webResponse.Headers ["Location"];
 
 			if (uriString == null)
@@ -1340,8 +1376,7 @@ namespace System.Net
 						bodyBuffer = null;
 						writeStream.Close ();
 					}
-				} else if (method != "HEAD" && method != "GET" && method != "MKCOL" && method != "CONNECT" &&
-				          method != "TRACE") {
+				} else if (MethodWithBuffer) {
 					if (getResponseCalled && !writeStream.RequestWritten)
 						return writeStream.WriteRequestAsync (result);
 				}
@@ -1640,12 +1675,28 @@ namespace System.Net
 					(ProxyQuery && !proxy_auth_state.IsCompleted && code == HttpStatusCode.ProxyAuthenticationRequired)) {
 					if (!usedPreAuth && CheckAuthorization (webResponse, code)) {
 						// Keep the written body, so it can be rewritten in the retry
-						if (InternalAllowBuffering) {
-							if (writeStream.WriteBufferLength > 0) {
-								bodyBuffer = writeStream.WriteBuffer;
-								bodyBufferLength = writeStream.WriteBufferLength;
+						if (MethodWithBuffer) {
+							if (AllowWriteStreamBuffering) {
+								if (writeStream.WriteBufferLength > 0) {
+									bodyBuffer = writeStream.WriteBuffer;
+									bodyBufferLength = writeStream.WriteBufferLength;
+								}
+
+								return true;
+							}
+
+							//
+							// Buffering is not allowed but we have alternative way to get same content (we
+							// need to resent it due to NTLM Authentication).
+					 		//
+							if (ResendContentFactory != null) {
+								using (var ms = new MemoryStream ()) {
+									ResendContentFactory (ms);
+									bodyBuffer = ms.ToArray ();
+									bodyBufferLength = bodyBuffer.Length;
+								}
+								return true;
 							}
-							return true;
 						} else if (method != "PUT" && method != "POST") {
 							bodyBuffer = null;
 							return true;
