@@ -30,10 +30,11 @@
 namespace System.Net {
 
 	[Flags]
-	[Serializable]
 	public
 	enum SecurityProtocolType {
 		Ssl3 = 48,
-		Tls = 192
+		Tls = 192,
+		Tls11 = 768,
+		Tls12 = 3072
 	}
 }
