@@ -31,13 +31,11 @@
 using System.IO;
 using System.Collections;
 using System.Net.Sockets;
-using System.Reflection;
 using System.Security.Cryptography.X509Certificates;
 using System.Text;
 using System.Threading;
-#if SECURITY_DEP
-using Mono.Security.Protocol.Tls;
-#endif
+using System.Diagnostics;
+using Mono.Net.Security;
 
 namespace System.Net
 {
@@ -46,15 +44,17 @@ namespace System.Net
 		None,
 		Status,
 		Headers,
-		Content
+		Content,
+		Aborted
 	}
 
 	class WebConnection
 	{
 		ServicePoint sPoint;
 		Stream nstream;
-		Socket socket;
+		internal Socket socket;
 		object socketLock = new object ();
+		IWebConnectionState state;
 		WebExceptionStatus status;
 		WaitCallback initConn;
 		bool keepAlive;
@@ -62,51 +62,51 @@ namespace System.Net
 		static AsyncCallback readDoneDelegate = new AsyncCallback (ReadDone);
 		EventHandler abortHandler;
 		AbortHelper abortHelper;
-		ReadState readState;
 		internal WebConnectionData Data;
 		bool chunkedRead;
 		ChunkStream chunkStream;
 		Queue queue;
 		bool reused;
 		int position;
-		bool busy;
-		HttpWebRequest priority_request;
+		HttpWebRequest priority_request;		
 		NetworkCredential ntlm_credentials;
 		bool ntlm_authenticated;
 		bool unsafe_sharing;
 
-		bool ssl;
-		bool certsAvailable;
+		enum NtlmAuthState
+		{
+			None,
+			Challenge,
+			Response
+		}
+		NtlmAuthState connect_ntlm_auth_state;
+		HttpWebRequest connect_request;
+
 		Exception connect_exception;
 		static object classLock = new object ();
-		static Type sslStream;
-		static PropertyInfo piClient;
-		static PropertyInfo piServer;
-		static PropertyInfo piTrustFailure;
+		MonoTlsStream tlsStream;
 
-#if MONOTOUCH
-                static MethodInfo start_wwan;
-
-                static WebConnection ()
-                {
-                        Type type = Type.GetType ("MonoTouch.ObjCRuntime.Runtime, monotouch");
-			if (type != null)
-	                        start_wwan = type.GetMethod ("StartWWAN");
-                }
+#if MONOTOUCH && !MONOTOUCH_TV && !MONOTOUCH_WATCH
+		[System.Runtime.InteropServices.DllImport ("__Internal")]
+		static extern void xamarin_start_wwan (string uri);
 #endif
 
-		public WebConnection (WebConnectionGroup group, ServicePoint sPoint)
+		internal ChunkStream ChunkStream {
+			get { return chunkStream; }
+		}
+
+		public WebConnection (IWebConnectionState wcs, ServicePoint sPoint)
 		{
+			this.state = wcs;
 			this.sPoint = sPoint;
 			buffer = new byte [4096];
-			readState = ReadState.None;
 			Data = new WebConnectionData ();
 			initConn = new WaitCallback (state => {
 				try {
 					InitConnection (state);
 				} catch {}
 				});
-			queue = group.Queue;
+			queue = wcs.Group.Queue;
 			abortHelper = new AbortHelper ();
 			abortHelper.Connection = this;
 			abortHandler = new EventHandler (abortHelper.Abort);
@@ -152,27 +152,38 @@ namespace System.Net
 				IPHostEntry hostEntry = sPoint.HostEntry;
 
 				if (hostEntry == null) {
-#if MONOTOUCH
-					if (start_wwan != null) {
-						start_wwan.Invoke (null, new object [1] { sPoint.Address });
-						hostEntry = sPoint.HostEntry;
-					}
+#if MONOTOUCH && !MONOTOUCH_TV && !MONOTOUCH_WATCH
+					xamarin_start_wwan (sPoint.Address.ToString ());
+					hostEntry = sPoint.HostEntry;
 					if (hostEntry == null) {
 #endif
 						status = sPoint.UsesProxy ? WebExceptionStatus.ProxyNameResolutionFailure :
 									    WebExceptionStatus.NameResolutionFailure;
 						return;
-#if MONOTOUCH
+#if MONOTOUCH && !MONOTOUCH_TV && !MONOTOUCH_WATCH
 					}
 #endif
 				}
 
 				//WebConnectionData data = Data;
 				foreach (IPAddress address in hostEntry.AddressList) {
-					socket = new Socket (address.AddressFamily, SocketType.Stream, ProtocolType.Tcp);
+					try {
+						socket = new Socket (address.AddressFamily, SocketType.Stream, ProtocolType.Tcp);
+					} catch (Exception se) {
+						// The Socket ctor can throw if we run out of FD's
+						if (!request.Aborted)
+								status = WebExceptionStatus.ConnectFailure;
+						connect_exception = se;
+						return;
+					}
 					IPEndPoint remote = new IPEndPoint (address, sPoint.Address.Port);
-					socket.SetSocketOption (SocketOptionLevel.Tcp, SocketOptionName.NoDelay, sPoint.UseNagleAlgorithm ? 0 : 1);
 					socket.NoDelay = !sPoint.UseNagleAlgorithm;
+					try {
+						sPoint.KeepAliveSetup (socket);
+					} catch {
+						// Ignore. Not supported in all platforms.
+					}
+
 					if (!sPoint.CallEndPointDelegate (socket, remote)) {
 						socket.Close ();
 						socket = null;
@@ -208,33 +219,8 @@ namespace System.Net
 			}
 		}
 
-		static void EnsureSSLStreamAvailable ()
-		{
-			lock (classLock) {
-				if (sslStream != null)
-					return;
-
-#if NET_2_1 && SECURITY_DEP
-				sslStream = typeof (Mono.Security.Protocol.Tls.HttpsClientStream);
-#else
-				// HttpsClientStream is an internal glue class in Mono.Security.dll
-				sslStream = Type.GetType ("Mono.Security.Protocol.Tls.HttpsClientStream, " +
-							Consts.AssemblyMono_Security, false);
-
-				if (sslStream == null) {
-					string msg = "Missing Mono.Security.dll assembly. " +
-							"Support for SSL/TLS is unavailable.";
-
-					throw new NotSupportedException (msg);
-				}
-#endif
-				piClient = sslStream.GetProperty ("SelectedClientCertificate");
-				piServer = sslStream.GetProperty ("ServerCertificate");
-				piTrustFailure = sslStream.GetProperty ("TrustFailure");
-			}
-		}
-
-		bool CreateTunnel (HttpWebRequest request, Stream stream, out byte [] buffer)
+		bool CreateTunnel (HttpWebRequest request, Uri connectUri,
+		                   Stream stream, out byte[] buffer)
 		{
 			StringBuilder sb = new StringBuilder ();
 			sb.Append ("CONNECT ");
@@ -249,21 +235,44 @@ namespace System.Net
 
 			sb.Append ("\r\nHost: ");
 			sb.Append (request.Address.Authority);
-			string challenge = Data.Challenge;
+
+			bool ntlm = false;
+			var challenge = Data.Challenge;
 			Data.Challenge = null;
-			bool have_auth = (request.Headers ["Proxy-Authorization"] != null);
+			var auth_header = request.Headers ["Proxy-Authorization"];
+			bool have_auth = auth_header != null;
 			if (have_auth) {
 				sb.Append ("\r\nProxy-Authorization: ");
-				sb.Append (request.Headers ["Proxy-Authorization"]);
+				sb.Append (auth_header);
+				ntlm = auth_header.ToUpper ().Contains ("NTLM");
 			} else if (challenge != null && Data.StatusCode == 407) {
-				have_auth = true;
 				ICredentials creds = request.Proxy.Credentials;
-				Authorization auth = AuthenticationManager.Authenticate (challenge, request, creds);
-				if (auth != null) {
+				have_auth = true;
+
+				if (connect_request == null) {
+					// create a CONNECT request to use with Authenticate
+					connect_request = (HttpWebRequest)WebRequest.Create (
+						connectUri.Scheme + "://" + connectUri.Host + ":" + connectUri.Port + "/");
+					connect_request.Method = "CONNECT";
+					connect_request.Credentials = creds;
+				}
+
+				for (int i = 0; i < challenge.Length; i++) {
+					var auth = AuthenticationManager.Authenticate (challenge [i], connect_request, creds);
+					if (auth == null)
+						continue;
+					ntlm = (auth.Module.AuthenticationType == "NTLM");
 					sb.Append ("\r\nProxy-Authorization: ");
 					sb.Append (auth.Message);
+					break;
 				}
 			}
+
+			if (ntlm) {
+				sb.Append ("\r\nProxy-Connection: keep-alive");
+				connect_ntlm_auth_state++;
+			}
+
 			sb.Append ("\r\n\r\n");
 
 			Data.StatusCode = 0;
@@ -271,10 +280,19 @@ namespace System.Net
 			stream.Write (connectBytes, 0, connectBytes.Length);
 
 			int status;
-			WebHeaderCollection result = ReadHeaders (request, stream, out buffer, out status);
-			if (!have_auth && result != null && status == 407) { // Needs proxy auth
+			WebHeaderCollection result = ReadHeaders (stream, out buffer, out status);
+			if ((!have_auth || connect_ntlm_auth_state == NtlmAuthState.Challenge) &&
+			    result != null && status == 407) { // Needs proxy auth
+				var connectionHeader = result ["Connection"];
+				if (socket != null && !string.IsNullOrEmpty (connectionHeader) &&
+				    connectionHeader.ToLower() == "close") {
+					// The server is requesting that this connection be closed
+					socket.Close();
+					socket = null;
+				}
+
 				Data.StatusCode = status;
-				Data.Challenge = result ["Proxy-Authenticate"];
+				Data.Challenge = result.GetValues_internal ("Proxy-Authenticate", false);
 				return false;
 			} else if (status != 200) {
 				string msg = String.Format ("The remote server returned a {0} status code.", status);
@@ -285,15 +303,13 @@ namespace System.Net
 			return (result != null);
 		}
 
-		WebHeaderCollection ReadHeaders (HttpWebRequest request, Stream stream, out byte [] retBuffer, out int status)
+		WebHeaderCollection ReadHeaders (Stream stream, out byte [] retBuffer, out int status)
 		{
 			retBuffer = null;
 			status = 200;
 
 			byte [] buffer = new byte [1024];
 			MemoryStream ms = new MemoryStream ();
-			bool gotStatus = false;
-			WebHeaderCollection headers = null;
 
 			while (true) {
 				int n = stream.Read (buffer, 0, 1024);
@@ -305,13 +321,29 @@ namespace System.Net
 				ms.Write (buffer, 0, n);
 				int start = 0;
 				string str = null;
-				headers = new WebHeaderCollection ();
+				bool gotStatus = false;
+				WebHeaderCollection headers = new WebHeaderCollection ();
 				while (ReadLine (ms.GetBuffer (), ref start, (int) ms.Length, ref str)) {
 					if (str == null) {
-						if (ms.Length - start > 0) {
-							retBuffer = new byte [ms.Length - start];
-							Buffer.BlockCopy (ms.GetBuffer (), start, retBuffer, 0, retBuffer.Length);
+						int contentLen = 0;
+						try	{
+							contentLen = int.Parse(headers["Content-Length"]);
 						}
+						catch {
+							contentLen = 0;
+						}
+
+						if (ms.Length - start - contentLen > 0)	{
+							// we've read more data than the response header and conents,
+							// give back extra data to the caller
+							retBuffer = new byte[ms.Length - start - contentLen];
+							Buffer.BlockCopy(ms.GetBuffer(), start + contentLen, retBuffer, 0, retBuffer.Length);
+						}
+						else {
+							// haven't read in some or all of the contents for the response, do so now
+							FlushContents(stream, contentLen - (int)(ms.Length - start));
+						}
+
 						return headers;
 					}
 
@@ -320,56 +352,74 @@ namespace System.Net
 						continue;
 					}
 
-					int spaceidx = str.IndexOf (' ');
-					if (spaceidx == -1) {
+					string[] parts = str.Split (' ');
+					if (parts.Length < 2) {
 						HandleError (WebExceptionStatus.ServerProtocolViolation, null, "ReadHeaders2");
 						return null;
 					}
 
-					status = (int) UInt32.Parse (str.Substring (spaceidx + 1, 3));
+					if (String.Compare (parts [0], "HTTP/1.1", true) == 0)
+						Data.ProxyVersion = HttpVersion.Version11;
+					else if (String.Compare (parts [0], "HTTP/1.0", true) == 0)
+						Data.ProxyVersion = HttpVersion.Version10;
+					else {
+						HandleError (WebExceptionStatus.ServerProtocolViolation, null, "ReadHeaders2");
+						return null;
+					}
+
+					status = (int)UInt32.Parse (parts [1]);
 					gotStatus = true;
 				}
 			}
 		}
 
+		void FlushContents(Stream stream, int contentLength)
+		{
+			while (contentLength > 0) {
+				byte[] contentBuffer = new byte[contentLength];
+				int bytesRead = stream.Read(contentBuffer, 0, contentLength);
+				if (bytesRead > 0) {
+					contentLength -= bytesRead;
+				}
+				else {
+					break;
+				}
+			}
+		}
+
 		bool CreateStream (HttpWebRequest request)
 		{
 			try {
 				NetworkStream serverStream = new NetworkStream (socket, false);
 
 				if (request.Address.Scheme == Uri.UriSchemeHttps) {
-					ssl = true;
-					EnsureSSLStreamAvailable ();
-					if (!reused || nstream == null || nstream.GetType () != sslStream) {
+#if SECURITY_DEP
+					if (!reused || nstream == null || tlsStream == null) {
 						byte [] buffer = null;
 						if (sPoint.UseConnect) {
-							bool ok = CreateTunnel (request, serverStream, out buffer);
+							bool ok = CreateTunnel (request, sPoint.Address, serverStream, out buffer);
 							if (!ok)
 								return false;
 						}
-
-						object[] args = new object [4] { serverStream,
-										request.ClientCertificates,
-										request, buffer};
-						nstream = (Stream) Activator.CreateInstance (sslStream, args);
-#if SECURITY_DEP
-						SslClientStream scs = (SslClientStream) nstream;
-						var helper = new ServicePointManager.ChainValidationHelper (request);
-						scs.ServerCertValidation2 += new CertificateValidationCallback2 (helper.ValidateChain);
-#endif
-						certsAvailable = false;
+						tlsStream = new MonoTlsStream (request, serverStream);
+						nstream = tlsStream.CreateStream (buffer);
 					}
 					// we also need to set ServicePoint.Certificate 
 					// and ServicePoint.ClientCertificate but this can
 					// only be done later (after handshake - which is
 					// done only after a read operation).
+#else
+					throw new NotSupportedException ();
+#endif
 				} else {
-					ssl = false;
 					nstream = serverStream;
 				}
-			} catch (Exception) {
-				if (!request.Aborted)
+			} catch (Exception ex) {
+				if (tlsStream != null)
+					status = tlsStream.ExceptionStatus;
+				else if (!request.Aborted)
 					status = WebExceptionStatus.ConnectFailure;
+				connect_exception = ex;
 				return false;
 			}
 
@@ -386,11 +436,7 @@ namespace System.Net
 
 			if (e == null) { // At least we now where it comes from
 				try {
-#if TARGET_JVM
-					throw new Exception ();
-#else
 					throw new Exception (new System.Diagnostics.StackTrace ().ToString ());
-#endif
 				} catch (Exception e2) {
 					e = e2;
 				}
@@ -409,7 +455,7 @@ namespace System.Net
 		
 		static void ReadDone (IAsyncResult result)
 		{
-			WebConnection cnc = (WebConnection) result.AsyncState;
+			WebConnection cnc = (WebConnection)result.AsyncState;
 			WebConnectionData data = cnc.Data;
 			Stream ns = cnc.nstream;
 			if (ns == null) {
@@ -420,7 +466,12 @@ namespace System.Net
 			int nread = -1;
 			try {
 				nread = ns.EndRead (result);
+			} catch (ObjectDisposedException) {
+				return;
 			} catch (Exception e) {
+				if (e.InnerException is ObjectDisposedException)
+					return;
+
 				cnc.HandleError (WebExceptionStatus.ReceiveFailure, e, "ReadDone1");
 				return;
 			}
@@ -437,43 +488,55 @@ namespace System.Net
 
 			int pos = -1;
 			nread += cnc.position;
-			if (cnc.readState == ReadState.None) { 
+			if (data.ReadState == ReadState.None) { 
 				Exception exc = null;
 				try {
-					pos = cnc.GetResponse (cnc.buffer, nread);
+					pos = GetResponse (data, cnc.sPoint, cnc.buffer, nread);
 				} catch (Exception e) {
 					exc = e;
 				}
 
-				if (exc != null) {
+				if (exc != null || pos == -1) {
 					cnc.HandleError (WebExceptionStatus.ServerProtocolViolation, exc, "ReadDone4");
 					return;
 				}
 			}
 
-			if (cnc.readState != ReadState.Content) {
+			if (data.ReadState == ReadState.Aborted) {
+				cnc.HandleError (WebExceptionStatus.RequestCanceled, null, "ReadDone");
+				return;
+			}
+
+			if (data.ReadState != ReadState.Content) {
 				int est = nread * 2;
 				int max = (est < cnc.buffer.Length) ? cnc.buffer.Length : est;
 				byte [] newBuffer = new byte [max];
 				Buffer.BlockCopy (cnc.buffer, 0, newBuffer, 0, nread);
 				cnc.buffer = newBuffer;
 				cnc.position = nread;
-				cnc.readState = ReadState.None;
+				data.ReadState = ReadState.None;
 				InitRead (cnc);
 				return;
 			}
 
 			cnc.position = 0;
 
-			WebConnectionStream stream = new WebConnectionStream (cnc);
+			WebConnectionStream stream = new WebConnectionStream (cnc, data);
+			bool expect_content = ExpectContent (data.StatusCode, data.request.Method);
+			string tencoding = null;
+			if (expect_content)
+				tencoding = data.Headers ["Transfer-Encoding"];
 
-			string contentType = data.Headers ["Transfer-Encoding"];
-			cnc.chunkedRead = (contentType != null && contentType.ToLower ().IndexOf ("chunked") != -1);
+			cnc.chunkedRead = (tencoding != null && tencoding.IndexOf ("chunked", StringComparison.OrdinalIgnoreCase) != -1);
 			if (!cnc.chunkedRead) {
 				stream.ReadBuffer = cnc.buffer;
 				stream.ReadBufferOffset = pos;
 				stream.ReadBufferSize = nread;
-				stream.CheckResponseInBuffer ();
+				try {
+					stream.CheckResponseInBuffer ();
+				} catch (Exception e) {
+					cnc.HandleError (WebExceptionStatus.ReceiveFailure, e, "ReadDone7");
+				}
 			} else if (cnc.chunkStream == null) {
 				try {
 					cnc.chunkStream = new ChunkStream (cnc.buffer, pos, nread, data.Headers);
@@ -493,26 +556,19 @@ namespace System.Net
 
 			data.stream = stream;
 			
-			if (!ExpectContent (data.StatusCode) || data.request.Method == "HEAD")
+			if (!expect_content)
 				stream.ForceCompletion ();
 
 			data.request.SetResponseData (data);
 		}
 
-		static bool ExpectContent (int statusCode)
+		static bool ExpectContent (int statusCode, string method)
 		{
+			if (method == "HEAD")
+				return false;
 			return (statusCode >= 200 && statusCode != 204 && statusCode != 304);
 		}
 
-		internal void GetCertificates () 
-		{
-			// here the SSL negotiation have been done
-			X509Certificate client = (X509Certificate) piClient.GetValue (nstream, null);
-			X509Certificate server = (X509Certificate) piServer.GetValue (nstream, null);
-			sPoint.SetCertificates (client, server);
-			certsAvailable = (server != null);
-		}
-
 		internal static void InitRead (object state)
 		{
 			WebConnection cnc = (WebConnection) state;
@@ -526,7 +582,8 @@ namespace System.Net
 			}
 		}
 		
-		int GetResponse (byte [] buffer, int max)
+		static int GetResponse (WebConnectionData data, ServicePoint sPoint,
+		                        byte [] buffer, int max)
 		{
 			int pos = 0;
 			string line = null;
@@ -534,45 +591,47 @@ namespace System.Net
 			bool isContinue = false;
 			bool emptyFirstLine = false;
 			do {
-				if (readState == ReadState.None) {
+				if (data.ReadState == ReadState.Aborted)
+					return -1;
+
+				if (data.ReadState == ReadState.None) {
 					lineok = ReadLine (buffer, ref pos, max, ref line);
 					if (!lineok)
-						return -1;
+						return 0;
 
 					if (line == null) {
 						emptyFirstLine = true;
 						continue;
 					}
 					emptyFirstLine = false;
-
-					readState = ReadState.Status;
+					data.ReadState = ReadState.Status;
 
 					string [] parts = line.Split (' ');
 					if (parts.Length < 2)
 						return -1;
 
 					if (String.Compare (parts [0], "HTTP/1.1", true) == 0) {
-						Data.Version = HttpVersion.Version11;
+						data.Version = HttpVersion.Version11;
 						sPoint.SetVersion (HttpVersion.Version11);
 					} else {
-						Data.Version = HttpVersion.Version10;
+						data.Version = HttpVersion.Version10;
 						sPoint.SetVersion (HttpVersion.Version10);
 					}
 
-					Data.StatusCode = (int) UInt32.Parse (parts [1]);
+					data.StatusCode = (int) UInt32.Parse (parts [1]);
 					if (parts.Length >= 3)
-						Data.StatusDescription = String.Join (" ", parts, 2, parts.Length - 2);
+						data.StatusDescription = String.Join (" ", parts, 2, parts.Length - 2);
 					else
-						Data.StatusDescription = "";
+						data.StatusDescription = "";
 
 					if (pos >= max)
 						return pos;
 				}
 
 				emptyFirstLine = false;
-				if (readState == ReadState.Status) {
-					readState = ReadState.Headers;
-					Data.Headers = new WebHeaderCollection ();
+				if (data.ReadState == ReadState.Status) {
+					data.ReadState = ReadState.Headers;
+					data.Headers = new WebHeaderCollection ();
 					ArrayList headers = new ArrayList ();
 					bool finished = false;
 					while (!finished) {
@@ -598,28 +657,28 @@ namespace System.Net
 					}
 
 					if (!finished)
-						return -1;
+						return 0;
 
 					foreach (string s in headers)
-						Data.Headers.SetInternal (s);
+						data.Headers.SetInternal (s);
 
-					if (Data.StatusCode == (int) HttpStatusCode.Continue) {
+					if (data.StatusCode == (int) HttpStatusCode.Continue) {
 						sPoint.SendContinue = true;
 						if (pos >= max)
 							return pos;
 
-						if (Data.request.ExpectContinue) {
-							Data.request.DoContinueDelegate (Data.StatusCode, Data.Headers);
+						if (data.request.ExpectContinue) {
+							data.request.DoContinueDelegate (data.StatusCode, data.Headers);
 							// Prevent double calls when getting the
 							// headers in several packets.
-							Data.request.ExpectContinue = false;
+							data.request.ExpectContinue = false;
 						}
 
-						readState = ReadState.None;
+						data.ReadState = ReadState.None;
 						isContinue = true;
 					}
 					else {
-						readState = ReadState.Content;
+						data.ReadState = ReadState.Content;
 						return pos;
 					}
 				}
@@ -632,13 +691,14 @@ namespace System.Net
 		{
 			HttpWebRequest request = (HttpWebRequest) state;
 			request.WebConnection = this;
+			if (request.ReuseConnection)
+				request.StoredConnection = this;
 
 			if (request.Aborted)
 				return;
 
 			keepAlive = request.KeepAlive;
-			Data = new WebConnectionData ();
-			Data.request = request;
+			Data = new WebConnectionData (request);
 		retry:
 			Connect (request);
 			if (request.Aborted)
@@ -667,22 +727,30 @@ namespace System.Net
 				return;
 			}
 
-			readState = ReadState.None;
 			request.SetWriteStream (new WebConnectionStream (this, request));
 		}
-		
+
+#if MONOTOUCH
+		static bool warned_about_queue = false;
+#endif
+
 		internal EventHandler SendRequest (HttpWebRequest request)
 		{
 			if (request.Aborted)
 				return null;
 
 			lock (this) {
-				if (!busy) {
-					busy = true;
+				if (state.TrySetBusy ()) {
 					status = WebExceptionStatus.Success;
 					ThreadPool.QueueUserWorkItem (initConn, request);
 				} else {
 					lock (queue) {
+#if MONOTOUCH
+						if (!warned_about_queue) {
+							warned_about_queue = true;
+							Console.WriteLine ("WARNING: An HttpWebRequest was added to the ConnectionGroup queue because the connection limit was reached.");
+						}
+#endif
 						queue.Enqueue (request);
 					}
 				}
@@ -703,21 +771,24 @@ namespace System.Net
 		internal void NextRead ()
 		{
 			lock (this) {
-				Data.request.FinishedReading = true;
+				if (Data.request != null)
+					Data.request.FinishedReading = true;
 				string header = (sPoint.UsesProxy) ? "Proxy-Connection" : "Connection";
 				string cncHeader = (Data.Headers != null) ? Data.Headers [header] : null;
 				bool keepAlive = (Data.Version == HttpVersion.Version11 && this.keepAlive);
+				if (Data.ProxyVersion != null && Data.ProxyVersion != HttpVersion.Version11)
+					keepAlive = false;
 				if (cncHeader != null) {
 					cncHeader = cncHeader.ToLower ();
-					keepAlive = (this.keepAlive && cncHeader.IndexOf ("keep-alive") != -1);
+					keepAlive = (this.keepAlive && cncHeader.IndexOf ("keep-alive", StringComparison.Ordinal) != -1);
 				}
 
 				if ((socket != null && !socket.Connected) ||
-				   (!keepAlive || (cncHeader != null && cncHeader.IndexOf ("close") != -1))) {
+				   (!keepAlive || (cncHeader != null && cncHeader.IndexOf ("close", StringComparison.Ordinal) != -1))) {
 					Close (false);
 				}
 
-				busy = false;
+				state.SetIdle ();
 				if (priority_request != null) {
 					SendRequest (priority_request);
 					priority_request = null;
@@ -772,17 +843,19 @@ namespace System.Net
 
 		internal IAsyncResult BeginRead (HttpWebRequest request, byte [] buffer, int offset, int size, AsyncCallback cb, object state)
 		{
+			Stream s = null;
 			lock (this) {
 				if (Data.request != request)
 					throw new ObjectDisposedException (typeof (NetworkStream).FullName);
 				if (nstream == null)
 					return null;
+				s = nstream;
 			}
 
 			IAsyncResult result = null;
-			if (!chunkedRead || chunkStream.WantMore) {
+			if (!chunkedRead || (!chunkStream.DataAvailable && chunkStream.WantMore)) {
 				try {
-					result = nstream.BeginRead (buffer, offset, size, cb, state);
+					result = s.BeginRead (buffer, offset, size, cb, state);
 					cb = null;
 				} catch (Exception) {
 					HandleError (WebExceptionStatus.ReceiveFailure, null, "chunked BeginRead");
@@ -806,28 +879,33 @@ namespace System.Net
 		
 		internal int EndRead (HttpWebRequest request, IAsyncResult result)
 		{
+			Stream s = null;
 			lock (this) {
 				if (Data.request != request)
 					throw new ObjectDisposedException (typeof (NetworkStream).FullName);
 				if (nstream == null)
 					throw new ObjectDisposedException (typeof (NetworkStream).FullName);
+				s = nstream;
 			}
 
 			int nbytes = 0;
+			bool done = false;
 			WebAsyncResult wr = null;
 			IAsyncResult nsAsync = ((WebAsyncResult) result).InnerAsyncResult;
 			if (chunkedRead && (nsAsync is WebAsyncResult)) {
 				wr = (WebAsyncResult) nsAsync;
 				IAsyncResult inner = wr.InnerAsyncResult;
-				if (inner != null && !(inner is WebAsyncResult))
-					nbytes = nstream.EndRead (inner);
+				if (inner != null && !(inner is WebAsyncResult)) {
+					nbytes = s.EndRead (inner);
+					done = nbytes == 0;
+				}
 			} else if (!(nsAsync is WebAsyncResult)) {
-				nbytes = nstream.EndRead (nsAsync);
+				nbytes = s.EndRead (nsAsync);
 				wr = (WebAsyncResult) result;
+				done = nbytes == 0;
 			}
 
 			if (chunkedRead) {
-				bool done = (nbytes == 0);
 				try {
 					chunkStream.WriteAndReadBack (wr.Buffer, wr.Offset, wr.Size, ref nbytes);
 					if (!done && nbytes == 0 && chunkStream.WantMore)
@@ -893,16 +971,18 @@ namespace System.Net
 
 		internal IAsyncResult BeginWrite (HttpWebRequest request, byte [] buffer, int offset, int size, AsyncCallback cb, object state)
 		{
+			Stream s = null;
 			lock (this) {
 				if (Data.request != request)
 					throw new ObjectDisposedException (typeof (NetworkStream).FullName);
 				if (nstream == null)
 					return null;
+				s = nstream;
 			}
 
 			IAsyncResult result = null;
 			try {
-				result = nstream.BeginWrite (buffer, offset, size, cb, state);
+				result = s.BeginWrite (buffer, offset, size, cb, state);
 			} catch (Exception) {
 				status = WebExceptionStatus.SendFailure;
 				throw;
@@ -911,63 +991,46 @@ namespace System.Net
 			return result;
 		}
 
-		internal void EndWrite2 (HttpWebRequest request, IAsyncResult result)
+		internal bool EndWrite (HttpWebRequest request, bool throwOnError, IAsyncResult result)
 		{
-			if (request.FinishedReading)
-				return;
-
+			Stream s = null;
 			lock (this) {
+				if (status == WebExceptionStatus.RequestCanceled)
+					return true;
 				if (Data.request != request)
 					throw new ObjectDisposedException (typeof (NetworkStream).FullName);
 				if (nstream == null)
 					throw new ObjectDisposedException (typeof (NetworkStream).FullName);
+				s = nstream;
 			}
 
 			try {
-				nstream.EndWrite (result);
+				s.EndWrite (result);
+				return true;
 			} catch (Exception exc) {
 				status = WebExceptionStatus.SendFailure;
-				if (exc.InnerException != null)
+				if (throwOnError && exc.InnerException != null)
 					throw exc.InnerException;
-				throw;
-			}
-		}
-
-		internal bool EndWrite (HttpWebRequest request, IAsyncResult result)
-		{
-			if (request.FinishedReading)
-				return true;
-
-			lock (this) {
-				if (Data.request != request)
-					throw new ObjectDisposedException (typeof (NetworkStream).FullName);
-				if (nstream == null)
-					throw new ObjectDisposedException (typeof (NetworkStream).FullName);
-			}
-
-			try {
-				nstream.EndWrite (result);
-				return true;
-			} catch {
-				status = WebExceptionStatus.SendFailure;
 				return false;
 			}
 		}
 
 		internal int Read (HttpWebRequest request, byte [] buffer, int offset, int size)
 		{
+			Stream s = null;
 			lock (this) {
 				if (Data.request != request)
 					throw new ObjectDisposedException (typeof (NetworkStream).FullName);
 				if (nstream == null)
 					return 0;
+				s = nstream;
 			}
 
 			int result = 0;
 			try {
 				bool done = false;
 				if (!chunkedRead) {
-					result = nstream.Read (buffer, offset, size);
+					result = s.Read (buffer, offset, size);
 					done = (result == 0);
 				}
 
@@ -996,18 +1059,17 @@ namespace System.Net
 		internal bool Write (HttpWebRequest request, byte [] buffer, int offset, int size, ref string err_msg)
 		{
 			err_msg = null;
+			Stream s = null;
 			lock (this) {
 				if (Data.request != request)
 					throw new ObjectDisposedException (typeof (NetworkStream).FullName);
-				if (nstream == null)
+				s = nstream;
+				if (s == null)
 					return false;
 			}
 
 			try {
-				nstream.Write (buffer, offset, size);
-				// here SSL handshake should have been done
-				if (ssl && !certsAvailable)
-					GetCertificates ();
+				s.Write (buffer, offset, size);
 			} catch (Exception e) {
 				err_msg = e.Message;
 				WebExceptionStatus wes = WebExceptionStatus.SendFailure;
@@ -1017,12 +1079,6 @@ namespace System.Net
 					return false;
 				}
 
-				// if SSL is in use then check for TrustFailure
-				if (ssl && (bool) piTrustFailure.GetValue (nstream, null)) {
-					wes = WebExceptionStatus.TrustFailure;
-					msg = "Trust failure";
-				}
-
 				HandleError (wes, e, msg);
 				return false;
 			}
@@ -1032,6 +1088,11 @@ namespace System.Net
 		internal void Close (bool sendNext)
 		{
 			lock (this) {
+				if (Data != null && Data.request != null && Data.request.ReuseConnection) {
+					Data.request.ReuseConnection = false;
+					return;
+				}
+
 				if (nstream != null) {
 					try {
 						nstream.Close ();
@@ -1048,10 +1109,18 @@ namespace System.Net
 
 				if (ntlm_authenticated)
 					ResetNtlm ();
-				busy = false;
+				if (Data != null) {
+					lock (Data) {
+						Data.ReadState = ReadState.Aborted;
+					}
+				}
+				state.SetIdle ();
 				Data = new WebConnectionData ();
 				if (sendNext)
 					SendNext ();
+				
+				connect_request = null;
+				connect_ntlm_auth_state = NtlmAuthState.None;
 			}
 		}
 
@@ -1060,7 +1129,7 @@ namespace System.Net
 			lock (this) {
 				lock (queue) {
 					HttpWebRequest req = (HttpWebRequest) sender;
-					if (Data.request == req) {
+					if (Data.request == req || Data.request == null) {
 						if (!req.FinishedReading) {
 							status = WebExceptionStatus.RequestCanceled;
 							Close (false);
@@ -1095,10 +1164,6 @@ namespace System.Net
 			unsafe_sharing = false;
 		}
 
-		internal bool Busy {
-			get { lock (this) return busy; }
-		}
-		
 		internal bool Connected {
 			get {
 				lock (this) {
