@@ -96,6 +96,12 @@ namespace System {
 			set { s_IriParsing = value; }
 		}
 
+		// Do not rename this.
+		// User code might set this to true with reflection.
+		// When set to true an Uri constructed with UriKind.RelativeOrAbsolute 
+		// and paths such as "/foo" is assumed relative.
+		private static bool useDotNetRelativeOrAbsolute;
+
 #if BOOTSTRAP_BASIC
 		private static readonly string hexUpperChars = "0123456789ABCDEF";
 		private static readonly string [] Empty = new string [0];
@@ -147,6 +153,8 @@ namespace System {
 				IriParsing = true;
 			else if (iriparsingVar == "false")
 				IriParsing = false;
+
+			useDotNetRelativeOrAbsolute = Environment.GetEnvironmentVariable ("MONO_URI_DOTNETRELATIVEORABSOLUTE") == "true";
 		}
 
 		public Uri (string uriString) : this (uriString, false) 
@@ -173,12 +181,21 @@ namespace System {
 		// When used instead of UriKind.RelativeOrAbsolute paths such as "/foo" are assumed relative.
 		const UriKind DotNetRelativeOrAbsolute = (UriKind) 300;
 
+		private void ProcessUriKind (string uriString, ref UriKind uriKind)
+		{
+			if (uriString == null)
+			   return;
+		
+			if (uriKind == DotNetRelativeOrAbsolute ||
+				(uriKind == UriKind.RelativeOrAbsolute && useDotNetRelativeOrAbsolute))
+				uriKind = (uriString.StartsWith ("/", StringComparison.Ordinal))? UriKind.Relative : UriKind.RelativeOrAbsolute;
+		}
+
 		public Uri (string uriString, UriKind uriKind)
 		{
 			source = uriString;
 
-			if (uriString != null && uriKind == DotNetRelativeOrAbsolute)
-				uriKind = (uriString.StartsWith ("/", StringComparison.Ordinal))? UriKind.Relative : UriKind.RelativeOrAbsolute;
+			ProcessUriKind (uriString, ref uriKind);
 
 			ParseUri (uriKind);
 
@@ -212,8 +229,7 @@ namespace System {
 				return;
 			}
 
-			if (uriKind == DotNetRelativeOrAbsolute)
-				uriKind = (uriString.StartsWith ("/", StringComparison.Ordinal))? UriKind.Relative : UriKind.RelativeOrAbsolute;
+			ProcessUriKind (uriString, ref uriKind);
 
 			if (uriKind != UriKind.RelativeOrAbsolute &&
 				uriKind != UriKind.Absolute &&
@@ -355,7 +371,7 @@ namespace System {
 			if ((path.Length == 0 || path [0] != '/') && baseEl.delimiter == SchemeDelimiter)
 				path = "/" + path;
 
-			source += UriHelper.Reduce (path, true);
+			source += UriHelper.Reduce (path, !IriParsing);
 
 			if (relativeEl.query != null) {
 				canUseBase = false;
@@ -784,6 +800,18 @@ namespace System {
 			return (((i >= 0x41) && (i <= 0x5A)) || ((i >= 0x61) && (i <= 0x7A)));
 		}
 
+		// taken from referencesource/System/net/System/URI.cs
+		private static bool IsAsciiLetter(char character) {
+
+			return  (character >= 'a' && character <= 'z') ||
+					(character >= 'A' && character <= 'Z');
+		}
+
+		internal static bool IsAsciiLetterOrDigit(char character) {
+			return IsAsciiLetter (character) || (character >= '0' && character <= '9');
+		}
+		//
+
 		public override bool Equals (object comparand) 
 		{
 			if (comparand == null) 
@@ -1761,7 +1789,7 @@ namespace System {
 
 		// static methods
 
-		private const int MaxUriLength = 32766;
+		private const int MaxUriLength = 0xfff0;
 
 		public static int Compare (Uri uri1, Uri uri2, UriComponents partsToCompare, UriFormat compareFormat, StringComparison comparisonType)
 		{
@@ -1809,7 +1837,7 @@ namespace System {
 			if (stringToEscape == null)
 				throw new ArgumentNullException ("stringToEscape");
 
-			if (stringToEscape.Length > MaxUriLength) {
+			if (stringToEscape.Length >= MaxUriLength) {
 				throw new UriFormatException (string.Format ("Uri is longer than the maximum {0} characters.", MaxUriLength));
 			}
 
@@ -1866,7 +1894,7 @@ namespace System {
 			if (stringToEscape == null)
 				throw new ArgumentNullException ("stringToEscape");
 
-			if (stringToEscape.Length > MaxUriLength) {
+			if (stringToEscape.Length >= MaxUriLength) {
 				throw new UriFormatException (string.Format ("Uri is longer than the maximum {0} characters.", MaxUriLength));
 			}
 
