@@ -431,12 +431,13 @@ namespace System.Reflection {
 
 		public string CultureName {
 			get {
-				if (cultureinfo == null)
-					return null;
-				if (cultureinfo.LCID == CultureInfo.InvariantCulture.LCID)
-					return "neutral";
-				return cultureinfo.Name;
+				return (cultureinfo == null)? null : cultureinfo.Name;
 			}
+#if NETSTANDARD
+			set {
+				throw new NotImplementedException ();
+			}
+#endif
 		}
 
 		[ComVisibleAttribute(false)]
