@@ -39,6 +39,7 @@ using System.IO;
 using System.Collections.Generic;
 using System.Reflection;
 using System.Security;
+using System.Diagnostics;
 using System.Runtime.ConstrainedExecution;
 
 namespace System.Threading {
@@ -67,7 +68,6 @@ namespace System.Threading {
 		private IntPtr start_notify;
 		private IntPtr stack_ptr;
 		private UIntPtr static_data; /* GC-tracked */
-		private IntPtr jit_data;
 		private IntPtr runtime_thread_info;
 		/* current System.Runtime.Remoting.Contexts.Context instance
 		   keep as an object to avoid triggering its class constructor when not needed */
@@ -78,14 +78,9 @@ namespace System.Threading {
 		internal int _serialized_principal_version;
 		private IntPtr appdomain_refs;
 		private int interruption_requested;
-		private IntPtr suspend_event;
-		private IntPtr suspended_event;
-		private IntPtr resume_event;
 		private IntPtr synch_cs;
 		internal bool threadpool_thread;
-		private bool thread_dump_requested;
 		private bool thread_interrupt_requested;
-		private IntPtr end_stack;
 		/* These are used from managed code */
 		internal int stack_size;
 		internal byte apartment_state;
@@ -95,14 +90,11 @@ namespace System.Threading {
 		private IntPtr manage_callback;
 		private IntPtr interrupt_on_stop;
 		private IntPtr flags;
-		private IntPtr android_tid;
 		private IntPtr thread_pinning_ref;
-		private int ignore_next_signal;
 		/* 
 		 * These fields are used to avoid having to increment corlib versions
 		 * when a new field is added to the unmanaged MonoThread structure.
 		 */
-		private IntPtr unused0;
 		private IntPtr unused1;
 		private IntPtr unused2;
 		#endregion
@@ -118,20 +110,12 @@ namespace System.Threading {
 		}
 	}
 
-	[ClassInterface (ClassInterfaceType.None)]
-	[ComVisible (true)]
-	[ComDefaultInterface (typeof (_Thread))]
 	[StructLayout (LayoutKind.Sequential)]
-#if MOBILE
-	public sealed class Thread : CriticalFinalizerObject {
-#else
-	public sealed class Thread : CriticalFinalizerObject, _Thread {
-#endif
+	public sealed partial class Thread {
 #pragma warning disable 414		
 		#region Sync with metadata/object-internals.h
 		private InternalThread internal_thread;
-		object start_obj;
-		private ExecutionContext ec_to_set;
+		object m_ThreadStartArg;
 		#endregion
 #pragma warning restore 414
 
@@ -142,28 +126,21 @@ namespace System.Threading {
 		CultureInfo current_culture;
 		CultureInfo current_ui_culture;
 
-		// the name of local_slots, current_thread and _ec is
+		// the name of current_thread is
 		// important because they are used by the runtime.
-		[ThreadStatic]
-		static object[] local_slots;
 
 		[ThreadStatic]
 		static Thread current_thread;
 
-		/* The actual ExecutionContext of the thread.  It's
-		   ThreadStatic so that it's not shared between
-		   AppDomains. */
-		[ThreadStatic]
-		static ExecutionContext _ec;
-
-		static NamedDataSlot namedDataSlot;		
-
 		static internal CultureInfo default_culture;
 		static internal CultureInfo default_ui_culture;
 
 		// can be both a ThreadStart and a ParameterizedThreadStart
-		private MulticastDelegate threadstart;
-		//private string thread_name=null;
+		private MulticastDelegate m_Delegate;
+
+		private ExecutionContext m_ExecutionContext;    // this call context follows the logical thread
+
+		private bool m_ExecutionContextBelongsToOuterScope;
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern void ConstructInternalThread ();
@@ -312,12 +289,6 @@ namespace System.Threading {
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern static InternalThread CurrentInternalThread_internal();
 
-		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		internal extern static uint AllocTlsData (Type type);
-
-		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		internal extern static void DestroyTlsData (uint offset);
-
 		public static Thread CurrentThread {
 			[ReliabilityContract (Consistency.WillNotCorruptState, Cer.MayFail)]
 			get {
@@ -332,64 +303,7 @@ namespace System.Threading {
 				return (int)(CurrentThread.internal_thread.thread_id);
 			}
 		}
-		
-		static NamedDataSlot NamedDataSlot {
-			get {
-				if (namedDataSlot == null)
-					Interlocked.CompareExchange (ref namedDataSlot, new NamedDataSlot (), null);
 
-				return namedDataSlot;
-			}
-		}
-		
-		public static LocalDataStoreSlot AllocateNamedDataSlot (string name)
-		{
-			return NamedDataSlot.Allocate (name);
-		}
-
-		public static void FreeNamedDataSlot (string name)
-		{
-			NamedDataSlot.Free (name);
-		}
-
-		public static LocalDataStoreSlot AllocateDataSlot ()
-		{
-			return new LocalDataStoreSlot (true);
-		}
-
-		public static object GetData (LocalDataStoreSlot slot) {
-			object[] slots = local_slots;
-			if (slot == null)
-				throw new ArgumentNullException ("slot");
-			if (slots != null && slot.slot < slots.Length)
-				return slots [slot.slot];
-			return null;
-		}
-
-		public static void SetData (LocalDataStoreSlot slot, object data) {
-			object[] slots = local_slots;
-			if (slot == null)
-				throw new ArgumentNullException ("slot");
-			if (slots == null) {
-				slots = new object [slot.slot + 2];
-				local_slots = slots;
-			} else if (slot.slot >= slots.Length) {
-				object[] nslots = new object [slot.slot + 2];
-				slots.CopyTo (nslots, 0);
-				slots = nslots;
-				local_slots = slots;
-			}
-			slots [slot.slot] = data;
-		}
-
-		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		internal extern static void FreeLocalSlotValues (int slot, bool thread_local);
-
-		public static LocalDataStoreSlot GetNamedDataSlot(string name)
-	 	{
-	 		return NamedDataSlot.Get (name);
-		}
-		
 		public static AppDomain GetDomain() {
 			return AppDomain.CurrentDomain;
 		}
@@ -397,52 +311,10 @@ namespace System.Threading {
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		public extern static int GetDomainID();
 
-		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		private extern static void ResetAbort_internal();
-
-		[SecurityPermission (SecurityAction.Demand, ControlThread=true)]
-		public static void ResetAbort ()
-		{
-			ResetAbort_internal ();
-		}
-
-		[HostProtectionAttribute (SecurityAction.LinkDemand, Synchronization = true, ExternalThreading = true)]
-		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		[ReliabilityContract (Consistency.WillNotCorruptState, Cer.Success)]
-		public extern static bool Yield ();
-
-
-		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		private extern static void Sleep_internal(int ms);
-
-		public static void Sleep (int millisecondsTimeout)
-		{
-			if (millisecondsTimeout < Timeout.Infinite)
-				throw new ArgumentOutOfRangeException ("millisecondsTimeout", "Negative timeout");
-
-			Sleep_internal (millisecondsTimeout);
-		}
-
-		public static void Sleep (TimeSpan timeout)
-		{
-			long ms = (long) timeout.TotalMilliseconds;
-			if (ms < Timeout.Infinite || ms > Int32.MaxValue)
-				throw new ArgumentOutOfRangeException ("timeout", "timeout out of range");
-
-			Sleep_internal ((int) ms);
-		}
-
 		// Returns the system thread handle
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern IntPtr Thread_internal (MulticastDelegate start);
 
-		public Thread(ThreadStart start) {
-			if(start==null) {
-				throw new ArgumentNullException("Null ThreadStart");
-			}
-			threadstart=start;
-		}
-
 		private Thread (InternalThread it) {
 			internal_thread = it;
 		}
@@ -583,32 +455,16 @@ namespace System.Threading {
 			}
 		}
 
-		public ThreadPriority Priority {
-			get {
-				return (ThreadPriority)GetPriority (Internal);
-			}
-			
-			set {
-				// FIXME: This doesn't do anything yet
-				SetPriority (Internal, (int)value);
-			}
-		}
-
 		public ThreadState ThreadState {
 			get {
 				return GetState (Internal);
 			}
 		}
 
+#if MONO_FEATURE_THREAD_ABORT
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
 		private extern static void Abort_internal (InternalThread thread, object stateInfo);
 
-		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		private extern static int GetPriority (InternalThread thread);
-
-		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		private extern static void SetPriority (InternalThread thread, int priority);
-
 		[SecurityPermission (SecurityAction.Demand, ControlThread=true)]
 		public void Abort () 
 		{
@@ -622,56 +478,36 @@ namespace System.Threading {
 		}
 
 		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		internal extern object GetAbortExceptionState ();
+		extern object GetAbortExceptionState ();
 
-		[MethodImplAttribute (MethodImplOptions.InternalCall)]
-		private extern static void Interrupt_internal (InternalThread thread);
-		
-		[SecurityPermission (SecurityAction.Demand, ControlThread=true)]
-		public void Interrupt ()
-		{
-			Interrupt_internal (Internal);
+		internal object AbortReason {
+			get {
+				return GetAbortExceptionState ();
+			}
 		}
 
-		// The current thread joins with 'this'. Set ms to 0 to block
-		// until this actually exits.
-		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		private extern static bool Join_internal(InternalThread thread, int ms, IntPtr handle);
-		
-		public void Join()
+		void ClearAbortReason ()
 		{
-			Join_internal(Internal, Timeout.Infinite, Internal.system_thread_handle);
+		}
+#else
+		[Obsolete ("Thread.Abort is not supported on the current platform.", true)]
+		public void Abort ()
+		{
+			throw new PlatformNotSupportedException ("Thread.Abort is not supported on the current platform.");
 		}
 
-		public bool Join(int millisecondsTimeout)
+		[Obsolete ("Thread.Abort is not supported on the current platform.", true)]
+		public void Abort (object stateInfo)
 		{
-			if (millisecondsTimeout < Timeout.Infinite)
-				throw new ArgumentOutOfRangeException ("millisecondsTimeout", "Timeout less than zero");
-
-			return Join_internal (Internal, millisecondsTimeout, Internal.system_thread_handle);
+			throw new PlatformNotSupportedException ("Thread.Abort is not supported on the current platform.");
 		}
 
-		public bool Join(TimeSpan timeout)
+		[Obsolete ("Thread.ResetAbort is not supported on the current platform.", true)]
+		public static void ResetAbort ()
 		{
-			long ms = (long) timeout.TotalMilliseconds;
-			if (ms < Timeout.Infinite || ms > Int32.MaxValue)
-				throw new ArgumentOutOfRangeException ("timeout", "timeout out of range");
-
-			return Join_internal (Internal, (int) ms, Internal.system_thread_handle);
-		}
-
-		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		public extern static void MemoryBarrier ();
-
-		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		private extern void Resume_internal();
-
-		[Obsolete ("")]
-		[SecurityPermission (SecurityAction.Demand, ControlThread=true)]
-		public void Resume () 
-		{
-			Resume_internal ();
+			throw new PlatformNotSupportedException ("Thread.ResetAbort is not supported on the current platform.");
 		}
+#endif // MONO_FEATURE_THREAD_ABORT
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		private extern static void SpinWait_nop ();
@@ -688,35 +524,17 @@ namespace System.Threading {
 			}
 		}
 
-		private void StartInternal ()
+		void StartInternal (IPrincipal principal, ref StackCrawlMark stackMark)
 		{
-			current_thread = this;
-
-			if (threadstart is ThreadStart) {
-				((ThreadStart) threadstart) ();
-			} else {
-				((ParameterizedThreadStart) threadstart) (start_obj);
-			}
-		}
-
-		public void Start() {
-			// propagate informations from the original thread to the new thread
-			ec_to_set = ExecutionContext.Capture (false, true);
+#if FEATURE_ROLE_BASED_SECURITY
 			Internal._serialized_principal = CurrentThread.Internal._serialized_principal;
+#endif
 
 			// Thread_internal creates and starts the new thread, 
-			if (Thread_internal((ThreadStart) StartInternal) == (IntPtr) 0)
+			if (Thread_internal(m_Delegate) == IntPtr.Zero)
 				throw new SystemException ("Thread creation failed.");
-		}
 
-		[MethodImplAttribute(MethodImplOptions.InternalCall)]
-		private extern static void Suspend_internal(InternalThread thread);
-
-		[Obsolete ("")]
-		[SecurityPermission (SecurityAction.Demand, ControlThread=true)]
-		public void Suspend ()
-		{
-			Suspend_internal (Internal);
+			m_ThreadStartArg = null;
 		}
 
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
@@ -819,10 +637,10 @@ namespace System.Threading {
 		[MethodImplAttribute (MethodImplOptions.InternalCall)]
 		extern static int SystemMaxStackStize ();
 
-		static int CheckStackSize (int maxStackSize)
+		static int GetProcessDefaultStackSize (int maxStackSize)
 		{
-			if (maxStackSize < 0)
-				throw new ArgumentOutOfRangeException ("less than zero", "maxStackSize");
+			if (maxStackSize == 0)
+				return 0;
 
 			if (maxStackSize < 131072) // make sure stack is at least 128k big
 				return 131072;
@@ -836,68 +654,10 @@ namespace System.Threading {
 			return Math.Min (maxStackSize, SystemMaxStackStize ());
 		}
 
-		public Thread (ThreadStart start, int maxStackSize)
+		void SetStart (MulticastDelegate start, int maxStackSize)
 		{
-			if (start == null)
-				throw new ArgumentNullException ("start");
-
-			threadstart = start;
-			Internal.stack_size = CheckStackSize (maxStackSize);;
-		}
-
-		public Thread (ParameterizedThreadStart start)
-		{
-			if (start == null)
-				throw new ArgumentNullException ("start");
-
-			threadstart = start;
-		}
-
-		public Thread (ParameterizedThreadStart start, int maxStackSize)
-		{
-			if (start == null)
-				throw new ArgumentNullException ("start");
-
-			threadstart = start;
-			Internal.stack_size = CheckStackSize (maxStackSize);
-		}
-
-		public ExecutionContext ExecutionContext {
-			[ReliabilityContract (Consistency.WillNotCorruptState, Cer.MayFail)]
-			get {
-				if (_ec == null)
-					_ec = new ExecutionContext ();
-				return _ec;
-			}
-			internal set {
-				_ec = value;
-			}
-		}
-
-		internal bool HasExecutionContext {
-			get {
-				return _ec != null;
-			}
-		}
-
-		internal void BranchExecutionContext (out ExecutionContext.Switcher switcher)
-		{
-			if (_ec == null) {
-				switcher =  new ExecutionContext.Switcher ();
-			} else {
-				switcher = new ExecutionContext.Switcher (_ec);
-				_ec.CopyOnWrite = true;
-			}
-		}
-
-		internal void RestoreExecutionContext (ref ExecutionContext.Switcher switcher)
-		{
-			if (switcher.IsEmpty) {
-				_ec = null;
-				return;
-			}
-
-			switcher.Restore (_ec);
+			m_Delegate = start;
+			Internal.stack_size = maxStackSize;
 		}
 
 		public int ManagedThreadId {
@@ -962,66 +722,38 @@ namespace System.Threading {
 			return ManagedThreadId;
 		}
 
-		public void Start (object parameter)
+		internal CultureInfo GetCurrentUICultureNoAppX ()
 		{
-			start_obj = parameter;
-			Start ();
+			return CultureInfo.CurrentUICulture;
 		}
 
-		// NOTE: This method doesn't show in the class library status page because
-		// it cannot be "found" with the StrongNameIdentityPermission for ECMA key.
-		// But it's there!
-		[SecurityPermission (SecurityAction.LinkDemand, UnmanagedCode = true)]
-		[StrongNameIdentityPermission (SecurityAction.LinkDemand, PublicKey="00000000000000000400000000000000")]
-		[Obsolete ("see CompressedStack class")]
-		public CompressedStack GetCompressedStack ()
-		{
-#if MOBILE
-			throw new NotSupportedException ();
-#else			
-			// Note: returns null if no CompressedStack has been set.
-			// However CompressedStack.GetCompressedStack returns an 
-			// (empty?) CompressedStack instance.
-			CompressedStack cs = ExecutionContext.SecurityContext.CompressedStack;
-			return ((cs == null) || cs.IsEmpty ()) ? null : cs.CreateCopy ();
-#endif
+		[MethodImplAttribute(MethodImplOptions.InternalCall)]
+		internal static extern void GetStackTraces (out Thread[] threads, out object[] stack_frames);
+
+		// This is a mono extension to gather the stack traces for all running threads
+		internal static Dictionary<Thread, StackTrace> Mono_GetStackTraces () {
+			Thread[] threads;
+			object[] stack_frames;
+
+			GetStackTraces (out threads, out stack_frames);
+
+			var res = new Dictionary<Thread, StackTrace> ();
+			for (int i = 0; i < threads.Length; ++i)
+				res [threads [i]] = new StackTrace ((StackFrame[])stack_frames [i]);
+			return res;
 		}
 
-		// NOTE: This method doesn't show in the class library status page because
-		// it cannot be "found" with the StrongNameIdentityPermission for ECMA key.
-		// But it's there!
-		[SecurityPermission (SecurityAction.LinkDemand, UnmanagedCode = true)]
-		[StrongNameIdentityPermission (SecurityAction.LinkDemand, PublicKey="00000000000000000400000000000000")]
-		[Obsolete ("see CompressedStack class")]
-		public void SetCompressedStack (CompressedStack stack)
+#if !MONO_FEATURE_THREAD_SUSPEND_RESUME
+		[Obsolete ("Thread.Suspend is not supported on the current platform.", true)]
+		public void Suspend ()
 		{
-#if MOBILE
-			throw new NotSupportedException ();
-#else
-			ExecutionContext.SecurityContext.CompressedStack = stack;
-#endif
+			throw new PlatformNotSupportedException ("Thread.Suspend is not supported on the current platform.");
 		}
 
-#if !MOBILE
-		void _Thread.GetIDsOfNames ([In] ref Guid riid, IntPtr rgszNames, uint cNames, uint lcid, IntPtr rgDispId)
+		[Obsolete ("Thread.Resume is not supported on the current platform.", true)]
+		public void Resume ()
 		{
-			throw new NotImplementedException ();
-		}
-
-		void _Thread.GetTypeInfo (uint iTInfo, uint lcid, IntPtr ppTInfo)
-		{
-			throw new NotImplementedException ();
-		}
-
-		void _Thread.GetTypeInfoCount (out uint pcTInfo)
-		{
-			throw new NotImplementedException ();
-		}
-
-		void _Thread.Invoke (uint dispIdMember, [In] ref Guid riid, uint lcid, short wFlags, IntPtr pDispParams,
-			IntPtr pVarResult, IntPtr pExcepInfo, IntPtr puArgErr)
-		{
-			throw new NotImplementedException ();
+			throw new PlatformNotSupportedException ("Thread.Resume is not supported on the current platform.");
 		}
 #endif
 	}
