@@ -96,11 +96,6 @@ namespace System
 
 		static Console ()
 		{
-#if NET_2_1
-			Encoding inputEncoding;
-			Encoding outputEncoding;
-#endif
-
 			if (Environment.IsRunningOnWindows) {
 				//
 				// On Windows, follow the Windows tradition
@@ -152,10 +147,7 @@ namespace System
 				stdin = new CStreamReader (OpenStandardInput (0), inputEncoding);
 			} else {
 #endif
-// FULL_AOT_RUNTIME is used (instead of MONOTOUCH) since we only want this code when running on 
-// iOS (simulator or devices) and *not* when running tools (e.g. btouch #12179) that needs to use 
-// the mscorlib.dll shipped with Xamarin.iOS
-#if MONOTOUCH && FULL_AOT_RUNTIME
+#if MONOTOUCH
 				stdout = new NSLogWriter ();
 #else
 				stdout = new UnexceptionalStreamWriter (OpenStandardOutput (0), outputEncoding);
@@ -163,7 +155,7 @@ namespace System
 #endif
 				stdout = TextWriter.Synchronized (stdout);
 
-#if MONOTOUCH && FULL_AOT_RUNTIME
+#if MONOTOUCH
 				stderr = new NSLogWriter ();
 #else
 				stderr = new UnexceptionalStreamWriter (OpenStandardError (0), outputEncoding); 
@@ -210,7 +202,8 @@ namespace System
 		private static Stream Open (IntPtr handle, FileAccess access, int bufferSize)
 		{
 			try {
-				return new FileStream (handle, access, false, bufferSize, false, bufferSize == 0);
+				// TODO: Should use __ConsoleStream from reference sources
+				return new FileStream (handle, access, false, bufferSize, false, true);
 			} catch (IOException) {
 				return Stream.Null;
 			}
@@ -542,7 +535,6 @@ namespace System
 
 #endif
 
-#if !NET_2_1
 		// FIXME: Console should use these encodings when changed
 		static Encoding inputEncoding;
 		static Encoding outputEncoding;
@@ -563,6 +555,7 @@ namespace System
 			}
 		}
 
+#if !NET_2_1
 		public static ConsoleColor BackgroundColor {
 			get { return ConsoleDriver.BackgroundColor; }
 			set { ConsoleDriver.BackgroundColor = value; }
