@@ -16,6 +16,7 @@ using System.IO;
 using System.Runtime.InteropServices;
 using System.Text;
 using System.Threading;
+using Microsoft.Win32.SafeHandles;
 
 namespace MonoTests.System.IO
 {
@@ -87,12 +88,13 @@ namespace MonoTests.System.IO
 		[Test]
 		public void CtorFileNotFoundException_Mode_Open ()
 		{
+			const string file_name = "thisfileshouldnotexist.test";
 			// absolute path
-			string path = TempFolder + DSC + "thisfileshouldnotexists.test";
+			string path = TempFolder + DSC + file_name;
 			DeleteFile (path);
 			FileStream stream = null;
 			try {
-				stream = new FileStream (TempFolder + DSC + "thisfileshouldnotexists.test", FileMode.Open);
+				stream = new FileStream (TempFolder + DSC + file_name, FileMode.Open);
 				Assert.Fail ("#A1");
 			} catch (FileNotFoundException ex) {
 				Assert.AreEqual (typeof (FileNotFoundException), ex.GetType (), "#A2");
@@ -111,14 +113,18 @@ namespace MonoTests.System.IO
 			// relative path
 			string orignalCurrentDir = Directory.GetCurrentDirectory ();
 			Directory.SetCurrentDirectory (TempFolder);
+
+			// If TempFolder is a symlink, Mono will follow it and ex.FileName below will contain
+			// the real directory name, not that of the TempFolder symlink and the test will fail
+			// (happens e.g. on Android M)
+			string realTempDir = Directory.GetCurrentDirectory ();
+			path = realTempDir + DSC + file_name;
+
 			try {
-				stream = new FileStream ("thisfileshouldnotexists.test", FileMode.Open);
+				stream = new FileStream (file_name, FileMode.Open);
 				Assert.Fail ("#B1");
 			} catch (FileNotFoundException ex) {
 				Assert.AreEqual (typeof (FileNotFoundException), ex.GetType (), "#B2");
-				// under OSX 'var' is a symlink to 'private/var'
-				if (MacOSX)
-					path = "/private" + path;
 				Assert.AreEqual (path, ex.FileName, "#B3");
 				Assert.IsNull (ex.InnerException, "#B4");
 				Assert.IsNotNull (ex.Message, "#B5");
@@ -135,8 +141,9 @@ namespace MonoTests.System.IO
 		[Test]
 		public void CtorFileNotFoundException_Mode_Truncate ()
 		{
+			const string file_name = "thisfileshouldNOTexist.test";
 			// absolute path
-			string path = TempFolder + DSC + "thisfileshouldNOTexists.test";
+			string path = TempFolder + DSC + file_name;
 			DeleteFile (path);
 			FileStream stream = null;
 			try {
@@ -159,14 +166,18 @@ namespace MonoTests.System.IO
 			// relative path
 			string orignalCurrentDir = Directory.GetCurrentDirectory ();
 			Directory.SetCurrentDirectory (TempFolder);
+
+			// If TempFolder is a symlink, Mono will follow it and ex.FileName below will contain
+			// the real directory name, not that of the TempFolder symlink and the test will fail
+			// (happens e.g. on Android M)
+			string realTempDir = Directory.GetCurrentDirectory ();
+			path = realTempDir + DSC + file_name;
+
 			try {
-				stream = new FileStream ("thisfileshouldNOTexists.test", FileMode.Truncate);
+				stream = new FileStream (file_name, FileMode.Truncate);
 				Assert.Fail ("#B1");
 			} catch (FileNotFoundException ex) {
 				Assert.AreEqual (typeof (FileNotFoundException), ex.GetType (), "#B2");
-				// under OSX 'var' is a symlink to 'private/var'
-				if (MacOSX)
-					path = "/private" + path;
 				Assert.AreEqual (path, ex.FileName, "#B3");
 				Assert.IsNull (ex.InnerException, "#B4");
 				Assert.IsNotNull (ex.Message, "#B5");
@@ -183,8 +194,9 @@ namespace MonoTests.System.IO
 		[Test]
 		public void CtorIOException1 ()
 		{
+			const string file_name = "thisfileshouldexists.test";
 			// absolute path
-			string path = TempFolder + DSC + "thisfileshouldexists.test";
+			string path = TempFolder + DSC + file_name;
 			FileStream stream = null;
 			DeleteFile (path);
 			try {
@@ -209,11 +221,18 @@ namespace MonoTests.System.IO
 			// relative path
 			string orignalCurrentDir = Directory.GetCurrentDirectory ();
 			Directory.SetCurrentDirectory (TempFolder);
+
+			// If TempFolder is a symlink, Mono will follow it and ex.FileName below will contain
+			// the real directory name, not that of the TempFolder symlink and the test will fail
+			// (happens e.g. on Android M)
+			string realTempDir = Directory.GetCurrentDirectory ();
+			path = realTempDir + DSC + file_name;
+
 			try {
-				stream = new FileStream ("thisfileshouldexists.test", FileMode.CreateNew);
+				stream = new FileStream (file_name, FileMode.CreateNew);
 				stream.Close ();
 				stream = null;
-				stream = new FileStream ("thisfileshouldexists.test", FileMode.CreateNew);
+				stream = new FileStream (file_name, FileMode.CreateNew);
 				Assert.Fail ("#B1");
 			} catch (IOException ex) {
 				Assert.AreEqual (typeof (IOException), ex.GetType (), "#B2");
@@ -327,8 +346,14 @@ namespace MonoTests.System.IO
 		{
 			string orignalCurrentDir = Directory.GetCurrentDirectory ();
 			Directory.SetCurrentDirectory (TempFolder);
+
+			// If TempFolder is a symlink, Mono will follow it and ex.FileName below will contain
+			// the real directory name, not that of the TempFolder symlink and the test will fail
+			// (happens e.g. on Android M)
+			string realTempDir = Directory.GetCurrentDirectory ();
+			
 			string relativePath = "DirectoryDoesNotExist" + Path.DirectorySeparatorChar + "file.txt";
-			string fullPath = Path.Combine (TempFolder, relativePath);
+			string fullPath = Path.Combine (realTempDir, relativePath);
 			try {
 				new FileStream (relativePath, FileMode.Open);
 				Assert.Fail ("#A1");
@@ -1507,15 +1532,23 @@ namespace MonoTests.System.IO
 		static IAsyncResult DoBeginWrite(Stream stream, ManualResetEvent mre, byte[] RandomBuffer)
 		{
 			return stream.BeginWrite (RandomBuffer, 0, RandomBuffer.Length, ar => {
-				stream.EndWrite (ar);
+				IAsyncResult begin_write_recursive_ares;
 
-				// we don't supply an ManualResetEvent so this will throw an NRE on the second run
-				// which nunit-console will ignore (but other test runners don't like that)
-				if (mre == null)
-					return;
+				try {
+					stream.EndWrite (ar);
 
-				DoBeginWrite (stream, null, RandomBuffer).AsyncWaitHandle.WaitOne ();
-				mre.Set ();
+					// we don't supply an ManualResetEvent so this will throw an NRE on the second run
+					// which nunit-console will ignore (but other test runners don't like that)
+					if (mre == null)
+						return;
+
+					begin_write_recursive_ares = DoBeginWrite (stream, null, RandomBuffer);
+					begin_write_recursive_ares.AsyncWaitHandle.WaitOne ();
+
+					mre.Set ();
+				} catch (ObjectDisposedException e) {
+					Console.WriteLine ("stream was disposed: {0}", e);
+				}
 			}, null);
 		}
 
@@ -1524,12 +1557,19 @@ namespace MonoTests.System.IO
 		{
 			string path = TempFolder + Path.DirectorySeparatorChar + "temp";
 			DeleteFile (path);
-	
-			using (FileStream stream = new FileStream (path, FileMode.OpenOrCreate, FileAccess.Write)) {
-				var mre = new ManualResetEvent (false);	
-				var RandomBuffer = new byte[1024];			
-				DoBeginWrite (stream, mre, RandomBuffer);
-				Assert.IsTrue (mre.WaitOne (5000), "#1");
+
+			IAsyncResult begin_write_ares = null;
+
+			try {
+				using (FileStream stream = new FileStream (path, FileMode.OpenOrCreate, FileAccess.Write)) {
+					var mre = new ManualResetEvent (false);
+					var RandomBuffer = new byte[1024];
+					begin_write_ares = DoBeginWrite (stream, mre, RandomBuffer);
+					Assert.IsTrue (mre.WaitOne (5000), "#1");
+				}
+			} finally {
+				if (begin_write_ares != null)
+					begin_write_ares.AsyncWaitHandle.WaitOne ();
 			}
 		}
 
@@ -1620,6 +1660,28 @@ namespace MonoTests.System.IO
 			
 		}
 
+		[Test]
+		public void OpenCharDeviceRepeatedly ()
+		{
+			// https://bugzilla.xamarin.com/show_bug.cgi?id=38408
+			try {
+				using (var f = new FileStream ("/dev/zero", FileMode.Open))
+				{
+				}
+			} catch (FileNotFoundException) {
+				// Only run this test on platforms where /dev/zero exists
+				Assert.Ignore();
+			} catch (DirectoryNotFoundException) {
+				// Only run this test on platforms where /dev exists
+				Assert.Ignore();
+			}
+
+			// this shouldn't throw
+			using (var g = new FileStream ("/dev/zero", FileMode.Open))
+			{
+			}
+		}
+
 #if !MOBILE
 		[Test]
 		public void WriteWithExposedHandle ()
@@ -1654,6 +1716,17 @@ namespace MonoTests.System.IO
 				DeleteFile (path);
 			}
 		}
+
+		[Test]
+		public void Ctor_InvalidSafeHandle ()
+		{
+			var sf = new SafeFileHandle (IntPtr.Zero, true);
+			try {
+				new FileStream (sf, FileAccess.ReadWrite);
+				Assert.Fail ("#1");
+			} catch (ArgumentException) {
+			}
+		}
 #endif
 	}
 }
