@@ -18,13 +18,10 @@ using System.Runtime.InteropServices;
 using NUnit.Framework;
 using System.Runtime.CompilerServices;
 
-#if NET_2_0
 using System.Collections.Generic;
-#endif
 
 namespace MonoTests.System.Reflection.Emit
 {
-#if NET_2_0
 	[TestFixture]
 	public class PointerTypeTest
 	{
@@ -526,10 +523,7 @@ namespace MonoTests.System.Reflection.Emit
 			Assert.IsFalse (ptr.IsGenericType, "#9");
 			Assert.IsFalse (ptr.IsGenericTypeDefinition, "#10");
 
-			try {
-				var x = ptr.Attributes; 
-				Assert.Fail ("#11");
-			} catch (NotSupportedException) {}
+			Assert.AreEqual (TypeAttributes.Public, ptr.Attributes, "#11");
 
 			Assert.IsTrue (ptr.HasElementType, "#12");
 			Assert.IsTrue (ptr.IsPointer, "#13");
@@ -979,6 +973,8 @@ namespace MonoTests.System.Reflection.Emit
 
 		[Test]
 		[Category ("NotDotNet")]
+		// CompilerContext no longer supported
+		[Category ("NotWorking")]
 		public void ByRefOfAttriburesUnderCompilerContext ()
 		{
 			SetUp (AssemblyBuilderAccess.RunAndSave | (AssemblyBuilderAccess)0x800);
@@ -1030,10 +1026,8 @@ namespace MonoTests.System.Reflection.Emit
 			Assert.IsFalse (byref.IsGenericType, "#9");
 			Assert.IsFalse (byref.IsGenericTypeDefinition, "#10");
 
-			try {
-				var x = byref.Attributes; 
-				Assert.Fail ("#11");
-			} catch (NotSupportedException) {}
+
+			Assert.AreEqual (TypeAttributes.Public, byref.Attributes, "#11");
 
 			Assert.IsTrue (byref.HasElementType, "#12");
 			Assert.IsTrue (byref.IsByRef, "#13");
@@ -1288,6 +1282,8 @@ namespace MonoTests.System.Reflection.Emit
 
 		[Test]
 		[Category ("NotDotNet")]
+		// CompilerContext no longer supported
+		[Category ("NotWorking")]
 		public void AttributeValuesUnderCompilerContext ()
 		{
 			SetUp (AssemblyBuilderAccess.RunAndSave | (AssemblyBuilderAccess)0x800);
@@ -1653,10 +1649,7 @@ namespace MonoTests.System.Reflection.Emit
 			Assert.IsFalse (arr.IsGenericType, "#9");
 			Assert.IsFalse (arr.IsGenericTypeDefinition, "#10");
 
-			try {
-				var x = arr.Attributes; //This is because GenericTypeParameterBuilder doesn't support Attributes 
-				Assert.Fail ("#11");
-			} catch (NotSupportedException) {}
+			Assert.AreEqual (TypeAttributes.Public, arr.Attributes, "#11");
 
 			Assert.IsTrue (arr.HasElementType, "#12");
 			Assert.IsTrue (arr.IsArray, "#13");
@@ -1728,5 +1721,4 @@ namespace MonoTests.System.Reflection.Emit
 		}
 
 	}
-#endif
 }
