@@ -5,9 +5,11 @@
 // 	Gonzalo Paniagua Javier (gonzalo@ximian.com)
 //	Philippe Lavoie (philippe.lavoie@cactus.ca)
 //	Sebastien Pouliot (sebastien@ximian.com)
+//      Aleksey Kliger (aleksey@xamarin.com)
 //
 // (c) 2003 Ximian, Inc. (http://www.ximian.com)
 // Copyright (C) 2004-2005 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2015 Xamarin, Inc. (http://www.xamarin.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -35,12 +37,18 @@ using System.Configuration.Assemblies;
 using System.Globalization;
 using System.IO;
 using System.Reflection;
-#if !TARGET_JVM
+#if !MONOTOUCH
 using System.Reflection.Emit;
 #endif
 using System.Threading;
 using System.Runtime.Serialization;
+using System.Runtime.CompilerServices;
 using System.Security;
+using System.Linq;
+using System.Resources;
+
+// Used by GetType_TypeForwarder_Nested ()
+[assembly: System.Runtime.CompilerServices.TypeForwardedToAttribute(typeof(System.Globalization.CultureInfo))]
 
 namespace MonoTests.System.Reflection
 {
@@ -87,23 +95,63 @@ namespace MonoTests.System.Reflection
 		}
 
 		[Test] // bug #49114
-#if NET_2_0
-		[Category ("NotWorking")]
 		[ExpectedException (typeof (ArgumentException))]
-#else
-		[ExpectedException (typeof (TypeLoadException))]
-#endif
 		public void GetType_TypeName_Invalid () 
 		{
 			typeof (int).Assembly.GetType ("&blabla", true, true);
 		}
 
+		[Test] // bug #17571
+		[ExpectedException (typeof (ArgumentException))]
+		public void GetType_Invalid_RefPtr () {
+			typeof (int).Assembly.GetType ("System.Int32&*", true, true);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void GetType_Invalid_RefArray () {
+			typeof (int).Assembly.GetType ("System.Int32&[]", true, true);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void GetType_Invalid_RefGeneric () {
+			typeof (int).Assembly.GetType ("System.Tuple`1&[System.Int32]", true, true);
+		}
+
+		[Test]
+		[ExpectedException (typeof (ArgumentException))]
+		public void GetType_Invalid_PtrGeneric () {
+			typeof (int).Assembly.GetType ("System.Tuple`1*[System.Int32]", true, true);
+		}
+
+		[Test]
+		public void GetType_ComposeModifiers () {
+			var a = typeof(int).Assembly;
+
+			var e1 = typeof (Int32).MakePointerType().MakeByRefType();
+			var t1 = a.GetType ("System.Int32*&", true, true);
+			Assert.AreEqual (e1, t1, "#1");
+
+			var e2 = typeof (Int32).MakeArrayType(2).MakeByRefType();
+			var t2 = a.GetType ("System.Int32[,]&", true, true);
+			Assert.AreEqual (e2, t2, "#2");
+
+			var e3 = typeof (Int32).MakePointerType().MakeArrayType();
+			var t3 = a.GetType ("System.Int32*[]", true, true);
+			Assert.AreEqual (e3, t3, "#3");
+
+			var e4 = typeof (Int32).MakeArrayType().MakePointerType().MakePointerType().MakeArrayType().MakePointerType().MakeByRefType();
+			var t4 = a.GetType ("System.Int32[]**[]*&", true, true);
+			Assert.AreEqual (e4, t4, "#4");
+				
+		}
+		
 		[Test] // bug #334203
 		public void GetType_TypeName_AssemblyName ()
 		{
 			Assembly a = typeof (int).Assembly;
 			string typeName = typeof (string).AssemblyQualifiedName;
-#if NET_2_0
 			try {
 				a.GetType (typeName, true, false);
 				Assert.Fail ("#A1");
@@ -113,17 +161,6 @@ namespace MonoTests.System.Reflection
 				Assert.IsNotNull (ex.Message, "#A4");
 				Assert.IsNull (ex.ParamName, "#A5");
 			}
-#else
-			try {
-				a.GetType (typeName, true, false);
-				Assert.Fail ("#A1");
-			} catch (TypeLoadException ex) {
-				Assert.AreEqual (typeof (TypeLoadException), ex.GetType (), "#A2");
-				Assert.IsNull (ex.InnerException, "#A3");
-				Assert.IsNotNull (ex.Message, "#A4");
-				Assert.IsTrue (ex.Message.IndexOf (typeName) != -1, "#A5");
-			}
-#endif
 
 			Type type = a.GetType (typeName, false);
 			Assert.IsNull (type, "#B1");
@@ -131,28 +168,38 @@ namespace MonoTests.System.Reflection
 			Assert.IsNull (type, "#B2");
 		}
 
+		[Test]
+		public void GetType_TypeForwarder_Nested () {
+			// System.Globalization is a PCL assembly
+			Type t = typeof (AssemblyTest).Assembly.GetType ("System.Globalization.CultureInfo/Data");
+			Assert.IsNotNull (t);
+			Assert.AreEqual ("System.Globalization.CultureInfo+Data", t.FullName);
+		}
+
 		[Test]
 		public void GetEntryAssembly ()
 		{
 			// note: only available in default appdomain
 			// http://weblogs.asp.net/asanto/archive/2003/09/08/26710.aspx
 			// Not sure we should emulate this behavior.
+#if !MONODROID
 			string fname = AppDomain.CurrentDomain.FriendlyName;
 			if (fname.EndsWith (".dll")) { // nunit-console
 				Assert.IsNull (Assembly.GetEntryAssembly (), "GetEntryAssembly");
-#if NET_2_0 && !TARGET_JVM // IsDefaultAppDomain not supported for TARGET_JVM
 				Assert.IsFalse (AppDomain.CurrentDomain.IsDefaultAppDomain (), "!default appdomain");
-#endif
 			} else { // gnunit
 				Assert.IsNotNull (Assembly.GetEntryAssembly (), "GetEntryAssembly");
-#if NET_2_0 && !TARGET_JVM // IsDefaultAppDomain not supported for TARGET_JVM
 				Assert.IsTrue (AppDomain.CurrentDomain.IsDefaultAppDomain (), "!default appdomain");
-#endif
 			}
+#else
+			Assert.IsNull (Assembly.GetEntryAssembly (), "GetEntryAssembly");
+			Assert.IsTrue (AppDomain.CurrentDomain.IsDefaultAppDomain (), "!default appdomain");
+#endif
 		}
 
-#if !TARGET_JVM // Reflection.Emit is not supported.
+#if !MONOTOUCH // Reflection.Emit is not supported.
 		[Test]
+		[Category("AndroidNotWorking")] // Missing Mono.CompilerServices.SymbolWriter
 		public void GetModules_MissingFile ()
 		{
 			AssemblyName newName = new AssemblyName ();
@@ -176,10 +223,7 @@ namespace MonoTests.System.Reflection
 		}
 #endif
 
-#if !TARGET_JVM // ManifestModule not supported under TARGET_JVM.
-#if NET_2_0
 		[Category ("NotWorking")]
-#endif
 		[Test]
 		public void Corlib () 
 		{
@@ -192,7 +236,6 @@ namespace MonoTests.System.Reflection
 
 			// corlib doesn't reference anything
 			Assert.AreEqual (0, corlib.GetReferencedAssemblies ().Length, "GetReferencedAssemblies");
-#if NET_2_0
 			Assert.AreEqual ("mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", corlib.FullName, "FullName");
 			// not really "true" but it's even more trusted so...
 			Assert.IsTrue (corlib.GlobalAssemblyCache, "GlobalAssemblyCache");
@@ -200,33 +243,31 @@ namespace MonoTests.System.Reflection
 			Assert.AreEqual ("v2.0.50727", corlib.ImageRuntimeVersion, "ImageRuntimeVersion");
 			Assert.IsFalse (corlib.ReflectionOnly, "ReflectionOnly");
 			Assert.AreEqual (0x1, corlib.ManifestModule.MetadataToken);
-#elif NET_1_1
-			Assert.IsFalse (corlib.GlobalAssemblyCache, "GlobalAssemblyCache");
-			Assert.AreEqual ("mscorlib, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", corlib.FullName, "FullName");
-			Assert.AreEqual ("v1.1.4322", corlib.ImageRuntimeVersion, "ImageRuntimeVersion");
-#endif
 		}
 
 		[Test]
 		public void Corlib_test ()
 		{
 			Assembly corlib_test = Assembly.GetExecutingAssembly ();
+#if MONODROID
+			Assert.IsNull (corlib_test.EntryPoint, "EntryPoint");
+			Assert.IsNull (corlib_test.Evidence, "Evidence");
+#elif MOBILE
+			Assert.IsNotNull (corlib_test.EntryPoint, "EntryPoint");
+			Assert.IsNull (corlib_test.Evidence, "Evidence");
+#else
 			Assert.IsNull (corlib_test.EntryPoint, "EntryPoint");
 			Assert.IsNotNull (corlib_test.Evidence, "Evidence");
+#endif
 			Assert.IsFalse (corlib_test.GlobalAssemblyCache, "GlobalAssemblyCache");
 
 			Assert.IsTrue (corlib_test.GetReferencedAssemblies ().Length > 0, "GetReferencedAssemblies");
 			Assert.AreEqual (0, corlib_test.HostContext, "HostContext");
-#if NET_4_0
 			Assert.AreEqual ("v4.0.30319", corlib_test.ImageRuntimeVersion, "ImageRuntimeVersion");
-#else
-			Assert.AreEqual ("v2.0.50727", corlib_test.ImageRuntimeVersion, "ImageRuntimeVersion");
-#endif
 
 			Assert.IsNotNull (corlib_test.ManifestModule, "ManifestModule");
 			Assert.IsFalse (corlib_test.ReflectionOnly, "ReflectionOnly");
 		}
-#endif
 
 		[Test]
 		public void GetAssembly ()
@@ -236,7 +277,6 @@ namespace MonoTests.System.Reflection
 		}
 
 		[Test]
-		[Category("TargetJvmNotWorking")] // Not yet supported for TARGET_JVM
 		public void GetFile_Null ()
 		{
 			try {
@@ -251,7 +291,6 @@ namespace MonoTests.System.Reflection
 		}
 
 		[Test]
-		[Category("TargetJvmNotWorking")] // Not yet supported for TARGET_JVM
 		public void GetFile_Empty ()
 		{
 			try {
@@ -267,7 +306,7 @@ namespace MonoTests.System.Reflection
 		}
 
 		[Test]
-		[Category("TargetJvmNotWorking")] // Not yet supported for TARGET_JVM
+		[Category ("AndroidNotWorking")] // Assemblies in Xamarin.Android cannot be accessed as FileStream
 		public void GetFiles_False ()
 		{
 			Assembly corlib = typeof (int).Assembly;
@@ -280,7 +319,7 @@ namespace MonoTests.System.Reflection
 		}
 
 		[Test]
-		[Category("TargetJvmNotWorking")] // Not yet supported for TARGET_JVM
+		[Category ("AndroidNotWorking")] // Assemblies in Xamarin.Android cannot be accessed as FileStream
 		public void GetFiles_True ()
 		{
 			Assembly corlib = typeof (int).Assembly;
@@ -366,9 +405,6 @@ namespace MonoTests.System.Reflection
 		}
 
 		[Test] // bug #78517
-#if ONLY_1_1
-		[Category ("NotDotNet")] // MS.NET 1.x throws FileLoadException
-#endif
 		public void LoadFrom_Empty_Assembly ()
 		{
 			string tempFile = Path.GetTempFileName ();
@@ -421,7 +457,7 @@ namespace MonoTests.System.Reflection
 		[Test]
 		public void LoadWithPartialName ()
 		{
-			string [] names = { "corlib_test_net_1_1", "corlib_test_net_2_0", "corlib_test_net_4_0", "corlib_plattest" };
+			string [] names = { "corlib_test_net_1_1", "corlib_test_net_2_0", "corlib_test_net_4_0", "corlib_test_net_4_5", "corlib_test_net_4_x", "corlib_plattest", "mscorlibtests", "BclTests" };
 
 			foreach (string s in names)
 				if (Assembly.LoadWithPartialName (s) != null)
@@ -429,7 +465,6 @@ namespace MonoTests.System.Reflection
 			Assert.Fail ("Was not able to load any corlib test");
 		}
 
-#if !TARGET_JVM // GetObjectData currently not implemented for Assembly.
 		[Test]
 		public void GetObjectData_Info_Null ()
 		{
@@ -446,7 +481,6 @@ namespace MonoTests.System.Reflection
 				Assert.AreEqual ("info", ex.ParamName, "#6");
 			}
 		}
-#endif // TARGET_JVM
 
 		[Test]
 		public void GetReferencedAssemblies ()
@@ -468,7 +502,7 @@ namespace MonoTests.System.Reflection
 			}
 		}
 
-#if !TARGET_JVM // Reflection.Emit is not supported.
+#if !MONOTOUCH // Reflection.Emit is not supported.
 		[Test]
 		public void Location_Empty() {
 			string assemblyFileName = Path.Combine (
@@ -497,6 +531,63 @@ namespace MonoTests.System.Reflection
 			}
 		}
 
+		[Test]
+		public void SateliteAssemblyForInMemoryAssembly ()
+		{
+			string assemblyFileName = Path.Combine (
+				Path.GetTempPath (), "AssemblyLocation1.dll");
+
+			try {
+				AssemblyName assemblyName = new AssemblyName ();
+				assemblyName.Name = "AssemblyLocation1";
+
+				AssemblyBuilder ab = AppDomain.CurrentDomain
+					.DefineDynamicAssembly (assemblyName,
+						AssemblyBuilderAccess.Save,
+						Path.GetTempPath ());
+
+				ModuleBuilder moduleBuilder = ab.DefineDynamicModule (assemblyName.Name, assemblyName.Name + ".dll");
+				TypeBuilder typeBuilder = moduleBuilder.DefineType ("Program", TypeAttributes.Public);
+
+				MethodBuilder methodBuilder = typeBuilder.DefineMethod ("TestCall", MethodAttributes.Public | MethodAttributes.Static, typeof(void), Type.EmptyTypes);
+				ILGenerator gen = methodBuilder.GetILGenerator ();
+
+				//
+				// 	var resourceManager = new ResourceManager (typeof (Program));
+				//	resourceManager.GetString ("test");
+				//
+				gen.Emit (OpCodes.Ldtoken, typeBuilder);
+				gen.Emit (OpCodes.Call, typeof(Type).GetMethod ("GetTypeFromHandle"));
+				gen.Emit (OpCodes.Newobj, typeof(ResourceManager).GetConstructor (new Type[] { typeof(Type) }));
+				gen.Emit (OpCodes.Ldstr, "test");
+				gen.Emit (OpCodes.Callvirt, typeof(ResourceManager).GetMethod ("GetString", new Type[] { typeof(string) }));
+				gen.Emit (OpCodes.Pop);
+				gen.Emit (OpCodes.Ret);
+
+				typeBuilder.CreateType ();
+
+				ab.Save (Path.GetFileName (assemblyFileName));
+
+				using (FileStream fs = File.OpenRead (assemblyFileName)) {
+					byte[] buffer = new byte[fs.Length];
+					fs.Read (buffer, 0, buffer.Length);
+					Assembly assembly = Assembly.Load (buffer);
+
+					var mm = assembly.GetType ("Program").GetMethod ("TestCall");
+					try {
+						mm.Invoke (null, null);
+						Assert.Fail ();
+					} catch (TargetInvocationException e) {
+						Assert.IsTrue (e.InnerException is MissingManifestResourceException);
+					}
+
+					fs.Close ();
+				}
+			} finally {
+				File.Delete (assemblyFileName);
+			}
+		}
+
 		[Test]
 		[Category ("NotWorking")]
 		public void bug78464 ()
@@ -519,6 +610,7 @@ namespace MonoTests.System.Reflection
 		}
 
 		[Test]
+		[Category("MobileNotWorking")]
 		public void bug78465 ()
 		{
 			string assemblyFileName = Path.Combine (
@@ -556,6 +648,7 @@ namespace MonoTests.System.Reflection
 		}
 
 		[Test]
+		[Category("MobileNotWorking")]
 		public void bug78468 ()
 		{
 			string assemblyFileNameA = Path.Combine (Path.GetTempPath (),
@@ -599,15 +692,11 @@ namespace MonoTests.System.Reflection
 				Assert.AreEqual ("readme.txt", resInfo.FileName, "#A6");
 				Assert.IsNull (resInfo.ReferencedAssembly, "#A7");
 				Assert.AreEqual ((ResourceLocation) 0, resInfo.ResourceLocation, "#A8");
-#if NET_2_0
 				try {
 					assembly.GetManifestResourceStream ("read");
 					Assert.Fail ("#A9");
 				} catch (FileNotFoundException) {
 				}
-#else
-				Assert.IsNull (assembly.GetManifestResourceStream ("read"), "#A9");
-#endif
 				try {
 					assembly.GetFile ("readme.txt");
 					Assert.Fail ("#A10");
@@ -631,7 +720,6 @@ namespace MonoTests.System.Reflection
 			}
 		}
 
-#if NET_2_0
 		[Test]
 		[Category ("NotWorking")]
 		public void ReflectionOnlyLoad ()
@@ -643,6 +731,7 @@ namespace MonoTests.System.Reflection
 		}
 
 		[Test]
+		[Category ("AndroidNotWorking")] // Assemblies in Xamarin.Android cannot be directly as files
 		public void ReflectionOnlyLoadFrom ()
 		{
 			string loc = typeof (AssemblyTest).Assembly.Location;
@@ -660,7 +749,6 @@ namespace MonoTests.System.Reflection
 			Assembly assembly = Assembly.ReflectionOnlyLoad (typeof (AssemblyTest).Assembly.FullName);
 			assembly.CreateInstance ("MonoTests.System.Reflection.AssemblyTest");
 		}
-#endif
 
 		[Test]
 		[Category ("NotWorking")] // patch for bug #79720 must be committed first
@@ -770,11 +858,7 @@ namespace MonoTests.System.Reflection
 		}
 
 		[Test] // bug #79712
-#if NET_2_0
 		[Category ("NotWorking")] // in non-default domain, MS throws FileNotFoundException
-#else
-		[Category ("NotWorking")]
-#endif
 		public void Load_Culture_Mismatch ()
 		{
 			string tempDir = Path.Combine (Path.GetTempPath (),
@@ -797,11 +881,7 @@ namespace MonoTests.System.Reflection
 				aname = new AssemblyName ();
 				aname.Name = "bug79712a";
 				aname.CultureInfo = CultureInfo.InvariantCulture;
-#if NET_2_0
 				Assert.IsTrue (cdt.AssertFileNotFoundException (aname), "#A1");
-#else
-				Assert.IsTrue (cdt.AssertFileLoadException (aname), "#A2");
-#endif
 
 				// PART B
 
@@ -813,11 +893,7 @@ namespace MonoTests.System.Reflection
 				aname = new AssemblyName ();
 				aname.Name = "bug79712b";
 				aname.CultureInfo = new CultureInfo ("en-US");
-#if NET_2_0
 				Assert.IsTrue (cdt.AssertFileNotFoundException (aname), "#B1");
-#else
-				Assert.IsTrue (cdt.AssertFileLoadException (aname), "#B2");
-#endif
 			} finally {
 				AppDomain.Unload (ad);
 				if (Directory.Exists (tempDir))
@@ -827,6 +903,7 @@ namespace MonoTests.System.Reflection
 
 
 		[Test] // bug #79715
+		[Category("MobileNotWorking")]
 		public void Load_PartialVersion ()
 		{
 			string tempDir = Path.Combine (Path.GetTempPath (),
@@ -1087,7 +1164,7 @@ namespace MonoTests.System.Reflection
 
 			Directory.Delete (outdir, true);
 		}
-#endif // TARGET_JVM
+#endif
 
 		[Test]
 		public void ManifestModule ()
@@ -1096,13 +1173,11 @@ namespace MonoTests.System.Reflection
 			Module module = assembly.ManifestModule;
 			Assert.IsNotNull (module, "#1");
 
-#if NET_4_0
 			Assert.AreEqual ("MonoModule", module.GetType ().Name, "#2");
-#else
-			Assert.AreEqual (typeof (Module), module.GetType (), "#2");
-#endif
 
+#if !MONOTOUCH
 			Assert.AreEqual ("mscorlib.dll", module.Name, "#3");
+#endif
 			Assert.IsFalse (module.IsResource (), "#4");
 			Assert.IsTrue (assembly.GetModules ().Length > 0, "#5");
 			Assert.AreSame (module, assembly.GetModules () [0], "#6");
@@ -1170,5 +1245,80 @@ namespace MonoTests.System.Reflection
 			} catch (ArgumentException) {}
 		}
 
+		class GetCallingAssemblyCallee {
+			static int _dummy;
+
+			static void sideEffect () {
+				_dummy++;
+			}
+
+			// GetCallingAssembly may see an unpredictable
+			// view of the stack if it's called in tail
+			// position, or if its caller or the caller's
+			// caller is inlined.  So we put in a side
+			// effect to get out of tail position, and we
+			// tag the methods NoInlining to discourage
+			// the inliner.
+			[MethodImplAttribute (MethodImplOptions.NoInlining)]
+			public static Assembly Leaf () {
+				var a = Assembly.GetCallingAssembly ();
+				sideEffect();
+				return a;
+			}
+
+			[MethodImplAttribute (MethodImplOptions.NoInlining)]
+			public static Assembly DirectCall () {
+				var a = Leaf();
+				sideEffect();
+				return a;
+			}
+
+			[MethodImplAttribute (MethodImplOptions.NoInlining)]
+			public static Assembly InvokeCall () {
+				var ty = typeof (GetCallingAssemblyCallee);
+				var mi = ty.GetMethod("Leaf");
+				var o = mi.Invoke(null, null);
+				sideEffect();
+				return (Assembly)o;
+			}
+		}
+
+		[Test]
+		public void GetCallingAssembly_Direct() {
+			var a = GetCallingAssemblyCallee.DirectCall ();
+			Assert.IsNotNull (a);
+
+			Assert.AreEqual (GetType().Assembly, a);
+		}
+
+		[Test]
+		public void GetCallingAssembly_SkipsReflection () {
+			// check that the calling assembly is this
+			// one, not mscorlib (aka, the reflection
+			// API).
+			var a = GetCallingAssemblyCallee.InvokeCall ();
+			Assert.IsNotNull (a);
+
+			var invokeAssembly =
+				typeof (MethodInfo).Assembly;
+			Assert.AreNotEqual (invokeAssembly, a);
+
+			Assert.AreEqual (GetType().Assembly, a);
+		}
+
+#if NET_4_5
+		[Test]
+		public void DefinedTypes_Equality ()
+		{
+			var x1 = Assembly.GetExecutingAssembly ().DefinedTypes.Where (l => l.FullName == "MonoTests.System.Reflection.TestDefinedTypes").Single ();
+			var x2 = Assembly.GetExecutingAssembly ().GetTypes ().Where (l => l.FullName == "MonoTests.System.Reflection.TestDefinedTypes").Single ();
+
+			Assert.AreSame (x1, x2, "#1");
+		}
+#endif
+	}
+
+	public class TestDefinedTypes
+	{
 	}
 }
