@@ -27,7 +27,6 @@
 //
 //
 
-#if NET_4_0
 
 using System;
 using System.Threading;
@@ -306,4 +305,3 @@ namespace MonoTests.System.Threading
 		}
 	}
 }
-#endif
