@@ -262,9 +262,7 @@ namespace Mono.CSharp
 			if (sf.IsHiddenLocation (loc))
 				return false;
 
-#if NET_4_0
 			methodSymbols.MarkSequencePoint (ig.ILOffset, sf.SourceFileEntry, loc.Row, loc.Column, false);
-#endif
 			return true;
 		}
 
@@ -324,9 +322,7 @@ namespace Mono.CSharp
 			if ((flags & Options.OmitDebugInfo) != 0)
 				return;
 
-#if NET_4_0
 			methodSymbols.StartBlock (CodeBlockEntry.Type.Lexical, ig.ILOffset);
-#endif
 		}
 
 		public void BeginCompilerScope ()
@@ -334,9 +330,7 @@ namespace Mono.CSharp
 			if ((flags & Options.OmitDebugInfo) != 0)
 				return;
 
-#if NET_4_0
 			methodSymbols.StartBlock (CodeBlockEntry.Type.CompilerGenerated, ig.ILOffset);
-#endif
 		}
 
 		public void EndExceptionBlock ()
@@ -349,9 +343,7 @@ namespace Mono.CSharp
 			if ((flags & Options.OmitDebugInfo) != 0)
 				return;
 
-#if NET_4_0
 			methodSymbols.EndBlock (ig.ILOffset);
-#endif
 		}
 
 		public void CloseConditionalAccess (TypeSpec type)
@@ -398,6 +390,22 @@ namespace Mono.CSharp
 			if (IsAnonymousStoreyMutateRequired)
 				type = CurrentAnonymousMethod.Storey.Mutator.Mutate (type);
 
+			if (pinned) {
+				//
+				// This is for .net compatibility. I am not sure why pinned
+				// pointer temps are converted to & even if they are pointers to
+				// pointers.
+				//
+				var pt = type as PointerContainer;
+				if (pt != null) {
+					type = pt.Element;
+					if (type.Kind == MemberKind.Void)
+						type = Module.Compiler.BuiltinTypes.IntPtr;
+					
+					return ig.DeclareLocal (type.GetMetaInfo ().MakeByRefType (), true);
+				}
+			}
+
 			return ig.DeclareLocal (type.GetMetaInfo (), pinned);
 		}
 
@@ -1236,7 +1244,7 @@ namespace Mono.CSharp
 					EmitLoad (ec, !conditionalAccess);
 
 					if (conditionalAccess) {
-						conditional_access_dup = !IsInexpensiveLoad ();
+						conditional_access_dup = !ExpressionAnalyzer.IsInexpensiveLoad (instance);
 						if (conditional_access_dup)
 							ec.Emit (OpCodes.Dup);
 					}
@@ -1268,10 +1276,12 @@ namespace Mono.CSharp
 					instance_address.AddressOf (ec, AddressOp.Load);
 				} else if (unwrap != null) {
 					unwrap.Emit (ec);
-					var tmp = ec.GetTemporaryLocal (unwrap.Type);
-					ec.Emit (OpCodes.Stloc, tmp);
-					ec.Emit (OpCodes.Ldloca, tmp);
-					ec.FreeTemporaryLocal (tmp, unwrap.Type);
+					if (addressRequired) {
+						var tmp = ec.GetTemporaryLocal (unwrap.Type);
+						ec.Emit (OpCodes.Stloc, tmp);
+						ec.Emit (OpCodes.Ldloca, tmp);
+						ec.FreeTemporaryLocal (tmp, unwrap.Type);
+					}
 				} else if (!conditional_access_dup) {
 					instance.Emit (ec);
 				}
@@ -1309,7 +1319,7 @@ namespace Mono.CSharp
 			instance.Emit (ec);
 
 			// Only to make verifier happy
-			if (boxInstance && RequiresBoxing ()) {
+			if (boxInstance && ExpressionAnalyzer.RequiresBoxing (instance)) {
 				ec.Emit (OpCodes.Box, instance_type);
 			}
 		}
@@ -1327,7 +1337,12 @@ namespace Mono.CSharp
 			return instance_type;
 		}
 
-		bool RequiresBoxing ()
+
+	}
+
+	static class ExpressionAnalyzer
+	{
+		public static bool RequiresBoxing (Expression instance)
 		{
 			var instance_type = instance.Type;
 			if (instance_type.IsGenericParameter && !(instance is This) && TypeSpec.IsReferenceType (instance_type))
@@ -1342,12 +1357,12 @@ namespace Mono.CSharp
 		//
 		// Returns true for cheap race-free load, where we can avoid using dup
 		//
-		bool IsInexpensiveLoad ()
+		public static bool IsInexpensiveLoad (Expression instance)
 		{
 			if (instance is Constant)
 				return instance.IsSideEffectFree;
 
-			if (RequiresBoxing ())
+			if (RequiresBoxing (instance))
 				return false;
 
 			var vr = instance as VariableReference;
