@@ -11,6 +11,6 @@ interface IBar : IFoo
 
 class X
 {
-	static void Main ()
+	public static void Main ()
 	{ }
 }
