@@ -123,7 +123,9 @@ namespace Mono.Linker.Steps {
 		{
 			while (!QueueIsEmpty ()) {
 				MethodDefinition method = (MethodDefinition) _methods.Dequeue ();
+				Annotations.Push (method);
 				ProcessMethod (method);
+				Annotations.Pop ();
 			}
 		}
 
@@ -334,6 +336,9 @@ namespace Mono.Linker.Steps {
 					return;
 
 				MarkType (et);
+				if (argument.Value == null)
+					return;
+
 				foreach (var cac in (CustomAttributeArgument[]) argument.Value)
 					MarkWithResolvedScope ((TypeReference) cac.Value);
 			} else if (at.Namespace == "System" && at.Name == "Type") {
@@ -346,9 +351,24 @@ namespace Mono.Linker.Steps {
 		// even if we (just before saving) will resolve all type references (bug #26752)
 		void MarkWithResolvedScope (TypeReference type)
 		{
-			// we cannot set the Scope of a TypeSpecification so there's no point in resolving it
-			if ((type == null) || (type is TypeSpecification))
+			if (type == null)
 				return;
+
+			// a GenericInstanceType can could contains generic arguments with scope that
+			// needs to be updated out of the PCL facade (bug #28823)
+			var git = (type as GenericInstanceType);
+			if ((git != null) && git.HasGenericArguments) {
+				foreach (var ga in git.GenericArguments)
+					MarkWithResolvedScope (ga);
+			}
+			// we cannot set the Scope of a TypeSpecification but it's element type can be set
+			// e.g. System.String[] -> System.String
+			var ts = (type as TypeSpecification);
+			if (ts != null) {
+				MarkWithResolvedScope (ts.GetElementType ());
+				return;
+			}
+
 			var td = type.Resolve ();
 			if (td != null)
 				type.Scope = td.Scope;
@@ -467,6 +487,8 @@ namespace Mono.Linker.Steps {
 			if (CheckProcessed (type))
 				return null;
 
+			Annotations.Push (type);
+
 			MarkScope (type.Scope);
 			MarkType (type.BaseType);
 			MarkType (type.DeclaringType);
@@ -498,6 +520,10 @@ namespace Mono.Linker.Steps {
 				MarkMethodsIf (type.Methods, IsStaticConstructorPredicate);
 			}
 
+			DoAdditionalTypeProcessing (type);
+
+			Annotations.Pop ();
+
 			Annotations.Mark (type);
 
 			ApplyPreserveInfo (type);
@@ -505,6 +531,11 @@ namespace Mono.Linker.Steps {
 			return type;
 		}
 
+		// Allow subclassers to mark additional things when marking a method
+		protected virtual void DoAdditionalTypeProcessing (TypeDefinition method)
+		{
+		}
+
 		void MarkTypeSpecialCustomAttributes (TypeDefinition type)
 		{
 			if (!type.HasCustomAttributes)
@@ -554,17 +585,21 @@ namespace Mono.Linker.Steps {
 			return argument != null;
 		}
 
-		protected void MarkNamedMethod (TypeDefinition type, string method_name)
+		protected int MarkNamedMethod (TypeDefinition type, string method_name)
 		{
 			if (!type.HasMethods)
-				return;
+				return 0;
 
+			int count = 0;
 			foreach (MethodDefinition method in type.Methods) {
 				if (method.Name != method_name)
 					continue;
 
 				MarkMethod (method);
+				count++;
 			}
+
+			return count;
 		}
 
 		void MarkSoapHeader (MethodDefinition method, CustomAttribute attribute)
@@ -851,6 +886,7 @@ namespace Mono.Linker.Steps {
 			if (reference.DeclaringType is ArrayType)
 				return null;
 
+			Annotations.Push (reference);
 			if (reference.DeclaringType is GenericInstanceType)
 				MarkType (reference.DeclaringType);
 
@@ -859,13 +895,18 @@ namespace Mono.Linker.Steps {
 
 			MethodDefinition method = ResolveMethodDefinition (reference);
 
-			if (method == null)
+			if (method == null) {
+				Annotations.Pop ();
 				throw new ResolutionException (reference);
+			}
 
 			if (Annotations.GetAction (method) == MethodAction.Nothing)
 				Annotations.SetAction (method, MethodAction.Parse);
 
 			EnqueueMethod (method);
+
+			Annotations.Pop ();
+
 			return method;
 		}
 
@@ -941,11 +982,18 @@ namespace Mono.Linker.Steps {
 			if (ShouldParseMethodBody (method))
 				MarkMethodBody (method.Body);
 
+			DoAdditionalMethodProcessing (method);
+
 			Annotations.Mark (method);
 
 			ApplyPreserveMethods (method);
 		}
 
+		// Allow subclassers to mark additional things when marking a method
+		protected virtual void DoAdditionalMethodProcessing (MethodDefinition method)
+		{
+		}
+
 		void MarkBaseMethods (MethodDefinition method)
 		{
 			IList base_methods = Annotations.GetBaseMethods (method);
