@@ -83,35 +83,35 @@
 #include <mono/io-layer/wapi.h>
 #include <mono/io-layer/wapi-private.h>
 #include <mono/io-layer/handles-private.h>
-#include <mono/io-layer/misc-private.h>
 #include <mono/io-layer/process-private.h>
 #include <mono/io-layer/threads.h>
+#include <mono/io-layer/io-trace.h>
 #include <mono/utils/strenc.h>
 #include <mono/utils/mono-path.h>
 #include <mono/io-layer/timefuncs-private.h>
 #include <mono/utils/mono-time.h>
 #include <mono/utils/mono-membar.h>
-#include <mono/utils/mono-mutex.h>
+#include <mono/utils/mono-os-mutex.h>
 #include <mono/utils/mono-signal-handler.h>
 #include <mono/utils/mono-proclib.h>
+#include <mono/utils/mono-once.h>
+#include <mono/utils/mono-logger-internals.h>
 
 /* The process' environment strings */
-#if defined(__APPLE__) && !defined (__arm__) && !defined (__aarch64__)
+#if defined(__APPLE__)
+#if defined (TARGET_OSX)
 /* Apple defines this in crt_externs.h but doesn't provide that header for 
  * arm-apple-darwin9.  We'll manually define the symbol on Apple as it does
  * in fact exist on all implementations (so far) 
  */
-char ***_NSGetEnviron(void);
+gchar ***_NSGetEnviron(void);
 #define environ (*_NSGetEnviron())
 #else
-extern char **environ;
-#endif
-
-#if 0
-#define DEBUG(...) g_message(__VA_ARGS__)
-#define DEBUG_ENABLED 1
+static char *mono_environ[1] = { NULL };
+#define environ mono_environ
+#endif /* defined (TARGET_OSX) */
 #else
-#define DEBUG(...)
+extern char **environ;
 #endif
 
 static guint32 process_wait (gpointer handle, guint32 timeout, gboolean alertable);
@@ -178,7 +178,9 @@ is_pid_valid (pid_t pid)
 {
 	gboolean result = FALSE;
 
-#if defined(PLATFORM_MACOSX) || defined(__OpenBSD__) || defined(__FreeBSD__)
+#if defined(HOST_WATCHOS)
+	result = TRUE; // TODO: Rewrite using sysctl
+#elif defined(PLATFORM_MACOSX) || defined(__OpenBSD__) || defined(__FreeBSD__)
 	if (((kill(pid, 0) == 0) || (errno == EPERM)) && pid != 0)
 		result = TRUE;
 #elif defined(__HAIKU__)
@@ -552,15 +554,16 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 			WapiStartupInfo *startup,
 			WapiProcessInformation *process_info)
 {
+#if defined (HAVE_FORK) && defined (HAVE_EXECVE)
 	char *cmd = NULL, *prog = NULL, *full_prog = NULL, *args = NULL, *args_after_prog = NULL;
 	char *dir = NULL, **env_strings = NULL, **argv = NULL;
 	guint32 i, env_count = 0;
 	gboolean ret = FALSE;
-	gpointer handle;
+	gpointer handle = NULL;
 	WapiHandle_process process_handle = {0}, *process_handle_data;
 	GError *gerr = NULL;
 	int in_fd, out_fd, err_fd;
-	pid_t pid;
+	pid_t pid = 0;
 	int thr_ret;
 	int startup_pipe [2] = {-1, -1};
 	int dummy;
@@ -598,7 +601,7 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 	if (appname != NULL) {
 		cmd = mono_unicode_to_external (appname);
 		if (cmd == NULL) {
-			DEBUG ("%s: unicode conversion returned NULL",
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: unicode conversion returned NULL",
 				   __func__);
 
 			SetLastError (ERROR_PATH_NOT_FOUND);
@@ -611,7 +614,7 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 	if (cmdline != NULL) {
 		args = mono_unicode_to_external (cmdline);
 		if (args == NULL) {
-			DEBUG ("%s: unicode conversion returned NULL", __func__);
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: unicode conversion returned NULL", __func__);
 
 			SetLastError (ERROR_PATH_NOT_FOUND);
 			goto free_strings;
@@ -621,7 +624,7 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 	if (cwd != NULL) {
 		dir = mono_unicode_to_external (cwd);
 		if (dir == NULL) {
-			DEBUG ("%s: unicode conversion returned NULL", __func__);
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: unicode conversion returned NULL", __func__);
 
 			SetLastError (ERROR_PATH_NOT_FOUND);
 			goto free_strings;
@@ -651,7 +654,7 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 
 			/* Executable existing ? */
 			if (!is_readable_or_executable (prog)) {
-				DEBUG ("%s: Couldn't find executable %s",
+				MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Couldn't find executable %s",
 					   __func__, prog);
 				g_free (unquoted);
 				SetLastError (ERROR_FILE_NOT_FOUND);
@@ -668,7 +671,7 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 
 			/* And make sure it's readable */
 			if (!is_readable_or_executable (prog)) {
-				DEBUG ("%s: Couldn't find executable %s",
+				MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Couldn't find executable %s",
 					   __func__, prog);
 				g_free (unquoted);
 				SetLastError (ERROR_FILE_NOT_FOUND);
@@ -733,7 +736,7 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 		
 		if (token == NULL) {
 			/* Give up */
-			DEBUG ("%s: Couldn't find what to exec", __func__);
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Couldn't find what to exec", __func__);
 
 			SetLastError (ERROR_PATH_NOT_FOUND);
 			goto free_strings;
@@ -759,7 +762,7 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 			
 			/* Executable existing ? */
 			if (!is_readable_or_executable (prog)) {
-				DEBUG ("%s: Couldn't find executable %s",
+				MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Couldn't find executable %s",
 					   __func__, token);
 				g_free (token);
 				SetLastError (ERROR_FILE_NOT_FOUND);
@@ -786,7 +789,7 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 				g_free (prog);
 				prog = g_find_program_in_path (token);
 				if (prog == NULL) {
-					DEBUG ("%s: Couldn't find executable %s", __func__, token);
+					MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Couldn't find executable %s", __func__, token);
 
 					g_free (token);
 					SetLastError (ERROR_FILE_NOT_FOUND);
@@ -798,7 +801,7 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 		g_free (token);
 	}
 
-	DEBUG ("%s: Exec prog [%s] args [%s]", __func__, prog,
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Exec prog [%s] args [%s]", __func__, prog,
 		   args_after_prog);
 	
 	/* Check for CLR binaries; if found, we will try to invoke
@@ -815,11 +818,11 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 
 		if (newapp != NULL) {
 			if (appname != NULL) {
-				newcmd = utf16_concat (newapp, utf16_space,
+				newcmd = utf16_concat (utf16_quote, newapp, utf16_quote, utf16_space,
 						       appname, utf16_space,
 						       cmdline, NULL);
 			} else {
-				newcmd = utf16_concat (newapp, utf16_space,
+				newcmd = utf16_concat (utf16_quote, newapp, utf16_quote, utf16_space,
 						       cmdline, NULL);
 			}
 			
@@ -841,7 +844,7 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 		}
 	} else {
 		if (!is_executable (prog)) {
-			DEBUG ("%s: Executable permisson not set on %s", __func__, prog);
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Executable permisson not set on %s", __func__, prog);
 			SetLastError (ERROR_ACCESS_DENIED);
 			goto free_strings;
 		}
@@ -952,7 +955,7 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 		/* Could not create the pipe to synchroniz process startup. We'll just not synchronize.
 		 * This is just for a very hard to hit race condition in the first place */
 		startup_pipe [0] = startup_pipe [1] = -1;
-		DEBUG ("%s: new process startup not synchronized. We may not notice if the newly created process exits immediately.", __func__);
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: new process startup not synchronized. We may not notice if the newly created process exits immediately.", __func__);
 	}
 
 	thr_ret = _wapi_handle_lock_shared_handles ();
@@ -971,7 +974,7 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 		if (startup_pipe [0] != -1) {
 			/* Wait until the parent has updated it's internal data */
 			ssize_t _i G_GNUC_UNUSED = read (startup_pipe [0], &dummy, 1);
-			DEBUG ("%s: child: parent has completed its setup", __func__);
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: child: parent has completed its setup", __func__);
 			close (startup_pipe [0]);
 			close (startup_pipe [1]);
 		}
@@ -992,7 +995,7 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 			close (i);
 
 #ifdef DEBUG_ENABLED
-		DEBUG ("%s: exec()ing [%s] in dir [%s]", __func__, cmd,
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: exec()ing [%s] in dir [%s]", __func__, cmd,
 			   dir == NULL?".":dir);
 		for (i = 0; argv[i] != NULL; i++)
 			g_message ("arg %d: [%s]", i, argv[i]);
@@ -1029,7 +1032,7 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 	mono_process = (struct MonoProcess *) g_malloc0 (sizeof (struct MonoProcess));
 	mono_process->pid = pid;
 	mono_process->handle_count = 1;
-	if (MONO_SEM_INIT (&mono_process->exit_sem, 0) != 0) {
+	if (mono_os_sem_init (&mono_process->exit_sem, 0) != 0) {
 		/* If we can't create the exit semaphore, we just don't add anything
 		 * to our list of mono processes. Waiting on the process will return 
 		 * immediately. */
@@ -1043,10 +1046,10 @@ gboolean CreateProcess (const gunichar2 *appname, const gunichar2 *cmdline,
 
 		process_handle_data->mono_process = mono_process;
 
-		mono_mutex_lock (&mono_processes_mutex);
+		mono_os_mutex_lock (&mono_processes_mutex);
 		mono_process->next = mono_processes;
 		mono_processes = mono_process;
-		mono_mutex_unlock (&mono_processes_mutex);
+		mono_os_mutex_unlock (&mono_processes_mutex);
 	}
 	
 	if (process_info != NULL) {
@@ -1089,13 +1092,16 @@ free_strings:
 	if (argv)
 		g_strfreev (argv);
 	
-	DEBUG ("%s: returning handle %p for pid %d", __func__, handle,
-		   pid);
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: returning handle %p for pid %d", __func__, handle, pid);
 
 	/* Check if something needs to be cleaned up. */
 	mono_processes_cleanup ();
 	
 	return ret;
+#else
+	SetLastError (ERROR_NOT_SUPPORTED);
+	return FALSE;
+#endif // defined (HAVE_FORK) && defined (HAVE_EXECVE)
 }
 		
 static void
@@ -1106,7 +1112,7 @@ process_set_name (WapiHandle_process *process_handle)
 	progname = g_get_prgname ();
 	utf8_progname = mono_utf8_from_external (progname);
 
-	DEBUG ("%s: using [%s] as prog name", __func__, progname);
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: using [%s] as prog name", __func__, progname);
 
 	if (utf8_progname) {
 		slash = strrchr (utf8_progname, '/');
@@ -1125,8 +1131,7 @@ wapi_processes_init (void)
 	WapiHandle_process process_handle = {0};
 
 	_wapi_handle_register_capabilities (WAPI_HANDLE_PROCESS,
-					    WAPI_HANDLE_CAP_WAIT |
-					    WAPI_HANDLE_CAP_SPECIAL_WAIT);
+		(WapiHandleCapability)(WAPI_HANDLE_CAP_WAIT | WAPI_HANDLE_CAP_SPECIAL_WAIT));
 	
 	process_handle.id = pid;
 
@@ -1136,6 +1141,8 @@ wapi_processes_init (void)
 	current_process = _wapi_handle_new (WAPI_HANDLE_PROCESS,
 					    &process_handle);
 	g_assert (current_process);
+
+	mono_os_mutex_init (&mono_processes_mutex);
 }
 
 gpointer
@@ -1183,7 +1190,7 @@ process_open_compare (gpointer handle, gpointer user_data)
 	process_handle = lookup_process_handle (handle);
 	g_assert (process_handle);
 	
-	DEBUG ("%s: looking at process %d", __func__, process_handle->id);
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: looking at process %d", __func__, process_handle->id);
 
 	checking_pid = process_handle->id;
 
@@ -1225,7 +1232,7 @@ OpenProcess (guint32 req_access G_GNUC_UNUSED, gboolean inherit G_GNUC_UNUSED, g
 	/* Find the process handle that corresponds to pid */
 	gpointer handle = NULL;
 	
-	DEBUG ("%s: looking for process %d", __func__, pid);
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: looking for process %d", __func__, pid);
 
 	handle = _wapi_search_handle (WAPI_HANDLE_PROCESS,
 				      process_open_compare,
@@ -1237,7 +1244,7 @@ OpenProcess (guint32 req_access G_GNUC_UNUSED, gboolean inherit G_GNUC_UNUSED, g
 			 */
 			return WAPI_PID_TO_HANDLE (pid);
 		} else {
-			DEBUG ("%s: Can't find pid %d", __func__, pid);
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Can't find pid %d", __func__, pid);
 
 			SetLastError (ERROR_PROC_NOT_FOUND);
 	
@@ -1273,7 +1280,7 @@ GetExitCodeProcess (gpointer process, guint32 *code)
 
 	process_handle = lookup_process_handle (process);
 	if (!process_handle) {
-		DEBUG ("%s: Can't find process %p", __func__, process);
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Can't find process %p", __func__, process);
 		
 		return FALSE;
 	}
@@ -1327,7 +1334,7 @@ GetProcessTimes (gpointer process, WapiFileTime *create_time,
 
 	process_handle = lookup_process_handle (process);
 	if (!process_handle) {
-		DEBUG ("%s: Can't find process %p", __func__, process);
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Can't find process %p", __func__, process);
 		
 		return FALSE;
 	}
@@ -1492,7 +1499,7 @@ static GSList *load_modules (void)
 		mod->inode = i;
 		mod->filename = g_strdup (info->dlpi_name); 
 
-		DEBUG ("%s: inode=%d, filename=%s, address_start=%p, address_end=%p", __func__,
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: inode=%d, filename=%s, address_start=%p, address_end=%p", __func__,
 				   mod->inode, mod->filename, mod->address_start, mod->address_end);
 
 		free(info);
@@ -1673,7 +1680,7 @@ static gboolean match_procname_to_modulename (char *procname, char *modulename)
 	if (procname == NULL || modulename == NULL)
 		return (FALSE);
 
-	DEBUG ("%s: procname=\"%s\", modulename=\"%s\"", __func__, procname, modulename);
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: procname=\"%s\", modulename=\"%s\"", __func__, procname, modulename);
 	pname = mono_path_resolve_symlinks (procname);
 	mname = mono_path_resolve_symlinks (modulename);
 
@@ -1702,7 +1709,7 @@ static gboolean match_procname_to_modulename (char *procname, char *modulename)
 	g_free (pname);
 	g_free (mname);
 
-	DEBUG ("%s: result is %d", __func__, result);
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: result is %d", __func__, result);
 	return result;
 }
 
@@ -1763,12 +1770,12 @@ gboolean EnumProcessModules (gpointer process, gpointer *modules,
 	} else {
 		process_handle = lookup_process_handle (process);
 		if (!process_handle) {
-			DEBUG ("%s: Can't find process %p", __func__, process);
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Can't find process %p", __func__, process);
 		
 			return FALSE;
 		}
 		pid = process_handle->id;
-		proc_name = process_handle->proc_name;
+		proc_name = g_strdup (process_handle->proc_name);
 	}
 	
 #if defined(PLATFORM_MACOSX) || defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__HAIKU__)
@@ -1786,6 +1793,7 @@ gboolean EnumProcessModules (gpointer process, gpointer *modules,
 		 */
 		modules[0] = NULL;
 		*needed = sizeof(gpointer);
+		g_free (proc_name);
 		return TRUE;
 	}
 	mods = load_modules (fp);
@@ -1816,10 +1824,11 @@ gboolean EnumProcessModules (gpointer process, gpointer *modules,
 	}
 		
 	for (i = 0; i < count; i++) {
-		free_procmodule (g_slist_nth_data (mods, i));
+		free_procmodule ((WapiProcModule *)g_slist_nth_data (mods, i));
 	}
 	g_slist_free (mods);
-
+	g_free (proc_name);
+	
 	return TRUE;
 }
 
@@ -1898,7 +1907,7 @@ get_process_name_from_proc (pid_t pid)
 	if (sysctl (mib, 4, pi, &size, NULL, 0) < 0) {
 		if (errno == ENOMEM) {
 			free(pi);
-			DEBUG ("%s: Didn't allocate enough memory for kproc info", __func__);
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Didn't allocate enough memory for kproc info", __func__);
 		}
 		return(ret);
 	}
@@ -1914,7 +1923,7 @@ get_process_name_from_proc (pid_t pid)
 	mib [2] = KERN_PROC_PID;
 	mib [3] = pid;
 	if (sysctl(mib, 4, NULL, &size, NULL, 0) < 0) {
-		DEBUG ("%s: sysctl() failed: %d", __func__, errno);
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: sysctl() failed: %d", __func__, errno);
 		return(ret);
 	}
 
@@ -1924,7 +1933,7 @@ get_process_name_from_proc (pid_t pid)
 	if (sysctl (mib, 4, pi, &size, NULL, 0) < 0) {
 		if (errno == ENOMEM) {
 			free(pi);
-			DEBUG ("%s: Didn't allocate enough memory for kproc info", __func__);
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Didn't allocate enough memory for kproc info", __func__);
 		}
 		return(ret);
 	}
@@ -1942,7 +1951,7 @@ get_process_name_from_proc (pid_t pid)
 
 retry:
 	if (sysctl(mib, 6, NULL, &size, NULL, 0) < 0) {
-		DEBUG ("%s: sysctl() failed: %d", __func__, errno);
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: sysctl() failed: %d", __func__, errno);
 		return(ret);
 	}
 
@@ -2082,7 +2091,7 @@ get_module_name (gpointer process, gpointer module,
 	int i;
 	char *proc_name = NULL;
 	
-	DEBUG ("%s: Getting module base name, process handle %p module %p",
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Getting module base name, process handle %p module %p",
 		   __func__, process, module);
 
 	size = size * sizeof (gunichar2); /* adjust for unicode characters */
@@ -2097,7 +2106,7 @@ get_module_name (gpointer process, gpointer module,
 	} else {
 		process_handle = lookup_process_handle (process);
 		if (!process_handle) {
-			DEBUG ("%s: Can't find process %p", __func__,
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Can't find process %p", __func__,
 				   process);
 			
 			return 0;
@@ -2158,7 +2167,7 @@ get_module_name (gpointer process, gpointer module,
 	g_free (proc_name);
 
 	if (procname_ext) {
-		DEBUG ("%s: Process name is [%s]", __func__,
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Process name is [%s]", __func__,
 			   procname_ext);
 
 		procname = mono_unicode_from_external (procname_ext, &bytes);
@@ -2174,11 +2183,11 @@ get_module_name (gpointer process, gpointer module,
 		bytes += 2;
 		
 		if (size < bytes) {
-			DEBUG ("%s: Size %d smaller than needed (%ld); truncating", __func__, size, bytes);
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Size %d smaller than needed (%ld); truncating", __func__, size, bytes);
 
 			memcpy (basename, procname, size);
 		} else {
-			DEBUG ("%s: Size %d larger than needed (%ld)",
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Size %d larger than needed (%ld)",
 				   __func__, size, bytes);
 
 			memcpy (basename, procname, bytes);
@@ -2225,11 +2234,11 @@ get_module_filename (gpointer process, gpointer module,
 	bytes += 2;
 
 	if (size < bytes) {
-		DEBUG ("%s: Size %d smaller than needed (%ld); truncating", __func__, size, bytes);
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Size %d smaller than needed (%ld); truncating", __func__, size, bytes);
 
 		memcpy (basename, proc_path, size);
 	} else {
-		DEBUG ("%s: Size %d larger than needed (%ld)",
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Size %d larger than needed (%ld)",
 			   __func__, size, bytes);
 
 		memcpy (basename, proc_path, bytes);
@@ -2270,7 +2279,7 @@ GetModuleInformation (gpointer process, gpointer module,
 	gboolean ret = FALSE;
 	char *proc_name = NULL;
 	
-	DEBUG ("%s: Getting module info, process handle %p module %p",
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Getting module info, process handle %p module %p",
 		   __func__, process, module);
 
 	if (modinfo == NULL || size < sizeof (WapiModuleInfo))
@@ -2282,7 +2291,7 @@ GetModuleInformation (gpointer process, gpointer module,
 	} else {
 		process_handle = lookup_process_handle (process);
 		if (!process_handle) {
-			DEBUG ("%s: Can't find process %p", __func__,
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Can't find process %p", __func__,
 				   process);
 			
 			return FALSE;
@@ -2346,7 +2355,7 @@ GetProcessWorkingSetSize (gpointer process, size_t *min, size_t *max)
 	
 	process_handle = lookup_process_handle (process);
 	if (!process_handle) {
-		DEBUG ("%s: Can't find process %p", __func__, process);
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Can't find process %p", __func__, process);
 		
 		return FALSE;
 	}
@@ -2369,7 +2378,7 @@ SetProcessWorkingSetSize (gpointer process, size_t min, size_t max)
 
 	process_handle = lookup_process_handle (process);
 	if (!process_handle) {
-		DEBUG ("%s: Can't find process %p", __func__, process);
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Can't find process %p", __func__, process);
 		
 		return FALSE;
 	}
@@ -2384,6 +2393,7 @@ SetProcessWorkingSetSize (gpointer process, size_t min, size_t max)
 gboolean
 TerminateProcess (gpointer process, gint32 exitCode)
 {
+#if defined(HAVE_KILL)
 	WapiHandle_process *process_handle;
 	int signo;
 	int ret;
@@ -2395,7 +2405,7 @@ TerminateProcess (gpointer process, gint32 exitCode)
 	} else {
 		process_handle = lookup_process_handle (process);
 		if (!process_handle) {
-			DEBUG ("%s: Can't find process %p", __func__, process);
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s: Can't find process %p", __func__, process);
 			SetLastError (ERROR_INVALID_HANDLE);
 			return FALSE;
 		}
@@ -2421,6 +2431,10 @@ TerminateProcess (gpointer process, gint32 exitCode)
 	}
 	
 	return (ret == 0);
+#else
+	g_error ("kill() is not supported by this platform");
+	return FALSE;
+#endif
 }
 
 guint32
@@ -2556,7 +2570,7 @@ mono_processes_cleanup (void)
 	GSList *l;
 	gpointer unref_handle;
 
-	DEBUG ("%s", __func__);
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s", __func__);
 
 	/* Ensure we're not in here in multiple threads at once, nor recursive. */
 	if (InterlockedCompareExchange (&mono_processes_cleaning_up, 1, 0) != 0)
@@ -2566,10 +2580,10 @@ mono_processes_cleanup (void)
 		if (mp->pid == 0 && mp->handle) {
 			/* This process has exited and we need to remove the artifical ref
 			 * on the handle */
-			mono_mutex_lock (&mono_processes_mutex);
+			mono_os_mutex_lock (&mono_processes_mutex);
 			unref_handle = mp->handle;
 			mp->handle = NULL;
-			mono_mutex_unlock (&mono_processes_mutex);
+			mono_os_mutex_unlock (&mono_processes_mutex);
 			if (unref_handle)
 				_wapi_handle_unref (unref_handle);
 		}
@@ -2581,7 +2595,7 @@ mono_processes_cleanup (void)
 	 * asynchronously. The handler requires that the mono_processes list
 	 * remain valid.
 	 */
-	mono_mutex_lock (&mono_processes_mutex);
+	mono_os_mutex_lock (&mono_processes_mutex);
 
 	mp = mono_processes;
 	while (mp) {
@@ -2612,15 +2626,15 @@ mono_processes_cleanup (void)
 		 * they have the 'finished' flag set, which means the sigchld handler is done
 		 * accessing them.
 		 */
-		mp = l->data;
-		MONO_SEM_DESTROY (&mp->exit_sem);
+		mp = (MonoProcess *)l->data;
+		mono_os_sem_destroy (&mp->exit_sem);
 		g_free (mp);
 	}
 	g_slist_free (finished);
 
-	mono_mutex_unlock (&mono_processes_mutex);
+	mono_os_mutex_unlock (&mono_processes_mutex);
 
-	DEBUG ("%s done", __func__);
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s done", __func__);
 
 	InterlockedDecrement (&mono_processes_cleaning_up);
 }
@@ -2630,7 +2644,7 @@ process_close (gpointer handle, gpointer data)
 {
 	WapiHandle_process *process_handle;
 
-	DEBUG ("%s", __func__);
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s", __func__);
 
 	process_handle = (WapiHandle_process *) data;
 	g_free (process_handle->proc_name);
@@ -2647,7 +2661,7 @@ MONO_SIGNAL_HANDLER_FUNC (static, mono_sigchld_signal_handler, (int _dummy, sigi
 	int pid;
 	struct MonoProcess *p;
 
-	DEBUG ("SIG CHILD handler for pid: %i\n", info->si_pid);
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "SIG CHILD handler for pid: %i\n", info->si_pid);
 
 	do {
 		do {
@@ -2657,7 +2671,7 @@ MONO_SIGNAL_HANDLER_FUNC (static, mono_sigchld_signal_handler, (int _dummy, sigi
 		if (pid <= 0)
 			break;
 
-		DEBUG ("child ended: %i", pid);
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "child ended: %i", pid);
 
 		/*
 		 * This can run concurrently with the code in the rest of this module.
@@ -2670,14 +2684,14 @@ MONO_SIGNAL_HANDLER_FUNC (static, mono_sigchld_signal_handler, (int _dummy, sigi
 		if (p) {
 			p->pid = 0; /* this pid doesn't exist anymore, clear it */
 			p->status = status;
-			MONO_SEM_POST (&p->exit_sem);
+			mono_os_sem_post (&p->exit_sem);
 			mono_memory_barrier ();
 			/* Mark this as freeable, the pointer becomes invalid afterwards */
 			p->freeable = TRUE;
 		}
 	} while (1);
 
-	DEBUG ("SIG CHILD handler: done looping.");
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "SIG CHILD handler: done looping.");
 }
 
 #endif
@@ -2692,7 +2706,7 @@ process_add_sigchld_handler (void)
 	sigemptyset (&sa.sa_mask);
 	sa.sa_flags = SA_NOCLDSTOP | SA_SIGINFO;
 	g_assert (sigaction (SIGCHLD, &sa, &previous_chld_sa) != -1);
-	DEBUG ("Added SIGCHLD handler");
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "Added SIGCHLD handler");
 #endif
 }
 
@@ -2710,7 +2724,7 @@ process_wait (gpointer handle, guint32 timeout, gboolean alertable)
 	 * but WaitFor*Object won't call us for pseudo handles. */
 	g_assert ((GPOINTER_TO_UINT (handle) & _WAPI_PROCESS_UNHANDLED) != _WAPI_PROCESS_UNHANDLED);
 
-	DEBUG ("%s (%p, %u)", __func__, handle, timeout);
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s (%p, %u)", __func__, handle, timeout);
 
 	process_handle = lookup_process_handle (handle);
 	if (!process_handle) {
@@ -2720,13 +2734,13 @@ process_wait (gpointer handle, guint32 timeout, gboolean alertable)
 
 	if (process_handle->exited) {
 		/* We've already done this one */
-		DEBUG ("%s (%p, %u): Process already exited", __func__, handle, timeout);
+		MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s (%p, %u): Process already exited", __func__, handle, timeout);
 		return WAIT_OBJECT_0;
 	}
 
 	pid = process_handle->id;
 
-	DEBUG ("%s (%p, %u): PID: %d", __func__, handle, timeout, pid);
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s (%p, %u): PID: %d", __func__, handle, timeout, pid);
 
 	/* We don't need to lock mono_processes here, the entry
 	 * has a handle_count > 0 which means it will not be freed. */
@@ -2738,47 +2752,46 @@ process_wait (gpointer handle, guint32 timeout, gboolean alertable)
 
 	while (1) {
 		if (timeout != INFINITE) {
-			DEBUG ("%s (%p, %u): waiting on semaphore for %li ms...", 
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s (%p, %u): waiting on semaphore for %li ms...", 
 				   __func__, handle, timeout, (timeout - (now - start)));
-
-			ret = MONO_SEM_TIMEDWAIT_ALERTABLE (&mp->exit_sem, (timeout - (now - start)), alertable);
+			ret = mono_os_sem_timedwait (&mp->exit_sem, (timeout - (now - start)), alertable ? MONO_SEM_FLAGS_ALERTABLE : MONO_SEM_FLAGS_NONE);
 		} else {
-			DEBUG ("%s (%p, %u): waiting on semaphore forever...", 
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s (%p, %u): waiting on semaphore forever...", 
 				   __func__, handle, timeout);
-			ret = MONO_SEM_WAIT_ALERTABLE (&mp->exit_sem, alertable);
+			ret = mono_os_sem_wait (&mp->exit_sem, alertable ? MONO_SEM_FLAGS_ALERTABLE : MONO_SEM_FLAGS_NONE);
 		}
 
 		if (ret == -1 && errno != EINTR && errno != ETIMEDOUT) {
-			DEBUG ("%s (%p, %u): sem_timedwait failure: %s", 
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s (%p, %u): sem_timedwait failure: %s", 
 				   __func__, handle, timeout, g_strerror (errno));
 			/* Should we return a failure here? */
 		}
 
 		if (ret == 0) {
 			/* Success, process has exited */
-			MONO_SEM_POST (&mp->exit_sem);
+			mono_os_sem_post (&mp->exit_sem);
 			break;
 		}
 
 		if (timeout == 0) {
-			DEBUG ("%s (%p, %u): WAIT_TIMEOUT (timeout = 0)", __func__, handle, timeout);
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s (%p, %u): WAIT_TIMEOUT (timeout = 0)", __func__, handle, timeout);
 			return WAIT_TIMEOUT;
 		}
 
 		now = mono_msec_ticks ();
 		if (now - start >= timeout) {
-			DEBUG ("%s (%p, %u): WAIT_TIMEOUT", __func__, handle, timeout);
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s (%p, %u): WAIT_TIMEOUT", __func__, handle, timeout);
 			return WAIT_TIMEOUT;
 		}
 		
 		if (alertable && _wapi_thread_cur_apc_pending ()) {
-			DEBUG ("%s (%p, %u): WAIT_IO_COMPLETION", __func__, handle, timeout);
+			MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s (%p, %u): WAIT_IO_COMPLETION", __func__, handle, timeout);
 			return WAIT_IO_COMPLETION;
 		}
 	}
 
 	/* Process must have exited */
-	DEBUG ("%s (%p, %u): Waited successfully", __func__, handle, timeout);
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s (%p, %u): Waited successfully", __func__, handle, timeout);
 
 	ret = _wapi_handle_lock_shared_handles ();
 	g_assert (ret == 0);
@@ -2792,7 +2805,7 @@ process_wait (gpointer handle, guint32 timeout, gboolean alertable)
 
 	process_handle->exited = TRUE;
 
-	DEBUG ("%s (%p, %u): Setting pid %d signalled, exit status %d",
+	MONO_TRACE (G_LOG_LEVEL_DEBUG, MONO_TRACE_IO_LAYER, "%s (%p, %u): Setting pid %d signalled, exit status %d",
 		   __func__, handle, timeout, process_handle->id, process_handle->exitstatus);
 
 	_wapi_handle_set_signal_state (handle, TRUE, TRUE);
