@@ -11,6 +11,7 @@
 #include <mono/metadata/object-internals.h>
 #include <mono/metadata/class-internals.h>
 #include <mono/metadata/domain-internals.h>
+#include <mono/metadata/gc-internals.h>
 
 #include <mono/metadata/console-io.h>
 #include <mono/metadata/exception.h>
@@ -20,11 +21,14 @@ mono_console_init (void)
 {
 }
 
+void
+mono_console_handle_async_ops (void)
+{
+}
+
 MonoBoolean
 ves_icall_System_ConsoleDriver_Isatty (HANDLE handle)
 {
-	MONO_ARCH_SAVE_REGS;
-
 	return (GetFileType (handle) == FILE_TYPE_CHAR);
 }
 
