@@ -9,13 +9,7 @@
 
 #include <config.h>
 
-#ifdef PLATFORM_WIN32
-
-#if _WIN32_WINNT < 0x0501
-/* Required for ACTCTX. */
-#undef _WIN32_WINNT
-#define _WIN32_WINNT 0x0501
-#endif /* _WIN32_WINNT < 0x0501 */
+#ifdef HOST_WIN32
 
 #include <string.h>
 #include <glib.h>
@@ -33,6 +27,8 @@
 #include "environment.h"
 #include "coree.h"
 
+#include <shellapi.h>
+
 HMODULE coree_module_handle = NULL;
 
 static gboolean init_from_coree = FALSE;
@@ -226,10 +222,12 @@ STDAPI _CorValidateImage(PVOID *ImageBase, LPCWSTR FileName)
 {
 	IMAGE_DOS_HEADER* DosHeader;
 	IMAGE_NT_HEADERS32* NtHeaders32;
-	IMAGE_NT_HEADERS64* NtHeaders64;
 	IMAGE_DATA_DIRECTORY* CliHeaderDir;
+#ifdef _WIN64
+	IMAGE_NT_HEADERS64* NtHeaders64;
 	MonoCLIHeader* CliHeader;
 	DWORD SizeOfHeaders;
+#endif
 	DWORD* Address;
 	DWORD OldProtect;
 
@@ -410,7 +408,9 @@ HMODULE WINAPI MonoLoadImage(LPCWSTR FileName)
 	HANDLE MapHandle;
 	IMAGE_DOS_HEADER* DosHeader;
 	IMAGE_NT_HEADERS32* NtHeaders32;
+#ifdef _WIN64
 	IMAGE_NT_HEADERS64* NtHeaders64;
+#endif
 	HMODULE ModuleHandle;
 
 	FileHandle = CreateFile(FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
@@ -458,7 +458,9 @@ InvalidImageFormat:
 		goto UnmapView;
 	}
 
+#ifdef _WIN64
 ValidImage:
+#endif
 	UnmapViewOfFile(DosHeader);
 	CloseHandle(MapHandle);
 
@@ -923,4 +925,4 @@ mono_fixup_exe_image (MonoImage* image)
 		MonoFixupExe ((HMODULE) image->raw_data);
 }
 
-#endif /* PLATFORM_WIN32 */
+#endif /* HOST_WIN32 */
