@@ -1,5 +1,5 @@
 /*
- * console-io.c: ConsoleDriver internal calls
+ * debug-helpers.c:
  *
  * Author:
  *	Mono Project (http://www.mono-project.com)
@@ -12,13 +12,14 @@
 #include "mono/metadata/opcodes.h"
 #include "mono/metadata/metadata-internals.h"
 #include "mono/metadata/class-internals.h"
+#include "mono/metadata/object-internals.h"
 #include "mono/metadata/mono-endian.h"
 #include "mono/metadata/debug-helpers.h"
 #include "mono/metadata/tabledefs.h"
 #include "mono/metadata/appdomain.h"
 
 struct MonoMethodDesc {
-	char *namespace;
+	char *name_space;
 	char *klass;
 	char *name;
 	char *args;
@@ -71,19 +72,21 @@ wrapper_type_to_str (guint32 wrapper_type)
 #endif
 
 static void
-append_class_name (GString *res, MonoClass *class, gboolean include_namespace)
+append_class_name (GString *res, MonoClass *klass, gboolean include_namespace)
 {
-	if (!class) {
+	if (!klass) {
 		g_string_append (res, "Unknown");
 		return;
 	}
-	if (class->nested_in) {
-		append_class_name (res, class->nested_in, include_namespace);
+	if (klass->nested_in) {
+		append_class_name (res, klass->nested_in, include_namespace);
 		g_string_append_c (res, '/');
 	}
-	if (include_namespace && *(class->name_space))
-		g_string_append_printf (res, "%s.", class->name_space);
-	g_string_append_printf (res, "%s", class->name);
+	if (include_namespace && *(klass->name_space)) {
+		g_string_append (res, klass->name_space);
+		g_string_append_c (res, '.');
+	}
+	g_string_append (res, klass->name);
 }
 
 static MonoClass*
@@ -197,15 +200,18 @@ mono_type_get_desc (GString *res, MonoType *type, gboolean include_namespace)
 	case MONO_TYPE_VAR:
 	case MONO_TYPE_MVAR:
 		if (type->data.generic_param) {
-			MonoGenericParamInfo *info = mono_generic_param_info (type->data.generic_param);
-			if (info)
-				g_string_append (res, info->name);
+			const char *name = mono_generic_param_name (type->data.generic_param);
+			if (name)
+				g_string_append (res, name);
 			else
 				g_string_append_printf (res, "%s%d", type->type == MONO_TYPE_VAR ? "!" : "!!", mono_generic_param_num (type->data.generic_param));
 		} else {
 			g_string_append (res, "<unknown>");
 		}
 		break;
+	case MONO_TYPE_TYPEDBYREF:
+		g_string_append (res, "typedbyref");
+		break;
 	default:
 		break;
 	}
@@ -245,6 +251,31 @@ mono_signature_get_desc (MonoMethodSignature *sig, gboolean include_namespace)
 	return result;
 }
 
+char*
+mono_signature_full_name (MonoMethodSignature *sig)
+{
+	int i;
+	char *result;
+	GString *res;
+
+	if (!sig)
+		return g_strdup ("<invalid signature>");
+
+	res = g_string_new ("");
+
+	mono_type_get_desc (res, sig->ret, TRUE);
+	g_string_append_c (res, '(');
+	for (i = 0; i < sig->param_count; ++i) {
+		if (i > 0)
+			g_string_append_c (res, ',');
+		mono_type_get_desc (res, sig->params [i], TRUE);
+	}
+	g_string_append_c (res, ')');
+	result = res->str;
+	g_string_free (res, FALSE);
+	return result;
+}
+
 static void
 ginst_get_desc (GString *str, MonoGenericInst *ginst)
 {
@@ -322,10 +353,10 @@ mono_method_desc_new (const char *name, gboolean include_namespace)
 		g_free (class_nspace);
 		return NULL;
 	}
-	*method_name++ = 0;
 	/* allow two :: to separate the method name */
-	if (*method_name == ':')
-		method_name++;
+	if (method_name != class_nspace && method_name [-1] == ':')
+		method_name [-1] = 0;
+	*method_name++ = 0;
 	class_name = strrchr (class_nspace, '.');
 	if (class_name) {
 		*class_name++ = 0;
@@ -338,7 +369,7 @@ mono_method_desc_new (const char *name, gboolean include_namespace)
 	result->include_namespace = include_namespace;
 	result->name = method_name;
 	result->klass = class_name;
-	result->namespace = use_namespace? class_nspace: NULL;
+	result->name_space = use_namespace? class_nspace: NULL;
 	result->args = use_args? use_args: NULL;
 	if (strstr (result->name, "*"))
 		result->name_glob = TRUE;
@@ -367,7 +398,7 @@ mono_method_desc_from_method (MonoMethod *method)
 	result->include_namespace = TRUE;
 	result->name = g_strdup (method->name);
 	result->klass = g_strdup (method->klass->name);
-	result->namespace = g_strdup (method->klass->name_space);
+	result->name_space = g_strdup (method->klass->name_space);
 
 	return result;
 }
@@ -381,8 +412,8 @@ mono_method_desc_from_method (MonoMethod *method)
 void
 mono_method_desc_free (MonoMethodDesc *desc)
 {
-	if (desc->namespace)
-		g_free (desc->namespace);
+	if (desc->name_space)
+		g_free (desc->name_space);
 	else if (desc->klass)
 		g_free (desc->klass);
 	g_free (desc);
@@ -448,7 +479,7 @@ match_class (MonoMethodDesc *desc, int pos, MonoClass *klass)
 	if (!p) {
 		if (strncmp (desc->klass, klass->name, pos))
 			return FALSE;
-		if (desc->namespace && strcmp (desc->namespace, klass->name_space))
+		if (desc->name_space && strcmp (desc->name_space, klass->name_space))
 			return FALSE;
 		return TRUE;
 	}
@@ -464,6 +495,8 @@ match_class (MonoMethodDesc *desc, int pos, MonoClass *klass)
 gboolean
 mono_method_desc_full_match (MonoMethodDesc *desc, MonoMethod *method)
 {
+	if (!desc->klass)
+		return FALSE;
 	if (!match_class (desc, strlen (desc->klass), method->klass))
 		return FALSE;
 
@@ -486,26 +519,26 @@ MonoMethod*
 mono_method_desc_search_in_image (MonoMethodDesc *desc, MonoImage *image)
 {
 	MonoClass *klass;
-	const MonoTableInfo *tdef;
 	const MonoTableInfo *methods;
 	MonoMethod *method;
 	int i;
 
 	/* Handle short names for system classes */
-	if (!desc->namespace && image == mono_defaults.corlib) {
+	if (!desc->name_space && image == mono_defaults.corlib) {
 		klass = find_system_class (desc->klass);
 		if (klass)
 			return mono_method_desc_search_in_class (desc, klass);
 	}
 
-	if (desc->namespace && desc->klass) {
-		klass = mono_class_from_name (image, desc->namespace, desc->klass);
+	if (desc->name_space && desc->klass) {
+		klass = mono_class_from_name (image, desc->name_space, desc->klass);
 		if (!klass)
 			return NULL;
 		return mono_method_desc_search_in_class (desc, klass);
 	}
 
-	tdef = mono_image_get_table_info (image, MONO_TABLE_TYPEDEF);
+	/* FIXME: Is this call necessary?  We don't use its result. */
+	mono_image_get_table_info (image, MONO_TABLE_TYPEDEF);
 	methods = mono_image_get_table_info (image, MONO_TABLE_METHOD);
 	for (i = 0; i < mono_table_info_get_rows (methods); ++i) {
 		guint32 token = mono_metadata_decode_row_col (methods, i, MONO_METHOD_NAME);
@@ -569,7 +602,7 @@ dis_one (GString *str, MonoDisHelper *dh, MonoMethod *method, const unsigned cha
 		size_t len2;
 		char *blob2 = NULL;
 
-		if (!method->klass->image->dynamic) {
+		if (!image_is_dynamic (method->klass->image) && !method_is_dynamic (method)) {
 			token = read32 (ip);
 			blob = mono_metadata_user_string (method->klass->image, mono_metadata_token_index (token));
 
@@ -581,7 +614,7 @@ dis_one (GString *str, MonoDisHelper *dh, MonoMethod *method, const unsigned cha
 			blob2 = g_malloc ((len2 * 2) + 1);
 			memcpy (blob2, blob, len2 * 2);
 #else
-			blob2 = blob;
+			blob2 = (char*)blob;
 #endif
 
 #if G_BYTE_ORDER != G_LITTLE_ENDIAN
@@ -680,6 +713,7 @@ dis_one (GString *str, MonoDisHelper *dh, MonoMethod *method, const unsigned cha
 	if (dh->newline)
 		g_string_append (str, dh->newline);
 
+	mono_metadata_free_mh (header);
 	return ip;
 }
 
@@ -741,18 +775,29 @@ mono_field_full_name (MonoClassField *field)
 }
 
 char *
-mono_method_full_name (MonoMethod *method, gboolean signature)
+mono_method_get_name_full (MonoMethod *method, gboolean signature, gboolean ret, MonoTypeNameFormat format)
 {
 	char *res;
 	char wrapper [64];
-	char *klass_desc = mono_type_full_name (&method->klass->byval_arg);
+	char *klass_desc;
 	char *inst_desc = NULL;
 
+	if (format == MONO_TYPE_NAME_FORMAT_IL)
+		klass_desc = mono_type_full_name (&method->klass->byval_arg);
+	else
+		klass_desc = mono_type_get_name_full (&method->klass->byval_arg, format);
+
 	if (method->is_inflated && ((MonoMethodInflated*)method)->context.method_inst) {
 		GString *str = g_string_new ("");
-		g_string_append (str, "<");
+		if (format == MONO_TYPE_NAME_FORMAT_IL)
+			g_string_append (str, "<");
+		else
+			g_string_append (str, "[");
 		ginst_get_desc (str, ((MonoMethodInflated*)method)->context.method_inst);
-		g_string_append (str, ">");
+		if (format == MONO_TYPE_NAME_FORMAT_IL)
+			g_string_append_c (str, '>');
+		else
+			g_string_append_c (str, ']');
 
 		inst_desc = str->str;
 		g_string_free (str, FALSE);
@@ -760,9 +805,15 @@ mono_method_full_name (MonoMethod *method, gboolean signature)
 		MonoGenericContainer *container = mono_method_get_generic_container (method);
 
 		GString *str = g_string_new ("");
-		g_string_append (str, "<");
+		if (format == MONO_TYPE_NAME_FORMAT_IL)
+			g_string_append (str, "<");
+		else
+			g_string_append (str, "[");
 		ginst_get_desc (str, container->context.method_inst);
-		g_string_append (str, ">");
+		if (format == MONO_TYPE_NAME_FORMAT_IL)
+			g_string_append_c (str, '>');
+		else
+			g_string_append_c (str, ']');
 
 		inst_desc = str->str;
 		g_string_free (str, FALSE);
@@ -780,8 +831,15 @@ mono_method_full_name (MonoMethod *method, gboolean signature)
 			sprintf (wrapper, "(wrapper %s) ", wrapper_type_to_str (method->wrapper_type));
 		else
 			strcpy (wrapper, "");
-		res = g_strdup_printf ("%s%s:%s%s (%s)", wrapper, klass_desc, 
-							   method->name, inst_desc ? inst_desc : "", tmpsig);
+		if (ret) {
+			char *ret_str = mono_type_full_name (mono_method_signature (method)->ret);
+			res = g_strdup_printf ("%s%s %s:%s%s (%s)", wrapper, ret_str, klass_desc,
+								   method->name, inst_desc ? inst_desc : "", tmpsig);
+			g_free (ret_str);
+		} else {
+			res = g_strdup_printf ("%s%s:%s%s (%s)", wrapper, klass_desc,
+								   method->name, inst_desc ? inst_desc : "", tmpsig);
+		}
 		g_free (tmpsig);
 	} else {
 		res = g_strdup_printf ("%s%s:%s%s", wrapper, klass_desc,
@@ -794,6 +852,18 @@ mono_method_full_name (MonoMethod *method, gboolean signature)
 	return res;
 }
 
+char *
+mono_method_full_name (MonoMethod *method, gboolean signature)
+{
+	return mono_method_get_name_full (method, signature, FALSE, MONO_TYPE_NAME_FORMAT_IL);
+}
+
+char *
+mono_method_get_full_name (MonoMethod *method)
+{
+	return mono_method_get_name_full (method, TRUE, TRUE, MONO_TYPE_NAME_FORMAT_IL);
+}
+
 static const char*
 print_name_space (MonoClass *klass)
 {
@@ -839,7 +909,7 @@ mono_object_describe (MonoObject *obj)
 		MonoArray *array = (MonoArray*)obj;
 		sep = print_name_space (klass);
 		g_print ("%s%s", sep, klass->name);
-		g_print (" at %p, rank: %d, length: %d\n", obj, klass->rank, mono_array_length (array));
+		g_print (" at %p, rank: %d, length: %d\n", obj, klass->rank, (int)mono_array_length (array));
 	} else {
 		sep = print_name_space (klass);
 		g_print ("%s%s", sep, klass->name);
@@ -924,23 +994,24 @@ print_field_value (const char *field_ptr, MonoClassField *field, int type_offset
 }
 
 static void
-objval_describe (MonoClass *class, const char *addr)
+objval_describe (MonoClass *klass, const char *addr)
 {
 	MonoClassField *field;
 	MonoClass *p;
 	const char *field_ptr;
 	gssize type_offset = 0;
-	if (class->valuetype)
+
+	if (klass->valuetype)
 		type_offset = -sizeof (MonoObject);
 
-	for (p = class; p != NULL; p = p->parent) {
+	for (p = klass; p != NULL; p = p->parent) {
 		gpointer iter = NULL;
 		int printed_header = FALSE;
 		while ((field = mono_class_get_fields (p, &iter))) {
 			if (field->type->attrs & (FIELD_ATTRIBUTE_STATIC | FIELD_ATTRIBUTE_HAS_FIELD_RVA))
 				continue;
 
-			if (p != class && !printed_header) {
+			if (p != klass && !printed_header) {
 				const char *sep;
 				g_print ("In class ");
 				sep = print_name_space (p);
@@ -963,8 +1034,8 @@ objval_describe (MonoClass *class, const char *addr)
 void
 mono_object_describe_fields (MonoObject *obj)
 {
-	MonoClass *class = mono_object_class (obj);
-	objval_describe (class, (char*)obj);
+	MonoClass *klass = mono_object_class (obj);
+	objval_describe (klass, (char*)obj);
 }
 
 /**
@@ -998,7 +1069,7 @@ mono_class_describe_statics (MonoClass* klass)
 
 	if (!vtable)
 		return;
-	if (!(addr = vtable->data))
+	if (!(addr = (const char *)mono_vtable_get_static_field_data (vtable)))
 		return;
 
 	for (p = klass; p != NULL; p = p->parent) {
@@ -1016,3 +1087,24 @@ mono_class_describe_statics (MonoClass* klass)
 	}
 }
 
+/**
+ * mono_print_method_code
+ * @MonoMethod: a pointer to the method
+ *
+ * This method is used from a debugger to print the code of the method.
+ *
+ * This prints the IL code of the method in the standard output.
+ */
+void
+mono_method_print_code (MonoMethod *method)
+{
+	char *code;
+	MonoMethodHeader *header = mono_method_get_header (method);
+	if (!header) {
+		printf ("METHOD HEADER NOT FOUND\n");
+		return;
+	}
+	code = mono_disasm_code (0, method, header->code, header->code + header->code_size);
+	printf ("CODE FOR %s:\n%s\n", mono_method_full_name (method, TRUE), code);
+	g_free (code);
+}
