@@ -7,6 +7,7 @@
  *
  * Copyright 2001-2003 Ximian, Inc (http://www.ximian.com)
  * Copyright 2004-2009 Novell, Inc (http://www.novell.com)
+ * Copyright 2012 Xamarin Inc (http://www.xamarin.com)
  */
 
 #include <config.h>
@@ -31,6 +32,7 @@
 #include <mono/metadata/appdomain.h>
 #include <mono/metadata/marshal.h>
 #include <mono/utils/strenc.h>
+#include <utils/mono-io-portability.h>
 
 #undef DEBUG
 
@@ -173,21 +175,13 @@ static gint64 convert_filetime (const FILETIME *filetime)
 	return (gint64)ticks;
 }
 
-static void convert_win32_file_attribute_data (const WIN32_FILE_ATTRIBUTE_DATA *data, const gunichar2 *name, MonoIOStat *stat)
+static void convert_win32_file_attribute_data (const WIN32_FILE_ATTRIBUTE_DATA *data, MonoIOStat *stat)
 {
-	int len;
-	
 	stat->attributes = data->dwFileAttributes;
 	stat->creation_time = convert_filetime (&data->ftCreationTime);
 	stat->last_access_time = convert_filetime (&data->ftLastAccessTime);
 	stat->last_write_time = convert_filetime (&data->ftLastWriteTime);
 	stat->length = ((gint64)data->nFileSizeHigh << 32) | data->nFileSizeLow;
-
-	len = 0;
-	while (name [len])
-		++ len;
-
-	MONO_STRUCT_SETREF (stat, name, mono_string_new_utf16 (mono_domain_get (), name, len));
 }
 
 /* Managed file attributes have nearly but not quite the same values
@@ -196,7 +190,7 @@ static void convert_win32_file_attribute_data (const WIN32_FILE_ATTRIBUTE_DATA *
 static guint32 convert_attrs(MonoFileAttributes attrs)
 {
 	if(attrs & FileAttributes_Encrypted) {
-		attrs |= FILE_ATTRIBUTE_ENCRYPTED;
+		attrs = (MonoFileAttributes)(attrs | FILE_ATTRIBUTE_ENCRYPTED);
 	}
 	
 	return(attrs);
@@ -275,16 +269,16 @@ MonoBoolean
 ves_icall_System_IO_MonoIO_CreateDirectory (MonoString *path, gint32 *error)
 {
 	gboolean ret;
+	MONO_PREPARE_BLOCKING;
 	
-	MONO_ARCH_SAVE_REGS;
-
 	*error=ERROR_SUCCESS;
 	
 	ret=CreateDirectory (mono_string_chars (path), NULL);
 	if(ret==FALSE) {
 		*error=GetLastError ();
 	}
-	
+
+	MONO_FINISH_BLOCKING;
 	return(ret);
 }
 
@@ -292,78 +286,67 @@ MonoBoolean
 ves_icall_System_IO_MonoIO_RemoveDirectory (MonoString *path, gint32 *error)
 {
 	gboolean ret;
+	MONO_PREPARE_BLOCKING;
 	
-	MONO_ARCH_SAVE_REGS;
-
 	*error=ERROR_SUCCESS;
 	
 	ret=RemoveDirectory (mono_string_chars (path));
 	if(ret==FALSE) {
 		*error=GetLastError ();
 	}
-	
+
+	MONO_FINISH_BLOCKING;
 	return(ret);
 }
 
 static gchar *
-get_search_dir (MonoString *pattern)
+get_search_dir (const gunichar2 *pattern)
 {
 	gchar *p;
 	gchar *result;
 
-	p = mono_string_to_utf8 (pattern);
+	p = g_utf16_to_utf8 (pattern, -1, NULL, NULL, NULL);
 	result = g_path_get_dirname (p);
 	g_free (p);
 	return result;
 }
 
-MonoArray *
-ves_icall_System_IO_MonoIO_GetFileSystemEntries (MonoString *path,
-						 MonoString *path_with_pattern,
+static GPtrArray *
+get_filesystem_entries (const gunichar2 *path,
+						 const gunichar2 *path_with_pattern,
 						 gint attrs, gint mask,
 						 gint32 *error)
 {
-	MonoDomain *domain;
-	MonoArray *result;
 	int i;
 	WIN32_FIND_DATA data;
 	HANDLE find_handle;
-	GPtrArray *names;
-	gchar *utf8_path, *utf8_result, *full_name;
+	GPtrArray *names = NULL;
+	gchar *utf8_path = NULL, *utf8_result, *full_name;
 	gint32 attributes;
-	
-	MONO_ARCH_SAVE_REGS;
 
-	*error = ERROR_SUCCESS;
-
-	domain = mono_domain_get ();
-	mask = convert_attrs (mask);
-	attributes = get_file_attributes (mono_string_chars (path));
+	mask = convert_attrs ((MonoFileAttributes)mask);
+	attributes = get_file_attributes (path);
 	if (attributes != -1) {
 		if ((attributes & FILE_ATTRIBUTE_DIRECTORY) == 0) {
 			*error = ERROR_INVALID_NAME;
-			return (NULL);
+			goto fail;
 		}
 	} else {
 		*error = GetLastError ();
-		return (NULL);
+		goto fail;
 	}
 	
-	find_handle = FindFirstFile (mono_string_chars (path_with_pattern), &data);
+	find_handle = FindFirstFile (path_with_pattern, &data);
 	if (find_handle == INVALID_HANDLE_VALUE) {
 		gint32 find_error = GetLastError ();
 		
 		if (find_error == ERROR_FILE_NOT_FOUND || find_error == ERROR_NO_MORE_FILES) {
 			/* No files, so just return an empty array */
-			result = mono_array_new (domain,
-						 mono_defaults.string_class,
-						 0);
-
-			return(result);
+			goto fail;
 		}
 		
 		*error = find_error;
-		return(NULL);
+		goto fail;
 	}
 
 	utf8_path = get_search_dir (path_with_pattern);
@@ -390,20 +373,52 @@ ves_icall_System_IO_MonoIO_GetFileSystemEntries (MonoString *path,
 
 	if (FindClose (find_handle) == FALSE) {
 		*error = GetLastError ();
-		result = NULL;
-	} else {
-		result = mono_array_new (domain, mono_defaults.string_class, names->len);
-		for (i = 0; i < names->len; i++) {
-			mono_array_setref (result, i, mono_string_new (domain, g_ptr_array_index (names, i)));
-		}
+		goto fail;
 	}
 
+	g_free (utf8_path);
+	return names;
+fail:
+	if (names) {
+		for (i = 0; i < names->len; i++)
+			g_free (g_ptr_array_index (names, i));
+		g_ptr_array_free (names, TRUE);
+	}
+	g_free (utf8_path);
+	return FALSE;
+}
+
+
+MonoArray *
+ves_icall_System_IO_MonoIO_GetFileSystemEntries (MonoString *path,
+						 MonoString *path_with_pattern,
+						 gint attrs, gint mask,
+						 gint32 *error)
+{
+	MonoDomain *domain = mono_domain_get ();
+	MonoArray *result;
+	int i;
+	GPtrArray *names;
+	
+	*error = ERROR_SUCCESS;
+
+	MONO_PREPARE_BLOCKING;
+	names = get_filesystem_entries (mono_string_chars (path), mono_string_chars (path_with_pattern), attrs, mask, error);
+	MONO_FINISH_BLOCKING;
+
+	if (!names) {
+		// If there's no array and no error, then return an empty array.
+		if (*error == ERROR_SUCCESS)
+			return mono_array_new (domain, mono_defaults.string_class, 0);
+		return NULL;
+	}
+
+	result = mono_array_new (domain, mono_defaults.string_class, names->len);
 	for (i = 0; i < names->len; i++) {
+		mono_array_setref (result, i, mono_string_new (domain, (const char *)g_ptr_array_index (names, i)));
 		g_free (g_ptr_array_index (names, i));
 	}
 	g_ptr_array_free (names, TRUE);
-	g_free (utf8_path);
-	
 	return result;
 }
 
@@ -434,6 +449,7 @@ incremental_find_check_match (IncrementalFind *handle, WIN32_FIND_DATA *data, Mo
 	return TRUE;
 }
 
+/* FIXME make gc suspendable */
 MonoString *
 ves_icall_System_IO_MonoIO_FindFirst (MonoString *path,
 				      MonoString *path_with_pattern,
@@ -479,14 +495,15 @@ ves_icall_System_IO_MonoIO_FindFirst (MonoString *path,
 	return result;
 }
 
+/* FIXME make gc suspendable */
 MonoString *
 ves_icall_System_IO_MonoIO_FindNext (gpointer handle, gint32 *result_attr, gint32 *error)
 {
-	IncrementalFind *ifh = handle;
+	IncrementalFind *ifh = (IncrementalFind *)handle;
 	WIN32_FIND_DATA data;
 	MonoString *result;
 
-	error = ERROR_SUCCESS;
+	*error = ERROR_SUCCESS;
 	do {
 		if (FindNextFile (ifh->find_handle, &data) == FALSE){
 			int e = GetLastError ();
@@ -503,15 +520,17 @@ ves_icall_System_IO_MonoIO_FindNext (gpointer handle, gint32 *result_attr, gint3
 int
 ves_icall_System_IO_MonoIO_FindClose (gpointer handle)
 {
-	IncrementalFind *ifh = handle;
+	IncrementalFind *ifh = (IncrementalFind *)handle;
 	gint32 error;
-	
+
+	MONO_PREPARE_BLOCKING;
 	if (FindClose (ifh->find_handle) == FALSE){
 		error = GetLastError ();
 	} else
 		error = ERROR_SUCCESS;
 	g_free (ifh->utf8_path);
 	g_free (ifh);
+	MONO_FINISH_BLOCKING;
 
 	return error;
 }
@@ -523,8 +542,6 @@ ves_icall_System_IO_MonoIO_GetCurrentDirectory (gint32 *error)
 	gunichar2 *buf;
 	int len, res_len;
 
-	MONO_ARCH_SAVE_REGS;
-
 	len = MAX_PATH + 1; /*FIXME this is too smal under most unix systems.*/
 	buf = g_new (gunichar2, len);
 	
@@ -559,8 +576,6 @@ ves_icall_System_IO_MonoIO_SetCurrentDirectory (MonoString *path,
 {
 	gboolean ret;
 	
-	MONO_ARCH_SAVE_REGS;
-
 	*error=ERROR_SUCCESS;
 	
 	ret=SetCurrentDirectory (mono_string_chars (path));
@@ -576,16 +591,16 @@ ves_icall_System_IO_MonoIO_MoveFile (MonoString *path, MonoString *dest,
 				     gint32 *error)
 {
 	gboolean ret;
+	MONO_PREPARE_BLOCKING;
 	
-	MONO_ARCH_SAVE_REGS;
-
 	*error=ERROR_SUCCESS;
-	
+
 	ret=MoveFile (mono_string_chars (path), mono_string_chars (dest));
 	if(ret==FALSE) {
 		*error=GetLastError ();
 	}
-	
+
+	MONO_FINISH_BLOCKING;
 	return(ret);
 }
 
@@ -597,8 +612,7 @@ ves_icall_System_IO_MonoIO_ReplaceFile (MonoString *sourceFileName, MonoString *
 	gboolean ret;
 	gunichar2 *utf16_sourceFileName = NULL, *utf16_destinationFileName = NULL, *utf16_destinationBackupFileName = NULL;
 	guint32 replaceFlags = REPLACEFILE_WRITE_THROUGH;
-
-	MONO_ARCH_SAVE_REGS;
+	MONO_PREPARE_BLOCKING;
 
 	if (sourceFileName)
 		utf16_sourceFileName = mono_string_chars (sourceFileName);
@@ -611,11 +625,13 @@ ves_icall_System_IO_MonoIO_ReplaceFile (MonoString *sourceFileName, MonoString *
 	if (ignoreMetadataErrors)
 		replaceFlags |= REPLACEFILE_IGNORE_MERGE_ERRORS;
 
+	/* FIXME: source and destination file names must not be NULL, but apparently they might be! */
 	ret = ReplaceFile (utf16_destinationFileName, utf16_sourceFileName, utf16_destinationBackupFileName,
 			 replaceFlags, NULL, NULL);
 	if (ret == FALSE)
 		*error = GetLastError ();
 
+	MONO_FINISH_BLOCKING;
 	return ret;
 }
 
@@ -624,9 +640,8 @@ ves_icall_System_IO_MonoIO_CopyFile (MonoString *path, MonoString *dest,
 				     MonoBoolean overwrite, gint32 *error)
 {
 	gboolean ret;
+	MONO_PREPARE_BLOCKING;
 	
-	MONO_ARCH_SAVE_REGS;
-
 	*error=ERROR_SUCCESS;
 	
 	ret=CopyFile (mono_string_chars (path), mono_string_chars (dest), !overwrite);
@@ -634,6 +649,7 @@ ves_icall_System_IO_MonoIO_CopyFile (MonoString *path, MonoString *dest,
 		*error=GetLastError ();
 	}
 	
+	MONO_FINISH_BLOCKING;
 	return(ret);
 }
 
@@ -641,9 +657,8 @@ MonoBoolean
 ves_icall_System_IO_MonoIO_DeleteFile (MonoString *path, gint32 *error)
 {
 	gboolean ret;
+	MONO_PREPARE_BLOCKING;
 	
-	MONO_ARCH_SAVE_REGS;
-
 	*error=ERROR_SUCCESS;
 	
 	ret=DeleteFile (mono_string_chars (path));
@@ -651,6 +666,7 @@ ves_icall_System_IO_MonoIO_DeleteFile (MonoString *path, gint32 *error)
 		*error=GetLastError ();
 	}
 	
+	MONO_FINISH_BLOCKING;
 	return(ret);
 }
 
@@ -658,8 +674,7 @@ gint32
 ves_icall_System_IO_MonoIO_GetFileAttributes (MonoString *path, gint32 *error)
 {
 	gint32 ret;
-	
-	MONO_ARCH_SAVE_REGS;
+	MONO_PREPARE_BLOCKING;
 
 	*error=ERROR_SUCCESS;
 	
@@ -676,6 +691,7 @@ ves_icall_System_IO_MonoIO_GetFileAttributes (MonoString *path, gint32 *error)
 		*error=GetLastError ();
 	}
 	
+	MONO_FINISH_BLOCKING;
 	return(ret);
 }
 
@@ -684,17 +700,17 @@ ves_icall_System_IO_MonoIO_SetFileAttributes (MonoString *path, gint32 attrs,
 					      gint32 *error)
 {
 	gboolean ret;
+	MONO_PREPARE_BLOCKING;
 	
-	MONO_ARCH_SAVE_REGS;
-
 	*error=ERROR_SUCCESS;
 	
 	ret=SetFileAttributes (mono_string_chars (path),
-			       convert_attrs (attrs));
+		convert_attrs ((MonoFileAttributes)attrs));
 	if(ret==FALSE) {
 		*error=GetLastError ();
 	}
-	
+
+	MONO_FINISH_BLOCKING;
 	return(ret);
 }
 
@@ -702,8 +718,7 @@ gint32
 ves_icall_System_IO_MonoIO_GetFileType (HANDLE handle, gint32 *error)
 {
 	gboolean ret;
-	
-	MONO_ARCH_SAVE_REGS;
+	MONO_PREPARE_BLOCKING;
 
 	*error=ERROR_SUCCESS;
 	
@@ -715,6 +730,7 @@ ves_icall_System_IO_MonoIO_GetFileType (HANDLE handle, gint32 *error)
 		*error=GetLastError ();
 	}
 	
+	MONO_FINISH_BLOCKING;
 	return(ret);
 }
 
@@ -724,22 +740,20 @@ ves_icall_System_IO_MonoIO_GetFileStat (MonoString *path, MonoIOStat *stat,
 {
 	gboolean result;
 	WIN32_FILE_ATTRIBUTE_DATA data;
-
-	MONO_ARCH_SAVE_REGS;
+	MONO_PREPARE_BLOCKING;
 
 	*error=ERROR_SUCCESS;
 	
 	result = get_file_attributes_ex (mono_string_chars (path), &data);
 
 	if (result) {
-		convert_win32_file_attribute_data (&data,
-						   mono_string_chars (path),
-						   stat);
+		convert_win32_file_attribute_data (&data, stat);
 	} else {
 		*error=GetLastError ();
 		memset (stat, 0, sizeof (MonoIOStat));
 	}
 
+	MONO_FINISH_BLOCKING;
 	return result;
 }
 
@@ -750,10 +764,10 @@ ves_icall_System_IO_MonoIO_Open (MonoString *filename, gint32 mode,
 {
 	HANDLE ret;
 	int attributes, attrs;
-	gunichar2 *chars = mono_string_chars (filename);
-	
-	MONO_ARCH_SAVE_REGS;
+	gunichar2 *chars;
+	MONO_PREPARE_BLOCKING;
 
+	chars = mono_string_chars (filename);	
 	*error=ERROR_SUCCESS;
 
 	if (options != 0){
@@ -789,13 +803,14 @@ ves_icall_System_IO_MonoIO_Open (MonoString *filename, gint32 mode,
 		}
 	}
 	
-	ret=CreateFile (chars, convert_access (access_mode),
-			convert_share (share), NULL, convert_mode (mode),
+	ret=CreateFile (chars, convert_access ((MonoFileAccess)access_mode),
+			convert_share ((MonoFileShare)share), NULL, convert_mode ((MonoFileMode)mode),
 			attributes, NULL);
 	if(ret==INVALID_HANDLE_VALUE) {
 		*error=GetLastError ();
 	} 
 	
+	MONO_FINISH_BLOCKING;
 	return(ret);
 }
 
@@ -803,8 +818,7 @@ MonoBoolean
 ves_icall_System_IO_MonoIO_Close (HANDLE handle, gint32 *error)
 {
 	gboolean ret;
-	
-	MONO_ARCH_SAVE_REGS;
+	MONO_PREPARE_BLOCKING;
 
 	*error=ERROR_SUCCESS;
 	
@@ -813,6 +827,7 @@ ves_icall_System_IO_MonoIO_Close (HANDLE handle, gint32 *error)
 		*error=GetLastError ();
 	}
 	
+	MONO_FINISH_BLOCKING;
 	return(ret);
 }
 
@@ -825,17 +840,20 @@ ves_icall_System_IO_MonoIO_Read (HANDLE handle, MonoArray *dest,
 	gboolean result;
 	guint32 n;
 
-	MONO_ARCH_SAVE_REGS;
-
 	*error=ERROR_SUCCESS;
 
-	MONO_CHECK_ARG_NULL (dest);
-	
-	if (dest_offset + count > mono_array_length (dest))
+	MONO_CHECK_ARG_NULL (dest, 0);
+
+	if (dest_offset > mono_array_length (dest) - count) {
+		mono_set_pending_exception (mono_get_exception_argument ("array", "array too small. numBytes/offset wrong."));
 		return 0;
+	}
 
 	buffer = mono_array_addr (dest, guchar, dest_offset);
+
+	MONO_PREPARE_BLOCKING;
 	result = ReadFile (handle, buffer, count, &n, NULL);
+	MONO_FINISH_BLOCKING;
 
 	if (!result) {
 		*error=GetLastError ();
@@ -854,17 +872,19 @@ ves_icall_System_IO_MonoIO_Write (HANDLE handle, MonoArray *src,
 	gboolean result;
 	guint32 n;
 
-	MONO_ARCH_SAVE_REGS;
-
 	*error=ERROR_SUCCESS;
 
-	MONO_CHECK_ARG_NULL (src);
+	MONO_CHECK_ARG_NULL (src, 0);
 	
-	if (src_offset + count > mono_array_length (src))
+	if (src_offset > mono_array_length (src) - count) {
+		mono_set_pending_exception (mono_get_exception_argument ("array", "array too small. numBytes/offset wrong."));
 		return 0;
+	}
 	
 	buffer = mono_array_addr (src, guchar, src_offset);
+	MONO_PREPARE_BLOCKING;
 	result = WriteFile (handle, buffer, count, &n, NULL);
+	MONO_FINISH_BLOCKING;
 
 	if (!result) {
 		*error=GetLastError ();
@@ -879,19 +899,19 @@ ves_icall_System_IO_MonoIO_Seek (HANDLE handle, gint64 offset, gint32 origin,
 				 gint32 *error)
 {
 	gint32 offset_hi;
-
-	MONO_ARCH_SAVE_REGS;
+	MONO_PREPARE_BLOCKING;
 
 	*error=ERROR_SUCCESS;
 	
 	offset_hi = offset >> 32;
 	offset = SetFilePointer (handle, (gint32) (offset & 0xFFFFFFFF), &offset_hi,
-				 convert_seekorigin (origin));
+				 convert_seekorigin ((MonoSeekOrigin)origin));
 
 	if(offset==INVALID_SET_FILE_POINTER) {
 		*error=GetLastError ();
 	}
-	
+
+	MONO_FINISH_BLOCKING;
 	return offset | ((gint64)offset_hi << 32);
 }
 
@@ -899,8 +919,7 @@ MonoBoolean
 ves_icall_System_IO_MonoIO_Flush (HANDLE handle, gint32 *error)
 {
 	gboolean ret;
-	
-	MONO_ARCH_SAVE_REGS;
+	MONO_PREPARE_BLOCKING;
 
 	*error=ERROR_SUCCESS;
 	
@@ -909,6 +928,7 @@ ves_icall_System_IO_MonoIO_Flush (HANDLE handle, gint32 *error)
 		*error=GetLastError ();
 	}
 	
+	MONO_FINISH_BLOCKING;
 	return(ret);
 }
 
@@ -917,8 +937,7 @@ ves_icall_System_IO_MonoIO_GetLength (HANDLE handle, gint32 *error)
 {
 	gint64 length;
 	guint32 length_hi;
-
-	MONO_ARCH_SAVE_REGS;
+	MONO_PREPARE_BLOCKING;
 
 	*error=ERROR_SUCCESS;
 	
@@ -927,9 +946,11 @@ ves_icall_System_IO_MonoIO_GetLength (HANDLE handle, gint32 *error)
 		*error=GetLastError ();
 	}
 	
+	MONO_FINISH_BLOCKING;
 	return length | ((gint64)length_hi << 32);
 }
 
+/* FIXME make gc suspendable */
 MonoBoolean
 ves_icall_System_IO_MonoIO_SetLength (HANDLE handle, gint64 length,
 				      gint32 *error)
@@ -939,8 +960,6 @@ ves_icall_System_IO_MonoIO_SetLength (HANDLE handle, gint64 length,
 	gint32 length_hi;
 	gboolean result;
 
-	MONO_ARCH_SAVE_REGS;
-
 	*error=ERROR_SUCCESS;
 	
 	/* save file pointer */
@@ -989,8 +1008,7 @@ ves_icall_System_IO_MonoIO_SetFileTime (HANDLE handle, gint64 creation_time,
 	const FILETIME *creation_filetime;
 	const FILETIME *last_access_filetime;
 	const FILETIME *last_write_filetime;
-
-	MONO_ARCH_SAVE_REGS;
+	MONO_PREPARE_BLOCKING;
 
 	*error=ERROR_SUCCESS;
 	
@@ -1013,49 +1031,45 @@ ves_icall_System_IO_MonoIO_SetFileTime (HANDLE handle, gint64 creation_time,
 	if(ret==FALSE) {
 		*error=GetLastError ();
 	}
-	
+
+	MONO_FINISH_BLOCKING;
 	return(ret);
 }
 
 HANDLE 
 ves_icall_System_IO_MonoIO_get_ConsoleOutput ()
 {
-	MONO_ARCH_SAVE_REGS;
-
 	return GetStdHandle (STD_OUTPUT_HANDLE);
 }
 
 HANDLE 
 ves_icall_System_IO_MonoIO_get_ConsoleInput ()
 {
-	MONO_ARCH_SAVE_REGS;
-
 	return GetStdHandle (STD_INPUT_HANDLE);
 }
 
 HANDLE 
 ves_icall_System_IO_MonoIO_get_ConsoleError ()
 {
-	MONO_ARCH_SAVE_REGS;
-
 	return GetStdHandle (STD_ERROR_HANDLE);
 }
 
 MonoBoolean
-ves_icall_System_IO_MonoIO_CreatePipe (HANDLE *read_handle,
-				       HANDLE *write_handle)
+ves_icall_System_IO_MonoIO_CreatePipe (HANDLE *read_handle, HANDLE *write_handle, gint32 *error)
 {
 	SECURITY_ATTRIBUTES attr;
 	gboolean ret;
 	
-	MONO_ARCH_SAVE_REGS;
-
 	attr.nLength=sizeof(SECURITY_ATTRIBUTES);
 	attr.bInheritHandle=TRUE;
 	attr.lpSecurityDescriptor=NULL;
-	
+
+	MONO_PREPARE_BLOCKING;
 	ret=CreatePipe (read_handle, write_handle, &attr, 0);
+	MONO_FINISH_BLOCKING;
+
 	if(ret==FALSE) {
+		*error = GetLastError ();
 		/* FIXME: throw an exception? */
 		return(FALSE);
 	}
@@ -1063,17 +1077,19 @@ ves_icall_System_IO_MonoIO_CreatePipe (HANDLE *read_handle,
 	return(TRUE);
 }
 
-MonoBoolean ves_icall_System_IO_MonoIO_DuplicateHandle (HANDLE source_process_handle, 
-						HANDLE source_handle, HANDLE target_process_handle, HANDLE *target_handle, 
-						gint32 access, gint32 inherit, gint32 options)
+MonoBoolean
+ves_icall_System_IO_MonoIO_DuplicateHandle (HANDLE source_process_handle, HANDLE source_handle,
+		HANDLE target_process_handle, HANDLE *target_handle, gint32 access, gint32 inherit, gint32 options, gint32 *error)
 {
 	/* This is only used on Windows */
 	gboolean ret;
 	
-	MONO_ARCH_SAVE_REGS;
-	
+	MONO_PREPARE_BLOCKING;
 	ret=DuplicateHandle (source_process_handle, source_handle, target_process_handle, target_handle, access, inherit, options);
+	MONO_FINISH_BLOCKING;
+
 	if(ret==FALSE) {
+		*error = GetLastError ();
 		/* FIXME: throw an exception? */
 		return(FALSE);
 	}
@@ -1107,7 +1123,10 @@ ves_icall_System_IO_MonoIO_get_AltDirectorySeparatorChar ()
 #if defined (TARGET_WIN32)
 	return (gunichar2) '/';	/* forward slash */
 #else
-	return (gunichar2) '/';	/* slash, same as DirectorySeparatorChar */
+	if (IS_PORTABILITY_SET)
+		return (gunichar2) '\\';	/* backslash */
+	else
+		return (gunichar2) '/';	/* forward slash */
 #endif
 }
 
@@ -1149,8 +1168,6 @@ ves_icall_System_IO_MonoIO_get_InvalidPathChars ()
 	MonoDomain *domain;
 	int i, n;
 
-	MONO_ARCH_SAVE_REGS;
-
 	domain = mono_domain_get ();
 	n = sizeof (invalid_path_chars) / sizeof (gunichar2);
 	chars = mono_array_new (domain, mono_defaults.char_class, n);
@@ -1161,40 +1178,11 @@ ves_icall_System_IO_MonoIO_get_InvalidPathChars ()
 	return chars;
 }
 
-gint32
-ves_icall_System_IO_MonoIO_GetTempPath (MonoString **mono_name)
-{
-	gunichar2 *name;
-	int ret;
-
-	name=g_new0 (gunichar2, 256);
-	
-	ret=GetTempPath (256, name);
-	if(ret>255) {
-		/* Buffer was too short. Try again... */
-		g_free (name);
-		name=g_new0 (gunichar2, ret+2);	/* include the terminator */
-		ret=GetTempPath (ret, name);
-	}
-	
-	if(ret>0) {
-#ifdef DEBUG
-		g_message ("%s: Temp path is [%s] (len %d)", __func__, name, ret);
-#endif
-
-		mono_gc_wbarrier_generic_store ((gpointer) mono_name,
-				(MonoObject*) mono_string_new_utf16 (mono_domain_get (), name, ret));
-	}
-
-	g_free (name);
-	
-	return(ret);
-}
-
 void ves_icall_System_IO_MonoIO_Lock (HANDLE handle, gint64 position,
 				      gint64 length, gint32 *error)
 {
 	gboolean ret;
+	MONO_PREPARE_BLOCKING;
 	
 	*error=ERROR_SUCCESS;
 	
@@ -1203,12 +1191,15 @@ void ves_icall_System_IO_MonoIO_Lock (HANDLE handle, gint64 position,
 	if (ret == FALSE) {
 		*error = GetLastError ();
 	}
+
+	MONO_FINISH_BLOCKING;
 }
 
 void ves_icall_System_IO_MonoIO_Unlock (HANDLE handle, gint64 position,
 					gint64 length, gint32 *error)
 {
 	gboolean ret;
+	MONO_PREPARE_BLOCKING;
 	
 	*error=ERROR_SUCCESS;
 	
@@ -1217,5 +1208,56 @@ void ves_icall_System_IO_MonoIO_Unlock (HANDLE handle, gint64 position,
 	if (ret == FALSE) {
 		*error = GetLastError ();
 	}
+
+	MONO_FINISH_BLOCKING;
 }
 
+//Support for io-layer free mmap'd files.
+
+#if defined (TARGET_IOS) || defined (TARGET_ANDROID)
+
+gint64
+mono_filesize_from_path (MonoString *string)
+{
+	struct stat buf;
+	gint64 res;
+	char *path = mono_string_to_utf8 (string);
+
+	MONO_PREPARE_BLOCKING;
+	if (stat (path, &buf) == -1)
+		res = -1;
+	else
+		res = (gint64)buf.st_size;
+
+	g_free (path);
+
+	MONO_FINISH_BLOCKING;
+	return res;
+}
+
+gint64
+mono_filesize_from_fd (int fd)
+{
+	struct stat buf;
+	int res;
+
+	MONO_PREPARE_BLOCKING;
+	res = fstat (fd, &buf);
+	MONO_FINISH_BLOCKING;
+	
+	if (res == -1)
+		return (gint64)-1;
+
+	return (gint64)buf.st_size;
+}
+
+#endif
+
+void _wapi_handle_dump (void);
+
+void ves_icall_System_IO_MonoIO_DumpHandles (void)
+{
+#ifndef HOST_WIN32
+	_wapi_handle_dump ();
+#endif
+}
