@@ -11,9 +11,8 @@
 #include <config.h>
 #include <glib.h>
 #include <string.h>
-#include <errno.h>
 
-#include <mono/metadata/gc-internal.h>
+#include <mono/metadata/gc-internals.h>
 #include <mono/metadata/mono-gc.h>
 #include <mono/metadata/threads.h>
 #include <mono/metadata/tabledefs.h>
@@ -23,21 +22,23 @@
 #include <mono/metadata/class-internals.h>
 #include <mono/metadata/metadata-internals.h>
 #include <mono/metadata/mono-mlist.h>
-#include <mono/metadata/threadpool.h>
-#include <mono/metadata/threadpool-internals.h>
 #include <mono/metadata/threads-types.h>
-#include <mono/metadata/sgen-conf.h>
-#include <mono/utils/mono-logger-internal.h>
-#include <mono/metadata/gc-internal.h>
+#include <mono/metadata/threadpool-ms.h>
+#include <mono/sgen/sgen-conf.h>
+#include <mono/sgen/sgen-gc.h>
+#include <mono/utils/mono-logger-internals.h>
+#include <mono/metadata/gc-internals.h>
 #include <mono/metadata/marshal.h> /* for mono_delegate_free_ftnptr () */
 #include <mono/metadata/attach.h>
 #include <mono/metadata/console-io.h>
-#include <mono/utils/mono-semaphore.h>
+#include <mono/utils/mono-os-semaphore.h>
 #include <mono/utils/mono-memory-model.h>
 #include <mono/utils/mono-counters.h>
+#include <mono/utils/mono-time.h>
 #include <mono/utils/dtrace.h>
 #include <mono/utils/mono-threads.h>
 #include <mono/utils/atomic.h>
+#include <mono/utils/mono-coop-semaphore.h>
 
 #ifndef HOST_WIN32
 #include <pthread.h>
@@ -48,47 +49,53 @@ typedef struct DomainFinalizationReq {
 	HANDLE done_event;
 } DomainFinalizationReq;
 
-#ifdef PLATFORM_WINCE /* FIXME: add accessors to gc.dll API */
-extern void (*__imp_GC_finalizer_notifier)(void);
-#define GC_finalizer_notifier __imp_GC_finalizer_notifier
-extern int __imp_GC_finalize_on_demand;
-#define GC_finalize_on_demand __imp_GC_finalize_on_demand
-#endif
-
 static gboolean gc_disabled = FALSE;
 
 static gboolean finalizing_root_domain = FALSE;
 
-#define mono_finalizer_lock() mono_mutex_lock (&finalizer_mutex)
-#define mono_finalizer_unlock() mono_mutex_unlock (&finalizer_mutex)
-static mono_mutex_t finalizer_mutex;
-static mono_mutex_t reference_queue_mutex;
+gboolean log_finalizers = FALSE;
+gboolean mono_do_not_finalize = FALSE;
+gchar **mono_do_not_finalize_class_names = NULL;
+
+#define mono_finalizer_lock() mono_coop_mutex_lock (&finalizer_mutex)
+#define mono_finalizer_unlock() mono_coop_mutex_unlock (&finalizer_mutex)
+static MonoCoopMutex finalizer_mutex;
+static MonoCoopMutex reference_queue_mutex;
 
 static GSList *domains_to_finalize= NULL;
 static MonoMList *threads_to_finalize = NULL;
 
+static gboolean finalizer_thread_exited;
+/* Uses finalizer_mutex */
+static MonoCoopCond exited_cond;
+
 static MonoInternalThread *gc_thread;
 
 static void object_register_finalizer (MonoObject *obj, void (*callback)(void *, void*));
 
-static void mono_gchandle_set_target (guint32 gchandle, MonoObject *obj);
-
 static void reference_queue_proccess_all (void);
 static void mono_reference_queue_cleanup (void);
 static void reference_queue_clear_for_domain (MonoDomain *domain);
-#ifndef HAVE_NULL_GC
 static HANDLE pending_done_event;
-static HANDLE shutdown_event;
-#endif
 
-GCStats gc_stats;
+static guint32
+guarded_wait (HANDLE handle, guint32 timeout, gboolean alertable)
+{
+	guint32 result;
+
+	MONO_PREPARE_BLOCKING;
+	result = WaitForSingleObjectEx (handle, timeout, alertable);
+	MONO_FINISH_BLOCKING;
+
+	return result;
+}
 
 static void
 add_thread_to_finalize (MonoInternalThread *thread)
 {
 	mono_finalizer_lock ();
 	if (!threads_to_finalize)
-		MONO_GC_REGISTER_ROOT_SINGLE (threads_to_finalize);
+		MONO_GC_REGISTER_ROOT_SINGLE (threads_to_finalize, MONO_ROOT_SOURCE_FINALIZER_QUEUE, "finalizable threads list");
 	threads_to_finalize = mono_mlist_append (threads_to_finalize, (MonoObject*)thread);
 	mono_finalizer_unlock ();
 }
@@ -111,8 +118,31 @@ mono_gc_run_finalize (void *obj, void *data)
 	MonoDomain *domain;
 	RuntimeInvokeFunction runtime_invoke;
 
+	// This function is called from the innards of the GC, so our best alternative for now is to do polling here
+	mono_threads_safepoint ();
+
 	o = (MonoObject*)((char*)obj + GPOINTER_TO_UINT (data));
 
+	if (mono_do_not_finalize) {
+		if (!mono_do_not_finalize_class_names)
+			return;
+
+		size_t namespace_len = strlen (o->vtable->klass->name_space);
+		for (int i = 0; mono_do_not_finalize_class_names [i]; ++i) {
+			const char *name = mono_do_not_finalize_class_names [i];
+			if (strncmp (name, o->vtable->klass->name_space, namespace_len))
+				break;
+			if (name [namespace_len] != '.')
+				break;
+			if (strcmp (name + namespace_len + 1, o->vtable->klass->name))
+				break;
+			return;
+		}
+	}
+
+	if (log_finalizers)
+		g_log ("mono-gc-finalizers", G_LOG_LEVEL_DEBUG, "<%s at %p> Starting finalizer checks.", o->vtable->klass->name, o);
+
 	if (suspend_finalizers)
 		return;
 
@@ -121,7 +151,7 @@ mono_gc_run_finalize (void *obj, void *data)
 #ifndef HAVE_SGEN_GC
 	mono_domain_finalizers_lock (domain);
 
-	o2 = g_hash_table_lookup (domain->finalizable_objects_hash, o);
+	o2 = (MonoObject *)g_hash_table_lookup (domain->finalizable_objects_hash, o);
 
 	mono_domain_finalizers_unlock (domain);
 
@@ -131,7 +161,10 @@ mono_gc_run_finalize (void *obj, void *data)
 #endif
 
 	/* make sure the finalizer is not called again if the object is resurrected */
-	object_register_finalizer (obj, NULL);
+	object_register_finalizer ((MonoObject *)obj, NULL);
+
+	if (log_finalizers)
+		g_log ("mono-gc-finalizers", G_LOG_LEVEL_MESSAGE, "<%s at %p> Registered finalizer as processed.", o->vtable->klass->name, o);
 
 	if (o->vtable->klass == mono_defaults.internal_thread_class) {
 		MonoInternalThread *t = (MonoInternalThread*)o;
@@ -183,7 +216,6 @@ mono_gc_run_finalize (void *obj, void *data)
 
 	finalizer = mono_class_get_finalizer (o->vtable->klass);
 
-#ifndef DISABLE_COM
 	/* If object has a CCW but has no finalizer, it was only
 	 * registered for finalization in order to free the CCW.
 	 * Else it needs the regular finalizer run.
@@ -194,20 +226,22 @@ mono_gc_run_finalize (void *obj, void *data)
 		mono_domain_set_internal (caller_domain);
 		return;
 	}
-#endif
 
 	/* 
 	 * To avoid the locking plus the other overhead of mono_runtime_invoke (),
 	 * create and precompile a wrapper which calls the finalize method using
 	 * a CALLVIRT.
 	 */
+	if (log_finalizers)
+		g_log ("mono-gc-finalizers", G_LOG_LEVEL_MESSAGE, "<%s at %p> Compiling finalizer.", o->vtable->klass->name, o);
+
 	if (!domain->finalize_runtime_invoke) {
 		MonoMethod *invoke = mono_marshal_get_runtime_invoke (mono_class_get_method_from_name_flags (mono_defaults.object_class, "Finalize", 0, 0), TRUE);
 
 		domain->finalize_runtime_invoke = mono_compile_method (invoke);
 	}
 
-	runtime_invoke = domain->finalize_runtime_invoke;
+	runtime_invoke = (RuntimeInvokeFunction)domain->finalize_runtime_invoke;
 
 	mono_runtime_class_init (o->vtable);
 
@@ -216,10 +250,16 @@ mono_gc_run_finalize (void *obj, void *data)
 				o->vtable->klass->name_space, o->vtable->klass->name);
 	}
 
+	if (log_finalizers)
+		g_log ("mono-gc-finalizers", G_LOG_LEVEL_MESSAGE, "<%s at %p> Calling finalizer.", o->vtable->klass->name, o);
+
 	runtime_invoke (o, NULL, &exc, NULL);
 
+	if (log_finalizers)
+		g_log ("mono-gc-finalizers", G_LOG_LEVEL_MESSAGE, "<%s at %p> Returned from finalizer.", o->vtable->klass->name, o);
+
 	if (exc)
-		mono_internal_thread_unhandled_exception (exc);
+		mono_thread_internal_unhandled_exception (exc);
 
 	mono_domain_set_internal (caller_domain);
 }
@@ -347,10 +387,13 @@ mono_domain_finalize (MonoDomain *domain, guint32 timeout)
 	 * is still working and will take care of running the finalizers
 	 */ 
 	
-#ifndef HAVE_NULL_GC
 	if (gc_disabled)
 		return TRUE;
 
+	/* We don't support domain finalization without a GC */
+	if (mono_gc_is_null ())
+		return FALSE;
+
 	mono_gc_collect (mono_gc_max_generation ());
 
 	done_event = CreateEvent (NULL, TRUE, FALSE, NULL);
@@ -378,7 +421,7 @@ mono_domain_finalize (MonoDomain *domain, guint32 timeout)
 		timeout = INFINITE;
 
 	while (TRUE) {
-		res = WaitForSingleObjectEx (done_event, timeout, TRUE);
+		res = guarded_wait (done_event, timeout, TRUE);
 		/* printf ("WAIT RES: %d.\n", res); */
 
 		if (res == WAIT_IO_COMPLETION) {
@@ -395,15 +438,11 @@ mono_domain_finalize (MonoDomain *domain, guint32 timeout)
 	CloseHandle (done_event);
 
 	if (domain == mono_get_root_domain ()) {
-		mono_thread_pool_cleanup ();
+		mono_threadpool_ms_cleanup ();
 		mono_gc_finalize_threadpool_threads ();
 	}
 
 	return TRUE;
-#else
-	/* We don't support domain finalization without a GC */
-	return FALSE;
-#endif
 }
 
 void
@@ -415,8 +454,6 @@ ves_icall_System_GC_InternalCollect (int generation)
 gint64
 ves_icall_System_GC_GetTotalMemory (MonoBoolean forceCollection)
 {
-	MONO_ARCH_SAVE_REGS;
-
 	if (forceCollection)
 		mono_gc_collect (mono_gc_max_generation ());
 	return mono_gc_get_used_size ();
@@ -425,8 +462,6 @@ ves_icall_System_GC_GetTotalMemory (MonoBoolean forceCollection)
 void
 ves_icall_System_GC_KeepAlive (MonoObject *obj)
 {
-	MONO_ARCH_SAVE_REGS;
-
 	/*
 	 * Does nothing.
 	 */
@@ -435,8 +470,7 @@ ves_icall_System_GC_KeepAlive (MonoObject *obj)
 void
 ves_icall_System_GC_ReRegisterForFinalize (MonoObject *obj)
 {
-	if (!obj)
-		mono_raise_exception (mono_get_exception_argument_null ("obj"));
+	MONO_CHECK_ARG_NULL (obj,);
 
 	object_register_finalizer (obj, mono_gc_run_finalize);
 }
@@ -444,8 +478,7 @@ ves_icall_System_GC_ReRegisterForFinalize (MonoObject *obj)
 void
 ves_icall_System_GC_SuppressFinalize (MonoObject *obj)
 {
-	if (!obj)
-		mono_raise_exception (mono_get_exception_argument_null ("obj"));
+	MONO_CHECK_ARG_NULL (obj,);
 
 	/* delegates have no finalizers, but we register them to deal with the
 	 * unmanaged->managed trampoline. We don't let the user suppress it
@@ -464,7 +497,9 @@ ves_icall_System_GC_SuppressFinalize (MonoObject *obj)
 void
 ves_icall_System_GC_WaitForPendingFinalizers (void)
 {
-#ifndef HAVE_NULL_GC
+	if (mono_gc_is_null ())
+		return;
+
 	if (!mono_gc_pending_finalizers ())
 		return;
 
@@ -482,17 +517,18 @@ ves_icall_System_GC_WaitForPendingFinalizers (void)
 	ResetEvent (pending_done_event);
 	mono_gc_finalize_notify ();
 	/* g_print ("Waiting for pending finalizers....\n"); */
-	WaitForSingleObjectEx (pending_done_event, INFINITE, TRUE);
+	guarded_wait (pending_done_event, INFINITE, TRUE);
 	/* g_print ("Done pending....\n"); */
-#endif
 }
 
 void
 ves_icall_System_GC_register_ephemeron_array (MonoObject *array)
 {
 #ifdef HAVE_SGEN_GC
-	if (!mono_gc_ephemeron_array_add (array))
-		mono_raise_exception (mono_object_domain (array)->out_of_memory_ex);
+	if (!mono_gc_ephemeron_array_add (array)) {
+		mono_set_pending_exception (mono_object_domain (array)->out_of_memory_ex);
+		return;
+	}
 #endif
 }
 
@@ -502,20 +538,6 @@ ves_icall_System_GC_get_ephemeron_tombstone (void)
 	return mono_domain_get ()->ephemeron_tombstone;
 }
 
-#define mono_allocator_lock() mono_mutex_lock (&allocator_section)
-#define mono_allocator_unlock() mono_mutex_unlock (&allocator_section)
-static mono_mutex_t allocator_section;
-static mono_mutex_t handle_section;
-
-typedef enum {
-	HANDLE_WEAK,
-	HANDLE_WEAK_TRACK,
-	HANDLE_NORMAL,
-	HANDLE_PINNED
-} HandleType;
-
-static HandleType mono_gchandle_get_type (guint32 gchandle);
-
 MonoObject *
 ves_icall_System_GCHandle_GetTarget (guint32 handle)
 {
@@ -559,7 +581,7 @@ ves_icall_System_GCHandle_GetAddrOfPinnedObject (guint32 handle)
 {
 	MonoObject *obj;
 
-	if (mono_gchandle_get_type (handle) != HANDLE_PINNED)
+	if (MONO_GC_HANDLE_TYPE (handle) != HANDLE_PINNED)
 		return (gpointer)-2;
 	obj = mono_gchandle_get_target (handle);
 	if (obj) {
@@ -580,399 +602,12 @@ ves_icall_System_GCHandle_GetAddrOfPinnedObject (guint32 handle)
 }
 
 MonoBoolean
-ves_icall_Mono_Runtime_SetGCAllowSynchronousMajor (MonoBoolean flag)
-{
-	return mono_gc_set_allow_synchronous_major (flag);
-}
-
-typedef struct {
-	guint32  *bitmap;
-	gpointer *entries;
-	guint32   size;
-	guint8    type;
-	guint     slot_hint : 24; /* starting slot for search */
-	/* 2^16 appdomains should be enough for everyone (though I know I'll regret this in 20 years) */
-	/* we alloc this only for weak refs, since we can get the domain directly in the other cases */
-	guint16  *domain_ids;
-} HandleData;
-
-/* weak and weak-track arrays will be allocated in malloc memory 
- */
-static HandleData gc_handles [] = {
-	{NULL, NULL, 0, HANDLE_WEAK, 0},
-	{NULL, NULL, 0, HANDLE_WEAK_TRACK, 0},
-	{NULL, NULL, 0, HANDLE_NORMAL, 0},
-	{NULL, NULL, 0, HANDLE_PINNED, 0}
-};
-
-#define lock_handles(handles) mono_mutex_lock (&handle_section)
-#define unlock_handles(handles) mono_mutex_unlock (&handle_section)
-
-static int
-find_first_unset (guint32 bitmap)
-{
-	int i;
-	for (i = 0; i < 32; ++i) {
-		if (!(bitmap & (1 << i)))
-			return i;
-	}
-	return -1;
-}
-
-static void*
-make_root_descr_all_refs (int numbits, gboolean pinned)
-{
-#ifdef HAVE_SGEN_GC
-	if (pinned)
-		return NULL;
-#endif
-	return mono_gc_make_root_descr_all_refs (numbits);
-}
-
-static guint32
-alloc_handle (HandleData *handles, MonoObject *obj, gboolean track)
-{
-	gint slot, i;
-	guint32 res;
-	lock_handles (handles);
-	if (!handles->size) {
-		handles->size = 32;
-		if (handles->type > HANDLE_WEAK_TRACK) {
-			handles->entries = mono_gc_alloc_fixed (sizeof (gpointer) * handles->size, make_root_descr_all_refs (handles->size, handles->type == HANDLE_PINNED));
-		} else {
-			handles->entries = g_malloc0 (sizeof (gpointer) * handles->size);
-			handles->domain_ids = g_malloc0 (sizeof (guint16) * handles->size);
-		}
-		handles->bitmap = g_malloc0 (handles->size / 8);
-	}
-	i = -1;
-	for (slot = handles->slot_hint; slot < handles->size / 32; ++slot) {
-		if (handles->bitmap [slot] != 0xffffffff) {
-			i = find_first_unset (handles->bitmap [slot]);
-			handles->slot_hint = slot;
-			break;
-		}
-	}
-	if (i == -1 && handles->slot_hint != 0) {
-		for (slot = 0; slot < handles->slot_hint; ++slot) {
-			if (handles->bitmap [slot] != 0xffffffff) {
-				i = find_first_unset (handles->bitmap [slot]);
-				handles->slot_hint = slot;
-				break;
-			}
-		}
-	}
-	if (i == -1) {
-		guint32 *new_bitmap;
-		guint32 new_size = handles->size * 2; /* always double: we memset to 0 based on this below */
-
-		/* resize and copy the bitmap */
-		new_bitmap = g_malloc0 (new_size / 8);
-		memcpy (new_bitmap, handles->bitmap, handles->size / 8);
-		g_free (handles->bitmap);
-		handles->bitmap = new_bitmap;
-
-		/* resize and copy the entries */
-		if (handles->type > HANDLE_WEAK_TRACK) {
-			gpointer *entries;
-
-			entries = mono_gc_alloc_fixed (sizeof (gpointer) * new_size, make_root_descr_all_refs (new_size, handles->type == HANDLE_PINNED));
-			mono_gc_memmove_aligned (entries, handles->entries, sizeof (gpointer) * handles->size);
-
-			mono_gc_free_fixed (handles->entries);
-			handles->entries = entries;
-		} else {
-			gpointer *entries;
-			guint16 *domain_ids;
-			domain_ids = g_malloc0 (sizeof (guint16) * new_size);
-			entries = g_malloc0 (sizeof (gpointer) * new_size);
-			memcpy (domain_ids, handles->domain_ids, sizeof (guint16) * handles->size);
-			for (i = 0; i < handles->size; ++i) {
-				MonoObject *obj = mono_gc_weak_link_get (&(handles->entries [i]));
-				if (obj) {
-					mono_gc_weak_link_add (&(entries [i]), obj, track);
-					mono_gc_weak_link_remove (&(handles->entries [i]), track);
-				} else {
-					g_assert (!handles->entries [i]);
-				}
-			}
-			g_free (handles->entries);
-			g_free (handles->domain_ids);
-			handles->entries = entries;
-			handles->domain_ids = domain_ids;
-		}
-
-		/* set i and slot to the next free position */
-		i = 0;
-		slot = (handles->size + 1) / 32;
-		handles->slot_hint = handles->size + 1;
-		handles->size = new_size;
-	}
-	handles->bitmap [slot] |= 1 << i;
-	slot = slot * 32 + i;
-	handles->entries [slot] = NULL;
-	if (handles->type <= HANDLE_WEAK_TRACK) {
-		/*FIXME, what to use when obj == null?*/
-		handles->domain_ids [slot] = (obj ? mono_object_get_domain (obj) : mono_domain_get ())->domain_id;
-		if (obj)
-			mono_gc_weak_link_add (&(handles->entries [slot]), obj, track);
-	} else {
-		handles->entries [slot] = obj;
-	}
-
-#ifndef DISABLE_PERFCOUNTERS
-	mono_perfcounters->gc_num_handles++;
-#endif
-	unlock_handles (handles);
-	/*g_print ("allocated entry %d of type %d to object %p (in slot: %p)\n", slot, handles->type, obj, handles->entries [slot]);*/
-	res = (slot << 3) | (handles->type + 1);
-	mono_profiler_gc_handle (MONO_PROFILER_GC_HANDLE_CREATED, handles->type, res, obj);
-	return res;
-}
-
-/**
- * mono_gchandle_new:
- * @obj: managed object to get a handle for
- * @pinned: whether the object should be pinned
- *
- * This returns a handle that wraps the object, this is used to keep a
- * reference to a managed object from the unmanaged world and preventing the
- * object from being disposed.
- * 
- * If @pinned is false the address of the object can not be obtained, if it is
- * true the address of the object can be obtained.  This will also pin the
- * object so it will not be possible by a moving garbage collector to move the
- * object. 
- * 
- * Returns: a handle that can be used to access the object from
- * unmanaged code.
- */
-guint32
-mono_gchandle_new (MonoObject *obj, gboolean pinned)
-{
-	return alloc_handle (&gc_handles [pinned? HANDLE_PINNED: HANDLE_NORMAL], obj, FALSE);
-}
-
-/**
- * mono_gchandle_new_weakref:
- * @obj: managed object to get a handle for
- * @pinned: whether the object should be pinned
- *
- * This returns a weak handle that wraps the object, this is used to
- * keep a reference to a managed object from the unmanaged world.
- * Unlike the mono_gchandle_new the object can be reclaimed by the
- * garbage collector.  In this case the value of the GCHandle will be
- * set to zero.
- * 
- * If @pinned is false the address of the object can not be obtained, if it is
- * true the address of the object can be obtained.  This will also pin the
- * object so it will not be possible by a moving garbage collector to move the
- * object. 
- * 
- * Returns: a handle that can be used to access the object from
- * unmanaged code.
- */
-guint32
-mono_gchandle_new_weakref (MonoObject *obj, gboolean track_resurrection)
-{
-	guint32 handle = alloc_handle (&gc_handles [track_resurrection? HANDLE_WEAK_TRACK: HANDLE_WEAK], obj, track_resurrection);
-
-	return handle;
-}
-
-static HandleType
-mono_gchandle_get_type (guint32 gchandle)
-{
-	guint type = (gchandle & 7) - 1;
-
-	return type;
-}
-
-/**
- * mono_gchandle_get_target:
- * @gchandle: a GCHandle's handle.
- *
- * The handle was previously created by calling mono_gchandle_new or
- * mono_gchandle_new_weakref. 
- *
- * Returns a pointer to the MonoObject represented by the handle or
- * NULL for a collected object if using a weakref handle.
- */
-MonoObject*
-mono_gchandle_get_target (guint32 gchandle)
-{
-	guint slot = gchandle >> 3;
-	guint type = (gchandle & 7) - 1;
-	HandleData *handles = &gc_handles [type];
-	MonoObject *obj = NULL;
-	if (type > 3)
-		return NULL;
-	lock_handles (handles);
-	if (slot < handles->size && (handles->bitmap [slot / 32] & (1 << (slot % 32)))) {
-		if (handles->type <= HANDLE_WEAK_TRACK) {
-			obj = mono_gc_weak_link_get (&handles->entries [slot]);
-		} else {
-			obj = handles->entries [slot];
-		}
-	} else {
-		/* print a warning? */
-	}
-	unlock_handles (handles);
-	/*g_print ("get target of entry %d of type %d: %p\n", slot, handles->type, obj);*/
-	return obj;
-}
-
-static void
-mono_gchandle_set_target (guint32 gchandle, MonoObject *obj)
-{
-	guint slot = gchandle >> 3;
-	guint type = (gchandle & 7) - 1;
-	HandleData *handles = &gc_handles [type];
-	MonoObject *old_obj = NULL;
-
-	if (type > 3)
-		return;
-	lock_handles (handles);
-	if (slot < handles->size && (handles->bitmap [slot / 32] & (1 << (slot % 32)))) {
-		if (handles->type <= HANDLE_WEAK_TRACK) {
-			old_obj = handles->entries [slot];
-			if (handles->entries [slot])
-				mono_gc_weak_link_remove (&handles->entries [slot], handles->type == HANDLE_WEAK_TRACK);
-			if (obj)
-				mono_gc_weak_link_add (&handles->entries [slot], obj, handles->type == HANDLE_WEAK_TRACK);
-			/*FIXME, what to use when obj == null?*/
-			handles->domain_ids [slot] = (obj ? mono_object_get_domain (obj) : mono_domain_get ())->domain_id;
-		} else {
-			handles->entries [slot] = obj;
-		}
-	} else {
-		/* print a warning? */
-	}
-	/*g_print ("changed entry %d of type %d to object %p (in slot: %p)\n", slot, handles->type, obj, handles->entries [slot]);*/
-	unlock_handles (handles);
-}
-
-/**
- * mono_gchandle_is_in_domain:
- * @gchandle: a GCHandle's handle.
- * @domain: An application domain.
- *
- * Returns: true if the object wrapped by the @gchandle belongs to the specific @domain.
- */
-gboolean
-mono_gchandle_is_in_domain (guint32 gchandle, MonoDomain *domain)
-{
-	guint slot = gchandle >> 3;
-	guint type = (gchandle & 7) - 1;
-	HandleData *handles = &gc_handles [type];
-	gboolean result = FALSE;
-	if (type > 3)
-		return FALSE;
-	lock_handles (handles);
-	if (slot < handles->size && (handles->bitmap [slot / 32] & (1 << (slot % 32)))) {
-		if (handles->type <= HANDLE_WEAK_TRACK) {
-			result = domain->domain_id == handles->domain_ids [slot];
-		} else {
-			MonoObject *obj;
-			obj = handles->entries [slot];
-			if (obj == NULL)
-				result = TRUE;
-			else
-				result = domain == mono_object_domain (obj);
-		}
-	} else {
-		/* print a warning? */
-	}
-	unlock_handles (handles);
-	return result;
-}
-
-/**
- * mono_gchandle_free:
- * @gchandle: a GCHandle's handle.
- *
- * Frees the @gchandle handle.  If there are no outstanding
- * references, the garbage collector can reclaim the memory of the
- * object wrapped. 
- */
-void
-mono_gchandle_free (guint32 gchandle)
-{
-	guint slot = gchandle >> 3;
-	guint type = (gchandle & 7) - 1;
-	HandleData *handles = &gc_handles [type];
-	if (type > 3)
-		return;
-
-	lock_handles (handles);
-	if (slot < handles->size && (handles->bitmap [slot / 32] & (1 << (slot % 32)))) {
-		if (handles->type <= HANDLE_WEAK_TRACK) {
-			if (handles->entries [slot])
-				mono_gc_weak_link_remove (&handles->entries [slot], handles->type == HANDLE_WEAK_TRACK);
-		} else {
-			handles->entries [slot] = NULL;
-		}
-		handles->bitmap [slot / 32] &= ~(1 << (slot % 32));
-	} else {
-		/* print a warning? */
-	}
-#ifndef DISABLE_PERFCOUNTERS
-	mono_perfcounters->gc_num_handles--;
-#endif
-	/*g_print ("freed entry %d of type %d\n", slot, handles->type);*/
-	unlock_handles (handles);
-	mono_profiler_gc_handle (MONO_PROFILER_GC_HANDLE_DESTROYED, handles->type, gchandle, NULL);
-}
-
-/**
- * mono_gchandle_free_domain:
- * @domain: domain that is unloading
- *
- * Function used internally to cleanup any GC handle for objects belonging
- * to the specified domain during appdomain unload.
- */
-void
-mono_gchandle_free_domain (MonoDomain *domain)
-{
-	guint type;
-
-	for (type = 0; type < 3; ++type) {
-		guint slot;
-		HandleData *handles = &gc_handles [type];
-		lock_handles (handles);
-		for (slot = 0; slot < handles->size; ++slot) {
-			if (!(handles->bitmap [slot / 32] & (1 << (slot % 32))))
-				continue;
-			if (type <= HANDLE_WEAK_TRACK) {
-				if (domain->domain_id == handles->domain_ids [slot]) {
-					handles->bitmap [slot / 32] &= ~(1 << (slot % 32));
-					if (handles->entries [slot])
-						mono_gc_weak_link_remove (&handles->entries [slot], handles->type == HANDLE_WEAK_TRACK);
-				}
-			} else {
-				if (handles->entries [slot] && mono_object_domain (handles->entries [slot]) == domain) {
-					handles->bitmap [slot / 32] &= ~(1 << (slot % 32));
-					handles->entries [slot] = NULL;
-				}
-			}
-		}
-		unlock_handles (handles);
-	}
-
-}
-
-MonoBoolean
-GCHandle_CheckCurrentDomain (guint32 gchandle)
+mono_gc_GCHandle_CheckCurrentDomain (guint32 gchandle)
 {
 	return mono_gchandle_is_in_domain (gchandle, mono_domain_get ());
 }
 
-#ifndef HAVE_NULL_GC
-
-#ifdef MONO_HAS_SEMAPHORES
-static MonoSemType finalizer_sem;
-#endif
-static HANDLE finalizer_event;
+static MonoCoopSem finalizer_sem;
 static volatile gboolean finished=FALSE;
 
 void
@@ -982,11 +617,10 @@ mono_gc_finalize_notify (void)
 	g_message ( "%s: prodding finalizer", __func__);
 #endif
 
-#ifdef MONO_HAS_SEMAPHORES
-	MONO_SEM_POST (&finalizer_sem);
-#else
-	SetEvent (finalizer_event);
-#endif
+	if (mono_gc_is_null ())
+		return;
+
+	mono_coop_sem_post (&finalizer_sem);
 }
 
 #ifdef HAVE_BOEHM_GC
@@ -1070,29 +704,26 @@ finalizer_thread (gpointer unused)
 		 */
 
 		g_assert (mono_domain_get () == mono_get_root_domain ());
+		mono_gc_set_skip_thread (TRUE);
 
 		if (wait) {
-		/* An alertable wait is required so this thread can be suspended on windows */
-#ifdef MONO_HAS_SEMAPHORES
-			MONO_SEM_WAIT_ALERTABLE (&finalizer_sem, TRUE);
-#else
-			WaitForSingleObjectEx (finalizer_event, INFINITE, TRUE);
-#endif
+			/* An alertable wait is required so this thread can be suspended on windows */
+			mono_coop_sem_wait (&finalizer_sem, MONO_SEM_FLAGS_ALERTABLE);
 		}
 		wait = TRUE;
 
+		mono_gc_set_skip_thread (FALSE);
+
 		mono_threads_perform_thread_dump ();
 
 		mono_console_handle_async_ops ();
 
-#ifndef DISABLE_ATTACH
 		mono_attach_maybe_start ();
-#endif
 
 		if (domains_to_finalize) {
 			mono_finalizer_lock ();
 			if (domains_to_finalize) {
-				DomainFinalizationReq *req = domains_to_finalize->data;
+				DomainFinalizationReq *req = (DomainFinalizationReq *)domains_to_finalize->data;
 				domains_to_finalize = g_slist_remove (domains_to_finalize, req);
 				mono_finalizer_unlock ();
 
@@ -1111,19 +742,20 @@ finalizer_thread (gpointer unused)
 
 		reference_queue_proccess_all ();
 
-#ifdef MONO_HAS_SEMAPHORES
 		/* Avoid posting the pending done event until there are pending finalizers */
-		if (MONO_SEM_TIMEDWAIT (&finalizer_sem, 0) == 0)
+		if (mono_coop_sem_timedwait (&finalizer_sem, 0, MONO_SEM_FLAGS_NONE) == 0) {
 			/* Don't wait again at the start of the loop */
 			wait = FALSE;
-		else
+		} else {
 			SetEvent (pending_done_event);
-#else
-			SetEvent (pending_done_event);
-#endif
+		}
 	}
 
-	SetEvent (shutdown_event);
+	mono_finalizer_lock ();
+	finalizer_thread_exited = TRUE;
+	mono_coop_cond_signal (&exited_cond);
+	mono_finalizer_unlock ();
+
 	return 0;
 }
 
@@ -1140,28 +772,14 @@ mono_gc_init_finalizer_thread (void)
 void
 mono_gc_init (void)
 {
-	mono_mutex_init_recursive (&handle_section);
-	mono_mutex_init_recursive (&allocator_section);
+	mono_coop_mutex_init_recursive (&finalizer_mutex);
+	mono_coop_mutex_init_recursive (&reference_queue_mutex);
 
-	mono_mutex_init_recursive (&finalizer_mutex);
-	mono_mutex_init_recursive (&reference_queue_mutex);
-
-	MONO_GC_REGISTER_ROOT_FIXED (gc_handles [HANDLE_NORMAL].entries);
-	MONO_GC_REGISTER_ROOT_FIXED (gc_handles [HANDLE_PINNED].entries);
-
-	mono_counters_register ("Created object count", MONO_COUNTER_GC | MONO_COUNTER_ULONG, &mono_stats.new_object_count);
-	mono_counters_register ("Minor GC collections", MONO_COUNTER_GC | MONO_COUNTER_INT, &gc_stats.minor_gc_count);
-	mono_counters_register ("Major GC collections", MONO_COUNTER_GC | MONO_COUNTER_INT, &gc_stats.major_gc_count);
-	mono_counters_register ("Minor GC time", MONO_COUNTER_GC | MONO_COUNTER_LONG | MONO_COUNTER_TIME, &gc_stats.minor_gc_time);
-	mono_counters_register ("Major GC time", MONO_COUNTER_GC | MONO_COUNTER_LONG | MONO_COUNTER_TIME, &gc_stats.major_gc_time);
-#ifdef HEAVY_STATISTICS
-	mono_counters_register ("Gray Queue alloc section", MONO_COUNTER_GC | MONO_COUNTER_ULONG, &gc_stats.gray_queue_section_alloc);
-	mono_counters_register ("Gray Queue free section", MONO_COUNTER_GC | MONO_COUNTER_ULONG, &gc_stats.gray_queue_section_free);
-	mono_counters_register ("Gray Queue enqueue fast path", MONO_COUNTER_GC | MONO_COUNTER_ULONG, &gc_stats.gray_queue_enqueue_fast_path);
-	mono_counters_register ("Gray Queue dequeue fast path", MONO_COUNTER_GC | MONO_COUNTER_ULONG, &gc_stats.gray_queue_dequeue_fast_path);
-	mono_counters_register ("Gray Queue enqueue slow path", MONO_COUNTER_GC | MONO_COUNTER_ULONG, &gc_stats.gray_queue_enqueue_slow_path);
-	mono_counters_register ("Gray Queue dequeue slow path", MONO_COUNTER_GC | MONO_COUNTER_ULONG, &gc_stats.gray_queue_dequeue_slow_path);
-#endif
+	mono_counters_register ("Minor GC collections", MONO_COUNTER_GC | MONO_COUNTER_UINT, &gc_stats.minor_gc_count);
+	mono_counters_register ("Major GC collections", MONO_COUNTER_GC | MONO_COUNTER_UINT, &gc_stats.major_gc_count);
+	mono_counters_register ("Minor GC time", MONO_COUNTER_GC | MONO_COUNTER_ULONG | MONO_COUNTER_TIME, &gc_stats.minor_gc_time);
+	mono_counters_register ("Major GC time", MONO_COUNTER_GC | MONO_COUNTER_ULONG | MONO_COUNTER_TIME, &gc_stats.major_gc_time);
+	mono_counters_register ("Major GC time concurrent", MONO_COUNTER_GC | MONO_COUNTER_ULONG | MONO_COUNTER_TIME, &gc_stats.major_gc_time_concurrent);
 
 	mono_gc_base_init ();
 
@@ -1169,16 +787,11 @@ mono_gc_init (void)
 		gc_disabled = TRUE;
 		return;
 	}
-	
-	finalizer_event = CreateEvent (NULL, FALSE, FALSE, NULL);
+
 	pending_done_event = CreateEvent (NULL, TRUE, FALSE, NULL);
-	shutdown_event = CreateEvent (NULL, TRUE, FALSE, NULL);
-	if (finalizer_event == NULL || pending_done_event == NULL || shutdown_event == NULL) {
-		g_assert_not_reached ();
-	}
-#ifdef MONO_HAS_SEMAPHORES
-	MONO_SEM_INIT (&finalizer_sem, 0);
-#endif
+	g_assert (pending_done_event);
+	mono_coop_cond_init (&exited_cond);
+	mono_coop_sem_init (&finalizer_sem, 0);
 
 #ifndef LAZY_GC_THREAD_CREATION
 	mono_gc_init_finalizer_thread ();
@@ -1192,16 +805,34 @@ mono_gc_cleanup (void)
 	g_message ("%s: cleaning up finalizer", __func__);
 #endif
 
+	if (mono_gc_is_null ())
+		return;
+
 	if (!gc_disabled) {
-		ResetEvent (shutdown_event);
 		finished = TRUE;
 		if (mono_thread_internal_current () != gc_thread) {
 			gboolean timed_out = FALSE;
+			guint32 start_ticks = mono_msec_ticks ();
+			guint32 end_ticks = start_ticks + 2000;
 
 			mono_gc_finalize_notify ();
 			/* Finishing the finalizer thread, so wait a little bit... */
 			/* MS seems to wait for about 2 seconds */
-			if (WaitForSingleObjectEx (shutdown_event, 2000, FALSE) == WAIT_TIMEOUT) {
+			while (!finalizer_thread_exited) {
+				guint32 current_ticks = mono_msec_ticks ();
+				guint32 timeout;
+
+				if (current_ticks >= end_ticks)
+					break;
+				else
+					timeout = end_ticks - current_ticks;
+				mono_finalizer_lock ();
+				if (!finalizer_thread_exited)
+					mono_coop_cond_timedwait (&exited_cond, &finalizer_mutex, timeout);
+				mono_finalizer_unlock ();
+			}
+
+			if (!finalizer_thread_exited) {
 				int ret;
 
 				/* Set a flag which the finalizer thread can check */
@@ -1211,7 +842,7 @@ mono_gc_cleanup (void)
 				mono_thread_internal_stop (gc_thread);
 
 				/* Wait for it to stop */
-				ret = WaitForSingleObjectEx (gc_thread->handle, 100, TRUE);
+				ret = guarded_wait (gc_thread->handle, 100, TRUE);
 
 				if (ret == WAIT_TIMEOUT) {
 					/* 
@@ -1228,45 +859,23 @@ mono_gc_cleanup (void)
 				int ret;
 
 				/* Wait for the thread to actually exit */
-				ret = WaitForSingleObjectEx (gc_thread->handle, INFINITE, TRUE);
+				ret = guarded_wait (gc_thread->handle, INFINITE, TRUE);
 				g_assert (ret == WAIT_OBJECT_0);
 
-				mono_thread_join ((gpointer)gc_thread->tid);
+				mono_thread_join (GUINT_TO_POINTER (gc_thread->tid));
 			}
+			g_assert (finalizer_thread_exited);
 		}
 		gc_thread = NULL;
-#ifdef HAVE_BOEHM_GC
-		GC_finalizer_notifier = NULL;
-#endif
+		mono_gc_base_cleanup ();
 	}
 
 	mono_reference_queue_cleanup ();
 
-	mono_mutex_destroy (&handle_section);
-	mono_mutex_destroy (&allocator_section);
-	mono_mutex_destroy (&finalizer_mutex);
-	mono_mutex_destroy (&reference_queue_mutex);
+	mono_coop_mutex_destroy (&finalizer_mutex);
+	mono_coop_mutex_destroy (&reference_queue_mutex);
 }
 
-#else
-
-/* Null GC dummy functions */
-void
-mono_gc_finalize_notify (void)
-{
-}
-
-void mono_gc_init (void)
-{
-	mono_mutex_init_recursive (&handle_section);
-}
-
-void mono_gc_cleanup (void)
-{
-}
-
-#endif
-
 gboolean
 mono_gc_is_finalizer_internal_thread (MonoInternalThread *thread)
 {
@@ -1305,76 +914,6 @@ mono_gc_get_mach_exception_thread (void)
 }
 #endif
 
-/**
- * mono_gc_parse_environment_string_extract_number:
- *
- * @str: points to the first digit of the number
- * @out: pointer to the variable that will receive the value
- *
- * Tries to extract a number from the passed string, taking in to account m, k
- * and g suffixes
- *
- * Returns true if passing was successful
- */
-gboolean
-mono_gc_parse_environment_string_extract_number (const char *str, size_t *out)
-{
-	char *endptr;
-	int len = strlen (str), shift = 0;
-	size_t val;
-	gboolean is_suffix = FALSE;
-	char suffix;
-
-	if (!len)
-		return FALSE;
-
-	suffix = str [len - 1];
-
-	switch (suffix) {
-		case 'g':
-		case 'G':
-			shift += 10;
-		case 'm':
-		case 'M':
-			shift += 10;
-		case 'k':
-		case 'K':
-			shift += 10;
-			is_suffix = TRUE;
-			break;
-		default:
-			if (!isdigit (suffix))
-				return FALSE;
-			break;
-	}
-
-	errno = 0;
-	val = strtol (str, &endptr, 10);
-
-	if ((errno == ERANGE && (val == LONG_MAX || val == LONG_MIN))
-			|| (errno != 0 && val == 0) || (endptr == str))
-		return FALSE;
-
-	if (is_suffix) {
-		size_t unshifted;
-
-		if (val < 0)	/* negative numbers cannot be suffixed */
-			return FALSE;
-		if (*(endptr + 1)) /* Invalid string. */
-			return FALSE;
-
-		unshifted = (size_t)val;
-		val <<= shift;
-		if (val < 0)	/* overflow */
-			return FALSE;
-		if (((size_t)val >> shift) != unshifted) /* value too large */
-			return FALSE;
-	}
-
-	*out = val;
-	return TRUE;
-}
-
 #ifndef HAVE_SGEN_GC
 void*
 mono_gc_alloc_mature (MonoVTable *vtable)
@@ -1393,7 +932,7 @@ ref_list_remove_element (RefQueueEntry **prev, RefQueueEntry *element)
 		/* Guard if head is changed concurrently. */
 		while (*prev != element)
 			prev = &(*prev)->next;
-	} while (prev && InterlockedCompareExchangePointer ((void*)prev, element->next, element) != element);
+	} while (prev && InterlockedCompareExchangePointer ((volatile gpointer *)prev, element->next, element) != element);
 }
 
 static void
@@ -1404,7 +943,7 @@ ref_list_push (RefQueueEntry **head, RefQueueEntry *value)
 		current = *head;
 		value->next = current;
 		STORE_STORE_FENCE; /*Must make sure the previous store is visible before the CAS. */
-	} while (InterlockedCompareExchangePointer ((void*)head, value, current) != current);
+	} while (InterlockedCompareExchangePointer ((volatile gpointer *)head, value, current) != current);
 }
 
 static void
@@ -1413,13 +952,8 @@ reference_queue_proccess (MonoReferenceQueue *queue)
 	RefQueueEntry **iter = &queue->queue;
 	RefQueueEntry *entry;
 	while ((entry = *iter)) {
-#ifdef HAVE_SGEN_GC
-		if (queue->should_be_deleted || !mono_gc_weak_link_get (&entry->dis_link)) {
-			mono_gc_weak_link_remove (&entry->dis_link, TRUE);
-#else
 		if (queue->should_be_deleted || !mono_gchandle_get_target (entry->gchandle)) {
 			mono_gchandle_free ((guint32)entry->gchandle);
-#endif
 			ref_list_remove_element (iter, entry);
 			queue->callback (entry->user_data);
 			g_free (entry);
@@ -1438,7 +972,7 @@ reference_queue_proccess_all (void)
 		reference_queue_proccess (queue);
 
 restart:
-	mono_mutex_lock (&reference_queue_mutex);
+	mono_coop_mutex_lock (&reference_queue_mutex);
 	for (iter = &ref_queues; *iter;) {
 		queue = *iter;
 		if (!queue->should_be_deleted) {
@@ -1446,14 +980,14 @@ restart:
 			continue;
 		}
 		if (queue->queue) {
-			mono_mutex_unlock (&reference_queue_mutex);
+			mono_coop_mutex_unlock (&reference_queue_mutex);
 			reference_queue_proccess (queue);
 			goto restart;
 		}
 		*iter = queue->next;
 		g_free (queue);
 	}
-	mono_mutex_unlock (&reference_queue_mutex);
+	mono_coop_mutex_unlock (&reference_queue_mutex);
 }
 
 static void
@@ -1474,11 +1008,7 @@ reference_queue_clear_for_domain (MonoDomain *domain)
 		RefQueueEntry *entry;
 		while ((entry = *iter)) {
 			if (entry->domain == domain) {
-#ifdef HAVE_SGEN_GC
-				mono_gc_weak_link_remove (&entry->dis_link, TRUE);
-#else
 				mono_gchandle_free ((guint32)entry->gchandle);
-#endif
 				ref_list_remove_element (iter, entry);
 				queue->callback (entry->user_data);
 				g_free (entry);
@@ -1511,10 +1041,10 @@ mono_gc_reference_queue_new (mono_reference_queue_callback callback)
 	MonoReferenceQueue *res = g_new0 (MonoReferenceQueue, 1);
 	res->callback = callback;
 
-	mono_mutex_lock (&reference_queue_mutex);
+	mono_coop_mutex_lock (&reference_queue_mutex);
 	res->next = ref_queues;
 	ref_queues = res;
-	mono_mutex_unlock (&reference_queue_mutex);
+	mono_coop_mutex_unlock (&reference_queue_mutex);
 
 	return res;
 }
@@ -1542,12 +1072,8 @@ mono_gc_reference_queue_add (MonoReferenceQueue *queue, MonoObject *obj, void *u
 	entry->user_data = user_data;
 	entry->domain = mono_object_domain (obj);
 
-#ifdef HAVE_SGEN_GC
-	mono_gc_weak_link_add (&entry->dis_link, obj, TRUE);
-#else
 	entry->gchandle = mono_gchandle_new_weakref (obj, TRUE);
 	mono_object_register_finalizer (obj);
-#endif
 
 	ref_list_push (&queue->queue, entry);
 	return TRUE;
