@@ -39,12 +39,13 @@
 #include <mono/metadata/marshal.h>
 #include <mono/metadata/lock-tracer.h>
 #include <mono/metadata/verify-internals.h>
-#include <mono/utils/mono-logger-internal.h>
+#include <mono/utils/mono-logger-internals.h>
 #include <mono/utils/mono-dl.h>
 #include <mono/utils/mono-membar.h>
 #include <mono/utils/mono-counters.h>
 #include <mono/utils/mono-error-internals.h>
 #include <mono/utils/mono-tls.h>
+#include <mono/utils/mono-path.h>
 
 MonoDefaults mono_defaults;
 
@@ -55,7 +56,8 @@ MonoDefaults mono_defaults;
  * See domain-internals.h for locking policy in combination with the
  * domain lock.
  */
-static mono_mutex_t loader_mutex, global_loader_data_mutex;
+static MonoCoopMutex loader_mutex;
+static mono_mutex_t global_loader_data_mutex;
 static gboolean loader_lock_inited;
 
 /* Statistics */
@@ -81,13 +83,13 @@ static void dllmap_cleanup (void);
 static void
 global_loader_data_lock (void)
 {
-	mono_locks_acquire (&global_loader_data_mutex, LoaderGlobalDataLock);
+	mono_locks_os_acquire (&global_loader_data_mutex, LoaderGlobalDataLock);
 }
 
 static void
 global_loader_data_unlock (void)
 {
-	mono_locks_release (&global_loader_data_mutex, LoaderGlobalDataLock);
+	mono_locks_os_release (&global_loader_data_mutex, LoaderGlobalDataLock);
 }
 
 void
@@ -96,8 +98,8 @@ mono_loader_init ()
 	static gboolean inited;
 
 	if (!inited) {
-		mono_mutex_init_recursive (&loader_mutex);
-		mono_mutex_init_recursive (&global_loader_data_mutex);
+		mono_coop_mutex_init_recursive (&loader_mutex);
+		mono_os_mutex_init_recursive (&global_loader_data_mutex);
 		loader_lock_inited = TRUE;
 
 		mono_native_tls_alloc (&loader_error_thread_id, NULL);
@@ -125,8 +127,8 @@ mono_loader_cleanup (void)
 	mono_native_tls_free (loader_error_thread_id);
 	mono_native_tls_free (loader_lock_nest_id);
 
-	mono_mutex_destroy (&loader_mutex);
-	mono_mutex_destroy (&global_loader_data_mutex);
+	mono_coop_mutex_destroy (&loader_mutex);
+	mono_os_mutex_destroy (&global_loader_data_mutex);
 	loader_lock_inited = FALSE;	
 }
 
@@ -287,6 +289,17 @@ mono_loader_get_last_error (void)
 	return (MonoLoaderError*)mono_native_tls_get_value (loader_error_thread_id);
 }
 
+void
+mono_loader_assert_no_error (void)
+{
+	MonoLoaderError *error = mono_loader_get_last_error ();
+
+	if (error) {
+		g_print ("Unhandled loader error: %x, %s %s %s\n", error->exception_type, error->msg, error->assembly_name, error->class_name);
+		g_assert_not_reached ();
+	}
+}
+
 /**
  * mono_loader_clear_error:
  *
@@ -448,7 +461,7 @@ field_from_memberref (MonoImage *image, guint32 token, MonoClass **retklass,
 	MonoTableInfo *tables = image->tables;
 	MonoType *sig_type;
 	guint32 cols[6];
-	guint32 nindex, class, class_table;
+	guint32 nindex, class_index;
 	const char *fname;
 	const char *ptr;
 	guint32 idx = mono_metadata_token_index (token);
@@ -457,30 +470,27 @@ field_from_memberref (MonoImage *image, guint32 token, MonoClass **retklass,
 
 	mono_metadata_decode_row (&tables [MONO_TABLE_MEMBERREF], idx-1, cols, MONO_MEMBERREF_SIZE);
 	nindex = cols [MONO_MEMBERREF_CLASS] >> MONO_MEMBERREF_PARENT_BITS;
-	class = cols [MONO_MEMBERREF_CLASS] & MONO_MEMBERREF_PARENT_MASK;
+	class_index = cols [MONO_MEMBERREF_CLASS] & MONO_MEMBERREF_PARENT_MASK;
 
 	fname = mono_metadata_string_heap (image, cols [MONO_MEMBERREF_NAME]);
 
 	if (!mono_verifier_verify_memberref_field_signature (image, cols [MONO_MEMBERREF_SIGNATURE], NULL)) {
-		mono_error_set_bad_image (error, image, "Bad field '%s' signature 0x%08x", class, token);
+		mono_error_set_bad_image (error, image, "Bad field '%s' signature 0x%08x", class_index, token);
 		return NULL;
 	}
 
-	switch (class) {
+	switch (class_index) {
 	case MONO_MEMBERREF_PARENT_TYPEDEF:
-		class_table = MONO_TOKEN_TYPE_DEF;
 		klass = mono_class_get_checked (image, MONO_TOKEN_TYPE_DEF | nindex, error);
 		break;
 	case MONO_MEMBERREF_PARENT_TYPEREF:
-		class_table = MONO_TOKEN_TYPE_REF;
 		klass = mono_class_from_typeref_checked (image, MONO_TOKEN_TYPE_REF | nindex, error);
 		break;
 	case MONO_MEMBERREF_PARENT_TYPESPEC:
-		class_table = MONO_TOKEN_TYPE_SPEC;
 		klass = mono_class_get_and_inflate_typespec_checked (image, MONO_TOKEN_TYPE_SPEC | nindex, context, error);
 		break;
 	default:
-		mono_error_set_bad_image (error, image, "Bad field field '%s' signature 0x%08x", class, token);
+		mono_error_set_bad_image (error, image, "Bad field field '%s' signature 0x%08x", class_index, token);
 	}
 
 	if (!klass)
@@ -491,21 +501,21 @@ field_from_memberref (MonoImage *image, guint32 token, MonoClass **retklass,
 	/* we may want to check the signature here... */
 
 	if (*ptr++ != 0x6) {
-		mono_error_set_field_load (error, klass, fname, "Bad field signature class token %08x field name %s token %08x", class, fname, token);
+		mono_error_set_field_load (error, klass, fname, "Bad field signature class token %08x field name %s token %08x", class_index, fname, token);
 		return NULL;
 	}
 
 	/* FIXME: This needs a cache, especially for generic instances, since
 	 * mono_metadata_parse_type () allocates everything from a mempool.
 	 */
-	sig_type = find_cached_memberref_sig (image, cols [MONO_MEMBERREF_SIGNATURE]);
+	sig_type = (MonoType *)find_cached_memberref_sig (image, cols [MONO_MEMBERREF_SIGNATURE]);
 	if (!sig_type) {
 		sig_type = mono_metadata_parse_type (image, MONO_PARSE_TYPE, 0, ptr, &ptr);
 		if (sig_type == NULL) {
 			mono_error_set_field_load (error, klass, fname, "Could not parse field '%s' signature %08x", fname, token);
 			return NULL;
 		}
-		sig_type = cache_memberref_sig (image, cols [MONO_MEMBERREF_SIGNATURE], sig_type);
+		sig_type = (MonoType *)cache_memberref_sig (image, cols [MONO_MEMBERREF_SIGNATURE], sig_type);
 	}
 
 	mono_class_init (klass); /*FIXME is this really necessary?*/
@@ -514,7 +524,7 @@ field_from_memberref (MonoImage *image, guint32 token, MonoClass **retklass,
 	field = mono_class_get_field_from_name_full (klass, fname, sig_type);
 
 	if (!field) {
-		g_assert (!mono_loader_get_last_error ());
+		mono_loader_assert_no_error ();
 		mono_error_set_field_load (error, klass, fname, "Could not find field '%s'", fname);
 	}
 
@@ -530,7 +540,7 @@ mono_field_from_token (MonoImage *image, guint32 token, MonoClass **retklass, Mo
 {
 	MonoError error;
 	MonoClassField *res = mono_field_from_token_checked (image, token, retklass, context, &error);
-	g_assert (!mono_loader_get_last_error ());
+	mono_loader_assert_no_error ();
 	if (!mono_error_ok (&error)) {
 		mono_loader_set_error_from_mono_error (&error);
 		mono_error_cleanup (&error);
@@ -552,7 +562,7 @@ mono_field_from_token_checked (MonoImage *image, guint32 token, MonoClass **retk
 		MonoClass *handle_class;
 
 		*retklass = NULL;
-		result = mono_lookup_dynamic_token_class (image, token, TRUE, &handle_class, context);
+		result = (MonoClassField *)mono_lookup_dynamic_token_class (image, token, TRUE, &handle_class, context);
 		// This checks the memberref type as well
 		if (!result || handle_class != mono_defaults.fieldhandle_class) {
 			mono_error_set_bad_image (error, image, "Bad field token 0x%08x", token);
@@ -562,14 +572,14 @@ mono_field_from_token_checked (MonoImage *image, guint32 token, MonoClass **retk
 		return result;
 	}
 
-	if ((field = mono_conc_hashtable_lookup (image->field_cache, GUINT_TO_POINTER (token)))) {
+	if ((field = (MonoClassField *)mono_conc_hashtable_lookup (image->field_cache, GUINT_TO_POINTER (token)))) {
 		*retklass = field->parent;
 		return field;
 	}
 
 	if (mono_metadata_token_table (token) == MONO_TABLE_MEMBERREF) {
 		field = field_from_memberref (image, token, retklass, context, error);
-		g_assert (!mono_loader_get_last_error ());
+		mono_loader_assert_no_error ();
 	} else {
 		type = mono_metadata_typedef_from_field (image, mono_metadata_token_index (token));
 		if (!type) {
@@ -592,10 +602,13 @@ mono_field_from_token_checked (MonoImage *image, guint32 token, MonoClass **retk
 		}
 	}
 
-	if (field && field->parent && !field->parent->generic_class && !field->parent->generic_container)
+	if (field && field->parent && !field->parent->generic_class && !field->parent->generic_container) {
+		mono_image_lock (image);
 		mono_conc_hashtable_insert (image->field_cache, GUINT_TO_POINTER (token), field);
+		mono_image_unlock (image);
+	}
 
-	g_assert (!mono_loader_get_last_error ());
+	mono_loader_assert_no_error ();
 	return field;
 }
 
@@ -802,7 +815,7 @@ inflate_generic_signature_checked (MonoImage *image, MonoMethodSignature *sig, M
 	if (!context)
 		return sig;
 
-	res = g_malloc0 (MONO_SIZEOF_METHOD_SIGNATURE + ((gint32)sig->param_count) * sizeof (MonoType*));
+	res = (MonoMethodSignature *)g_malloc0 (MONO_SIZEOF_METHOD_SIGNATURE + ((gint32)sig->param_count) * sizeof (MonoType*));
 	res->param_count = sig->param_count;
 	res->sentinelpos = -1;
 	res->ret = mono_class_inflate_generic_type_checked (sig->ret, context, error);
@@ -860,21 +873,25 @@ mono_inflate_generic_signature (MonoMethodSignature *sig, MonoGenericContext *co
 static MonoMethodHeader*
 inflate_generic_header (MonoMethodHeader *header, MonoGenericContext *context)
 {
-	MonoMethodHeader *res;
-	int i;
-	res = g_malloc0 (MONO_SIZEOF_METHOD_HEADER + sizeof (gpointer) * header->num_locals);
+	size_t locals_size = sizeof (gpointer) * header->num_locals;
+	size_t clauses_size = header->num_clauses * sizeof (MonoExceptionClause);
+	size_t header_size = MONO_SIZEOF_METHOD_HEADER + locals_size + clauses_size; 
+	MonoMethodHeader *res = (MonoMethodHeader *)g_malloc0 (header_size);
+	res->num_locals = header->num_locals;
+	res->clauses = (MonoExceptionClause *) &res->locals [res->num_locals] ;
+	memcpy (res->clauses, header->clauses, clauses_size);
+
 	res->code = header->code;
 	res->code_size = header->code_size;
 	res->max_stack = header->max_stack;
 	res->num_clauses = header->num_clauses;
 	res->init_locals = header->init_locals;
-	res->num_locals = header->num_locals;
-	res->clauses = header->clauses;
-	for (i = 0; i < header->num_locals; ++i)
+
+	res->is_transient = TRUE;
+	for (int i = 0; i < header->num_locals; ++i)
 		res->locals [i] = mono_class_inflate_generic_type (header->locals [i], context);
 	if (res->num_clauses) {
-		res->clauses = g_memdup (header->clauses, sizeof (MonoExceptionClause) * res->num_clauses);
-		for (i = 0; i < header->num_clauses; ++i) {
+		for (int i = 0; i < header->num_clauses; ++i) {
 			MonoExceptionClause *clause = &res->clauses [i];
 			if (clause->flags != MONO_EXCEPTION_CLAUSE_NONE)
 				continue;
@@ -893,7 +910,7 @@ mono_method_get_signature_full (MonoMethod *method, MonoImage *image, guint32 to
 	MonoError error;
 	MonoMethodSignature *res = mono_method_get_signature_checked (method, image, token, context, &error);
 
-	g_assert (!mono_loader_get_last_error ());
+	mono_loader_assert_no_error ();
 
 	if (!res) {
 		g_assert (!mono_error_ok (&error));
@@ -938,14 +955,14 @@ mono_method_get_signature_checked (MonoMethod *method, MonoImage *image, guint32
 		mono_metadata_decode_row (&image->tables [MONO_TABLE_MEMBERREF], idx-1, cols, MONO_MEMBERREF_SIZE);
 		sig_idx = cols [MONO_MEMBERREF_SIGNATURE];
 
-		sig = find_cached_memberref_sig (image, sig_idx);
+		sig = (MonoMethodSignature *)find_cached_memberref_sig (image, sig_idx);
 		if (!sig) {
 			if (!mono_verifier_verify_memberref_method_signature (image, sig_idx, NULL)) {
-				guint32 class = cols [MONO_MEMBERREF_CLASS] & MONO_MEMBERREF_PARENT_MASK;
+				guint32 klass = cols [MONO_MEMBERREF_CLASS] & MONO_MEMBERREF_PARENT_MASK;
 				const char *fname = mono_metadata_string_heap (image, cols [MONO_MEMBERREF_NAME]);
 
 				//FIXME include the verification error
-				mono_error_set_bad_image (error, image, "Bad method signature class token 0x%08x field name %s token 0x%08x", class, fname, token);
+				mono_error_set_bad_image (error, image, "Bad method signature class token 0x%08x field name %s token 0x%08x", klass, fname, token);
 				return NULL;
 			}
 
@@ -956,14 +973,14 @@ mono_method_get_signature_checked (MonoMethod *method, MonoImage *image, guint32
 			if (!sig)
 				return NULL;
 
-			sig = cache_memberref_sig (image, sig_idx, sig);
+			sig = (MonoMethodSignature *)cache_memberref_sig (image, sig_idx, sig);
 		}
 		/* FIXME: we probably should verify signature compat in the dynamic case too*/
 		if (!mono_verifier_is_sig_compatible (image, method, sig)) {
-			guint32 class = cols [MONO_MEMBERREF_CLASS] & MONO_MEMBERREF_PARENT_MASK;
+			guint32 klass = cols [MONO_MEMBERREF_CLASS] & MONO_MEMBERREF_PARENT_MASK;
 			const char *fname = mono_metadata_string_heap (image, cols [MONO_MEMBERREF_NAME]);
 
-			mono_error_set_bad_image (error, image, "Incompatible method signature class token 0x%08x field name %s token 0x%08x", class, fname, token);
+			mono_error_set_bad_image (error, image, "Incompatible method signature class token 0x%08x field name %s token 0x%08x", klass, fname, token);
 			return NULL;
 		}
 	}
@@ -994,7 +1011,7 @@ mono_method_get_signature (MonoMethod *method, MonoImage *image, guint32 token)
 	MonoError error;
 	MonoMethodSignature *res = mono_method_get_signature_checked (method, image, token, NULL, &error);
 
-	g_assert (!mono_loader_get_last_error ());
+	mono_loader_assert_no_error ();
 
 	if (!res) {
 		g_assert (!mono_error_ok (&error));
@@ -1028,7 +1045,7 @@ method_from_memberref (MonoImage *image, guint32 idx, MonoGenericContext *typesp
 	MonoMethod *method = NULL;
 	MonoTableInfo *tables = image->tables;
 	guint32 cols[6];
-	guint32 nindex, class, sig_idx;
+	guint32 nindex, class_index, sig_idx;
 	const char *mname;
 	MonoMethodSignature *sig;
 	const char *ptr;
@@ -1037,7 +1054,7 @@ method_from_memberref (MonoImage *image, guint32 idx, MonoGenericContext *typesp
 
 	mono_metadata_decode_row (&tables [MONO_TABLE_MEMBERREF], idx-1, cols, 3);
 	nindex = cols [MONO_MEMBERREF_CLASS] >> MONO_MEMBERREF_PARENT_BITS;
-	class = cols [MONO_MEMBERREF_CLASS] & MONO_MEMBERREF_PARENT_MASK;
+	class_index = cols [MONO_MEMBERREF_CLASS] & MONO_MEMBERREF_PARENT_MASK;
 	/*g_print ("methodref: 0x%x 0x%x %s\n", class, nindex,
 		mono_metadata_string_heap (m, cols [MONO_MEMBERREF_NAME]));*/
 
@@ -1049,9 +1066,9 @@ method_from_memberref (MonoImage *image, guint32 idx, MonoGenericContext *typesp
 	 * method into a cache.
 	 */
 	if (used_context)
-		*used_context = class == MONO_MEMBERREF_PARENT_TYPESPEC;
+		*used_context = class_index == MONO_MEMBERREF_PARENT_TYPESPEC;
 
-	switch (class) {
+	switch (class_index) {
 	case MONO_MEMBERREF_PARENT_TYPEREF:
 		klass = mono_class_from_typeref_checked (image, MONO_TOKEN_TYPE_REF | nindex, error);
 		if (!klass)
@@ -1077,7 +1094,7 @@ method_from_memberref (MonoImage *image, guint32 idx, MonoGenericContext *typesp
 		return method;
 	}
 	default:
-		mono_error_set_bad_image (error, image, "Memberref parent unknown: class: %d, index %d", class, nindex);
+		mono_error_set_bad_image (error, image, "Memberref parent unknown: class: %d, index %d", class_index, nindex);
 		goto fail;
 	}
 
@@ -1094,16 +1111,16 @@ method_from_memberref (MonoImage *image, guint32 idx, MonoGenericContext *typesp
 	ptr = mono_metadata_blob_heap (image, sig_idx);
 	mono_metadata_decode_blob_size (ptr, &ptr);
 
-	sig = find_cached_memberref_sig (image, sig_idx);
+	sig = (MonoMethodSignature *)find_cached_memberref_sig (image, sig_idx);
 	if (!sig) {
 		sig = mono_metadata_parse_method_signature_full (image, NULL, 0, ptr, NULL, error);
 		if (sig == NULL)
 			goto fail;
 
-		sig = cache_memberref_sig (image, sig_idx, sig);
+		sig = (MonoMethodSignature *)cache_memberref_sig (image, sig_idx, sig);
 	}
 
-	switch (class) {
+	switch (class_index) {
 	case MONO_MEMBERREF_PARENT_TYPEREF:
 	case MONO_MEMBERREF_PARENT_TYPEDEF:
 		method = find_method (klass, NULL, mname, sig, klass, error);
@@ -1125,7 +1142,7 @@ method_from_memberref (MonoImage *image, guint32 idx, MonoGenericContext *typesp
 		break;
 	}
 	default:
-		mono_error_set_bad_image (error, image,"Memberref parent unknown: class: %d, index %d", class, nindex);
+		mono_error_set_bad_image (error, image,"Memberref parent unknown: class: %d, index %d", class_index, nindex);
 		goto fail;
 	}
 
@@ -1146,11 +1163,11 @@ method_from_memberref (MonoImage *image, guint32 idx, MonoGenericContext *typesp
 		g_free (msig);
 	}
 
-	g_assert (!mono_loader_get_last_error ());
+	mono_loader_assert_no_error ();
 	return method;
 
 fail:
-	g_assert (!mono_loader_get_last_error ());
+	mono_loader_assert_no_error ();
 	g_assert (!mono_error_ok (error));
 	return NULL;
 }
@@ -1184,12 +1201,9 @@ method_from_methodspec (MonoImage *image, MonoGenericContext *context, guint32 i
 	ptr++;
 	param_count = mono_metadata_decode_value (ptr, &ptr);
 
-	inst = mono_metadata_parse_generic_inst (image, NULL, param_count, ptr, &ptr);
-	if (!inst) {
-		g_assert (!mono_loader_get_last_error ());
-		mono_error_set_bad_image (error, image, "Cannot parse generic instance for methodspec 0x%08x", idx);
+	inst = mono_metadata_parse_generic_inst (image, NULL, param_count, ptr, &ptr, error);
+	if (!inst)
 		return NULL;
-	}
 
 	if (context && inst->is_open) {
 		inst = mono_metadata_inflate_generic_inst (inst, context, error);
@@ -1219,7 +1233,7 @@ method_from_methodspec (MonoImage *image, MonoGenericContext *context, guint32 i
 	new_context.method_inst = inst;
 
 	method = mono_class_inflate_generic_method_full_checked (method, klass, &new_context, error);
-	g_assert (!mono_loader_get_last_error ());
+	mono_loader_assert_no_error ();
 	return method;
 }
 
@@ -1322,7 +1336,7 @@ mono_dllmap_insert (MonoImage *assembly, const char *dll, const char *func, cons
 	mono_loader_init ();
 
 	if (!assembly) {
-		entry = g_malloc0 (sizeof (MonoDllMap));
+		entry = (MonoDllMap *)g_malloc0 (sizeof (MonoDllMap));
 		entry->dll = dll? g_strdup (dll): NULL;
 		entry->target = tdll? g_strdup (tdll): NULL;
 		entry->func = func? g_strdup (func): NULL;
@@ -1333,7 +1347,7 @@ mono_dllmap_insert (MonoImage *assembly, const char *dll, const char *func, cons
 		global_dll_map = entry;
 		global_loader_data_unlock ();
 	} else {
-		entry = mono_image_alloc0 (assembly, sizeof (MonoDllMap));
+		entry = (MonoDllMap *)mono_image_alloc0 (assembly, sizeof (MonoDllMap));
 		entry->dll = dll? mono_image_strdup (assembly, dll): NULL;
 		entry->target = tdll? mono_image_strdup (assembly, tdll): NULL;
 		entry->func = func? mono_image_strdup (assembly, func): NULL;
@@ -1380,7 +1394,7 @@ cached_module_load (const char *name, int flags, char **err)
 	global_loader_data_lock ();
 	if (!global_module_map)
 		global_module_map = g_hash_table_new (g_str_hash, g_str_equal);
-	res = g_hash_table_lookup (global_module_map, name);
+	res = (MonoDl *)g_hash_table_lookup (global_module_map, name);
 	if (res) {
 		global_loader_data_unlock ();
 		return res;
@@ -1420,7 +1434,7 @@ mono_lookup_pinvoke_call (MonoMethod *method, const char **exc_class, const char
 	const char *new_scope;
 	char *error_msg;
 	char *full_name, *file_name, *found_name = NULL;
-	int i;
+	int i,j;
 	MonoDl *module = NULL;
 	gboolean cached = FALSE;
 
@@ -1436,7 +1450,7 @@ mono_lookup_pinvoke_call (MonoMethod *method, const char **exc_class, const char
 
 	if (image_is_dynamic (method->klass->image)) {
 		MonoReflectionMethodAux *method_aux = 
-			g_hash_table_lookup (
+			(MonoReflectionMethodAux *)g_hash_table_lookup (
 				((MonoDynamicImage*)method->klass->image)->method_aux_hash, method);
 		if (!method_aux)
 			return NULL;
@@ -1467,8 +1481,8 @@ mono_lookup_pinvoke_call (MonoMethod *method, const char **exc_class, const char
 			image->pinvoke_scopes = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 			image->pinvoke_scope_filenames = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 		}
-		module = g_hash_table_lookup (image->pinvoke_scopes, new_scope);
-		found_name = g_hash_table_lookup (image->pinvoke_scope_filenames, new_scope);
+		module = (MonoDl *)g_hash_table_lookup (image->pinvoke_scopes, new_scope);
+		found_name = (char *)g_hash_table_lookup (image->pinvoke_scope_filenames, new_scope);
 		mono_image_unlock (image);
 		if (module)
 			cached = TRUE;
@@ -1491,7 +1505,7 @@ mono_lookup_pinvoke_call (MonoMethod *method, const char **exc_class, const char
 	/*
 	 * Try loading the module using a variety of names
 	 */
-	for (i = 0; i < 4; ++i) {
+	for (i = 0; i < 5; ++i) {
 		char *base_name = NULL, *dir_name = NULL;
 		gboolean is_absolute = is_absolute_path (new_scope);
 		
@@ -1525,6 +1539,14 @@ mono_lookup_pinvoke_call (MonoMethod *method, const char **exc_class, const char
 				break;
 			}
 			continue;
+		case 3:
+			if (!is_absolute && mono_dl_get_system_dir ()) {
+				dir_name = (char*)mono_dl_get_system_dir ();
+				file_name = g_path_get_basename (new_scope);
+				base_name = NULL;
+			} else
+				continue;
+			break;
 		default:
 #ifndef TARGET_WIN32
 			if (!g_ascii_strcasecmp ("user32.dll", new_scope) ||
@@ -1560,23 +1582,85 @@ mono_lookup_pinvoke_call (MonoMethod *method, const char **exc_class, const char
 		}
 
 		if (!module && !is_absolute) {
-			void *iter = NULL;
-			char *mdirname = g_path_get_dirname (image->name);
-			while ((full_name = mono_dl_build_path (mdirname, file_name, &iter))) {
-				module = cached_module_load (full_name, MONO_DL_LAZY, &error_msg);
-				if (!module) {
-					mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_DLLIMPORT,
-						"DllImport error loading library '%s': '%s'.",
-								full_name, error_msg);
-					g_free (error_msg);
-				} else {
-					found_name = g_strdup (full_name);
+			void *iter;
+			char *mdirname;
+
+			for (j = 0; j < 3; ++j) {
+				iter = NULL;
+				mdirname = NULL;
+				switch (j) {
+					case 0:
+						mdirname = g_path_get_dirname (image->name);
+						break;
+					case 1: /* @executable_path@/../lib */
+					{
+						char buf [4096];
+						int binl;
+						binl = mono_dl_get_executable_path (buf, sizeof (buf));
+						if (binl != -1) {
+							char *base, *newbase;
+							char *resolvedname;
+							buf [binl] = 0;
+							resolvedname = mono_path_resolve_symlinks (buf);
+
+							base = g_path_get_dirname (resolvedname);
+							newbase = g_path_get_dirname(base);
+							mdirname = g_strdup_printf ("%s/lib", newbase);
+
+							g_free (resolvedname);
+							g_free (base);
+							g_free (newbase);
+						}
+						break;
+					}
+#ifdef __MACH__
+					case 2: /* @executable_path@/../Libraries */
+					{
+						char buf [4096];
+						int binl;
+						binl = mono_dl_get_executable_path (buf, sizeof (buf));
+						if (binl != -1) {
+							char *base, *newbase;
+							char *resolvedname;
+							buf [binl] = 0;
+							resolvedname = mono_path_resolve_symlinks (buf);
+
+							base = g_path_get_dirname (resolvedname);
+							newbase = g_path_get_dirname(base);
+							mdirname = g_strdup_printf ("%s/Libraries", newbase);
+
+							g_free (resolvedname);
+							g_free (base);
+							g_free (newbase);
+						}
+						break;
+					}
+#endif
 				}
-				g_free (full_name);
+
+				if (!mdirname)
+					continue;
+
+				while ((full_name = mono_dl_build_path (mdirname, file_name, &iter))) {
+					module = cached_module_load (full_name, MONO_DL_LAZY, &error_msg);
+					if (!module) {
+						mono_trace (G_LOG_LEVEL_INFO, MONO_TRACE_DLLIMPORT,
+							"DllImport error loading library '%s': '%s'.",
+									full_name, error_msg);
+						g_free (error_msg);
+					} else {
+						found_name = g_strdup (full_name);
+					}
+					g_free (full_name);
+					if (module)
+						break;
+
+				}
+				g_free (mdirname);
 				if (module)
 					break;
 			}
-			g_free (mdirname);
+
 		}
 
 		if (!module) {
@@ -1779,7 +1863,6 @@ mono_get_method_from_token (MonoImage *image, guint32 token, MonoClass *klass,
 	MonoTableInfo *tables = image->tables;
 	MonoGenericContainer *generic_container = NULL, *container = NULL;
 	const char *sig = NULL;
-	int size;
 	guint32 cols [MONO_TYPEDEF_SIZE];
 
 	mono_error_init (error);
@@ -1787,8 +1870,8 @@ mono_get_method_from_token (MonoImage *image, guint32 token, MonoClass *klass,
 	if (image_is_dynamic (image)) {
 		MonoClass *handle_class;
 
-		result = mono_lookup_dynamic_token_class (image, token, TRUE, &handle_class, context);
-		g_assert (!mono_loader_get_last_error ());
+		result = (MonoMethod *)mono_lookup_dynamic_token_class (image, token, TRUE, &handle_class, context);
+		mono_loader_assert_no_error ();
 
 		// This checks the memberref type as well
 		if (result && handle_class != mono_defaults.methodhandle_class) {
@@ -1849,7 +1932,7 @@ mono_get_method_from_token (MonoImage *image, guint32 token, MonoClass *klass,
 
 	if (!sig) /* already taken from the methodref */
 		sig = mono_metadata_blob_heap (image, cols [4]);
-	size = mono_metadata_decode_blob_size (sig, &sig);
+	/* size = */ mono_metadata_decode_blob_size (sig, &sig);
 
 	container = klass->generic_container;
 
@@ -1859,11 +1942,12 @@ mono_get_method_from_token (MonoImage *image, guint32 token, MonoClass *klass,
 	 */
 	if (*sig & 0x10) {
 		generic_container = mono_metadata_load_generic_params (image, token, container);
-		g_assert (!mono_loader_get_last_error ()); /* FIXME don't swallow this error. */
+		mono_loader_assert_no_error (); /* FIXME don't swallow this error. */
 	}
 	if (generic_container) {
 		result->is_generic = TRUE;
 		generic_container->owner.method = result;
+		generic_container->is_anonymous = FALSE; // Method is now known, container is no longer anonymous
 		/*FIXME put this before the image alloc*/
 		if (!mono_metadata_load_generic_param_constraints_checked (image, token, generic_container, error))
 			return NULL;
@@ -1893,7 +1977,7 @@ mono_get_method_from_token (MonoImage *image, guint32 token, MonoClass *klass,
  	if (generic_container)
  		mono_method_set_generic_container (result, generic_container);
 
-	g_assert (!mono_loader_get_last_error ());
+	mono_loader_assert_no_error ();
 	return result;
 }
 
@@ -1909,7 +1993,7 @@ mono_get_method_full (MonoImage *image, guint32 token, MonoClass *klass,
 {
 	MonoError error;
 	MonoMethod *result = mono_get_method_checked (image, token, klass, context, &error);
-	g_assert (!mono_loader_get_last_error ());
+	mono_loader_assert_no_error ();
 	if (!mono_error_ok (&error)) {
 		mono_loader_set_error_from_mono_error (&error);
 		mono_error_cleanup (&error);
@@ -1932,11 +2016,11 @@ mono_get_method_checked (MonoImage *image, guint32 token, MonoClass *klass, Mono
 	if (mono_metadata_token_table (token) == MONO_TABLE_METHOD) {
 		if (!image->method_cache)
 			image->method_cache = g_hash_table_new (NULL, NULL);
-		result = g_hash_table_lookup (image->method_cache, GINT_TO_POINTER (mono_metadata_token_index (token)));
+		result = (MonoMethod *)g_hash_table_lookup (image->method_cache, GINT_TO_POINTER (mono_metadata_token_index (token)));
 	} else if (!image_is_dynamic (image)) {
 		if (!image->methodref_cache)
 			image->methodref_cache = g_hash_table_new (NULL, NULL);
-		result = g_hash_table_lookup (image->methodref_cache, GINT_TO_POINTER (token));
+		result = (MonoMethod *)g_hash_table_lookup (image->methodref_cache, GINT_TO_POINTER (token));
 	}
 	mono_image_unlock (image);
 
@@ -1953,9 +2037,9 @@ mono_get_method_checked (MonoImage *image, guint32 token, MonoClass *klass, Mono
 		MonoMethod *result2 = NULL;
 
 		if (mono_metadata_token_table (token) == MONO_TABLE_METHOD)
-			result2 = g_hash_table_lookup (image->method_cache, GINT_TO_POINTER (mono_metadata_token_index (token)));
+			result2 = (MonoMethod *)g_hash_table_lookup (image->method_cache, GINT_TO_POINTER (mono_metadata_token_index (token)));
 		else if (!image_is_dynamic (image))
-			result2 = g_hash_table_lookup (image->methodref_cache, GINT_TO_POINTER (token));
+			result2 = (MonoMethod *)g_hash_table_lookup (image->methodref_cache, GINT_TO_POINTER (token));
 
 		if (result2) {
 			mono_image_unlock (image);
@@ -2057,7 +2141,7 @@ mono_get_method_constrained (MonoImage *image, guint32 token, MonoClass *constra
 	MonoError error;
 	MonoMethod *result = mono_get_method_constrained_checked (image, token, constrained_class, context, cil_method, &error);
 
-	g_assert (!mono_loader_get_last_error ());
+	mono_loader_assert_no_error ();
 	if (!mono_error_ok (&error)) {
 		mono_loader_set_error_from_mono_error (&error);
 		mono_error_cleanup (&error);
@@ -2151,7 +2235,7 @@ mono_method_get_param_names (MonoMethod *method, const char **names)
 
 	if (image_is_dynamic (klass->image)) {
 		MonoReflectionMethodAux *method_aux = 
-			g_hash_table_lookup (
+			(MonoReflectionMethodAux *)g_hash_table_lookup (
 				((MonoDynamicImage*)method->klass->image)->method_aux_hash, method);
 		if (method_aux && method_aux->param_names) {
 			for (i = 0; i < mono_method_signature (method)->param_count; ++i)
@@ -2166,7 +2250,7 @@ mono_method_get_param_names (MonoMethod *method, const char **names)
 
 		mono_image_lock (klass->image);
 		if (klass->image->wrapper_param_names)
-			pnames = g_hash_table_lookup (klass->image->wrapper_param_names, method);
+			pnames = (char **)g_hash_table_lookup (klass->image->wrapper_param_names, method);
 		mono_image_unlock (klass->image);
 
 		if (pnames) {
@@ -2242,7 +2326,7 @@ mono_method_get_marshal_info (MonoMethod *method, MonoMarshalSpec **mspecs)
 
 	if (image_is_dynamic (method->klass->image)) {
 		MonoReflectionMethodAux *method_aux = 
-			g_hash_table_lookup (
+			(MonoReflectionMethodAux *)g_hash_table_lookup (
 				((MonoDynamicImage*)method->klass->image)->method_aux_hash, method);
 		if (method_aux && method_aux->param_marshall) {
 			MonoMarshalSpec **dyn_specs = method_aux->param_marshall;
@@ -2297,7 +2381,7 @@ mono_method_has_marshal_info (MonoMethod *method)
 
 	if (image_is_dynamic (method->klass->image)) {
 		MonoReflectionMethodAux *method_aux = 
-			g_hash_table_lookup (
+			(MonoReflectionMethodAux *)g_hash_table_lookup (
 				((MonoDynamicImage*)method->klass->image)->method_aux_hash, method);
 		MonoMarshalSpec **dyn_specs = method_aux->param_marshall;
 		if (dyn_specs) {
@@ -2342,7 +2426,7 @@ mono_method_get_wrapper_data (MonoMethod *method, guint32 id)
 
 	if (method->is_inflated)
 		method = ((MonoMethodInflated *) method)->declaring;
-	data = ((MonoMethodWrapper *)method)->method_data;
+	data = (void **)((MonoMethodWrapper *)method)->method_data;
 	g_assert (data != NULL);
 	g_assert (id <= GPOINTER_TO_UINT (*data));
 	return data [id];
@@ -2356,15 +2440,16 @@ typedef struct {
 static gboolean
 stack_walk_adapter (MonoStackFrameInfo *frame, MonoContext *ctx, gpointer data)
 {
-	StackWalkUserData *d = data;
+	StackWalkUserData *d = (StackWalkUserData *)data;
 
 	switch (frame->type) {
 	case FRAME_TYPE_DEBUGGER_INVOKE:
 	case FRAME_TYPE_MANAGED_TO_NATIVE:
+	case FRAME_TYPE_TRAMPOLINE:
 		return FALSE;
 	case FRAME_TYPE_MANAGED:
 		g_assert (frame->ji);
-		return d->func (mono_jit_info_get_method (frame->ji), frame->native_offset, frame->il_offset, frame->managed, d->user_data);
+		return d->func (frame->actual_method, frame->native_offset, frame->il_offset, frame->managed, d->user_data);
 		break;
 	default:
 		g_assert_not_reached ();
@@ -2395,19 +2480,21 @@ typedef struct {
 static gboolean
 async_stack_walk_adapter (MonoStackFrameInfo *frame, MonoContext *ctx, gpointer data)
 {
-	AsyncStackWalkUserData *d = data;
+	AsyncStackWalkUserData *d = (AsyncStackWalkUserData *)data;
 
 	switch (frame->type) {
 	case FRAME_TYPE_DEBUGGER_INVOKE:
 	case FRAME_TYPE_MANAGED_TO_NATIVE:
+	case FRAME_TYPE_TRAMPOLINE:
 		return FALSE;
 	case FRAME_TYPE_MANAGED:
 		if (!frame->ji)
 			return FALSE;
-		if (frame->ji->async)
+		if (frame->ji->async) {
 			return d->func (NULL, frame->domain, frame->ji->code_start, frame->native_offset, d->user_data);
-		else
-			return d->func (mono_jit_info_get_method (frame->ji), frame->domain, frame->ji->code_start, frame->native_offset, d->user_data);
+		} else {
+			return d->func (frame->actual_method, frame->domain, frame->ji->code_start, frame->native_offset, d->user_data);
+		}
 		break;
 	default:
 		g_assert_not_reached ();
@@ -2434,7 +2521,7 @@ mono_stack_walk_async_safe (MonoStackWalkAsyncSafe func, void *initial_sig_conte
 static gboolean
 last_managed (MonoMethod *m, gint no, gint ilo, gboolean managed, gpointer data)
 {
-	MonoMethod **dest = data;
+	MonoMethod **dest = (MonoMethod **)data;
 	*dest = m;
 	/*g_print ("In %s::%s [%d] [%d]\n", m->klass->name, m->name, no, ilo);*/
 
@@ -2459,7 +2546,7 @@ static gboolean loader_lock_track_ownership = FALSE;
 void
 mono_loader_lock (void)
 {
-	mono_locks_acquire (&loader_mutex, LoaderLock);
+	mono_locks_coop_acquire (&loader_mutex, LoaderLock);
 	if (G_UNLIKELY (loader_lock_track_ownership)) {
 		mono_native_tls_set_value (loader_lock_nest_id, GUINT_TO_POINTER (GPOINTER_TO_UINT (mono_native_tls_get_value (loader_lock_nest_id)) + 1));
 	}
@@ -2468,7 +2555,7 @@ mono_loader_lock (void)
 void
 mono_loader_unlock (void)
 {
-	mono_locks_release (&loader_mutex, LoaderLock);
+	mono_locks_coop_release (&loader_mutex, LoaderLock);
 	if (G_UNLIKELY (loader_lock_track_ownership)) {
 		mono_native_tls_set_value (loader_lock_nest_id, GUINT_TO_POINTER (GPOINTER_TO_UINT (mono_native_tls_get_value (loader_lock_nest_id)) - 1));
 	}
@@ -2530,7 +2617,6 @@ MonoMethodSignature*
 mono_method_signature_checked (MonoMethod *m, MonoError *error)
 {
 	int idx;
-	int size;
 	MonoImage* img;
 	const char *sig;
 	gboolean can_cache_signature;
@@ -2588,7 +2674,7 @@ mono_method_signature_checked (MonoMethod *m, MonoError *error)
 
 	if (can_cache_signature) {
 		mono_image_lock (img);
-		signature = g_hash_table_lookup (img->method_signatures, sig);
+		signature = (MonoMethodSignature *)g_hash_table_lookup (img->method_signatures, sig);
 		mono_image_unlock (img);
 	}
 
@@ -2598,7 +2684,7 @@ mono_method_signature_checked (MonoMethod *m, MonoError *error)
 		if (!mono_verifier_verify_method_signature (img, sig_offset, error))
 			return NULL;
 
-		size = mono_metadata_decode_blob_size (sig, &sig_body);
+		/* size = */ mono_metadata_decode_blob_size (sig, &sig_body);
 
 		signature = mono_metadata_parse_method_signature_full (img, container, idx, sig_body, NULL, error);
 		if (!signature)
@@ -2606,7 +2692,7 @@ mono_method_signature_checked (MonoMethod *m, MonoError *error)
 
 		if (can_cache_signature) {
 			mono_image_lock (img);
-			sig2 = g_hash_table_lookup (img->method_signatures, sig);
+			sig2 = (MonoMethodSignature *)g_hash_table_lookup (img->method_signatures, sig);
 			if (!sig2)
 				g_hash_table_insert (img->method_signatures, (gpointer)sig, signature);
 			mono_image_unlock (img);
@@ -2632,7 +2718,7 @@ mono_method_signature_checked (MonoMethod *m, MonoError *error)
 	if (m->iflags & METHOD_IMPL_ATTRIBUTE_INTERNAL_CALL)
 		signature->pinvoke = 1;
 	else if (m->flags & METHOD_ATTRIBUTE_PINVOKE_IMPL) {
-		MonoCallConvention conv = 0;
+		MonoCallConvention conv = (MonoCallConvention)0;
 		MonoMethodPInvoke *piinfo = (MonoMethodPInvoke *)m;
 		signature->pinvoke = 1;
 
@@ -2739,20 +2825,7 @@ mono_method_get_header (MonoMethod *method)
 		iheader = inflate_generic_header (header, mono_method_get_context (method));
 		mono_metadata_free_mh (header);
 
-		mono_image_lock (img);
-
-		if (imethod->header) {
-			mono_metadata_free_mh (iheader);
-			mono_image_unlock (img);
-			return imethod->header;
-		}
-
-		mono_memory_barrier ();
-		imethod->header = iheader;
-
-		mono_image_unlock (img);
-
-		return imethod->header;
+		return iheader;
 	}
 
 	if (method->wrapper_type != MONO_WRAPPER_NONE || method->sre_method) {
@@ -2783,7 +2856,7 @@ mono_method_get_header (MonoMethod *method)
 	container = mono_method_get_generic_container (method);
 	if (!container)
 		container = method->klass->generic_container;
-	header = mono_metadata_parse_mh_full (img, container, loc);
+	header = mono_metadata_parse_mh_full (img, container, (const char *)loc);
 
 	return header;
 }
