@@ -153,8 +153,8 @@ enum {
 };
 
 static mono_mutex_t perfctr_mutex;
-#define perfctr_lock() mono_mutex_lock (&perfctr_mutex)
-#define perfctr_unlock() mono_mutex_unlock (&perfctr_mutex)
+#define perfctr_lock() mono_os_mutex_lock (&perfctr_mutex)
+#define perfctr_unlock() mono_os_mutex_unlock (&perfctr_mutex)
 
 typedef struct {
 	char reserved [16];
@@ -359,9 +359,9 @@ load_sarea_for_pid (int pid)
 	perfctr_lock ();
 	if (pid_to_shared_area == NULL)
 		pid_to_shared_area = g_hash_table_new (NULL, NULL);
-	data = g_hash_table_lookup (pid_to_shared_area, GINT_TO_POINTER (pid));
+	data = (ExternalSArea *)g_hash_table_lookup (pid_to_shared_area, GINT_TO_POINTER (pid));
 	if (!data) {
-		area = mono_shared_area_for_pid (GINT_TO_POINTER (pid));
+		area = (MonoSharedArea *)mono_shared_area_for_pid (GINT_TO_POINTER (pid));
 		if (area) {
 			data = g_new (ExternalSArea, 1);
 			data->sarea = area;
@@ -369,7 +369,7 @@ load_sarea_for_pid (int pid)
 			g_hash_table_insert (pid_to_shared_area, GINT_TO_POINTER (pid), data);
 		}
 	} else {
-		area = data->sarea;
+		area = (MonoSharedArea *)data->sarea;
 		data->refcount ++;
 	}
 	perfctr_unlock ();
@@ -380,7 +380,7 @@ static void
 unref_pid_unlocked (int pid)
 {
 	ExternalSArea *data;
-	data = g_hash_table_lookup (pid_to_shared_area, GINT_TO_POINTER (pid));
+	data = (ExternalSArea *)g_hash_table_lookup (pid_to_shared_area, GINT_TO_POINTER (pid));
 	if (data) {
 		data->refcount--;
 		if (!data->refcount) {
@@ -498,13 +498,16 @@ mono_determine_physical_ram_available_size (void)
 	return ((guint64) value.t_free * page_size) / 1024;
 #elif defined (__APPLE__)
 	mach_msg_type_number_t count = HOST_VM_INFO_COUNT;
+	mach_port_t host = mach_host_self();
+	vm_size_t page_size;
 	vm_statistics_data_t vmstat;
-	if (KERN_SUCCESS != host_statistics(mach_host_self(), HOST_VM_INFO, (host_info_t)&vmstat, &count)) {
+	if (KERN_SUCCESS != host_statistics(host, HOST_VM_INFO, (host_info_t)&vmstat, &count)) {
 		g_warning ("Mono was unable to retrieve memory usage!");
 		return 0;
 	}
 
-	return (guint64) vmstat.free_count;
+	host_page_size(host, &page_size);
+	return (guint64) vmstat.free_count * page_size;
 
 #elif defined (HAVE_SYSCONF)
 	guint64 page_size = 0, num_pages = 0;
@@ -537,9 +540,9 @@ mono_perfcounters_init (void)
 	d_offset += 7;
 	d_offset &= ~7;
 
-	mono_mutex_init_recursive (&perfctr_mutex);
+	mono_os_mutex_init_recursive (&perfctr_mutex);
 
-	shared_area = mono_shared_area ();
+	shared_area = (MonoSharedArea *)mono_shared_area ();
 	shared_area->counters_start = G_STRUCT_OFFSET (MonoSharedArea, counters);
 	shared_area->counters_size = sizeof (MonoPerfCounters);
 	shared_area->data_start = d_offset;
@@ -645,7 +648,7 @@ typedef struct {
 static gboolean
 category_search (SharedHeader *header, void *data)
 {
-	CatSearch *search = data;
+	CatSearch *search = (CatSearch *)data;
 	if (header->ftype == FTYPE_CATEGORY) {
 		SharedCategory *cat = (SharedCategory*)header;
 		if (mono_string_compare_ascii (search->name, cat->name) == 0) {
@@ -669,7 +672,7 @@ find_custom_category (MonoString *name)
 static gboolean
 category_collect (SharedHeader *header, void *data)
 {
-	GSList **list = data;
+	GSList **list = (GSList **)data;
 	if (header->ftype == FTYPE_CATEGORY) {
 		*list = g_slist_prepend (*list, header);
 	}
@@ -718,7 +721,7 @@ typedef struct {
 static gboolean
 instance_search (SharedHeader *header, void *data)
 {
-	InstanceSearch *search = data;
+	InstanceSearch *search = (InstanceSearch *)data;
 	if (header->ftype == FTYPE_INSTANCE) {
 		SharedInstance *ins = (SharedInstance*)header;
 		if (search->cat_offset == ins->category_offset) {
@@ -890,7 +893,7 @@ network_cleanup (ImplVtable *vtable)
 	if (vtable == NULL)
 		return;
 
-	narg = vtable->arg;
+	narg = (NetworkVtableArg *)vtable->arg;
 	if (narg == NULL)
 		return;
 
@@ -1219,7 +1222,7 @@ static gint64
 custom_writable_update (ImplVtable *vtable, MonoBoolean do_incr, gint64 value)
 {
 	/* FIXME: check writability */
-	guint64 *ptr = vtable->arg;
+	guint64 *ptr = (guint64 *)vtable->arg;
 	if (ptr) {
 		if (do_incr) {
 			/* FIXME: we need to do this atomically */
@@ -1238,14 +1241,13 @@ custom_get_instance (SharedCategory *cat, SharedCounter *scounter, char* name)
 {
 	SharedInstance* inst;
 	char *p;
-	int size, data_offset;
+	int size;
 	inst = find_custom_instance (cat, name);
 	if (inst)
 		return inst;
 	size = sizeof (SharedInstance) + strlen (name);
 	size += 7;
 	size &= ~7;
-	data_offset = size;
 	size += (sizeof (guint64) * cat->num_counters);
 	perfctr_lock ();
 	inst = (SharedInstance*) shared_data_reserve_room (size, FTYPE_INSTANCE);
@@ -1305,7 +1307,7 @@ custom_get_impl (SharedCategory *cat, MonoString* counter, MonoString* instance,
 	g_free (name);
 	if (!inst)
 		return NULL;
-	return custom_vtable (scounter, inst, custom_get_value_address (scounter, inst));
+	return custom_vtable (scounter, inst, (char *)custom_get_value_address (scounter, inst));
 }
 
 static const CategoryDesc*
@@ -1362,7 +1364,7 @@ mono_perfcounter_get_impl (MonoString* category, MonoString* counter, MonoString
 MonoBoolean
 mono_perfcounter_get_sample (void *impl, MonoBoolean only_value, MonoCounterSample *sample)
 {
-	ImplVtable *vtable = impl;
+	ImplVtable *vtable = (ImplVtable *)impl;
 	if (vtable && vtable->sample)
 		return vtable->sample (vtable, only_value, sample);
 	return FALSE;
@@ -1371,7 +1373,7 @@ mono_perfcounter_get_sample (void *impl, MonoBoolean only_value, MonoCounterSamp
 gint64
 mono_perfcounter_update_value (void *impl, MonoBoolean do_incr, gint64 value)
 {
-	ImplVtable *vtable = impl;
+	ImplVtable *vtable = (ImplVtable *)impl;
 	if (vtable && vtable->update)
 		return vtable->update (vtable, do_incr, value);
 	return 0;
@@ -1380,7 +1382,7 @@ mono_perfcounter_update_value (void *impl, MonoBoolean do_incr, gint64 value)
 void
 mono_perfcounter_free_data (void *impl)
 {
-	ImplVtable *vtable = impl;
+	ImplVtable *vtable = (ImplVtable *)impl;
 	if (vtable && vtable->cleanup)
 		vtable->cleanup (vtable);
 	g_free (impl);
@@ -1597,7 +1599,7 @@ mono_perfcounter_category_names (MonoString *machine)
 		mono_array_setref (res, i, mono_string_new (domain, cdesc->name));
 	}
 	for (tmp = custom_categories; tmp; tmp = tmp->next) {
-		SharedCategory *scat = tmp->data;
+		SharedCategory *scat = (SharedCategory *)tmp->data;
 		mono_array_setref (res, i, mono_string_new (domain, scat->name));
 		i++;
 	}
@@ -1675,7 +1677,7 @@ get_string_array_of_strings (void **array, int count)
 	MonoDomain *domain = mono_domain_get ();
 	MonoArray * res = mono_array_new (mono_domain_get (), mono_get_string_class (), count);
 	for (i = 0; i < count; ++i) {
-		char* p = array[i];
+		char* p = (char *)array[i];
 		mono_array_setref (res, i, mono_string_new (domain, p));
 	}
 
@@ -1754,7 +1756,7 @@ get_custom_instances (MonoString *category)
 		int i = 0;
 		MonoArray *array = mono_array_new (mono_domain_get (), mono_get_string_class (), g_slist_length (list));
 		for (tmp = list; tmp; tmp = tmp->next) {
-			SharedInstance *inst = tmp->data;
+			SharedInstance *inst = (SharedInstance *)tmp->data;
 			mono_array_setref (array, i, mono_string_new (mono_domain_get (), inst->instance_name));
 			i++;
 		}
@@ -1797,14 +1799,13 @@ static gboolean
 mono_perfcounter_foreach_shared_item (SharedHeader *header, gpointer data)
 {
 	int i;
-	char *p, *name, *help;
+	char *p, *name;
 	unsigned char type;
-	int seq_num;
 	void *addr;
 	SharedCategory *cat;
 	SharedCounter *counter;
 	SharedInstance *inst;
-	PerfCounterForeachData *foreach_data = data;
+	PerfCounterForeachData *foreach_data = (PerfCounterForeachData *)data;
 
 	if (header->ftype == FTYPE_CATEGORY) {
 		cat = (SharedCategory*)header;
@@ -1816,10 +1817,10 @@ mono_perfcounter_foreach_shared_item (SharedHeader *header, gpointer data)
 		for (i = 0; i < cat->num_counters; ++i) {
 			counter = (SharedCounter*) p;
 			type = (unsigned char)*p++;
-			seq_num = (int)*p++;
+			/* seq_num = (int)* */ p++;
 			name = p;
 			p += strlen (p) + 1;
-			help = p;
+			/* help = p; */
 			p += strlen (p) + 1;
 
 			inst = custom_get_instance (cat, counter, name);
