@@ -23,7 +23,7 @@
 #include <mono/metadata/debug-helpers.h>
 #include <mono/metadata/profiler.h>
 #include <mono/metadata/profiler-private.h>
-#include <mono/metadata/gc-internal.h>
+#include <mono/metadata/gc-internals.h>
 
 /* This function is redirected to String.CreateString ()
    by mono_marshal_get_native_wrapper () */
@@ -66,9 +66,3 @@ ves_icall_System_String_GetLOSLimit (void)
 	return (limit - 2 - G_STRUCT_OFFSET (MonoString, chars)) / 2;
 }
 
-void
-ves_icall_System_String_InternalSetLength (MonoString *str, gint32 new_length)
-{
-	mono_gc_set_string_length (str, new_length);
-}
-
