@@ -220,7 +220,6 @@ mono_debug_close_method (MonoCompile *cfg)
 	MonoDebugMethodJitInfo *jit;
 	MonoMethodHeader *header;
 	MonoMethodSignature *sig;
-	MonoDebugMethodAddress *debug_info;
 	MonoMethod *method;
 	int i;
 
@@ -270,7 +269,7 @@ mono_debug_close_method (MonoCompile *cfg)
 	for (i = 0; i < jit->num_line_numbers; i++)
 		jit->line_numbers [i] = g_array_index (info->line_numbers, MonoDebugLineNumberEntry, i);
 
-	debug_info = mono_debug_add_method (cfg->method_to_register, jit, cfg->domain);
+	mono_debug_add_method (cfg->method_to_register, jit, cfg->domain);
 
 	mono_debug_add_vg_method (method, jit);
 
@@ -452,7 +451,7 @@ mono_debug_serialize_debug_info (MonoCompile *cfg, guint8 **out_buf, guint32 *bu
 	}
 
 	size = ((jit->num_params + jit->num_locals + 1) * 10) + (jit->num_line_numbers * 10) + 64;
-	p = buf = g_malloc (size);
+	p = buf = (guint8 *)g_malloc (size);
 	encode_value (jit->epilogue_begin, p, &p);
 	encode_value (jit->prologue_end, p, &p);
 	encode_value (jit->code_size, p, &p);
@@ -651,7 +650,7 @@ void
 mono_debug_print_vars (gpointer ip, gboolean only_arguments)
 {
 	MonoDomain *domain = mono_domain_get ();
-	MonoJitInfo *ji = mono_jit_info_table_find (domain, ip);
+	MonoJitInfo *ji = mono_jit_info_table_find (domain, (char *)ip);
 	MonoDebugMethodJitInfo *jit;
 	int i;
 
@@ -731,7 +730,7 @@ mono_debugger_method_has_breakpoint (MonoMethod *method)
 		return 0;
 
 	for (i = 0; i < breakpoints->len; i++) {
-		MiniDebugBreakpointInfo *info = g_ptr_array_index (breakpoints, i);
+		MiniDebugBreakpointInfo *info = (MiniDebugBreakpointInfo *)g_ptr_array_index (breakpoints, i);
 
 		if (!mono_method_desc_full_match (info->desc, method))
 			continue;
