@@ -99,7 +99,7 @@ load_file (const char *name) {
 			is_template = TRUE;
 			desc = g_new0 (OpDesc, 1);
 		} else {
-			desc = g_hash_table_lookup (table, str);
+			desc = (OpDesc *)g_hash_table_lookup (table, str);
 			if (!desc)
 				g_error ("Invalid opcode '%s' at line %d in %s\n", str, line, name);
 			if (desc->desc)
@@ -140,8 +140,12 @@ load_file (const char *name) {
 				*/
 			} else if (strncmp (p, "len:", 4) == 0) {
 				unsigned long size;
+				char* endptr;
 				p += 4;
-				size = strtoul (p, &p, 10);
+				size = strtoul (p, &endptr, 10);
+				if (size == 0 && p == endptr)
+					g_error ("Invalid length '%s' at line %d in %s\n", p, line, name);
+				p = endptr;
 				if (!nacl_length_set) {
 					desc->spec [MONO_INST_LEN] = size;
 				}
@@ -161,7 +165,7 @@ load_file (const char *name) {
 				tname = p;
 				while (*p && isalnum (*p)) ++p;
 				*p++ = 0;
-				tdesc = g_hash_table_lookup (template_table, tname);
+				tdesc = (OpDesc *)g_hash_table_lookup (template_table, tname);
 				if (!tdesc)
 					g_error ("Invalid template name %s at '%s' at line %d in %s\n", tname, p, line, name);
 				for (i = 0; i < MONO_INST_MAX; ++i) {
@@ -191,6 +195,7 @@ load_file (const char *name) {
 		if (is_template && !desc->name)
 			g_error ("Template without name at line %d in %s\n", line, name);
 	}
+	g_string_free (comment,TRUE);
 	fclose (f);
 	return 0;
 }
@@ -234,13 +239,13 @@ build_table (const char *fname, const char *name) {
 	if (!(f = fopen (fname, "w")))
 		g_error ("Cannot open file '%s'", fname);
 	fprintf (f, "/* File automatically generated by genmdesc, don't change */\n\n");
-	fprintf (f, "const char %s [] = {\n", name);
+	fprintf (f, "const char mono_%s [] = {\n", name);
 	fprintf (f, "\t\"");
 	for (j = 0; j < MONO_INST_MAX; ++j)
 		fprintf (f, "\\x0");
 	fprintf (f, "\"\t/* null entry */\n");
 	idx = 1;
-	g_string_append_printf (idx_array, "const guint16 %s_idx [] = {\n", name);
+	g_string_append_printf (idx_array, "const guint16 mono_%s_idx [] = {\n", name);
 
 	for (i = OP_LOAD; i < OP_LAST; ++i) {
 		desc = opcodes + i;
