@@ -48,13 +48,13 @@ class GFoo3<T> {
 
 // FIXME: Add mixed ref/noref tests, i.e. Dictionary<string, int>
 
-#if MOBILE
+#if __MOBILE__
 public class GSharedTests
 #else
 public class Tests
 #endif
 {
-#if !MOBILE
+#if !__MOBILE__
 	public static int Main (String[] args) {
 		return TestDriver.RunTests (typeof (Tests), args);
 	}
@@ -464,6 +464,21 @@ public class Tests
 		return 0;
 	}
 
+	class DelClass {
+		[MethodImplAttribute (MethodImplOptions.NoInlining)]
+		public static T return_t<T> (T t) {
+			return t;
+		}
+	}
+
+	public static int test_0_gsharedvt_in_delegates_reflection () {
+		var m = typeof(DelClass).GetMethod ("return_t").MakeGenericMethod (new Type [] { typeof (int) });
+		Func<int, int> f = (Func<int, int>)Delegate.CreateDelegate (typeof (Func<int,int>), null, m, false);
+		if (f (42) != 42)
+			return 1;
+		return 0;
+	}
+
 	[MethodImplAttribute (MethodImplOptions.NoInlining)]
 	static T return2_t<T> (T t) {
 		return return_t (t);
@@ -1048,6 +1063,11 @@ public class Tests
 		public Type gettype<T, T2>(T t, T2 t2) {
 			return t.GetType ();
 		}
+
+		[MethodImplAttribute (MethodImplOptions.NoInlining)]
+		public Type gettype2<T>(T t) {
+			return t.GetType ();
+		}
 	}
 
 	public static int test_0_constrained_gettype () {
@@ -1056,6 +1076,10 @@ public class Tests
 			return 1;
 		if (c.gettype<string, int> ("A", 1) != typeof (string))
 			return 2;
+		/* Partial sharing */
+		var c2 = new CGetType ();
+		if (c2.gettype2<long> (1) != typeof (long))
+			return 3;
 		return 0;
 	}
 
@@ -1291,6 +1315,7 @@ public class Tests
 
 	interface IFaceBox {
 		object box<T> (T t);
+		bool is_null<T> (T t);
 	}
 
 	class ClassBox : IFaceBox {
@@ -1298,6 +1323,12 @@ public class Tests
 			object o = t;
 			return o;
 		}
+
+		public bool is_null<T> (T t) {
+			if (!(default(T) == null))
+				return false;
+			return true;
+		}
 	}
 
 	public static int test_0_nullable_box () {
@@ -1320,6 +1351,15 @@ public class Tests
 		return 0;
 	}
 
+	public static int test_0_nullable_box_brtrue_opt () {
+		IFaceBox c = new ClassBox ();
+
+		if (c.is_null<double?> (null))
+			return 0;
+		else
+			return 1;
+	}
+
 	interface IFaceUnbox2 {
 		T unbox<T> (object o);
 	}
@@ -1655,6 +1695,23 @@ public class Tests
 		return 0;
 	}
 
+	interface ISmallArg {
+		T foo<T> (string s1, string s2, string s3, string s4, string s5, string s6, string s7, string s8,
+				  string s9, string s10, string s11, string s12, string s13, T t);
+	}
+
+	class SmallArgClass : ISmallArg {
+			public T foo<T> (string s1, string s2, string s3, string s4, string s5, string s6, string s7, string s8,
+							 string s9, string s10, string s11, string s12, string s13, T t) {
+				return t;
+			}
+		}
+
+	public static int test_1_small_gsharedvt_stack_arg_ios () {
+		ISmallArg o = new SmallArgClass ();
+		return o.foo<int> ("", "", "", "", "", "", "", "", "", "", "", "", "", 1);
+	}
+
 	// Passing vtype normal arguments on the stack
 	public static int test_0_arm64_vtype_stack_args () {
 		IFoo3<EmptyStruct> o = (IFoo3<EmptyStruct>)Activator.CreateInstance (typeof (Foo3<>).MakeGenericType (new Type [] { typeof (EmptyStruct) }));
@@ -1693,6 +1750,33 @@ public class Tests
 			return 2;
 		return 0;
 	}
+
+	public interface IFaceTest {
+		int iface_method ();
+	}
+
+	public interface IFaceConstrainedIFace {
+		int foo<T, T2> (ref T val) where T: IFaceTest;
+	}
+
+	class ConstrainedIFace : IFaceConstrainedIFace {
+		public int foo<T, T2> (ref T val) where T: IFaceTest {
+			return val.iface_method ();
+		}
+	}
+
+	class ClassTest : IFaceTest {
+		public int iface_method () {
+			return 42;
+		}
+	}
+
+	// Test constrained calls on an interface made from gsharedvt methods
+	public static int test_42_gsharedvt_constrained_iface () {
+		IFaceConstrainedIFace obj = new ConstrainedIFace ();
+		IFaceTest t = new ClassTest ();
+		return obj.foo<IFaceTest, int> (ref t);
+	}
 }
 
 // #13191
@@ -1710,7 +1794,7 @@ public class MobileServiceCollection<TTable, TCol>
 	}
 }
 
-#if !MOBILE
+#if !__MOBILE__
 public class GSharedTests : Tests {
 }
 #endif
