@@ -14,7 +14,6 @@ PATCH_INFO(CLASS, "class")
 PATCH_INFO(IMAGE, "image")
 PATCH_INFO(FIELD, "field")
 PATCH_INFO(VTABLE, "vtable")
-PATCH_INFO(CLASS_INIT, "class_init")
 PATCH_INFO(SFLDA, "sflda")
 PATCH_INFO(LDSTR, "ldstr")
 PATCH_INFO(LDTOKEN, "ldtoken")
@@ -36,13 +35,12 @@ PATCH_INFO(JIT_ICALL_ADDR, "jit_icall_addr")
 PATCH_INFO(INTERRUPTION_REQUEST_FLAG, "interruption_request_flag")
 PATCH_INFO(METHOD_RGCTX, "method_rgctx")
 PATCH_INFO(RGCTX_FETCH, "rgctx_fetch")
-PATCH_INFO(GENERIC_CLASS_INIT, "generic_class_init")
+PATCH_INFO(RGCTX_SLOT_INDEX, "rgctx_slot_index")
 PATCH_INFO(MONITOR_ENTER, "monitor_enter")
 PATCH_INFO(MONITOR_ENTER_V4, "monitor_enter_v4")
 PATCH_INFO(MONITOR_EXIT, "monitor_exit")
 PATCH_INFO(MSCORLIB_GOT_ADDR, "mscorlib_got_addr")
 PATCH_INFO(SEQ_POINT_INFO, "seq_point_info")
-PATCH_INFO(LLVM_IMT_TRAMPOLINE, "llvm_imt_trampoline")
 PATCH_INFO(GC_CARD_TABLE_ADDR, "gc_card_table_addr")
 PATCH_INFO(CASTCLASS_CACHE, "castclass_cache")
 PATCH_INFO(SIGNATURE, "signature")
@@ -53,4 +51,11 @@ PATCH_INFO(TLS_OFFSET, "tls_offset")
 PATCH_INFO(OBJC_SELECTOR_REF, "objc_selector_ref")
 PATCH_INFO(METHOD_CODE_SLOT, "method_code_slot")
 PATCH_INFO(LDSTR_LIT, "ldstr_lit")
+PATCH_INFO(GC_NURSERY_START, "gc_nursery_start")
+PATCH_INFO(VIRT_METHOD, "virt_method")
+PATCH_INFO(GC_SAFE_POINT_FLAG, "gc_safe_point_flag")
 PATCH_INFO(NONE, "none")
+PATCH_INFO(AOT_MODULE, "aot_module")
+PATCH_INFO(AOT_JIT_INFO, "aot_jit_info")
+PATCH_INFO(GC_NURSERY_BITS, "gc_nursery_bits")
+PATCH_INFO(GSHAREDVT_IN_WRAPPER, "gsharedvt_in_wrapper")
