@@ -21,6 +21,7 @@
 #include <mono/metadata/debug-helpers.h>
 #include <mono/metadata/assembly.h>
 #include <mono/utils/mono-time.h>
+#include <mono/utils/mono-memory-model.h>
 #include "trace.h"
 
 #if defined (PLATFORM_ANDROID) || (defined (TARGET_IOS) && defined (TARGET_IOS))
@@ -51,6 +52,8 @@
 
 static MonoTraceSpec trace_spec;
 
+static volatile gint32 output_lock = 0;
+
 gboolean
 mono_trace_eval_exception (MonoClass *klass)
 {
@@ -155,7 +158,7 @@ static void get_string (void)
 	}
 	if (value != NULL)
 		g_free (value);
-	value = g_malloc (input - start + 1);
+	value = (char *)g_malloc (input - start + 1);
 	strncpy (value, start, input-start);
 	value [input-start] = 0;
 }
@@ -357,7 +360,7 @@ static void indent (int diff) {
 		indent_level += diff;
 	if (start_time == 0)
 		start_time = mono_100ns_ticks ();
-	printf ("[%p: %.5f %d] ", (void*)GetCurrentThreadId (), seconds_since_start (), indent_level);
+	printf ("[%p: %.5f %d] ", (void*)mono_native_thread_id_get (), seconds_since_start (), indent_level);
 	if (diff > 0)
 		indent_level += diff;
 }
@@ -403,7 +406,7 @@ void
 mono_trace_enter_method (MonoMethod *method, char *ebp)
 {
 	int i, j;
-	MonoClass *class;
+	MonoClass *klass;
 	MonoObject *o;
 	MonoJitArgumentInfo *arg_info;
 	MonoMethodSignature *sig;
@@ -413,6 +416,9 @@ mono_trace_enter_method (MonoMethod *method, char *ebp)
 	if (!trace_spec.enabled)
 		return;
 
+	while (output_lock != 0 || InterlockedCompareExchange (&output_lock, 1, 0) != 0)
+		mono_thread_info_yield ();
+
 	fname = mono_method_full_name (method, TRUE);
 	indent (1);
 	printf ("ENTER: %s(", fname);
@@ -420,27 +426,27 @@ mono_trace_enter_method (MonoMethod *method, char *ebp)
 
 	if (!ebp) {
 		printf (") ip: %p\n", RETURN_ADDRESS_N (1));
-		return;
-	}	
+		goto unlock;
+	}
 
 	sig = mono_method_signature (method);
 
-	arg_info = alloca (sizeof (MonoJitArgumentInfo) * (sig->param_count + 1));
+	arg_info = (MonoJitArgumentInfo *)alloca (sizeof (MonoJitArgumentInfo) * (sig->param_count + 1));
 
 	if (method->is_inflated) {
 		/* FIXME: Might be better to pass the ji itself */
-		MonoJitInfo *ji = mini_jit_info_table_find (mono_domain_get (), RETURN_ADDRESS (), NULL);
+		MonoJitInfo *ji = mini_jit_info_table_find (mono_domain_get (), (char *)RETURN_ADDRESS (), NULL);
 		if (ji) {
 			gsctx = mono_jit_info_get_generic_sharing_context (ji);
-			if (gsctx && (gsctx->var_is_vt || gsctx->mvar_is_vt)) {
+			if (gsctx && gsctx->is_gsharedvt) {
 				/* Needs a ctx to get precise method */
 				printf (") <gsharedvt>\n");
-				return;
+				goto unlock;
 			}
 		}
 	}
 
-	mono_arch_get_argument_info (gsctx, sig, sig->param_count, arg_info);
+	mono_arch_get_argument_info (sig, sig->param_count, arg_info);
 
 	if (MONO_TYPE_ISSTRUCT (mono_method_signature (method)->ret)) {
 		g_assert (!mono_method_signature (method)->ret->byref);
@@ -449,23 +455,23 @@ mono_trace_enter_method (MonoMethod *method, char *ebp)
 	}
 
 	if (mono_method_signature (method)->hasthis) {
-		gpointer *this = (gpointer *)(ebp + arg_info [0].offset);
+		gpointer *this_obj = (gpointer *)(ebp + arg_info [0].offset);
 		if (method->klass->valuetype) {
-			printf ("value:%p, ", *arg_in_stack_slot(this, gpointer *));
+			printf ("value:%p, ", *arg_in_stack_slot(this_obj, gpointer *));
 		} else {
-			o = *arg_in_stack_slot(this, MonoObject *);
+			o = *arg_in_stack_slot(this_obj, MonoObject *);
 
 			if (o) {
-				class = o->vtable->klass;
+				klass = o->vtable->klass;
 
-				if (class == mono_defaults.string_class) {
+				if (klass == mono_defaults.string_class) {
 					MonoString *s = (MonoString*)o;
 					char *as = string_to_utf8 (s);
 
 					printf ("this:[STRING:%p:%s], ", o, as);
 					g_free (as);
 				} else {
-					printf ("this:%p[%s.%s %s], ", o, class->name_space, class->name, o->vtable->domain->friendly_name);
+					printf ("this:%p[%s.%s %s], ", o, klass->name_space, klass->name, o->vtable->domain->friendly_name);
 				}
 			} else 
 				printf ("this:NULL, ");
@@ -480,7 +486,7 @@ mono_trace_enter_method (MonoMethod *method, char *ebp)
 		
 		if (type->byref) {
 			printf ("[BYREF:%p], ", *arg_in_stack_slot(cpos, gpointer *));
-		} else switch (mono_type_get_underlying_type (type)->type) {
+		} else switch (mini_get_underlying_type (type)->type) {
 			
 		case MONO_TYPE_I:
 		case MONO_TYPE_U:
@@ -518,19 +524,19 @@ mono_trace_enter_method (MonoMethod *method, char *ebp)
 		case MONO_TYPE_OBJECT: {
 			o = *arg_in_stack_slot(cpos, MonoObject *);
 			if (o) {
-				class = o->vtable->klass;
+				klass = o->vtable->klass;
 		    
-				if (class == mono_defaults.string_class) {
+				if (klass == mono_defaults.string_class) {
 					char *as = string_to_utf8 ((MonoString*)o);
 
 					printf ("[STRING:%p:%s], ", o, as);
 					g_free (as);
-				} else if (class == mono_defaults.int32_class) {
+				} else if (klass == mono_defaults.int32_class) {
 					printf ("[INT32:%p:%d], ", o, *(gint32 *)((char *)o + sizeof (MonoObject)));
-				} else if (class == mono_defaults.monotype_class) {
+				} else if (klass == mono_defaults.monotype_class) {
 					printf ("[TYPE:%s], ", mono_type_full_name (((MonoReflectionType*)o)->type));
 				} else
-					printf ("[%s.%s:%p], ", class->name_space, class->name, o);
+					printf ("[%s.%s:%p], ", klass->name_space, klass->name, o);
 			} else {
 				printf ("%p, ", *arg_in_stack_slot(cpos, gpointer));
 			}
@@ -565,6 +571,9 @@ mono_trace_enter_method (MonoMethod *method, char *ebp)
 
 	printf (")\n");
 	fflush (stdout);
+
+unlock:
+	mono_atomic_store_release (&output_lock, 0);
 }
 
 void
@@ -578,6 +587,9 @@ mono_trace_leave_method (MonoMethod *method, ...)
 	if (!trace_spec.enabled)
 		return;
 
+	while (output_lock != 0 || InterlockedCompareExchange (&output_lock, 1, 0) != 0)
+		mono_thread_info_yield ();
+
 	va_start(ap, method);
 
 	fname = mono_method_full_name (method, TRUE);
@@ -587,20 +599,19 @@ mono_trace_leave_method (MonoMethod *method, ...)
 
 	if (method->is_inflated) {
 		/* FIXME: Might be better to pass the ji itself */
-		MonoJitInfo *ji = mini_jit_info_table_find (mono_domain_get (), RETURN_ADDRESS (), NULL);
+		MonoJitInfo *ji = mini_jit_info_table_find (mono_domain_get (), (char *)RETURN_ADDRESS (), NULL);
 		if (ji) {
 			gsctx = mono_jit_info_get_generic_sharing_context (ji);
-			if (gsctx && (gsctx->var_is_vt || gsctx->mvar_is_vt)) {
+			if (gsctx && gsctx->is_gsharedvt) {
 				/* Needs a ctx to get precise method */
 				printf (") <gsharedvt>\n");
-				return;
+				goto unlock;
 			}
 		}
 	}
 
-	type = mono_method_signature (method)->ret;
+	type = mini_get_underlying_type (mono_method_signature (method)->ret);
 
-handle_enum:
 	switch (type->type) {
 	case MONO_TYPE_VOID:
 		break;
@@ -682,20 +693,16 @@ handle_enum:
 		printf ("FP=%f", f);
 		break;
 	}
-	case MONO_TYPE_VALUETYPE: 
-		if (type->data.klass->enumtype) {
-			type = mono_class_enum_basetype (type->data.klass);
-			goto handle_enum;
-		} else {
-			guint8 *p = va_arg (ap, gpointer);
-			int j, size, align;
-			size = mono_type_size (type, &align);
-			printf ("[");
-			for (j = 0; p && j < size; j++)
-				printf ("%02x,", p [j]);
-			printf ("]");
-		}
+	case MONO_TYPE_VALUETYPE:  {
+		guint8 *p = (guint8 *)va_arg (ap, gpointer);
+		int j, size, align;
+		size = mono_type_size (type, &align);
+		printf ("[");
+		for (j = 0; p && j < size; j++)
+			printf ("%02x,", p [j]);
+		printf ("]");
 		break;
+	}
 	default:
 		printf ("(unknown return type %x)", mono_method_signature (method)->ret->type);
 	}
@@ -703,6 +710,9 @@ handle_enum:
 	//printf (" ip: %p\n", RETURN_ADDRESS_N (1));
 	printf ("\n");
 	fflush (stdout);
+
+unlock:
+	mono_atomic_store_release (&output_lock, 0);
 }
 
 void
