@@ -25,6 +25,7 @@
                    load/unload/name for assemblies
                removed TYPE_LOAD_ERR flag (profiler never generated it, now removed from the format itself)
                added TYPE_GC_HANDLE_{CREATED,DESTROYED}_BT
+               TYPE_JIT events are no longer guaranteed to have code start/size info (can be zero)
  */
 
 enum {
@@ -71,9 +72,11 @@ enum {
 	TYPE_CLAUSE       = 1 << 4,
 	TYPE_EXCEPTION_BT = 1 << 7,
 	/* extended type for TYPE_ALLOC */
-	TYPE_ALLOC_BT  = 1 << 4,
+	TYPE_ALLOC_NO_BT  = 0 << 4,
+	TYPE_ALLOC_BT     = 1 << 4,
 	/* extended type for TYPE_MONITOR */
-	TYPE_MONITOR_BT  = 1 << 7,
+	TYPE_MONITOR_NO_BT  = 0 << 7,
+	TYPE_MONITOR_BT     = 1 << 7,
 	/* extended type for TYPE_SAMPLE */
 	TYPE_SAMPLE_HIT           = 0 << 4,
 	TYPE_SAMPLE_USYM          = 1 << 4,
