@@ -133,18 +133,18 @@ namespace Mono.Tools.LocaleBuilder
 				Dump (writer, df.DayNames, "DayNames");
 				writer.WriteLine ("{0}: {1}", "FirstDayOfWeek", (DayOfWeek) df.FirstDayOfWeek);
 //				Dump (writer, df.GetAllDateTimePatterns (), "GetAllDateTimePatterns");
-				writer.WriteLine ("{0}: {1}", "LongDatePattern", df.LongDatePattern);
-				writer.WriteLine ("{0}: {1}", "LongTimePattern", df.LongTimePattern);
+//				writer.WriteLine ("{0}: {1}", "LongDatePattern", df.LongDatePattern);
+//				writer.WriteLine ("{0}: {1}", "LongTimePattern", df.LongTimePattern);
 				writer.WriteLine ("{0}: {1}", "MonthDayPattern", df.MonthDayPattern);
 				Dump (writer, df.MonthGenitiveNames, "MonthGenitiveNames");
 				Dump (writer, df.MonthNames, "MonthNames");
 				writer.WriteLine ("{0}: {1}", "NativeCalendarName", df.NativeCalendarName);
 				writer.WriteLine ("{0}: {1}", "PMDesignator", df.PMDesignator);
-				writer.WriteLine ("{0}: {1}", "ShortDatePattern", df.ShortDatePattern);
+//				writer.WriteLine ("{0}: {1}", "ShortDatePattern", df.ShortDatePattern);
 				Dump (writer, df.ShortestDayNames, "ShortestDayNames");
-				writer.WriteLine ("{0}: {1}", "ShortTimePattern", df.ShortTimePattern);
+//				writer.WriteLine ("{0}: {1}", "ShortTimePattern", df.ShortTimePattern);
 				writer.WriteLine ("{0}: {1}", "TimeSeparator", df.TimeSeparator);
-				writer.WriteLine ("{0}: {1}", "YearMonthPattern", df.YearMonthPattern);
+//				writer.WriteLine ("{0}: {1}", "YearMonthPattern", df.YearMonthPattern);
 
 				var ti = c.TextInfoEntry;
 				writer.WriteLine ("-- TextInfo --");
@@ -924,9 +924,6 @@ namespace Mono.Tools.LocaleBuilder
 				}
 			}
 
-			// It looks like it never changes
-			data.DateTimeFormatEntry.TimeSeparator = ":";
-
 			// TODO: Don't have input data available but most values are 2 with few exceptions for 1 and 3
 			// We don't add 3 as it's for some arabic states only
 			switch (data.ThreeLetterISOLanguageName) {
@@ -1130,13 +1127,16 @@ namespace Mono.Tools.LocaleBuilder
 				if (el != null) {
 					// CLDR uses unicode negative sign for some culture (e.g sv, is, lt, don't kwnow why) but .net always
 					// uses simple - sign
-					if (el.InnerText == "\u2212") {
+					switch (el.InnerText) {
+					case "\u2212":
+					case "\u200F\u002D": // Remove any right-to-left mark characters
+					case "\u200E\u002D":
 						ni.NegativeSign = "-";
-					} else if (el.InnerText ==  "\u200F\u002D") {
-						// Remove any right-to-left mark characters
-						ni.NegativeSign = "-";
-					} else
-						ni.NegativeSign = el.InnerText;					
+						break;
+					default:
+						ni.NegativeSign = el.InnerText;
+						break;
+					}
 				}
 
 				el = node.SelectSingleNode ("infinity");
@@ -1170,6 +1170,7 @@ namespace Mono.Tools.LocaleBuilder
 			// .net has incorrect separators for some countries and we want to be compatible
 			switch (ci.Name) {
 			case "es-ES":
+			case "es":
 				// es-ES does not have group separator but .net has '.'
 				value = ".";
 				break;
